SET AUTOCOMMIT=0;
SET FOREIGN_KEY_CHECKS=0;
-- List of users
-- GROUP: sysman
CREATE TABLE s_user (

    -- This identifies who the current user is
  s_user_id_c varchar(20) NOT NULL,
  s_email_address_c varchar(100),
  s_first_name_c varchar(64),
  s_last_name_c varchar(64),
    -- Hash value of the user's Salted password
  s_password_hash_c varchar(64),
    -- The Salt value used for hashing the password (two identical passwords will not yield the same Hash value with different Salt values)
  s_password_salt_c varchar(64),
    -- Version Number of the password hashing scheme (source code gives details about what algorithm/parameters combination is used for each number).
  s_pwd_scheme_version_i integer DEFAULT 1,
    -- Forces change of password at next login. Set when the SYSADMIN changes a password for a user. Also, data migration from legacy system will set this. Further to this it can *optionally* be set if a new password hashing scheme got introduced and the user should be forced to change to it by means of an enforced password change.
  s_password_needs_change_l boolean DEFAULT 0,
  s_failed_logins_i integer DEFAULT 0,
  s_account_locked_l boolean DEFAULT 0,
  s_retired_l boolean DEFAULT 0,
  s_last_login_time_i integer DEFAULT 0,
    -- The date the user last logged in.
  s_last_login_date_d date,
    -- This is the code used to identify a language.
  s_language_code_c varchar(20) DEFAULT '99',
    -- This defines if the code can be modified
  s_can_modify_l boolean DEFAULT 1,
  s_record_delete_l boolean DEFAULT 0,
    -- This code identifies the method of aquisition.
  s_acquisition_code_c varchar(16),
    -- This is used as a key field in most of the accounting system files
  a_default_ledger_number_i integer DEFAULT 0,
    -- The last time a user failed to log in
  s_failed_login_time_i integer DEFAULT 0,
    -- The last date a user failed to log in.
  s_failed_login_date_d date,
    -- If the user has a Partner record this is the key to it
  p_partner_key_n bigint DEFAULT NULL,
    -- If this token is set and it is still valid, then the user can reset his password using this token
  s_password_reset_token_c varchar(64),
    -- The date until the password reset token is valid
  s_password_reset_valid_until_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_pk
    PRIMARY KEY (s_user_id_c),
  CONSTRAINT s_user_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
KEY inx_p_language_fkcr_ref0 
   (s_user_id_c),
UNIQUE KEY inx_s_user_pk1 
   (s_user_id_c),
KEY inx_s_user_fk1_key2 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of language codes
-- GROUP: common
CREATE TABLE p_language (

    -- This is the code used to identify a language.
  p_language_code_c varchar(20) NOT NULL,
  p_language_description_c varchar(80) NOT NULL,
    -- This field indicates whether or not the language is one that is 'officially' used at conferences. These are the languages for which translation could be provided. 
  p_congress_language_l boolean DEFAULT 0,
    -- This defines if the language code can be deleted.
  p_deletable_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_language_pk
    PRIMARY KEY (p_language_code_c),
  CONSTRAINT p_language_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_language_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_language_pk0 
   (p_language_code_c),
KEY inx_p_language_fkcr_key1 
   (s_created_by_c),
KEY inx_p_language_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Units of time. Used in partner letters.  Also used to indicate how often a publication is produced or a receipt is sent to a donor.
-- GROUP: common
CREATE TABLE a_frequency (

  a_frequency_code_c varchar(24) NOT NULL,
  a_frequency_description_c varchar(64) NOT NULL,
  a_number_of_years_i integer DEFAULT 0 NOT NULL,
  a_number_of_months_i integer DEFAULT 0,
  a_number_of_days_i integer DEFAULT 0,
  a_number_of_hours_i integer DEFAULT 0,
  a_number_of_minutes_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_frequency_pk
    PRIMARY KEY (a_frequency_code_c),
  CONSTRAINT a_frequency_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_frequency_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_frequency_pk0 
   (a_frequency_code_c),
KEY inx_a_frequency_fkcr_key1 
   (s_created_by_c),
KEY inx_a_frequency_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Post office mailing zone classification
-- GROUP: common
CREATE TABLE p_international_postal_type (

  p_internat_postal_type_code_c varchar(16) NOT NULL,
  p_description_c varchar(64) NOT NULL,
    -- This defines if the international postal type code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_international_postal_type_pk
    PRIMARY KEY (p_internat_postal_type_code_c),
  CONSTRAINT p_international_postal_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_international_postal_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_international_postal_type_pk0 
   (p_internat_postal_type_code_c),
KEY inx_ational_postal_type_fkcr_key1 
   (s_created_by_c),
KEY inx_ational_postal_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of countries with their codes
-- GROUP: common
CREATE TABLE p_country (

    -- This is a code which identifies a country. It is the ISO code (ISO 3166)
  p_country_code_c varchar(8) NOT NULL,
    -- The name of the country
  p_country_name_c varchar(80) NOT NULL,
    -- Describes if the country is politically sensitive.
  p_undercover_l boolean DEFAULT 0,
    -- The telephone code needed to dial into a country
  p_internat_telephone_code_i integer DEFAULT 0,
  p_internat_postal_type_code_c varchar(16),
    -- The code needed to dial out of a country.
  p_internat_access_code_c varchar(8),
    -- Number of hours +/- GMT
  p_time_zone_minimum_n numeric(6, 2) DEFAULT 0,
    -- Number of hours +/- GMT
  p_time_zone_maximum_n numeric(6, 2) DEFAULT 0,
    -- This defines if the country code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- Tab order of the city, county, and post code fields on the Partner Edit screen
  p_address_order_i integer DEFAULT 0,
    -- The name of the country in the Local language
  p_country_name_local_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_country_pk
    PRIMARY KEY (p_country_code_c),
  CONSTRAINT p_country_fk1
    FOREIGN KEY (p_internat_postal_type_code_c)
    REFERENCES p_international_postal_type(p_internat_postal_type_code_c),
  CONSTRAINT p_country_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_country_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_country_pk0 
   (p_country_code_c),
KEY inx_p_country_fk1_key1 
   (p_internat_postal_type_code_c),
KEY inx_p_country_fkcr_key2 
   (s_created_by_c),
KEY inx_p_country_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Unit of money for various countries.
-- GROUP: common
CREATE TABLE a_currency (

    -- This defines which currency is being used
  a_currency_code_c varchar(16) NOT NULL,
    -- This is the name of the currency
  a_currency_name_c varchar(64) NOT NULL,
    -- This is the symbol used to show a currency. Eg $US or £
  a_currency_symbol_c varchar(8) NOT NULL,
    -- Country code
  p_country_code_c varchar(8) NOT NULL,
    -- The format in which to display and accept input on a currency (decimal values)
  a_display_format_c varchar(40) DEFAULT '->>>,>>>,>>>,>>9.99' NOT NULL,
    -- Indicates whether currency is part of the european exchange rate mechanism/ European Monetary Union
  a_in_emu_l boolean DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_currency_pk
    PRIMARY KEY (a_currency_code_c),
  CONSTRAINT a_currency_fk1
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT a_currency_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_currency_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_currency_pk0 
   (a_currency_code_c),
KEY inx_a_currency_fk1_key1 
   (p_country_code_c),
KEY inx_a_currency_fkcr_key2 
   (s_created_by_c),
KEY inx_a_currency_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Sessions
-- GROUP: sysman
CREATE TABLE s_session (

    -- This is the identifier of the session
  s_session_id_c varchar(128) NOT NULL,
    -- This session is valid till this point in time
  s_valid_until_d datetime NOT NULL,
    -- JSON encoded list of session values.
  s_session_values_c text,
    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_session_pk
    PRIMARY KEY (s_session_id_c),
  CONSTRAINT s_session_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_session_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_session_pk0 
   (s_session_id_c),
KEY inx_s_session_fkcr_key1 
   (s_created_by_c),
KEY inx_s_session_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Report Results
-- GROUP: sysman
CREATE TABLE s_report_result (

    -- This is the identifier of the report result
  s_report_id_c varchar(192) NOT NULL,
    -- This is the identifier of the session
  s_session_id_c varchar(128) NOT NULL,
    -- This session is valid till this point in time
  s_valid_until_d datetime NOT NULL,
    -- Parameters list in json
  s_parameter_list_c text,
    -- result represented in HTML
  s_result_html_c longtext,
    -- Did the report finish successfully to be calculated?
  s_success_l boolean DEFAULT 0,
    -- in case of failure, this contains the error message
  s_error_message_c text,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_report_result_pk
    PRIMARY KEY (s_report_id_c),
  CONSTRAINT s_report_result_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_report_result_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_report_result_pk0 
   (s_report_id_c),
KEY inx_s_report_result_fkcr_key1 
   (s_created_by_c),
KEY inx_s_report_result_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Logs activity on user accounts themselves (user logins and logouts are recorded in s_login)
-- GROUP: sysman
CREATE TABLE s_user_account_activity (

    -- User for which the user account activity got recorded.
  s_user_id_c varchar(20) NOT NULL,
    -- Date of the recorded user account activity.
  s_activity_date_d date NOT NULL,
    -- Time of the recorded user account activity.
  s_activity_time_i integer DEFAULT 0 NOT NULL,
    -- Type of the recorded account activity. This is a hard-coded constant value (there's no 'lookup table' for it); for available values and their meaning please check program code (TUserAccountActivityLog Class).
  s_activity_type_c varchar(50) NOT NULL,
    -- Details/description of the recorded account activity. This is a localised string, i.e. it can be recorded in the language of the Site! Refer to s_activity_type_c for exact identification of what the recorded account activity is about if text in here can't be understood because it is recorded in a foreign language.
  s_activity_details_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_account_activity_pk
    PRIMARY KEY (s_user_id_c,s_activity_date_d,s_activity_time_i,s_activity_type_c),
  CONSTRAINT s_user_account_activity_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_account_activity_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_user_account_activity_pk0 
   (s_user_id_c,s_activity_date_d,s_activity_time_i,s_activity_type_c),
KEY inx_er_account_activity_fkcr_key1 
   (s_created_by_c),
KEY inx_er_account_activity_fkmd_key2 
   (s_modified_by_c),
KEY s_user_account_activity_date0 
   (s_activity_date_d),
KEY s_user_account_activity_user1 
   (s_user_id_c),
KEY s_user_account_activity_type2 
   (s_activity_type_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of forms for printers
-- GROUP: sysman
CREATE TABLE s_form (

    -- This identifies the form
  s_form_name_c varchar(20) NOT NULL,
    -- This is a description of the form
  s_form_description_c varchar(80) NOT NULL,
  s_default_font_c varchar(64),
    -- Default font size (points)
  s_default_font_size_i integer DEFAULT 0 NOT NULL,
    -- Number of lines per unit of measure
  s_default_lpi_i integer DEFAULT 0 NOT NULL,
    -- Number of characters per unit of measure
  s_default_cpi_i integer DEFAULT 0 NOT NULL,
    -- Height of the page
  s_form_length_n numeric(6, 3) DEFAULT 0 NOT NULL,
    -- Width of the page
  s_form_width_n numeric(6, 3) DEFAULT 0 NOT NULL,
  s_form_orientation_c varchar(2) DEFAULT 'P',
    -- Unit of measure for the form.  True if inches, false if centimeters.
  s_unit_of_measure_l boolean DEFAULT 0,
    -- Top margin
  s_top_margin_n numeric(6, 3) DEFAULT 0,
    -- Bottom margin
  s_bottom_margin_n numeric(6, 3) DEFAULT 0,
    -- Left margin
  s_left_margin_n numeric(6, 3) DEFAULT 0,
    -- Right margin
  s_right_margin_n numeric(6, 3) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_form_pk
    PRIMARY KEY (s_form_name_c),
  CONSTRAINT s_form_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_form_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_form_pk0 
   (s_form_name_c),
KEY inx_s_form_fkcr_key1 
   (s_created_by_c),
KEY inx_s_form_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of groups to which users can belong
-- GROUP: sysman
CREATE TABLE s_group (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Describes a group
  s_group_name_c varchar(100),
    -- This defines if the code can be modified
  s_can_modify_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_pk
    PRIMARY KEY (s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_pk0 
   (s_group_id_c,s_unit_key_n),
KEY inx_s_group_fkcr_key1 
   (s_created_by_c),
KEY inx_s_group_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings of users to groups
-- GROUP: sysman
CREATE TABLE s_user_group (

    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_group_pk
    PRIMARY KEY (s_user_id_c,s_group_id_c,s_unit_key_n),
  CONSTRAINT s_user_group_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_group_fk2
    FOREIGN KEY (s_group_id_c,s_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_user_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_user_group_pk0 
   (s_user_id_c,s_group_id_c,s_unit_key_n),
KEY inx_s_user_group_fk1_key1 
   (s_user_id_c),
KEY inx_s_user_group_fk2_key2 
   (s_group_id_c,s_unit_key_n),
KEY inx_s_user_group_fkcr_key3 
   (s_created_by_c),
KEY inx_s_user_group_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of Petra software modules
-- GROUP: sysman
CREATE TABLE s_module (

    -- Identifies a module. A module is any part of aprogram which is related to each menu entry or to the sub-system. Eg, partner administration, AP, AR etc.
  s_module_id_c varchar(20) NOT NULL,
    -- This is the name of the module
  s_module_name_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_module_pk
    PRIMARY KEY (s_module_id_c),
  CONSTRAINT s_module_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_module_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_module_pk0 
   (s_module_id_c),
KEY inx_s_module_fkcr_key1 
   (s_created_by_c),
KEY inx_s_module_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists printer forms that are valid for each Petra module.
-- GROUP: sysman
CREATE TABLE s_valid_output_form (

  s_module_id_c varchar(20) NOT NULL,
  s_form_name_c varchar(20) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_valid_output_form_pk
    PRIMARY KEY (s_module_id_c,s_form_name_c),
  CONSTRAINT s_valid_output_form_fk1
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT s_valid_output_form_fk2
    FOREIGN KEY (s_form_name_c)
    REFERENCES s_form(s_form_name_c),
  CONSTRAINT s_valid_output_form_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_valid_output_form_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_valid_output_form_pk0 
   (s_module_id_c,s_form_name_c),
KEY inx_s_valid_output_form_fk1_key1 
   (s_module_id_c),
KEY inx_s_valid_output_form_fk2_key2 
   (s_form_name_c),
KEY inx_s_valid_output_form_fkcr_key3 
   (s_created_by_c),
KEY inx_s_valid_output_form_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings from groups to Petra modules
-- GROUP: sysman
CREATE TABLE s_group_module_access_permission (

  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
  s_module_id_c varchar(20) NOT NULL,
    -- Permission to access this module
  s_can_access_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_module_acc_perm_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,s_module_id_c),
  CONSTRAINT s_group_module_acc_perm_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_module_acc_perm_fk2
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT s_group_module_access_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_module_access_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_module_acc_perm_pk0 
   (s_group_id_c,s_group_unit_key_n,s_module_id_c),
KEY inx_roup_module_acc_perm_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_roup_module_acc_perm_fk2_key2 
   (s_module_id_c),
KEY inx_e_access_permission_fkcr_key3 
   (s_created_by_c),
KEY inx_e_access_permission_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings from groups to Petra database
-- GROUP: sysman
CREATE TABLE s_group_table_access_permission (

    -- This identifies the system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
  s_table_name_c varchar(64) NOT NULL,
    -- Permission to allow creation.
  s_can_create_l boolean DEFAULT 1,
    -- Permission to allow modification.
  s_can_modify_l boolean DEFAULT 1,
    -- Permission to allow deletion.
  s_can_delete_l boolean DEFAULT 1,
    -- Permission to allow inquiry.
  s_can_inquire_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_table_acc_perm_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,s_table_name_c),
  CONSTRAINT s_group_table_acc_perm_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_table_access_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_table_access_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_table_acc_perm_pk0 
   (s_group_id_c,s_group_unit_key_n,s_table_name_c),
KEY inx_group_table_acc_perm_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_e_access_permission_fkcr_ke22 
   (s_created_by_c),
KEY inx_e_access_permission_fkmd_ke23 
   (s_modified_by_c),
KEY gtap_OnTableName0 
   (s_table_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings from modules to Petra database
-- GROUP: sysman
CREATE TABLE s_module_table_access_permission (

  s_module_id_c varchar(20) NOT NULL,
  s_table_name_c varchar(64) NOT NULL,
    -- Permission to allow creation.
  s_can_create_l boolean DEFAULT 1,
    -- Permission to allow modification.
  s_can_modify_l boolean DEFAULT 1,
    -- Permission to allow deletion.
  s_can_delete_l boolean DEFAULT 1,
    -- Permission to allow inquiry.
  s_can_inquire_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_module_table_acc_perm_pk
    PRIMARY KEY (s_module_id_c,s_table_name_c),
  CONSTRAINT s_module_table_acc_perm_fk1
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT s_module_table_access_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_module_table_access_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_module_table_acc_perm_pk0 
   (s_module_id_c,s_table_name_c),
KEY inx_odule_table_acc_perm_fk1_key1 
   (s_module_id_c),
KEY inx_e_access_permission_fkcr_ke32 
   (s_created_by_c),
KEY inx_e_access_permission_fkmd_ke33 
   (s_modified_by_c),
KEY mtap_OnTableName0 
   (s_table_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings of users to Petra modules
-- GROUP: sysman
CREATE TABLE s_user_module_access_permission (

    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  s_module_id_c varchar(20) NOT NULL,
    -- Permission to access this module
  s_can_access_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_module_acc_perm_pk
    PRIMARY KEY (s_user_id_c,s_module_id_c),
  CONSTRAINT s_user_module_acc_perm_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_module_acc_perm_fk2
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT s_user_module_access_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_module_access_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_user_module_acc_perm_pk0 
   (s_user_id_c,s_module_id_c),
KEY inx_user_module_acc_perm_fk1_key1 
   (s_user_id_c),
KEY inx_user_module_acc_perm_fk2_key2 
   (s_module_id_c),
KEY inx_e_access_permission_fkcr_ke43 
   (s_created_by_c),
KEY inx_e_access_permission_fkmd_ke44 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Security mappings of users to Petra database
-- GROUP: sysman
CREATE TABLE s_user_table_access_permission (

    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  s_table_name_c varchar(64) NOT NULL,
    -- Permission to allow creation.
  s_can_create_l boolean DEFAULT 1,
    -- Permission to allow modification.
  s_can_modify_l boolean DEFAULT 1,
    -- Permission to allow deletion.
  s_can_delete_l boolean DEFAULT 1,
    -- Permission to allow inquiry.
  s_can_inquire_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_table_acc_perm_pk
    PRIMARY KEY (s_user_id_c,s_table_name_c),
  CONSTRAINT s_user_table_acc_perm_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_table_access_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_table_access_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_user_table_acc_perm_pk0 
   (s_user_id_c,s_table_name_c),
KEY inx__user_table_acc_perm_fk1_key1 
   (s_user_id_c),
KEY inx_e_access_permission_fkcr_ke52 
   (s_created_by_c),
KEY inx_e_access_permission_fkmd_ke53 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Definitions of fields that are language specific.
-- GROUP: sysman
CREATE TABLE s_language_specific (

    -- This is the code used to identify a language.
  s_language_code_c varchar(20) DEFAULT '99' NOT NULL,
    -- The language specific month name 1.
  s_month_name_c_1 varchar(40) NOT NULL,
    -- The language specific month name 2.
  s_month_name_c_2 varchar(40) NOT NULL,
    -- The language specific month name 3.
  s_month_name_c_3 varchar(40) NOT NULL,
    -- The language specific month name 4.
  s_month_name_c_4 varchar(40) NOT NULL,
    -- The language specific month name 5.
  s_month_name_c_5 varchar(40) NOT NULL,
    -- The language specific month name 6.
  s_month_name_c_6 varchar(40) NOT NULL,
    -- The language specific month name 7.
  s_month_name_c_7 varchar(40) NOT NULL,
    -- The language specific month name 8.
  s_month_name_c_8 varchar(40) NOT NULL,
    -- The language specific month name 9.
  s_month_name_c_9 varchar(40) NOT NULL,
    -- The language specific month name 10.
  s_month_name_c_10 varchar(40) NOT NULL,
    -- The language specific month name 11.
  s_month_name_c_11 varchar(40) NOT NULL,
    -- The language specific month name 12.
  s_month_name_c_12 varchar(40) NOT NULL,
    -- The language specific short month name 1.
  s_month_name_short_c_1 varchar(6) NOT NULL,
    -- The language specific short month name 2.
  s_month_name_short_c_2 varchar(6) NOT NULL,
    -- The language specific short month name 3.
  s_month_name_short_c_3 varchar(6) NOT NULL,
    -- The language specific short month name 4.
  s_month_name_short_c_4 varchar(6) NOT NULL,
    -- The language specific short month name 5.
  s_month_name_short_c_5 varchar(6) NOT NULL,
    -- The language specific short month name 6.
  s_month_name_short_c_6 varchar(6) NOT NULL,
    -- The language specific short month name 7.
  s_month_name_short_c_7 varchar(6) NOT NULL,
    -- The language specific short month name 8.
  s_month_name_short_c_8 varchar(6) NOT NULL,
    -- The language specific short month name 9.
  s_month_name_short_c_9 varchar(6) NOT NULL,
    -- The language specific short month name 10.
  s_month_name_short_c_10 varchar(6) NOT NULL,
    -- The language specific short month name 11.
  s_month_name_short_c_11 varchar(6) NOT NULL,
    -- The language specific short month name 12.
  s_month_name_short_c_12 varchar(6) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_language_specific_pk
    PRIMARY KEY (s_language_code_c),
  CONSTRAINT s_language_specific_fk1
    FOREIGN KEY (s_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT s_language_specific_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_language_specific_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_language_specific_pk0 
   (s_language_code_c),
KEY inx_s_language_specific_fkcr_key1 
   (s_created_by_c),
KEY inx_s_language_specific_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Log of all the log-ins/log-in attempts to the system, and of log-outs from the system (activities on user accounts themselves are recorded in s_user_account_activity).
-- GROUP: sysman
CREATE TABLE s_login (

    -- OpenPetra process ID; this is a unique key
  s_login_process_id_r integer NOT NULL,
    -- This is the system user id. Each user of the system is allocated one.
  s_user_id_c varchar(20) NOT NULL,
    -- Time of the login/login attempt/logout.
  s_time_i integer DEFAULT 0 NOT NULL,
    -- Date of the login/login attempt/logout.
  s_date_d date NOT NULL,
    -- Type of the login/logout record. This is a hard-coded constant value (there's no 'lookup table' for it); for available values and their meaning please check program code (TLoginLog Class).
  s_login_type_c varchar(50) NOT NULL,
    -- Details/description of the login/login attempt/logout. This is a localised string, i.e. it can be recorded in the language of the Site! Refer to s_login_type_c for exact identification of what the recorded account activity is about if text in here can't be understood because it is recorded in a foreign language.
  s_login_details_c varchar(500),
    -- Reference to s_login_process_id_r (OpenPetra process ID of the login record) - only set for a record of s_login_status_type_c 'LOGOUT' (connects the logout log entry with the corresponding login log entry.)
  s_login_process_id_ref_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_login_pk
    PRIMARY KEY (s_login_process_id_r),
  CONSTRAINT s_login_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_login_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_login_pk0 
   (s_login_process_id_r),
KEY inx_s_login_fkcr_key1 
   (s_created_by_c),
KEY inx_s_login_fkmd_key2 
   (s_modified_by_c),
KEY s_date0 
   (s_date_d),
KEY s_login_user1 
   (s_user_id_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of logon messages by language
-- GROUP: sysman
CREATE TABLE s_logon_message (

    -- This is the code used to identify a language.
  s_language_code_c varchar(20) DEFAULT '99' NOT NULL,
    -- Message displayed when a user logs onto to the system
  s_logon_message_c varchar(300),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_logon_message_pk
    PRIMARY KEY (s_language_code_c),
  CONSTRAINT s_logon_message_fk1
    FOREIGN KEY (s_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT s_logon_message_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_logon_message_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_logon_message_pk0 
   (s_language_code_c),
KEY inx_s_logon_message_fkcr_key1 
   (s_created_by_c),
KEY inx_s_logon_message_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Logs each patch procedure that is run
-- GROUP: sysman
CREATE TABLE s_patch_log (

  s_patch_name_c varchar(24) NOT NULL,
    -- The user who ran the patch
  s_user_id_c varchar(20),
    -- The date the patch was run.
  s_date_run_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_patch_log_pk
    PRIMARY KEY (s_patch_name_c),
  CONSTRAINT s_patch_log_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_patch_log_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_patch_log_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_patch_log_pk0 
   (s_patch_name_c),
KEY inx_s_patch_log_fk1_key1 
   (s_user_id_c),
KEY inx_s_patch_log_fkcr_key2 
   (s_created_by_c),
KEY inx_s_patch_log_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- filesystem for storage of XML report templates - those provided initially and also tweaks saved by users.
-- GROUP: sysman
CREATE TABLE s_report_template (

    -- Template Id
  s_template_id_i integer NOT NULL,
    -- Report Type
  s_report_type_c varchar(100) NOT NULL,
    -- Report Title
  s_report_variant_c varchar(100) NOT NULL,
    -- Tempate author
  s_author_c varchar(100) NOT NULL,
    -- Template will be selected by default
  s_default_l boolean DEFAULT 0 NOT NULL,
    -- Template only available to me
  s_private_l boolean DEFAULT 0 NOT NULL,
    -- Private template will be selected over public default
  s_private_default_l boolean DEFAULT 0 NOT NULL,
    -- Template cannot be modified
  s_readonly_l boolean DEFAULT 0 NOT NULL,
    -- Compressed XML text
  s_xml_text_c text NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_report_templates_pk
    PRIMARY KEY (s_template_id_i),
  CONSTRAINT s_report_template_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_report_template_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_report_templates_pk0 
   (s_template_id_i),
KEY inx_s_report_template_fkcr_key1 
   (s_created_by_c),
KEY inx_s_report_template_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains the titles of reports that should be archived, not deleted, when Purge Reports is run.
-- GROUP: sysman
CREATE TABLE s_reports_to_archive (

    -- Title of the Report
  s_report_title_c varchar(100) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_reports_to_archive_pk
    PRIMARY KEY (s_report_title_c),
  CONSTRAINT s_reports_to_archive_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_reports_to_archive_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_reports_to_archive_pk0 
   (s_report_title_c),
KEY inx__reports_to_archive_fkcr_key1 
   (s_created_by_c),
KEY inx__reports_to_archive_fkmd_key2 
   (s_modified_by_c),
KEY s_report_title_k0 
   (s_report_title_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Records current status of system (up, maintenance etc.)
-- GROUP: sysman
CREATE TABLE s_system_status (

    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  s_system_disabled_date_d date,
  s_system_disabled_time_i integer DEFAULT 0,
  s_system_disabled_reason_c varchar(160),
  s_system_available_date_d date,
  s_system_available_time_i integer DEFAULT 0,
    -- The log in status of the system.
  s_system_login_status_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_system_status_pk
    PRIMARY KEY (s_user_id_c),
  CONSTRAINT s_system_status_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_system_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_system_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_system_status_pk0 
   (s_user_id_c),
KEY inx_s_system_status_fkcr_key1 
   (s_created_by_c),
KEY inx_s_system_status_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores various default values and options for each user.
-- GROUP: sysman
CREATE TABLE s_user_defaults (

    -- This identifies who the current user is
  s_user_id_c varchar(20) NOT NULL,
    -- The name of the default
  s_default_code_c varchar(100) NOT NULL,
    -- The value of the default
  s_default_value_c varchar(500),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_user_defaults_pk
    PRIMARY KEY (s_user_id_c,s_default_code_c),
  CONSTRAINT s_user_defaults_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_defaults_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_user_defaults_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_user_defaults_pk0 
   (s_user_id_c,s_default_code_c),
KEY inx_s_user_defaults_fk1_key1 
   (s_user_id_c),
KEY inx_s_user_defaults_fkcr_key2 
   (s_created_by_c),
KEY inx_s_user_defaults_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The settings that are system wide (iso per user)
-- GROUP: sysman
CREATE TABLE s_system_defaults (

  s_default_code_c varchar(100) NOT NULL,
    -- Default Code in easy to read format
  s_default_code_local_c varchar(100),
    -- Default Code in easy to read format in English
  s_default_code_intl_c varchar(100),
    -- Description of what this code does
  s_default_description_c varchar(500),
    -- Category of code (e.g. Partner, Gift)
  s_category_c varchar(100),
  s_default_value_c varchar(500),
    -- Determines if the value can be edited in the GUI
  s_read_only_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_system_defaults_pk
    PRIMARY KEY (s_default_code_c),
  CONSTRAINT s_system_defaults_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_system_defaults_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_system_defaults_pk0 
   (s_default_code_c),
KEY inx_s_system_defaults_fkcr_key1 
   (s_created_by_c),
KEY inx_s_system_defaults_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Describes the GUI controls for editing the default values
-- GROUP: sysman
CREATE TABLE s_system_defaults_gui (

  s_default_code_c varchar(100) NOT NULL,
    -- An ID for the control array that determines the display order
  s_control_id_i integer NOT NULL,
    -- Label to display
  s_control_label_c varchar(100) NOT NULL,
    -- Type of control (e.g. txt, cmb, dtp)
  s_control_type_c varchar(20) NOT NULL,
    -- Optional values that apply to a control of type cmb
  s_control_optional_values_c varchar(200),
    -- List of comma separated additional control attributes, e.g. Width, Type etc.
  s_control_attributes_c varchar(500),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_system_defaults_gui_pk
    PRIMARY KEY (s_default_code_c,s_control_id_i),
  CONSTRAINT s_system_defaults_gui_fk1
    FOREIGN KEY (s_default_code_c)
    REFERENCES s_system_defaults(s_default_code_c),
  CONSTRAINT s_system_defaults_gui_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_system_defaults_gui_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_system_defaults_gui_pk0 
   (s_default_code_c,s_control_id_i),
KEY inx__system_defaults_gui_fk1_key1 
   (s_default_code_c),
KEY inx_system_defaults_gui_fkcr_key2 
   (s_created_by_c),
KEY inx_system_defaults_gui_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Log of captured runtime errors
-- GROUP: sysman
CREATE TABLE s_error_log (

  s_error_code_c varchar(12) NOT NULL,
    -- This is the software release number
  s_release_number_c varchar(24) NOT NULL,
    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  s_file_name_c varchar(80) NOT NULL,
  s_process_id_c varchar(16),
  s_date_d date NOT NULL,
  s_time_i integer DEFAULT 0 NOT NULL,
  s_message_line_1_c varchar(120) NOT NULL,
  s_message_line_2_c varchar(80),
  s_message_line_3_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_error_log_pk
    PRIMARY KEY (s_error_code_c,s_user_id_c,s_date_d,s_time_i),
  CONSTRAINT s_error_log_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_error_log_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_error_log_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_error_log_pk0 
   (s_error_code_c,s_user_id_c,s_date_d,s_time_i),
KEY inx_s_error_log_fk2_key1 
   (s_user_id_c),
KEY inx_s_error_log_fkcr_key2 
   (s_created_by_c),
KEY inx_s_error_log_fkmd_key3 
   (s_modified_by_c),
KEY s_error_date0 
   (s_date_d),
KEY s_error_no1 
   (s_error_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of statuses for partners
-- GROUP: partner
CREATE TABLE p_partner_status (

    -- This code describes the status of a partner. Eg,  Active, Deceased etc
  p_status_code_c varchar(16) NOT NULL,
  p_partner_status_description_c varchar(120) NOT NULL,
    -- If set to yes, then Partners with this status are considered as active Partners
  p_partner_is_active_l boolean DEFAULT 1,
  p_include_partner_on_report_l boolean DEFAULT 0,
    -- This defines if the partner status code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_status_pk
    PRIMARY KEY (p_status_code_c),
  CONSTRAINT p_partner_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_status_pk0 
   (p_status_code_c),
KEY inx_p_partner_status_fkcr_key1 
   (s_created_by_c),
KEY inx_p_partner_status_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table is used to describe how a partner first came into contact with the organisation.  Which department entered them.
-- GROUP: partner
CREATE TABLE p_acquisition (

    -- This code identifies the method of aquisition.
  p_acquisition_code_c varchar(16) NOT NULL,
    -- This describes the method of aquisition.
  p_acquisition_description_c varchar(160) NOT NULL,
    -- Defines if the acquisition code is still for use
  p_valid_acquisition_l boolean DEFAULT 1,
    -- This defines if the acquisition code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- This defines if the acquisition code listed represents a recruiting effort.
  p_recruiting_effort_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_acquisition_pk
    PRIMARY KEY (p_acquisition_code_c),
  CONSTRAINT p_acquisition_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_acquisition_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_acquisition_pk0 
   (p_acquisition_code_c),
KEY inx_p_acquisition_fkcr_key1 
   (s_created_by_c),
KEY inx_p_acquisition_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Ex. Fam - Family, SM - Single Male, etc.
-- GROUP: partner
CREATE TABLE p_addressee_type (

  p_addressee_type_code_c varchar(24) NOT NULL,
  p_description_c varchar(64) NOT NULL,
    -- This defines if the addressee type code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_addressee_type_pk
    PRIMARY KEY (p_addressee_type_code_c),
  CONSTRAINT p_addressee_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_addressee_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_addressee_type_pk0 
   (p_addressee_type_code_c),
KEY inx_p_addressee_type_fkcr_key1 
   (s_created_by_c),
KEY inx_p_addressee_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Titles available for use
-- GROUP: partner
CREATE TABLE p_title (

    -- Title
  p_title_c varchar(64) NOT NULL,
    -- Default addressee type code to be used for this title
  p_default_addressee_type_code_c varchar(24),
    -- This defines if the title is a common one to be used more often in this system
  p_common_title_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_title_pk
    PRIMARY KEY (p_title_c),
  CONSTRAINT p_title_fk1
    FOREIGN KEY (p_default_addressee_type_code_c)
    REFERENCES p_addressee_type(p_addressee_type_code_c),
  CONSTRAINT p_title_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_title_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_title_pk0 
   (p_title_c),
KEY inx_p_title_fk1_key1 
   (p_default_addressee_type_code_c),
KEY inx_p_title_fkcr_key2 
   (s_created_by_c),
KEY inx_p_title_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The class a partner is (PERSON, UNIT, etc.
-- GROUP: partner
CREATE TABLE p_partner_classes (

  p_partner_class_c varchar(24) NOT NULL,
  p_description_c varchar(100),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_classes_pk
    PRIMARY KEY (p_partner_class_c),
  CONSTRAINT p_partner_classes_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_classes_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_classes_pk0 
   (p_partner_class_c),
KEY inx_p_partner_classes_fkcr_key1 
   (s_created_by_c),
KEY inx_p_partner_classes_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Address and other data related to that address.
-- GROUP: partner
CREATE TABLE p_location (

    -- This is the key that tell what site created this location, it will help to merge addresses when doing imports
  p_site_key_n bigint DEFAULT 0 NOT NULL,
  p_location_key_i integer DEFAULT 0 NOT NULL,
    -- The name of the building
  p_building_1_c varchar(100),
    -- The name of the building (continued)
  p_building_2_c varchar(100),
    -- The name of the street that the house is located on
  p_street_name_c varchar(100),
    -- This is the first element of an address
  p_locality_c varchar(100),
    -- The name of the suburb
  p_suburb_c varchar(100),
    -- This can be a post town or city
  p_city_c varchar(64),
    -- This can be a county (UK), a state (US), province (CDN), canton (CH) etc.
  p_county_c varchar(64),
    -- This is the national post code
  p_postal_code_c varchar(40),
    -- This is a code which identifies a country. It is taken from the ISO 3166-1-alpha-2 code elements.
  p_country_code_c varchar(8),
    -- This is the third element of an address (if required)
  p_address_3_c varchar(100),
    -- The latitude of the location; a number between -90 and +90; precision is 6 digits (11cm)
  p_geo_latitude_n numeric(9, 6),
    -- The longitude of the location; a number between -180 and +180; precision is 6 digits (11cm)
  p_geo_longitude_n numeric(9, 6),
    -- The distance in km of this location to the location 0 if location 0 was on the same longitude; this is for improving query performance
  p_geo_km_x_i integer,
    -- The distance in km of this location to the location 0 if location 0 was on the same latitude; this is for improving query performance
  p_geo_km_y_i integer,
    -- The accuracy of the stored geo data;                 -2: server did not respond;                 -1: not processed yet;                 0: Unknown Location;                 1: Country level accuracy;                 2: Region;                 3: Sub-Region;                 4: Town/City/Village;                 5: Post code;                 6: Street;                 7: Intersection;                 8: Address level accuracy
  p_geo_accuracy_i integer DEFAULT -1,
    -- Indicates whether or not the location has restricted access. If it does then the access will be controlled by s_group_location.
  p_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_location_pk
    PRIMARY KEY (p_site_key_n,p_location_key_i),
  CONSTRAINT p_location_fk1
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_location_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_location_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_location_pk0 
   (p_site_key_n,p_location_key_i),
KEY inx_p_location_fkcr_key1 
   (s_created_by_c),
KEY inx_p_location_fkmd_key2 
   (s_modified_by_c),
KEY p_postal_code_k0 
   (p_postal_code_c),
KEY p_locality_k1 
   (p_locality_c),
KEY p_city_k2 
   (p_city_c),
KEY p_county_k3 
   (p_county_c),
KEY p_country_k4 
   (p_country_code_c),
KEY p_geo_location_km_k5 
   (p_geo_km_x_i,p_geo_km_y_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Types of address e.g. home, business
-- GROUP: partner
CREATE TABLE p_location_type (

  p_code_c varchar(24) NOT NULL,
  p_description_c varchar(60) NOT NULL,
  p_deletable_l boolean DEFAULT 1 NOT NULL,
  p_assignable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_location_type_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_location_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_location_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_location_type_pk0 
   (p_code_c),
KEY inx_p_location_type_fkcr_key1 
   (s_created_by_c),
KEY inx_p_location_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Holds categories that categorise Partner Attributes Types, thus allowing logical grouping of the latter (e.g. to have all Partner Attributes Types that are to do with phone numbers in one category).
-- GROUP: partner
CREATE TABLE p_partner_attribute_category (

    -- Code for Partner Attribute Category. Use plural words/phrases (e.g. 'Phone Numbers' and not 'Phone Number')
  p_category_code_c varchar(60) NOT NULL,
    -- Description for Partner Attribute Category.
  p_category_desc_c varchar(200) NOT NULL,
    -- Allows for manual ordering of Partner Attribute Categories by the user (e.g. list Partner Attribute Types for 'phone numbers' first, then list all for 'mobile numbers').
  p_index_i integer DEFAULT 0 NOT NULL,
    -- Flag to indicate whether Partner Attribute Types that are linked to this Partner Attribute Category are 'Partner Contact Detail' Attributes or not.
  p_partner_contact_category_l boolean NOT NULL,
    -- Flag to indicate whether this Partner Attribute Category is one that is internally used by OpenPetra. Records that have this Flag set will be hidden from the user altogether (and so will p_partner_attribute records that are linked to such a Category through a p_partner_attribute_type!!!).
  p_system_category_l boolean DEFAULT 0 NOT NULL,
    -- Flag to indicate whether this Partner Attribute Category is deletable by a user or not. Should be set to be true for records where p_system_category_l is true.
  p_deletable_l boolean DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_attribute_category_pk
    PRIMARY KEY (p_category_code_c),
  CONSTRAINT p_partner_attribute_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_attribute_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_artner_attribute_category_pk0 
   (p_category_code_c),
KEY inx__attribute_category_fkcr_key1 
   (s_created_by_c),
KEY inx__attribute_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Holds various Partner Attribute Types.
-- GROUP: partner
CREATE TABLE p_partner_attribute_type (

    -- Category of the Partner Attribute Type.
  p_category_code_c varchar(60) NOT NULL,
    -- Attribute Type. This can be anything really: Phone Number, Mobile Number, Email Address, etc (not limited to Partner Contact Details, though). Use singular words/phrases (e.g. 'Phone Number' and not 'Phone Numbers').
  p_attribute_type_c varchar(60) NOT NULL,
    -- Description of what this Partner Attribute Type is all about.
  p_description_c varchar(100) NOT NULL,
    -- Allows for manual ordering of Partner Attribute Types by the user (e.g. to list all Partner Attributes that have a Partner Attribute Type of 'phone number' *grouped together* first, then list all that have a Partner Attribute Type of 'assistant phone number' *grouped together*).
  p_index_i integer DEFAULT 0 NOT NULL,
    -- Describes the kind (type) of Value that the Partner Attributes that are linked to this Partner Attribute Type are. Valid values for Contact Detail Attributes: 'CONTACTDETAIL_GENERAL', 'CONTACTDETAIL_HYPERLINK', 'CONTACTDETAIL_HYPERLINK_WITHVALUE', 'CONTACTDETAIL_EMAILADDRESS', 'CONTACTDETAIL_SKYPEID'.
  p_attribute_type_value_kind_c varchar(80) NOT NULL,
    -- For use with p_attribute_type_value_kind_c 'CONTACTDETAIL_HYPERLINK_WITHVALUE' only. Specifies how to 'construct' a hyperlink where the Value of a Partner Attribute is part of the URL.
  p_hyperlink_format_c varchar(120),
    -- Label that should be displayed/printed instead of p_code_c if a Partner Attribute's 'specialised' Flag is set.
  p_special_label_c varchar(100),
    -- Can this Partner Attribute Type be assigned?
  p_unassignable_l boolean DEFAULT 0 NOT NULL,
    -- This is the date where the record was set to unassignable.
  p_unassignable_date_d date,
    -- Flag to indicate whether this Partner Attribute Type is deletable by a user or not. 
  p_deletable_l boolean DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_attribute_type_pk
    PRIMARY KEY (p_attribute_type_c),
  CONSTRAINT p_partner_attribute_type_fk1
    FOREIGN KEY (p_category_code_c)
    REFERENCES p_partner_attribute_category(p_category_code_c),
  CONSTRAINT p_partner_attribute_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_attribute_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_attribute_type_pk0 
   (p_attribute_type_c),
KEY inx_rtner_attribute_type_fk1_key1 
   (p_category_code_c),
KEY inx_tner_attribute_type_fkcr_key2 
   (s_created_by_c),
KEY inx_tner_attribute_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- General information about the unit such as unit type and entry conference. 
-- GROUP: partner
CREATE TABLE u_unit_type (

  u_unit_type_code_c varchar(24) NOT NULL,
  u_unit_type_name_c varchar(64) NOT NULL,
    -- This defines if the type code can be deleted. This can only be updated by the system manager.
  p_type_deletable_l boolean DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT u_unit_type_pk
    PRIMARY KEY (u_unit_type_code_c),
  CONSTRAINT u_unit_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT u_unit_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_u_unit_type_pk0 
   (u_unit_type_code_c),
KEY inx_u_unit_type_fkcr_key1 
   (s_created_by_c),
KEY inx_u_unit_type_fkmd_key2 
   (s_modified_by_c),
KEY um_type_k0 
   (u_unit_type_name_c,u_unit_type_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes indicating someones marital status.
-- GROUP: partner
CREATE TABLE pt_marital_status (

    -- This code indicates the different marital statuses.
  pt_code_c varchar(4) NOT NULL,
    -- This describes the marital statuses.
  pt_description_c varchar(80) NOT NULL,
    -- Indicates if this code can still be assigned?
  pt_assignable_flag_l boolean DEFAULT 1 NOT NULL,
    -- Date from which this code was made unassignable.
  pt_assignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_marital_status_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_marital_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_marital_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_family_fk2_ref0 
   (pt_code_c),
UNIQUE KEY inx_pt_marital_status_pk1 
   (pt_code_c),
KEY inx_pt_marital_status_fkcr_key2 
   (s_created_by_c),
KEY inx_pt_marital_status_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of occupations with codes
-- GROUP: partner
CREATE TABLE p_occupation (

  p_occupation_code_c varchar(32) DEFAULT 'UNKNOWN' NOT NULL,
  p_occupation_description_c varchar(80) NOT NULL,
  p_valid_occupation_l boolean DEFAULT 1,
    -- This defines if the occupation code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_occupation_pk
    PRIMARY KEY (p_occupation_code_c),
  CONSTRAINT p_occupation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_occupation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_occupation_pk0 
   (p_occupation_code_c),
KEY inx_p_occupation_fkcr_key1 
   (s_created_by_c),
KEY inx_p_occupation_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of denomination codes for churches
-- GROUP: partner
CREATE TABLE p_denomination (

  p_denomination_code_c varchar(16) NOT NULL,
  p_denomination_name_c varchar(80) NOT NULL,
    -- Defines if a denomination is still valid for use
  p_valid_denomination_l boolean DEFAULT 1 NOT NULL,
    -- This defines if the denomination code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_denomination_pk
    PRIMARY KEY (p_denomination_code_c),
  CONSTRAINT p_denomination_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_denomination_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_denomination_pk0 
   (p_denomination_code_c),
KEY inx_p_denomination_fkcr_key1 
   (s_created_by_c),
KEY inx_p_denomination_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of businesses with codes
-- GROUP: partner
CREATE TABLE p_business (

  p_business_code_c varchar(16) NOT NULL,
  p_business_description_c varchar(80) NOT NULL,
    -- This defines if the business code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_business_pk
    PRIMARY KEY (p_business_code_c),
  CONSTRAINT p_business_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_business_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_business_pk0 
   (p_business_code_c),
KEY inx_p_business_fkcr_key1 
   (s_created_by_c),
KEY inx_p_business_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Tells of what type this banking-detail is (bank-account, visa, mastercard, etc.
-- GROUP: partner
CREATE TABLE p_banking_type (

    -- The ID of this type
  p_id_i integer NOT NULL,
    -- Human readable form of this type
  p_type_c varchar(24) NOT NULL,
    -- What it means
  p_description_c varchar(50),
    -- A procedure to check the fields...
  p_check_procedure varchar(50),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_banking_type_pk
    PRIMARY KEY (p_id_i),
  CONSTRAINT p_banking_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_banking_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_banking_type_pk0 
   (p_id_i),
KEY inx_p_banking_type_fkcr_key1 
   (s_created_by_c),
KEY inx_p_banking_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Usage type information for Banking Details.
-- GROUP: partner
CREATE TABLE p_banking_details_usage_type (

    -- This identifies the usage type.
  p_type_c varchar(32) NOT NULL,
    -- This describes the usage type.
  p_type_description_c varchar(120) NOT NULL,
    -- Can this usage type be assigned?
  pc_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date where the record was set to unassignable.
  pc_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pc_deletable_flag_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_banking_details_usage_type_pk
    PRIMARY KEY (p_type_c),
  CONSTRAINT p_banking_details_usage_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_banking_details_usage_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_anking_details_usage_type_pk0 
   (p_type_c),
KEY inx__details_usage_type_fkcr_key1 
   (s_created_by_c),
KEY inx__details_usage_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the categories of partner (special) types
-- GROUP: partner
CREATE TABLE p_type_category (

    -- This code indicates the category of a type.
  p_code_c varchar(40) NOT NULL,
    -- This describes the type category.
  p_description_c varchar(100),
    -- Indicates if this code can still be assigned?
  p_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  p_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  p_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_type_category_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_type_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_type_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_type_fk1_ref0 
   (p_code_c),
UNIQUE KEY inx_p_partner_type_category_pk1 
   (p_code_c),
KEY inx_p_type_category_fkcr_key2 
   (s_created_by_c),
KEY inx_p_type_category_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of all possible special types for a partner.
-- GROUP: partner
CREATE TABLE p_type (

    -- This code identifies the type
  p_type_code_c varchar(20) NOT NULL,
    -- This describes the method of aquisition.
  p_type_description_c varchar(120) NOT NULL,
    -- This is a category, by which partner types can be grouped.
  p_category_code_c varchar(40),
    -- Defines if the type code is still for use
  p_valid_type_l boolean DEFAULT 1,
    -- This type can only be assigned by the system
  p_system_type_l boolean DEFAULT 0,
    -- This defines if the type code can be deleted. This can only be updated by the system manager.
  p_type_deletable_l boolean DEFAULT 1,
    -- This Mot Group will be selected by default when entering gifts for Partners with this Special Type
  p_type_motivation_group_c varchar(16),
    -- This Mot Detail will be selected by default when entering gifts for Partners with this Special Type
  p_type_motivation_detail_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_type_pk
    PRIMARY KEY (p_type_code_c),
  CONSTRAINT p_type_fk1
    FOREIGN KEY (p_category_code_c)
    REFERENCES p_type_category(p_code_c),
  CONSTRAINT p_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_type_pk0 
   (p_type_code_c),
KEY inx_p_type_fk1_key1 
   (p_category_code_c),
KEY inx_p_type_fkcr_key2 
   (s_created_by_c),
KEY inx_p_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the categories of relations (grouping). 
-- GROUP: partner
CREATE TABLE p_relation_category (

    -- This code indicates the category of a relation.
  p_code_c varchar(40) NOT NULL,
    -- This describes the relation category.
  p_description_c varchar(100),
    -- Indicates if this code can still be assigned?
  p_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  p_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  p_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_relation_category_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_relation_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_relation_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_relation_category_pk0 
   (p_code_c),
KEY inx_p_relation_category_fkcr_key1 
   (s_created_by_c),
KEY inx_p_relation_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of relationships between partners.  Relations occur in one direction only.   The relation code is used in the p_partner_relationship record. 
-- GROUP: partner
CREATE TABLE p_relation (

    -- This code identifies the relation
  p_relation_name_c varchar(20) NOT NULL,
    -- This describes the relation.
  p_relation_description_c varchar(100) NOT NULL,
    -- This is a category, by which relations can be grouped.
  p_relation_category_c varchar(40),
    -- This defines if the relation name can be deleted. This can only be updated by the system manager. It default to Yes.
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- This describes the relation in the reverse direction, eg Husband and Wife.
  p_reciprocal_description_c varchar(100) NOT NULL,
    -- Determines whether the record is still assignable 
  p_valid_relation_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_relation_pk
    PRIMARY KEY (p_relation_name_c),
  CONSTRAINT p_relation_fk1
    FOREIGN KEY (p_relation_category_c)
    REFERENCES p_relation_category(p_code_c),
  CONSTRAINT p_relation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_relation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_relation_pk0 
   (p_relation_name_c),
KEY inx_p_relation_fk1_key1 
   (p_relation_category_c),
KEY inx_p_relation_fkcr_key2 
   (s_created_by_c),
KEY inx_p_relation_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains a list of extract type which is needed when extracts need to be rerun
-- GROUP: mailroom
CREATE TABLE m_extract_type (

    -- Extract Type Code
  m_code_c varchar(50) NOT NULL,
    -- Function that is run to create the extract (4GL function, Delphi, etc.)
  m_function_c varchar(500),
    -- Description of Extract Type
  m_description_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT m_extract_type_pk
    PRIMARY KEY (m_code_c),
  CONSTRAINT m_extract_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT m_extract_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_m_extract_master_fk4_ref0 
   (m_code_c),
UNIQUE KEY inx_m_extract_type_pk1 
   (m_code_c),
KEY inx_m_extract_type_fkcr_key2 
   (s_created_by_c),
KEY inx_m_extract_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Master file for extracts.  Contains names for the extract id
-- GROUP: mailroom
CREATE TABLE m_extract_master (

    -- Identifier for the extract
  m_extract_id_i integer DEFAULT 0 NOT NULL,
    -- Short name for the extract to be used in filenames
  m_extract_name_c varchar(50) NOT NULL,
    -- This is a long description for the extract
  m_extract_desc_c varchar(160),
  m_last_ref_d date,
  m_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The user can set the frozen field when the extract should not be updated.
  m_frozen_l boolean DEFAULT 0 NOT NULL,
  m_key_count_i integer DEFAULT 0,
  m_public_l boolean DEFAULT 1 NOT NULL,
    -- Indicates that the extract has been edited by a user
  m_manual_mod_l boolean DEFAULT 0 NOT NULL,
    -- Date the extract was manually modified
  m_manual_mod_d date,
    -- Who made the last manual modification ?
  m_manual_mod_by_c varchar(20),
    -- Indicate the extract type. Which function was the extract created through?
  m_extract_type_code_c varchar(50),
    -- Is this extract just a template that has not yet been run?
  m_template_l boolean DEFAULT 0,
    -- Indicates whether or not the extract has restricted access. If it does then the access will be controlled by s_group_extract
  m_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT m_extract_master_pk
    PRIMARY KEY (m_extract_id_i),
  CONSTRAINT m_extract_master_uk
    UNIQUE (m_extract_name_c),
  CONSTRAINT m_extract_master_fk3
    FOREIGN KEY (m_manual_mod_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT m_extract_master_fk4
    FOREIGN KEY (m_extract_type_code_c)
    REFERENCES m_extract_type(m_code_c),
  CONSTRAINT m_extract_master_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT m_extract_master_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_m_extract_master_pk0 
   (m_extract_id_i),
KEY inx_m_extract_master_fk3_key1 
   (m_manual_mod_by_c),
KEY inx_m_extract_master_fk4_key2 
   (m_extract_type_code_c),
KEY inx_m_extract_master_fkcr_key3 
   (s_created_by_c),
KEY inx_m_extract_master_fkmd_key4 
   (s_modified_by_c),
UNIQUE KEY inx_m_extract_master_uk0 
   (m_extract_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains a list of parameters that an extract was run with (so it can be rerun)
-- GROUP: mailroom
CREATE TABLE m_extract_parameter (

    -- Identifier for the extract
  m_extract_id_i integer DEFAULT 0 NOT NULL,
    -- Extract Parameter Code
  m_parameter_code_c varchar(50) NOT NULL,
    -- Index for Parameter Value. Only relevant if a parameter is a list of values in which case a new index is used for each list item.
  m_value_index_i integer DEFAULT 0 NOT NULL,
    -- Extract Parameter Value
  m_parameter_value_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT m_extract_parameter_pk
    PRIMARY KEY (m_extract_id_i,m_parameter_code_c,m_value_index_i),
  CONSTRAINT m_extract_parameter_fk1
    FOREIGN KEY (m_extract_id_i)
    REFERENCES m_extract_master(m_extract_id_i),
  CONSTRAINT m_extract_parameter_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT m_extract_parameter_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_m_extract_parameter_pk0 
   (m_extract_id_i,m_parameter_code_c,m_value_index_i),
KEY inx_m_extract_parameter_fk1_key1 
   (m_extract_id_i),
KEY inx_m_extract_parameter_fkcr_key2 
   (s_created_by_c),
KEY inx_m_extract_parameter_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists mailings that are being tracked.   When entering gifts, the mailing that motivated the gift can be indicated.
-- GROUP: mailroom
CREATE TABLE p_mailing (

    -- Mailing Code
  p_mailing_code_c varchar(50) NOT NULL,
    -- Mailing Description
  p_mailing_description_c varchar(160),
    -- Date Of Mailing
  p_mailing_date_d date,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16),
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16),
    -- Cost of Mailing
  p_mailing_cost_n numeric(19, 2) DEFAULT 0,
    -- Gift amount attributed to this mailing
  p_mailing_attributed_amount_n numeric(19, 2) DEFAULT 0,
    -- Indicates if the mailing is viewable in comboboxes where the user can select it
  p_viewable_l boolean DEFAULT 1,
    -- Date until this mailing is viewable for users (if p_viewable_l is set)
  p_viewable_until_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_mailing_pk
    PRIMARY KEY (p_mailing_code_c),
  CONSTRAINT p_mailing_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_mailing_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_match_fk3_ref0 
   (p_mailing_code_c),
UNIQUE KEY inx_p_mailing_pk1 
   (p_mailing_code_c),
KEY inx_p_mailing_fkcr_key2 
   (s_created_by_c),
KEY inx_p_mailing_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the address layouts generally available for the user.
-- GROUP: mailroom
CREATE TABLE p_address_layout_code (

    -- Address Layout Code
  p_code_c varchar(32) NOT NULL,
    -- Description for Address Layout Code
  p_description_c varchar(100),
    -- Index for Display Order (to determine the display position of the layout in a list)
  p_display_index_i integer NOT NULL,
    -- Comment for Address Layout Code
  p_comment_c varchar(600),
    -- Can this layout code be deleted by the user?
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_layout_code_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_address_layout_code_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_layout_code_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_layout_code_pk0 
   (p_code_c),
KEY inx_address_layout_code_fkcr_key1 
   (s_created_by_c),
KEY inx_address_layout_code_fkmd_key2 
   (s_modified_by_c),
KEY p_address_layout_code_k0 
   (p_display_index_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the address lines used in laying out an address. Eg a form letter address layout
-- GROUP: mailroom
CREATE TABLE p_address_layout (

  p_country_code_c varchar(8) NOT NULL,
  p_address_layout_code_c varchar(32) DEFAULT 'SmlLabel' NOT NULL,
  p_address_line_number_i integer DEFAULT 0 NOT NULL,
  p_address_line_code_c varchar(16) NOT NULL,
    -- This field is a short description of the Address Line Code record
  p_address_prompt_c varchar(30),
    -- System flag indicates a lock is on the record
  p_locked_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_layout_pk
    PRIMARY KEY (p_country_code_c,p_address_layout_code_c,p_address_line_number_i,p_address_line_code_c),
  CONSTRAINT p_address_layout_fk1
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_address_layout_fk2
    FOREIGN KEY (p_address_layout_code_c)
    REFERENCES p_address_layout_code(p_code_c),
  CONSTRAINT p_address_layout_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_layout_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_layout_pk0 
   (p_country_code_c,p_address_layout_code_c,p_address_line_number_i,p_address_line_code_c),
KEY inx_p_address_layout_fk1_key1 
   (p_country_code_c),
KEY inx_p_address_layout_fk2_key2 
   (p_address_layout_code_c),
KEY inx_p_address_layout_fkcr_key3 
   (s_created_by_c),
KEY inx_p_address_layout_fkmd_key4 
   (s_modified_by_c),
KEY p_country_code_c_k0 
   (p_country_code_c,p_address_layout_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the address lines used in laying out an address. Eg a form letter address layout
-- GROUP: mailroom
CREATE TABLE p_address_block (

  p_country_code_c varchar(8) NOT NULL,
  p_address_layout_code_c varchar(32) DEFAULT 'SmlLabel' NOT NULL,
    -- The complete set of address lines, including replaceable parameters
  p_address_block_text_c varchar(512) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_block_pk
    PRIMARY KEY (p_country_code_c,p_address_layout_code_c),
  CONSTRAINT p_address_block_fk1
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_address_block_fk2
    FOREIGN KEY (p_address_layout_code_c)
    REFERENCES p_address_layout_code(p_code_c),
  CONSTRAINT p_address_block_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_block_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_block_pk0 
   (p_country_code_c,p_address_layout_code_c),
KEY inx_p_address_block_fk1_key1 
   (p_country_code_c),
KEY inx_p_address_block_fk2_key2 
   (p_address_layout_code_c),
KEY inx_p_address_block_fkcr_key3 
   (s_created_by_c),
KEY inx_p_address_block_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This contains the elements which make up an address. Eg Name etc
-- GROUP: mailroom
CREATE TABLE p_address_element (

    -- This Code is used to identify the address element.
  p_address_element_code_c varchar(24) NOT NULL,
  p_address_element_description_c varchar(160),
    -- Address element field name
  p_address_element_field_name_c varchar(60),
    -- This is usually a ""."" or a "";"" or a "","" etc
  p_address_element_text_c varchar(2),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_element_pk
    PRIMARY KEY (p_address_element_code_c),
  CONSTRAINT p_address_element_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_element_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_element_pk0 
   (p_address_element_code_c),
KEY inx_p_address_element_fkcr_key1 
   (s_created_by_c),
KEY inx_p_address_element_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This contains the elements which make up an address. Eg Name etc
-- GROUP: mailroom
CREATE TABLE p_address_block_element (

    -- This Code is used to identify the address element.
  p_address_element_code_c varchar(48) NOT NULL,
  p_address_element_description_c varchar(160),
    -- System flag indicates the element is a print directive and not a data placeholder
  p_is_directive_l boolean NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_block_element_pk
    PRIMARY KEY (p_address_element_code_c),
  CONSTRAINT p_address_block_element_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_block_element_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_block_element_pk0 
   (p_address_element_code_c),
KEY inx_dress_block_element_fkcr_key1 
   (s_created_by_c),
KEY inx_dress_block_element_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is an address line which consists of address elements.  Used along with p_address_layout and p_address_element to define layout of an address for different countries.
-- GROUP: mailroom
CREATE TABLE p_address_line (

  p_address_line_code_c varchar(16) NOT NULL,
    -- This is the column number where the element field or text should be placed.
  p_address_element_position_i integer DEFAULT 0 NOT NULL,
    -- This Code is used to identify the address element.
  p_address_element_code_c varchar(24) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_address_line_pk
    PRIMARY KEY (p_address_line_code_c,p_address_element_position_i),
  CONSTRAINT p_address_line_fk1
    FOREIGN KEY (p_address_element_code_c)
    REFERENCES p_address_element(p_address_element_code_c),
  CONSTRAINT p_address_line_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_address_line_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_address_line_pk0 
   (p_address_line_code_c,p_address_element_position_i),
KEY inx_p_address_line_fk1_key1 
   (p_address_element_code_c),
KEY inx_p_address_line_fkcr_key2 
   (s_created_by_c),
KEY inx_p_address_line_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is used to override titles that might be different in the address than that in the letter. Eg      German     Herr   Herrn ""Sehr geehrter Herr Starling"" in the letter and ""Herrn Starling"" in the address.
-- GROUP: mailroom
CREATE TABLE p_addressee_title_override (

    -- This is the code used to identify a language.
  p_language_code_c varchar(20) DEFAULT '99' NOT NULL,
    -- The partner's title
  p_title_c varchar(64) NOT NULL,
    -- The title to override the partner
  p_title_override_c varchar(64) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_addressee_title_override_pk
    PRIMARY KEY (p_language_code_c,p_title_c),
  CONSTRAINT p_addressee_title_override_fk1
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT p_addressee_title_override_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_addressee_title_override_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__addressee_title_override_pk0 
   (p_language_code_c,p_title_c),
KEY inx_essee_title_override_fk1_key1 
   (p_language_code_c),
KEY inx_ssee_title_override_fkcr_key2 
   (s_created_by_c),
KEY inx_ssee_title_override_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains the text used in letters
-- GROUP: mailroom
CREATE TABLE p_formality (

    -- This is the code used to identify a language.
  p_language_code_c varchar(20) DEFAULT '99' NOT NULL,
    -- This is a code which identifies a country. It is taken from the ISO 3166-1-alpha-2 code elements.
  p_country_code_c varchar(8) NOT NULL,
  p_addressee_type_code_c varchar(24) NOT NULL,
  p_formality_level_i integer DEFAULT 1 NOT NULL CHECK ( p_formality_level_i >= 1 AND p_formality_level_i <= 6 ),
  p_salutation_text_c varchar(64),
  p_title_c varchar(24),
  p_complimentary_closing_text_c varchar(64),
  p_personal_pronoun_c varchar(24),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_formality_pk
    PRIMARY KEY (p_language_code_c,p_country_code_c,p_addressee_type_code_c,p_formality_level_i),
  CONSTRAINT p_formality_fk1
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT p_formality_fk2
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_formality_fk3
    FOREIGN KEY (p_addressee_type_code_c)
    REFERENCES p_addressee_type(p_addressee_type_code_c),
  CONSTRAINT p_formality_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_formality_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_formality_pk0 
   (p_language_code_c,p_country_code_c,p_addressee_type_code_c,p_formality_level_i),
KEY inx_p_formality_fk1_key1 
   (p_language_code_c),
KEY inx_p_formality_fk2_key2 
   (p_country_code_c),
KEY inx_p_formality_fk3_key3 
   (p_addressee_type_code_c),
KEY inx_p_formality_fkcr_key4 
   (s_created_by_c),
KEY inx_p_formality_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Defines the attributes of different label types.  Eg: for address labels.
-- GROUP: mailroom
CREATE TABLE p_label (

  p_code_c varchar(16) NOT NULL,
    -- This identifies the form
  s_form_name_c varchar(20) NOT NULL,
  p_gap_lines_i integer DEFAULT 0,
  p_height_i integer DEFAULT 0 NOT NULL,
  p_width_i integer DEFAULT 0 NOT NULL,
  p_gap_columns_i integer DEFAULT 0,
  p_labels_across_i integer DEFAULT 0 NOT NULL,
  p_labels_down_i integer DEFAULT 0 NOT NULL,
  p_description_c varchar(70) NOT NULL,
  p_start_column_i integer DEFAULT 0,
  p_start_line_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_label_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_label_fk1
    FOREIGN KEY (s_form_name_c)
    REFERENCES s_form(s_form_name_c),
  CONSTRAINT p_label_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_label_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_label_pk0 
   (p_code_c),
KEY inx_p_label_fk1_key1 
   (s_form_name_c),
KEY inx_p_label_fkcr_key2 
   (s_created_by_c),
KEY inx_p_label_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Master record for Mail Merge output creation
-- GROUP: mailroom
CREATE TABLE p_merge_form (

    -- Name of Merge Form
  p_merge_form_name_c varchar(16) NOT NULL,
    -- Form description
  p_merge_form_description_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_merge_form_pk
    PRIMARY KEY (p_merge_form_name_c),
  CONSTRAINT p_merge_form_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_merge_form_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_merge_form_pk0 
   (p_merge_form_name_c),
KEY inx_p_merge_form_fkcr_key1 
   (s_created_by_c),
KEY inx_p_merge_form_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Fields within a Mail Merge Form
-- GROUP: mailroom
CREATE TABLE p_merge_field (

    -- Name of Merge Form
  p_merge_form_name_c varchar(16) NOT NULL,
    -- Name of the field in the Word document which will be filled with the data 
  p_merge_field_name_c varchar(32) NOT NULL,
    -- Position to define order of merge fields
  p_merge_field_position_i integer DEFAULT 0,
    -- Type of this field.  This defines the parameters which are required to generate the insert
  p_merge_type_c varchar(32),
    -- List of parameters required to create the actual insert
  p_merge_parameters_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_merge_field_pk
    PRIMARY KEY (p_merge_form_name_c,p_merge_field_name_c),
  CONSTRAINT p_merge_field_fk1
    FOREIGN KEY (p_merge_form_name_c)
    REFERENCES p_merge_form(p_merge_form_name_c),
  CONSTRAINT p_merge_field_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_merge_field_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_merge_field_pk0 
   (p_merge_form_name_c,p_merge_field_name_c),
KEY inx_p_merge_field_fk1_key1 
   (p_merge_form_name_c),
KEY inx_p_merge_field_fkcr_key2 
   (s_created_by_c),
KEY inx_p_merge_field_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Postcode ranges for each region
-- GROUP: mailroom
CREATE TABLE p_postcode_range (

    -- Name of the postcode range
  p_range_c varchar(64) NOT NULL,
    -- Start of postcode range
  p_from_c varchar(40),
    -- End of postcode range
  p_to_c varchar(40),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_postcode_range_pk
    PRIMARY KEY (p_range_c),
  CONSTRAINT p_postcode_range_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_postcode_range_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_postcode_range_pk0 
   (p_range_c),
KEY inx_p_postcode_range_fkcr_key1 
   (s_created_by_c),
KEY inx_p_postcode_range_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List postcode regions
-- GROUP: mailroom
CREATE TABLE p_postcode_region (

    -- Name of a postcode region
  p_region_c varchar(64) NOT NULL,
    -- This describes the region.
  p_description_c varchar(100),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_postcode_region_pk
    PRIMARY KEY (p_region_c),
  CONSTRAINT p_postcode_region_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_postcode_region_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_postcode_region_pk0 
   (p_region_c),
KEY inx_p_postcode_region_fkcr_key1 
   (s_created_by_c),
KEY inx_p_postcode_region_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links ranges to a region.
-- GROUP: mailroom
CREATE TABLE p_postcode_region_range (

    -- Name of a postcode region
  p_region_c varchar(64) NOT NULL,
    -- A range for a postcode region
  p_range_c varchar(64) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_postcode_region_range_pk
    PRIMARY KEY (p_region_c,p_range_c),
  CONSTRAINT p_postcode_region_fk1
    FOREIGN KEY (p_region_c)
    REFERENCES p_postcode_region(p_region_c),
  CONSTRAINT p_postcode_region_fk2
    FOREIGN KEY (p_range_c)
    REFERENCES p_postcode_range(p_range_c),
  CONSTRAINT p_postcode_region_range_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_postcode_region_range_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_postcode_region_range_pk0 
   (p_region_c,p_range_c),
KEY inx_p_postcode_region_fk1_key1 
   (p_region_c),
KEY inx_p_postcode_region_fk2_key2 
   (p_range_c),
KEY inx_stcode_region_range_fkcr_key3 
   (s_created_by_c),
KEY inx_stcode_region_range_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of a publication
-- GROUP: mailroom
CREATE TABLE p_publication (

    -- This is the key to the publication table
  p_publication_code_c varchar(20) NOT NULL,
  p_number_of_issues_i integer DEFAULT 0,
    -- The number of free issues and reminders to send out.
  p_number_of_reminders_i integer DEFAULT 1,
    -- A short description of the publication
  p_publication_description_c varchar(80),
  p_valid_publication_l boolean DEFAULT 1,
  a_frequency_code_c varchar(24) NOT NULL,
    -- The publication short code that is used on an address label
  p_publication_label_code_c varchar(6),
  p_publication_language_c varchar(20),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_publication_pk
    PRIMARY KEY (p_publication_code_c),
  CONSTRAINT p_publication_fk1
    FOREIGN KEY (a_frequency_code_c)
    REFERENCES a_frequency(a_frequency_code_c),
  CONSTRAINT p_publication_fk2
    FOREIGN KEY (p_publication_language_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT p_publication_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_publication_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_publication_pk0 
   (p_publication_code_c),
KEY inx_p_publication_fk1_key1 
   (a_frequency_code_c),
KEY inx_p_publication_fk2_key2 
   (p_publication_language_c),
KEY inx_p_publication_fkcr_key3 
   (s_created_by_c),
KEY inx_p_publication_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Cost of a publication
-- GROUP: mailroom
CREATE TABLE p_publication_cost (

    -- The is the key to the publication table
  p_publication_code_c varchar(20) NOT NULL,
  p_date_effective_d date NOT NULL,
    -- This is a number of currency units
  p_publication_cost_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- The cost of posting each item
  p_postage_cost_n numeric(24, 10) DEFAULT 0,
    -- This defines which currency is being used
  p_currency_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_publication_cost_pk
    PRIMARY KEY (p_publication_code_c,p_date_effective_d),
  CONSTRAINT p_publication_cost_fk1
    FOREIGN KEY (p_publication_code_c)
    REFERENCES p_publication(p_publication_code_c),
  CONSTRAINT p_publication_cost_fk2
    FOREIGN KEY (p_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT p_publication_cost_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_publication_cost_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_publication_cost_pk0 
   (p_publication_code_c,p_date_effective_d),
KEY inx_p_publication_cost_fk1_key1 
   (p_publication_code_c),
KEY inx_p_publication_cost_fk2_key2 
   (p_currency_code_c),
KEY inx_p_publication_cost_fkcr_key3 
   (s_created_by_c),
KEY inx_p_publication_cost_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of reasons for giving a subscription
-- GROUP: mailroom
CREATE TABLE p_reason_subscription_given (

  p_code_c varchar(24) NOT NULL,
  p_description_c varchar(160) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_reason_subs_given_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_reason_subscription_given_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_reason_subscription_given_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_reason_subs_given_pk0 
   (p_code_c),
KEY inx__subscription_given_fkcr_key1 
   (s_created_by_c),
KEY inx__subscription_given_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of reasons for cancelling a subscription
-- GROUP: mailroom
CREATE TABLE p_reason_subscription_cancelled (

  p_code_c varchar(24) NOT NULL,
  p_description_c varchar(160) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_reason_subs_cancelled_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_reason_subscription_cancelled_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_reason_subscription_cancelled_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_reason_subs_cancelled_pk0 
   (p_code_c),
KEY inx_scription_cancelled_fkcr_key1 
   (s_created_by_c),
KEY inx_scription_cancelled_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of a membership
-- GROUP: partner
CREATE TABLE p_membership (

    -- This is the key to the membership table
  p_membership_code_c varchar(20) NOT NULL,
    -- A short description of the membership
  p_membership_description_c varchar(80),
  p_valid_membership_l boolean DEFAULT 1,
  a_frequency_code_c varchar(24) NOT NULL,
    -- This is the amount charged for each membership period.
  a_membership_fee_n numeric(24, 10) DEFAULT 0,
    -- This is the time required to do service for each membership period.
  a_membership_hours_service_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_membership_pk
    PRIMARY KEY (p_membership_code_c),
  CONSTRAINT p_membership_fk1
    FOREIGN KEY (a_frequency_code_c)
    REFERENCES a_frequency(a_frequency_code_c),
  CONSTRAINT p_membership_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_membership_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_membership_pk0 
   (p_membership_code_c),
KEY inx_p_membership_fk1_key1 
   (a_frequency_code_c),
KEY inx_p_membership_fkcr_key2 
   (s_created_by_c),
KEY inx_p_membership_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Possible attributes for partner contacts.  Gives the description of each attribute code.  An attribute is a type of contact that was made or which occurred with a partner.
-- GROUP: mailroom
CREATE TABLE p_contact_attribute (

    -- Contact Attribute Code
  p_contact_attribute_code_c varchar(32) NOT NULL,
    -- This is a contact attribute description.
  p_contact_attribute_descr_c varchar(64),
    -- allowed to use this attribute for new contacts?
  p_active_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_contact_attribute_pk
    PRIMARY KEY (p_contact_attribute_code_c),
  CONSTRAINT p_contact_attribute_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_contact_attribute_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_contact_attribute_pk0 
   (p_contact_attribute_code_c),
KEY inx_p_contact_attribute_fkcr_key1 
   (s_created_by_c),
KEY inx_p_contact_attribute_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Possible attribute details for each contact attribute.  Breaks down the attribute into more specifice information that applies to a contact with a partner.
-- GROUP: mailroom
CREATE TABLE p_contact_attribute_detail (

    -- Contact Attribute Code
  p_contact_attribute_code_c varchar(32) NOT NULL,
    -- code for attribute detail
  p_contact_attr_detail_code_c varchar(32) NOT NULL,
    -- This is a contact attribute detail description.
  p_contact_attr_detail_descr_c varchar(64),
    -- allowed to use this attribute detail for new contacts?
  p_active_l boolean DEFAULT 1 NOT NULL,
    -- Contact attribute detail comment
  p_comment_c varchar(8000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_contact_attribute_detail_pk
    PRIMARY KEY (p_contact_attribute_code_c,p_contact_attr_detail_code_c),
  CONSTRAINT p_contact_attribute_detail_fk1
    FOREIGN KEY (p_contact_attribute_code_c)
    REFERENCES p_contact_attribute(p_contact_attribute_code_c),
  CONSTRAINT p_contact_attribute_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_contact_attribute_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__contact_attribute_detail_pk0 
   (p_contact_attribute_code_c,p_contact_attr_detail_code_c),
KEY inx_act_attribute_detail_fk1_key1 
   (p_contact_attribute_code_c),
KEY inx_ct_attribute_detail_fkcr_key2 
   (s_created_by_c),
KEY inx_ct_attribute_detail_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- How contacts are made
-- GROUP: mailroom
CREATE TABLE p_method_of_contact (

  p_method_of_contact_code_c varchar(16) NOT NULL,
  p_description_c varchar(64),
  p_contact_type_c varchar(16),
  p_valid_method_l boolean DEFAULT 1 NOT NULL,
    -- This defines if the method of contact code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_method_of_contact_pk
    PRIMARY KEY (p_method_of_contact_code_c),
  CONSTRAINT p_method_of_contact_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_method_of_contact_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_method_of_contact_pk0 
   (p_method_of_contact_code_c),
KEY inx_p_method_of_contact_fkcr_key1 
   (s_created_by_c),
KEY inx_p_method_of_contact_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of contacts with partners
-- GROUP: mailroom
CREATE TABLE p_contact_log (

    -- identifying key for p_contact_log
  p_contact_log_id_i bigint DEFAULT 0 NOT NULL,
    -- Date of contact
  s_contact_date_d date NOT NULL,
    -- Time of contact
  s_contact_time_i integer DEFAULT 0,
    -- Contact code
  p_contact_code_c varchar(16) NOT NULL,
    -- User who made the contact
  p_contactor_c varchar(20),
    -- The Message ID (only applies if the type of contact is an email); this helps to identify the email and to interface with the EMail application
  p_contact_message_id_c varchar(200),
    -- Contact Comment (also used to hold contents of emails)
  p_contact_comment_c longtext,
    -- Identifies a module. A module is any part of aprogram which is related to each menu entry or to the sub-system. Eg, partner administration, AP, AR etc.
  s_module_id_c varchar(20),
    -- If set, this contact is restricted to one user.
  s_user_id_c varchar(20),
    -- The mailing code associated with the contact
  p_mailing_code_c varchar(50),
    -- Indicates whether or not the contact has restricted access. If it does then the access will be controlled by s_group_partner_contact
  p_restricted_l boolean DEFAULT 0,
    -- Location of contact
  p_contact_location_c varchar(8000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_contact_log_pk
    PRIMARY KEY (p_contact_log_id_i),
  CONSTRAINT p_contact_log_fk1
    FOREIGN KEY (p_mailing_code_c)
    REFERENCES p_mailing(p_mailing_code_c),
  CONSTRAINT p_contact_log_fk2
    FOREIGN KEY (p_contact_code_c)
    REFERENCES p_method_of_contact(p_method_of_contact_code_c),
  CONSTRAINT p_contact_log_fk3
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT p_contact_log_fk4
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_contact_log_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_contact_log_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_contact_log_pk0 
   (p_contact_log_id_i),
KEY inx_p_contact_log_fk1_key1 
   (p_mailing_code_c),
KEY inx_p_contact_log_fk2_key2 
   (p_contact_code_c),
KEY inx_p_contact_log_fk3_key3 
   (s_module_id_c),
KEY inx_p_contact_log_fk4_key4 
   (s_user_id_c),
KEY inx_p_contact_log_fkcr_key5 
   (s_created_by_c),
KEY inx_p_contact_log_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Associates a p_contact_attribute_detail with a p_contact_log.  A contact log may have more than one p_contact_attribute_detail associated with it.
-- GROUP: mailroom
CREATE TABLE p_partner_contact_attribute (

    -- identifying key for p_contact_log
  p_contact_id_i bigint DEFAULT 0 NOT NULL,
    -- Contact Attribute Code
  p_contact_attribute_code_c varchar(32) NOT NULL,
    -- code for attribute detail
  p_contact_attr_detail_code_c varchar(32) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_contact_attribute_pk
    PRIMARY KEY (p_contact_id_i,p_contact_attribute_code_c,p_contact_attr_detail_code_c),
  CONSTRAINT p_partner_contact_attribute_fk1
    FOREIGN KEY (p_contact_attribute_code_c,p_contact_attr_detail_code_c)
    REFERENCES p_contact_attribute_detail(p_contact_attribute_code_c,p_contact_attr_detail_code_c),
  CONSTRAINT p_partner_contact_attribute_fk2
    FOREIGN KEY (p_contact_id_i)
    REFERENCES p_contact_log(p_contact_log_id_i),
  CONSTRAINT p_partner_contact_attribute_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_contact_attribute_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_partner_contact_attribute_pk0 
   (p_contact_id_i,p_contact_attribute_code_c,p_contact_attr_detail_code_c),
KEY inx_er_contact_attribute_fk1_key1 
   (p_contact_attribute_code_c,p_contact_attr_detail_code_c),
KEY inx_er_contact_attribute_fk2_key2 
   (p_contact_id_i),
KEY inx_r_contact_attribute_fkcr_key3 
   (s_created_by_c),
KEY inx_r_contact_attribute_fkmd_key4 
   (s_modified_by_c),
UNIQUE KEY p_contact_log_attribute_k0 
   (p_contact_attribute_code_c,p_contact_attr_detail_code_c,p_contact_id_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The form description for user definable forms such as receipts
-- GROUP: mailroom
CREATE TABLE p_form (

    -- The code which defines the type of form described in the table
  p_form_code_c varchar(20) NOT NULL,
    -- The name of the form being created for the form code.
  p_form_name_c varchar(20) NOT NULL,
    -- The language that this form is written in.  Use 99 if the form can be used for unspecified languages.
  p_form_language_c varchar(20) NOT NULL,
    -- Description of the form
  p_form_description_c varchar(100),
    -- If there are several types of form then it can be specified here.  Eg an annual receipt and an individual receipt.
  p_form_type_code_c varchar(24) NOT NULL,
    -- The address layout code that defines the address block content.
  p_address_layout_code_c varchar(16),
    -- The formality level to use if the template contains greetings or salutations. 1=Informal, 6=Very formal
  p_formality_level_i integer DEFAULT 1 NOT NULL,
    -- Is the template available in the database.
  p_template_available_l boolean DEFAULT 0 NOT NULL,
    -- The binary template file encoded as Base64 text
  p_template_document_c longtext,
    -- The file type associated with the template.
  p_template_file_extension_c varchar(16),
    -- Date the template was uploaded to the database
  p_template_upload_date_d date,
    -- Time the template was uploaded to the database
  p_template_upload_time_i integer,
  p_template_uploaded_by_user_id_c varchar(16),
    -- The minimum amount that is acceptable on a receipt
  p_minimum_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Allows the exclusion of certain records from a report
  p_options_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_form_pk
    PRIMARY KEY (p_form_code_c,p_form_name_c,p_form_language_c),
  CONSTRAINT p_form_fk1
    FOREIGN KEY (p_form_language_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT p_form_fk2
    FOREIGN KEY (p_address_layout_code_c)
    REFERENCES p_address_layout_code(p_code_c),
  CONSTRAINT p_form_fk3
    FOREIGN KEY (p_template_uploaded_by_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_form_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_form_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_form_pk0 
   (p_form_code_c,p_form_name_c,p_form_language_c),
KEY inx_p_form_fk1_key1 
   (p_form_language_c),
KEY inx_p_form_fk2_key2 
   (p_address_layout_code_c),
KEY inx_p_form_fk3_key3 
   (p_template_uploaded_by_user_id_c),
KEY inx_p_form_fkcr_key4 
   (s_created_by_c),
KEY inx_p_form_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Subsystems to the general ledger which can be added and removed independantly.  Eg GL, AP, AR, GR
-- GROUP: account
CREATE TABLE a_sub_system (

    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24) NOT NULL,
    -- Name of a sub system
  a_sub_system_name_c varchar(64) NOT NULL,
    -- The filename of the process to call
  a_setup_sub_system_process_c varchar(24) NOT NULL,
    -- The filename of the process to call
  a_sub_system_to_call_c varchar(24),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_sub_system_pk
    PRIMARY KEY (a_sub_system_code_c),
  CONSTRAINT a_sub_system_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_sub_system_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_sub_system_pk0 
   (a_sub_system_code_c),
KEY inx_a_sub_system_fkcr_key1 
   (s_created_by_c),
KEY inx_a_sub_system_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- used for invoicing
-- GROUP: ar
CREATE TABLE a_tax_type (

    -- This is whether it is GST, VAT
  a_tax_type_code_c varchar(16) NOT NULL,
    -- This is a short description which is 32 characters long
  a_tax_type_description_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_tax_type_pk
    PRIMARY KEY (a_tax_type_code_c),
  CONSTRAINT a_tax_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_tax_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_tax_type_pk0 
   (a_tax_type_code_c),
KEY inx_a_tax_type_fkcr_key1 
   (s_created_by_c),
KEY inx_a_tax_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Basic information for each general ledger on the system. Also counters for ledger-specific variables (such as last receipt number).
-- GROUP: account
CREATE TABLE a_ledger (

    -- This is used as a key field in most of the accounting system files .It is created from the first 4 digits of a partner key of type ""ledger"".
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This is the ledger name
  a_ledger_name_c varchar(64),
    -- Defines if the ledger is in use
  a_ledger_status_l boolean DEFAULT 1 NOT NULL,
    -- This is the last batch number used within a ledger
  a_last_batch_number_i integer DEFAULT 0 NOT NULL,
  a_last_recurring_batch_number_i integer DEFAULT 0 NOT NULL,
  a_last_gift_number_i integer DEFAULT 0 NOT NULL,
  a_last_ap_inv_number_i integer DEFAULT 0 NOT NULL,
  a_last_header_r_number_i integer DEFAULT 0 NOT NULL,
  a_last_po_number_i integer DEFAULT 0 NOT NULL,
  a_last_so_number_i integer DEFAULT 0 NOT NULL,
    -- This is a number of currency units
  a_max_gift_aid_amount_n numeric(24, 10) DEFAULT 0,
    -- This is a number of currency units
  a_min_gift_aid_amount_n numeric(24, 10) DEFAULT 0,
  a_number_of_gifts_to_display_i integer DEFAULT 0 NOT NULL,
  a_tax_type_code_c varchar(16),
    -- The account for inter-ledger transfers.
  a_ilt_gl_account_code_c varchar(16),
    -- This identifies the account the financial transaction must be stored against
  a_profit_loss_gl_account_code_c varchar(16),
    -- This defines which accounting period is being used
  a_number_of_accounting_periods_i integer DEFAULT 0 NOT NULL,
    -- This identifies a country. It uses the ISO 3166-1-alpha-2 code elements.
  a_country_code_c varchar(8),
    -- This defines which currency is being used
  a_base_currency_c varchar(16),
    -- Used to get a yes no response from the user
  a_transaction_account_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_year_end_flag_l boolean DEFAULT 0 NOT NULL,
    -- The account for foreign exchange gains or losses
  a_forex_gains_losses_account_c varchar(16) NOT NULL,
    -- Used to get a yes no response from the user
  a_system_interface_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_suspense_account_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_bank_accounts_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_delete_ledger_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_new_financial_year_flag_l boolean DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_recalculate_gl_master_flag_l boolean DEFAULT 0 NOT NULL,
    -- Defines which installation is running on this database
  a_installation_id_c varchar(16),
  a_budget_control_flag_l boolean DEFAULT 0,
  a_budget_data_retention_i integer DEFAULT 0,
  a_cost_of_sales_gl_account_c varchar(16),
  a_creditor_gl_account_code_c varchar(16),
  a_current_financial_year_i integer DEFAULT 0,
  a_current_period_i integer DEFAULT 0,
  a_date_cr_dr_balances_d date,
  a_debtor_gl_account_code_c varchar(16),
  a_fa_depreciation_gl_account_c varchar(16),
  a_fa_gl_account_code_c varchar(16),
  a_fa_pl_on_sale_gl_account_c varchar(16),
  a_fa_prov_for_depn_gl_account_c varchar(16),
  a_ilt_account_flag_l boolean DEFAULT 0,
  a_last_ap_dn_number_i integer DEFAULT 0,
  a_last_po_ret_number_i integer DEFAULT 0,
  a_last_so_del_number_i integer DEFAULT 0,
  a_last_so_ret_number_i integer DEFAULT 0,
  a_last_special_gift_number_i integer DEFAULT 0,
  a_number_fwd_posting_periods_i integer DEFAULT 0,
  a_discount_allowed_pct_n numeric(5, 2) DEFAULT 0,
  a_discount_received_pct_n numeric(5, 2) DEFAULT 0,
  a_po_accrual_gl_account_code_c varchar(16),
    -- This flag is set between the completion of the last month of the year and the year itself. In this state some activities are disabled and some others are enabled.
  a_provisional_year_end_flag_l boolean DEFAULT 0,
  a_purchase_gl_account_code_c varchar(16),
  a_ret_earnings_gl_account_c varchar(16),
  a_sales_gl_account_code_c varchar(16),
  a_so_accrual_gl_account_code_c varchar(16),
  a_stock_accrual_gl_account_c varchar(16),
  a_stock_adj_gl_account_code_c varchar(16),
  a_stock_gl_account_code_c varchar(16),
  a_tax_excl_incl_l boolean DEFAULT 1,
  a_tax_excl_incl_indicator_l boolean DEFAULT 0,
  a_tax_input_gl_account_code_c varchar(16),
  a_tax_input_gl_cc_code_c varchar(16),
  a_tax_output_gl_account_code_c varchar(16),
  a_terms_of_payment_code_c varchar(16),
  a_last_po_rec_number_i integer DEFAULT 0,
  a_tax_gl_account_number_i integer DEFAULT 0,
  a_actuals_data_retention_i integer DEFAULT 11,
    -- Partner key which links the ledger to the partner type record where the type is a ledger.
  p_partner_key_n bigint DEFAULT 0,
  a_calendar_mode_l boolean DEFAULT 1,
    -- How far along is the year end process.
  a_year_end_process_status_i integer DEFAULT 0,
    -- last used cashbook payment header number
  a_last_header_p_number_i integer DEFAULT 0 NOT NULL,
    -- Is this ledger an ILT processing centre (ie, clearinghouse status)
  a_ilt_processing_centre_l boolean DEFAULT 0,
    -- The number of the last gift batch to be created.
  a_last_gift_batch_number_i integer DEFAULT 0,
    -- This defines which currency to use as a second ('international') base currency.
  a_intl_currency_c varchar(16),
    -- The number of the last gift batch to be created.
  a_last_rec_gift_batch_number_i integer DEFAULT 0,
    -- How many years to retain gift data.
  a_gift_data_retention_i integer DEFAULT 2,
    -- When recalculating the account report structure this indicates that all periods should be recalculated.
  a_recalculate_all_periods_l boolean DEFAULT 0,
    -- identifies the Last used ICH process number
  a_last_ich_number_i integer DEFAULT 0 NOT NULL,
    -- Indicates whether the ledger is just for storing consolidated accounts (rather than a 'real' ledger).
  a_consolidation_ledger_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ledger_pk
    PRIMARY KEY (a_ledger_number_i),
  CONSTRAINT a_ledger_fk1
    FOREIGN KEY (a_tax_type_code_c)
    REFERENCES a_tax_type(a_tax_type_code_c),
  CONSTRAINT a_ledger_fk2
    FOREIGN KEY (a_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT a_ledger_fk3
    FOREIGN KEY (a_base_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ledger_fk4
    FOREIGN KEY (a_intl_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ledger_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ledger_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_account_fk2_ref0 
   (a_ledger_number_i),
UNIQUE KEY inx_a_ledger_pk1 
   (a_ledger_number_i),
KEY inx_a_ledger_fk1_key2 
   (a_tax_type_code_c),
KEY inx_a_ledger_fk2_key3 
   (a_country_code_c),
KEY inx_a_ledger_fk3_key4 
   (a_base_currency_c),
KEY inx_a_ledger_fk4_key5 
   (a_intl_currency_c),
KEY inx_a_ledger_fkcr_key6 
   (s_created_by_c),
KEY inx_a_ledger_fkmd_key7 
   (s_modified_by_c),
UNIQUE KEY a_ledger_k20 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is used by the invoicing
-- GROUP: ar
CREATE TABLE a_tax_table (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The tax type is always the same, e.g. VAT
  a_tax_type_code_c varchar(16) NOT NULL,
    -- this describes whether it is e.g. the standard, reduced or zero rate of VAT
  a_tax_rate_code_c varchar(16) NOT NULL,
    -- this describes when this particular percentage rate has become valid by law
  a_tax_valid_from_d date NOT NULL,
    -- This is a short description which is 32 charcters long
  a_tax_rate_description_c varchar(64),
    -- Tax rate
  a_tax_rate_n numeric(24, 10) DEFAULT 0,
    -- flag that prevents this rate from being used, e.g. if it has been replaced by another rate
  a_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_tax_table_pk
    PRIMARY KEY (a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d),
  CONSTRAINT a_tax_table_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_tax_table_fk2
    FOREIGN KEY (a_tax_type_code_c)
    REFERENCES a_tax_type(a_tax_type_code_c),
  CONSTRAINT a_tax_table_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_tax_table_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_tax_table_pk0 
   (a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d),
KEY inx_a_tax_table_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_tax_table_fk2_key2 
   (a_tax_type_code_c),
KEY inx_a_tax_table_fkcr_key3 
   (s_created_by_c),
KEY inx_a_tax_table_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Ledger Init Flags
-- GROUP: account
CREATE TABLE a_ledger_init_flag (

  a_ledger_number_i integer NOT NULL,
  a_init_option_name_c varchar(64) NOT NULL,
  a_value_c varchar(128) DEFAULT 'IsSet' NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ledger_init_flag_pk
    PRIMARY KEY (a_ledger_number_i,a_init_option_name_c),
  CONSTRAINT a_ledger_init_flag_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ledger_init_flag_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ledger_init_flag_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ledger_init_flag_pk0 
   (a_ledger_number_i,a_init_option_name_c),
KEY inx_a_ledger_init_flag_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_ledger_init_flag_fkcr_key2 
   (s_created_by_c),
KEY inx_a_ledger_init_flag_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Used for initial set up of budgets, for how to calculate amounts for each period.  Some possible types are adhoc,same,percentage of annual.
-- GROUP: account
CREATE TABLE a_budget_type (

    -- The type. Adhoc, Split, Same, Inflate.
  a_budget_type_code_c varchar(16) NOT NULL,
  a_budget_type_description_c varchar(64) NOT NULL,
    -- The Petra programme filename of the process to call
  a_budget_process_to_call_c varchar(80) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_budget_type_pk
    PRIMARY KEY (a_budget_type_code_c),
  CONSTRAINT a_budget_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_budget_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_budget_type_pk0 
   (a_budget_type_code_c),
KEY inx_a_budget_type_fkcr_key1 
   (s_created_by_c),
KEY inx_a_budget_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Extra properties that might want to be saved with an account can be defined in this table.
-- GROUP: account
CREATE TABLE a_account_property_code (

    -- Code for the property
  a_property_code_c varchar(40) NOT NULL,
    -- Description of this property
  a_description_c varchar(400),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_account_property_code_pk
    PRIMARY KEY (a_property_code_c),
  CONSTRAINT a_account_property_code_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_account_property_code_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_account_property_code_pk0 
   (a_property_code_c),
KEY inx_count_property_code_fkcr_key1 
   (s_created_by_c),
KEY inx_count_property_code_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores standard and user-defined cost centre types.  For example: Foreign, Local.
-- GROUP: account
CREATE TABLE a_cost_centre_types (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Type of cost centre (Defaults: Foreign or Local)
  a_cost_centre_type_c varchar(16) DEFAULT 'Local' NOT NULL,
    -- The description of the cost centre type
  a_cc_description_c varchar(60),
    -- Can this cost centre type be deleted by the user?
  a_deletable_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_cost_centre_types_pk
    PRIMARY KEY (a_ledger_number_i,a_cost_centre_type_c),
  CONSTRAINT a_cost_centre_types_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_cost_centre_types_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_cost_centre_types_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_cost_centre_types_pk0 
   (a_ledger_number_i,a_cost_centre_type_c),
KEY inx_a_cost_centre_types_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_cost_centre_types_fkcr_key2 
   (s_created_by_c),
KEY inx_a_cost_centre_types_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Master cost centre records which contain details of each cost centre and their relationship to each other; determines the cost centre structure.
-- GROUP: account
CREATE TABLE a_cost_centre (

    -- The ledger in which the cost centre is used.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner but reflected as a character rather than a numeric
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This identifies which cost centre the cost centre summarises to.
  a_cost_centre_to_report_to_c varchar(24),
  a_cost_centre_name_c varchar(64) NOT NULL,
  a_posting_cost_centre_flag_l boolean DEFAULT 1 NOT NULL,
  a_cost_centre_active_flag_l boolean DEFAULT 1,
  a_project_status_l boolean DEFAULT 0,
  a_project_constraint_date_d date,
  a_project_constraint_amount_n numeric(24, 10) DEFAULT 0,
  a_system_cost_centre_flag_l boolean DEFAULT 0,
    -- Type of cost centre (Foreign or Local)
  a_cost_centre_type_c varchar(16) DEFAULT 'Local',
    -- ICH or non-ICH clearing
  a_clearing_account_c varchar(24) DEFAULT '8500',
    -- At Year End, Balance in this Cost Centre will roll up to this account
  a_ret_earnings_account_code_c varchar(24) DEFAULT '9700',
    -- Determine how to roll up the balance in this Cost Centre
  a_rollup_style_c varchar(24) DEFAULT 'Always',
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_cost_centre_pk
    PRIMARY KEY (a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_cost_centre_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_cost_centre_fk2
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_type_c)
    REFERENCES a_cost_centre_types(a_ledger_number_i,a_cost_centre_type_c),
  CONSTRAINT a_cost_centre_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_cost_centre_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_match_fk5_ref0 
   (a_ledger_number_i,a_cost_centre_code_c),
UNIQUE KEY inx_a_cost_centre_pk1 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_cost_centre_fk1_key2 
   (a_ledger_number_i),
KEY inx_a_cost_centre_fk2_key3 
   (a_ledger_number_i,a_cost_centre_type_c),
KEY inx_a_cost_centre_fkcr_key4 
   (s_created_by_c),
KEY inx_a_cost_centre_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This defines a certain revision of a budget in a ledger in a year.
-- GROUP: account
CREATE TABLE a_budget_revision (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The year that this budget applies to
  a_year_i integer NOT NULL,
    -- A budget can have several revisions per year
  a_revision_i integer DEFAULT 0 NOT NULL,
    -- A description of this revision of the budget
  a_description_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_budget_revision_pk
    PRIMARY KEY (a_ledger_number_i,a_year_i,a_revision_i),
  CONSTRAINT a_budget_revision_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_budget_revision_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_budget_revision_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_budget_revision_pk0 
   (a_ledger_number_i,a_year_i,a_revision_i),
KEY inx_a_budget_revision_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_budget_revision_fkcr_key2 
   (s_created_by_c),
KEY inx_a_budget_revision_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Information about each financial period in a ledger.
-- GROUP: account
CREATE TABLE a_accounting_period (

    -- The ledger that the period applies to.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The accounting period number.  Must be <= 20
  a_accounting_period_number_i integer DEFAULT 0 NOT NULL,
    -- This is a short description which is 32 charcters long
  a_accounting_period_desc_c varchar(64) NOT NULL,
  a_period_start_date_d date NOT NULL,
  a_period_end_date_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_accounting_period_pk
    PRIMARY KEY (a_ledger_number_i,a_accounting_period_number_i),
  CONSTRAINT a_accounting_period_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_accounting_period_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_accounting_period_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_accounting_period_pk0 
   (a_ledger_number_i,a_accounting_period_number_i),
KEY inx_a_accounting_period_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_accounting_period_fkcr_key2 
   (s_created_by_c),
KEY inx_a_accounting_period_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- One record describing the limitations imposed on the accounting system.
-- GROUP: account
CREATE TABLE a_accounting_system_parameter (

  a_number_of_accounting_periods_i integer DEFAULT 0,
  a_actuals_data_retention_i integer DEFAULT 0,
  a_budget_data_retention_i integer DEFAULT 0,
  a_number_fwd_posting_periods_i integer DEFAULT 0,
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Recipient gift statement text
  a_recipient_gift_statement_txt_c varchar(264),
    -- Recipient gift statement text 2
  a_recipient_gift_statement_tx2_c varchar(264),
    -- Donor gift statement text
  a_donor_gift_statement_txt_c varchar(264),
    -- Donor gift statement text 2
  a_donor_gift_statement_tx2_c varchar(264),
    -- Hosa statement text
  a_hosa_statement_txt_c varchar(264),
    -- Hosa (Home Office Statement of Accounts) statement text 2
  a_hosa_statement_tx2_c varchar(264),
    -- Hosa statement text 3
  a_hosa_statement_tx3_c varchar(264),
    -- Hosa statement text 4
  a_hosa_statement_tx4_c varchar(264),
    -- Text for the donor receipt heading.
  a_donor_receipt_txt_c varchar(64),
    -- Stewardship Report  text
  a_stewardship_report_txt_c varchar(264),
    -- Stewardship Report  text
  a_stewardship_report_tx2_c varchar(264),
    -- Text for the yearly donor receipt heading.
  a_donor_yearly_receipt_txt_c varchar(64),
    -- How many years to retain gift data.
  a_gift_data_retention_i integer DEFAULT 2,
    -- Text to put on receipt when addressing a deceased donor
  a_deceased_address_text_c varchar(400),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_accounting_system_param_pk
    PRIMARY KEY (a_ledger_number_i),
  CONSTRAINT a_accounting_system_param_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_accounting_system_parameter_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_accounting_system_parameter_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_accounting_system_param_pk0 
   (a_ledger_number_i),
KEY inx_ng_system_parameter_fkcr_key1 
   (s_created_by_c),
KEY inx_ng_system_parameter_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of tables in the financial system, meant to be used with analysis attributes. Not available.
-- GROUP: account
CREATE TABLE a_analysis_store_table (

  a_store_name_c varchar(64) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_analysis_store_table_pk
    PRIMARY KEY (a_store_name_c),
  CONSTRAINT a_analysis_store_table_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_analysis_store_table_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_analysis_store_table_pk0 
   (a_store_name_c),
KEY inx_nalysis_store_table_fkcr_key1 
   (s_created_by_c),
KEY inx_nalysis_store_table_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Types of analysis attributes.
-- GROUP: account
CREATE TABLE a_analysis_type (

    -- The number of the ledger in which the analysis type is used.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
  a_analysis_type_code_c varchar(16) NOT NULL,
    -- This is a short description which is 32 charcters long
  a_analysis_type_description_c varchar(64) NOT NULL,
    -- Shows what type of analysis attribute it is
  a_analysis_mode_l boolean DEFAULT 1 NOT NULL,
  a_analysis_store_c varchar(64),
    -- The name of the field within a table which will be used as the analysis attribute
  a_analysis_element_c varchar(64),
    -- To indicate whether the user or system has set up the analysis type.
  a_system_analysis_type_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_analysis_type_pk
    PRIMARY KEY (a_ledger_number_i,a_analysis_type_code_c),
  CONSTRAINT a_analysis_type_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_analysis_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_analysis_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_analysis_type_pk0 
   (a_ledger_number_i,a_analysis_type_code_c),
KEY inx_a_analysis_type_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_analysis_type_fkcr_key2 
   (s_created_by_c),
KEY inx_a_analysis_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Hold (monthly) corporate rates.
-- GROUP: account
CREATE TABLE a_corporate_exchange_rate (

    -- Defines the currency being exchanged
  a_from_currency_code_c varchar(16) NOT NULL,
    -- Defines which currency is being changed to
  a_to_currency_code_c varchar(16) NOT NULL,
    -- The rate of exchange
  a_rate_of_exchange_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Date the exchange rate becomes effective
  a_date_effective_from_d date NOT NULL,
    -- System generated time stamp.
  a_time_effective_from_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_corporate_exchange_rate_pk
    PRIMARY KEY (a_from_currency_code_c,a_to_currency_code_c,a_date_effective_from_d),
  CONSTRAINT a_corporate_exchange_rate_fk1
    FOREIGN KEY (a_from_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_corporate_exchange_rate_fk2
    FOREIGN KEY (a_to_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_corporate_exchange_rate_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_corporate_exchange_rate_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_corporate_exchange_rate_pk0 
   (a_from_currency_code_c,a_to_currency_code_c,a_date_effective_from_d),
KEY inx_porate_exchange_rate_fk1_key1 
   (a_from_currency_code_c),
KEY inx_porate_exchange_rate_fk2_key2 
   (a_to_currency_code_c),
KEY inx_orate_exchange_rate_fkcr_key3 
   (s_created_by_c),
KEY inx_orate_exchange_rate_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Ad hoc exchange rates.
-- GROUP: account
CREATE TABLE a_daily_exchange_rate (

    -- Defines the currency being exchanged
  a_from_currency_code_c varchar(16) NOT NULL,
    -- Defines which currency is being changed to
  a_to_currency_code_c varchar(16) NOT NULL,
    -- The rate of exchange
  a_rate_of_exchange_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Date the exchange rate becomes effective
  a_date_effective_from_d date NOT NULL,
    -- The date and time
  a_time_effective_from_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_daily_exchange_rate_pk
    PRIMARY KEY (a_from_currency_code_c,a_to_currency_code_c,a_date_effective_from_d,a_time_effective_from_i),
  CONSTRAINT a_daily_exchange_rate_fk1
    FOREIGN KEY (a_from_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_daily_exchange_rate_fk2
    FOREIGN KEY (a_to_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_daily_exchange_rate_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_daily_exchange_rate_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_daily_exchange_rate_pk0 
   (a_from_currency_code_c,a_to_currency_code_c,a_date_effective_from_d,a_time_effective_from_i),
KEY inx__daily_exchange_rate_fk1_key1 
   (a_from_currency_code_c),
KEY inx__daily_exchange_rate_fk2_key2 
   (a_to_currency_code_c),
KEY inx_daily_exchange_rate_fkcr_key3 
   (s_created_by_c),
KEY inx_daily_exchange_rate_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- email addresses of our organisation
-- GROUP: account
CREATE TABLE p_email (

  p_email_address_c varchar(80) DEFAULT 'first.last@field.om.org' NOT NULL,
  p_description_c varchar(160),
  p_valid_l boolean DEFAULT 1,
    -- This defines if the email code can be deleted. This can only be updated by the system manager. At the risk of serious operational integrity. Default to Yes
  p_deletable_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_email_pk
    PRIMARY KEY (p_email_address_c),
  CONSTRAINT p_email_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_email_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_email_pk0 
   (p_email_address_c),
KEY inx_p_email_fkcr_key1 
   (s_created_by_c),
KEY inx_p_email_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The form description for user definable forms such as receipts
-- GROUP: account
CREATE TABLE a_form (

    -- The code which defines the type of form described in the table
  a_form_code_c varchar(20) NOT NULL,
    -- The name of the form being created for the form code.
  a_form_name_c varchar(20) NOT NULL,
    -- Description of the form
  a_form_description_c varchar(100),
    -- File name used for the form. Includes path information.
  a_form_file_name_c varchar(2000),
    -- If there are several types of form then it can be specified here.  Eg an annual receipt and an individual receipt.
  a_form_type_code_c varchar(24) NOT NULL,
    -- The number of repeating lines that will be displayed on each page of a form.
  a_number_of_details_i integer DEFAULT 0,
    -- Is the report to be formatted to print in bold or not.
  a_print_in_bold_l boolean DEFAULT 0 NOT NULL,
    -- The total number of lines that can be displayed on the page
  a_lines_on_page_i integer DEFAULT 66 NOT NULL,
    -- The minimum amount that is acceptable on a receipt
  a_minimum_amount_n numeric(24, 10) DEFAULT 0,
    -- Allows the exclusion of certain records from a report
  a_options_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_form_pk
    PRIMARY KEY (a_form_code_c,a_form_name_c),
  CONSTRAINT a_form_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_form_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_form_pk0 
   (a_form_code_c,a_form_name_c),
KEY inx_a_form_fkcr_key1 
   (s_created_by_c),
KEY inx_a_form_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The element types that are available for placing in forms.
-- GROUP: account
CREATE TABLE a_form_element_type (

    -- The code which defines the type of form described in the table
  a_form_code_c varchar(20) NOT NULL,
    -- The code of an element type that can be positioned for use on a form.
  a_form_element_type_code_c varchar(40) NOT NULL,
    -- Description of Element Type
  a_form_element_type_desc_c varchar(160),
  a_default_length_i integer DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_form_element_type_pk
    PRIMARY KEY (a_form_code_c,a_form_element_type_code_c),
  CONSTRAINT a_form_element_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_form_element_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_form_element_type_pk0 
   (a_form_code_c,a_form_element_type_code_c),
KEY inx_a_form_element_type_fkcr_key1 
   (s_created_by_c),
KEY inx_a_form_element_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Each element that can be displayed on a form is defined on this table.
-- GROUP: account
CREATE TABLE a_form_element (

    -- The code which defines the type of form described in the table
  a_form_code_c varchar(20) NOT NULL,
    -- The name of the form being created for the form code.
  a_form_name_c varchar(20) NOT NULL,
    -- Unique identifier of each form element within a form
  a_form_sequence_i integer DEFAULT 0 NOT NULL,
    -- The code of an element type that can be possitioned for use on a form.
  a_form_element_type_code_c varchar(40) NOT NULL,
    -- The column that the element is to be displayed at on the form.
  a_column_i integer DEFAULT 1 NOT NULL,
    -- The row on the page that the element is to be displayed at.
  a_row_i integer DEFAULT 1 NOT NULL,
    -- The display length for the element.  e.g. it may be necessary to truncate fields.
  a_length_i integer DEFAULT 1 NOT NULL,
    -- This stores the number of characters to skip prior to printing.  It can be used to spread a description over two lines.
  a_skip_i integer DEFAULT 0,
    -- Indicates when the element is printed.  This would indicate detail lines, first, last page etc.
  a_when_print_c varchar(40) NOT NULL,
    -- Text to be displayed if the element is defined as a literal.
  a_literal_text_c varchar(2000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_form_element_pk
    PRIMARY KEY (a_form_code_c,a_form_name_c,a_form_sequence_i),
  CONSTRAINT a_form_element_fk1
    FOREIGN KEY (a_form_code_c,a_form_name_c)
    REFERENCES a_form(a_form_code_c,a_form_name_c),
  CONSTRAINT a_form_element_fk2
    FOREIGN KEY (a_form_code_c,a_form_element_type_code_c)
    REFERENCES a_form_element_type(a_form_code_c,a_form_element_type_code_c),
  CONSTRAINT a_form_element_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_form_element_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_form_element_pk0 
   (a_form_code_c,a_form_name_c,a_form_sequence_i),
KEY inx_a_form_element_fk1_key1 
   (a_form_code_c,a_form_name_c),
KEY inx_a_form_element_fk2_key2 
   (a_form_code_c,a_form_element_type_code_c),
KEY inx_a_form_element_fkcr_key3 
   (s_created_by_c),
KEY inx_a_form_element_fkmd_key4 
   (s_modified_by_c),
KEY a_form_element_pk0 
   (a_form_code_c,a_form_name_c,a_row_i,a_column_i,a_form_sequence_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Available analysis values that may be given to a transaction for each analysis type.
-- GROUP: account
CREATE TABLE a_freeform_analysis (

    -- Ledger Number 
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
  a_analysis_type_code_c varchar(32) NOT NULL,
    -- Value of analysis code
  a_analysis_value_c varchar(160) NOT NULL,
    -- Analysis attribute values cannot be deleted, because they are needed for existing transaction analysis attributes. But they can be deactivated.
  a_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_freeform_analysis_pk
    PRIMARY KEY (a_ledger_number_i,a_analysis_type_code_c,a_analysis_value_c),
  CONSTRAINT a_freeform_analysis_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_freeform_analysis_fk2
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c)
    REFERENCES a_analysis_type(a_ledger_number_i,a_analysis_type_code_c),
  CONSTRAINT a_freeform_analysis_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_freeform_analysis_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_freeform_analysis_pk0 
   (a_ledger_number_i,a_analysis_type_code_c,a_analysis_value_c),
KEY inx_a_freeform_analysis_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_freeform_analysis_fk2_key2 
   (a_ledger_number_i,a_analysis_type_code_c),
KEY inx_a_freeform_analysis_fkcr_key3 
   (s_created_by_c),
KEY inx_a_freeform_analysis_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains the calculation for the stewardship for a particular run.  Files and reports can be recreated from this.  ICH = International Clearing House (Handles transfering of funds and admin grant fees among offices.)
-- GROUP: account
CREATE TABLE a_ich_stewardship (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Accounting Year
  a_year_i integer DEFAULT 0 NOT NULL,
    -- This defines which accounting period is being used
  a_period_number_i integer DEFAULT 0 NOT NULL,
    -- identifies the ICH process number
  a_ich_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner but reflected as a character rather than a numeric
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This is the date the stewardship was processed.
  a_date_processed_d date NOT NULL,
    -- Income amount for foreign cost centre's stewardship
  a_income_amount_n numeric(24, 10) DEFAULT 0,
    -- Expense amount for foreign cost centre's stewardship
  a_expense_amount_n numeric(24, 10) DEFAULT 0,
    -- Direct Transfer amount for foreign cost centre's stewardship
  a_direct_xfer_amount_n numeric(24, 10) DEFAULT 0,
    -- Income amount for foreign cost centre's stewardship in International Currency
  a_income_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- Expense amount for foreign cost centre's stewardship in international currency
  a_expense_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- Direct Transfer amount for foreign cost centre's stewardship in international currency.
  a_direct_xfer_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ich_stewardship_pk
    PRIMARY KEY (a_ledger_number_i,a_period_number_i,a_ich_number_i,a_cost_centre_code_c),
  CONSTRAINT a_ich_stewardship_fk1
    FOREIGN KEY (a_ledger_number_i,a_period_number_i)
    REFERENCES a_accounting_period(a_ledger_number_i,a_accounting_period_number_i),
  CONSTRAINT a_ich_stewardship_fk2
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_ich_stewardship_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ich_stewardship_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ich_stewardship_pk0 
   (a_ledger_number_i,a_period_number_i,a_ich_number_i,a_cost_centre_code_c),
KEY inx_a_ich_stewardship_fk1_key1 
   (a_ledger_number_i,a_period_number_i),
KEY inx_a_ich_stewardship_fk2_key2 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_ich_stewardship_fkcr_key3 
   (s_created_by_c),
KEY inx_a_ich_stewardship_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Special payment programs the donor may give money through. (ie, Gift Aid in the UK). Currently not used in Petra release 1.
-- GROUP: account
CREATE TABLE a_method_of_giving (

    -- Defines how a gift is given
  a_method_of_giving_code_c varchar(24) NOT NULL,
    -- This is a short description which is 32 charcters long
  a_method_of_giving_desc_c varchar(64) NOT NULL,
    -- Shows if the method of giving involves a trust
  a_trust_flag_l boolean DEFAULT 0 NOT NULL,
    -- Shows if this method of giving involves a tax rebate
  a_tax_rebate_flag_l boolean DEFAULT 0 NOT NULL,
    -- Shows if this method of giving is used by recurring gifts
  a_recurring_method_flag_l boolean DEFAULT 0 NOT NULL,
    -- Shows whether this code is active
  a_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_method_of_giving_pk
    PRIMARY KEY (a_method_of_giving_code_c),
  CONSTRAINT a_method_of_giving_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_method_of_giving_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_match_fk6_ref0 
   (a_method_of_giving_code_c),
UNIQUE KEY inx_a_method_of_giving_pk1 
   (a_method_of_giving_code_c),
KEY inx_a_method_of_giving_fkcr_key2 
   (s_created_by_c),
KEY inx_a_method_of_giving_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Media types of money received. Eg: Cash, Check Credit Card
-- GROUP: gift
CREATE TABLE a_method_of_payment (

    -- This is how the partner paid. EgCash, Cheque etc
  a_method_of_payment_code_c varchar(16) NOT NULL,
    -- This is a short description which is 32 charcters long
  a_method_of_payment_desc_c varchar(64) NOT NULL,
  a_method_of_payment_type_c varchar(16),
    -- The filename of the process to call
  a_process_to_call_c varchar(80),
  a_special_method_of_pmt_l boolean DEFAULT 0 NOT NULL,
    -- Shows whether this code is active
  a_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_method_of_payment_pk
    PRIMARY KEY (a_method_of_payment_code_c),
  CONSTRAINT a_method_of_payment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_method_of_payment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_match_fk7_ref0 
   (a_method_of_payment_code_c),
UNIQUE KEY inx_a_method_of_payment_pk1 
   (a_method_of_payment_code_c),
KEY inx_a_method_of_payment_fkcr_key2 
   (s_created_by_c),
KEY inx_a_method_of_payment_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is used to track a partner's reason for contacting the organisation/sending money. Divided into Motivation Detail codes.
-- GROUP: gift
CREATE TABLE a_motivation_group (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16) NOT NULL,
    -- This is a long description and is 80 characters long.
  a_motivation_group_description_c varchar(100) NOT NULL,
    -- Defines whether the motivation group is still in use
  a_group_status_l boolean DEFAULT 1 NOT NULL,
    -- This is a long description and is 80 characters long in the local language.
  a_motivation_group_desc_local_c varchar(100),
    -- Indicates whether or not the motivation has restricted access. If it does then the access will be controlled by s_group_motivation
  a_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_motivation_group_pk
    PRIMARY KEY (a_ledger_number_i,a_motivation_group_code_c),
  CONSTRAINT a_motivation_group_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_motivation_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_motivation_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_motivation_group_pk0 
   (a_ledger_number_i,a_motivation_group_code_c),
KEY inx_a_motivation_group_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_motivation_group_fkcr_key2 
   (s_created_by_c),
KEY inx_a_motivation_group_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Templates of general ledger batches which can be copied into the ledger.
-- GROUP: account
CREATE TABLE a_recurring_batch (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- This is a long description and is 80 characters long.
  a_batch_description_c varchar(160),
    -- This is a number of currency units
  a_batch_control_total_n numeric(24, 10) DEFAULT 0,
    -- identifies the status of a batch
  a_batch_status_c varchar(24) DEFAULT 'Unposted' NOT NULL,
    -- This is a number of currency units
  a_batch_running_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is a number of currency units
  a_batch_debit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is a number of currency units
  a_batch_credit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This defines which accounting period is being used
  a_batch_period_i integer DEFAULT 0,
    -- Date the  batch comes into effect
  a_date_effective_d date NOT NULL,
    -- This identifies who the current user is
  s_user_id_c varchar(20),
  a_date_of_entry_d date,
  a_frequency_code_c varchar(24),
  a_date_batch_last_run_d date,
    -- Identifies a journal within a batch
  a_last_journal_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_batch_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_batch_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_recurring_batch_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_batch_fk3
    FOREIGN KEY (a_frequency_code_c)
    REFERENCES a_frequency(a_frequency_code_c),
  CONSTRAINT a_recurring_batch_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_batch_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_batch_pk0 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_recurring_batch_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_recurring_batch_fk2_key2 
   (s_user_id_c),
KEY inx_a_recurring_batch_fk3_key3 
   (a_frequency_code_c),
KEY inx_a_recurring_batch_fkcr_key4 
   (s_created_by_c),
KEY inx_a_recurring_batch_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Store current and forward period general ledger batches for a ledger.
-- GROUP: account
CREATE TABLE a_batch (

    -- The ledger that the batch belongs to.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies the batch.
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- This is a long description and is 80 characters long.
  a_batch_description_c varchar(160),
    -- Raw total amount of the batch.  If the journals within the batch have different currencies, this is just a raw numeric sum of all the journal amounts.  It does not reflect a monetary value in a specific currency.  Entered by the user.
  a_batch_control_total_n numeric(24, 10) DEFAULT 0,
    -- Running total amount of the batch calculated as transactions are enterd.  If the journals within the batch have different currencies, this is just a raw numeric sum of all the journal amounts.  It does not reflect a monetary value in a specific currency.
  a_batch_running_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Raw debit total amount of the batch.  If the journals within the batch have different currencies, this is just a raw numeric sum of all the journal amounts.  It does not reflect a monetary value in a specific currency.
  a_batch_debit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Raw credit total amount of the batch.  If the journals within the batch have different currencies, this is just a raw numeric sum of all the journal amounts.  It does not reflect a monetary value in a specific currency.
  a_batch_credit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This identifies which accounting period that the batch belongs to.
  a_batch_period_i integer DEFAULT 0 NOT NULL,
    -- The number of the accounting year
  a_batch_year_i integer DEFAULT 0,
    -- Date the  batch comes into effect
  a_date_effective_d date NOT NULL,
    -- Date the  batch was created.
  a_date_of_entry_d date NOT NULL,
    -- Has this batch been posted yet?
  a_batch_status_c varchar(24) DEFAULT 'Unposted',
    -- Identifies a journal within a batch
  a_last_journal_i integer DEFAULT 0 NOT NULL,
    -- Number of the originating gift batch that generated this GL batch.
  a_gift_batch_number_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_batch_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_batch_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_batch_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_batch_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_batch_pk0 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_batch_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_batch_fkcr_key2 
   (s_created_by_c),
KEY inx_a_batch_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Holds information used specifically in revaluations. The table extends the a_journal table.
-- GROUP: account
CREATE TABLE a_revaluation (

    -- The revaluation journal belongs to this ledger.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch the revaluation journal belongs to.
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies the revaluation journal within a batch (usually 1)
  a_journal_number_i integer NOT NULL,
    -- This defines which revaluation currency the rate applies to
  a_revaluation_currency_c varchar(16) NOT NULL,
    -- The rate of exchange from the revaluation currency (in a_revaluation_currency_c) to the ledger base currency.
  a_exchange_rate_to_base_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_revaluation_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_revaluation_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_revaluation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_revaluation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_revaluation_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_a_revaluation_fk1_key1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_revaluation_fkcr_key2 
   (s_created_by_c),
KEY inx_a_revaluation_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Transaction types which have special processing. Eg. Allocation, Reallocation, Gift, Revaluation.
-- GROUP: account
CREATE TABLE a_special_trans_type (

    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24) NOT NULL,
  a_transaction_type_code_c varchar(16) NOT NULL,
    -- This is a short description which is 32 charcters long
  a_transaction_type_description_c varchar(64) NOT NULL,
    -- The filename of the process to call
  a_spec_trans_process_to_call_c varchar(80) NOT NULL,
    -- Process to call to undo the work of the special transaction process, if needed
  a_spec_trans_undo_process_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_special_trans_type_pk
    PRIMARY KEY (a_sub_system_code_c,a_transaction_type_code_c),
  CONSTRAINT a_special_trans_type_fk1
    FOREIGN KEY (a_sub_system_code_c)
    REFERENCES a_sub_system(a_sub_system_code_c),
  CONSTRAINT a_special_trans_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_special_trans_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_special_trans_type_pk0 
   (a_sub_system_code_c,a_transaction_type_code_c),
KEY inx_a_special_trans_type_fk1_key1 
   (a_sub_system_code_c),
KEY inx__special_trans_type_fkcr_key2 
   (s_created_by_c),
KEY inx__special_trans_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Shows which systems are interfaced (have been added) to each general ledger.
-- GROUP: account
CREATE TABLE a_system_interface (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24) NOT NULL,
  a_set_up_complete_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_system_interface_pk
    PRIMARY KEY (a_ledger_number_i,a_sub_system_code_c),
  CONSTRAINT a_system_interface_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_system_interface_fk2
    FOREIGN KEY (a_sub_system_code_c)
    REFERENCES a_sub_system(a_sub_system_code_c),
  CONSTRAINT a_system_interface_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_system_interface_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_system_interface_pk0 
   (a_ledger_number_i,a_sub_system_code_c),
KEY inx_a_system_interface_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_system_interface_fk2_key2 
   (a_sub_system_code_c),
KEY inx_a_system_interface_fkcr_key3 
   (s_created_by_c),
KEY inx_a_system_interface_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Describes what is to be displayed for a currency and language combination when displaying the amount as text.
-- GROUP: account
CREATE TABLE a_currency_language (

    -- This defines which currency is being used
  a_currency_code_c varchar(16) NOT NULL,
    -- This is the code used to identify a language.
  p_language_code_c varchar(20) NOT NULL,
    -- The currency unit label when the amount is 1
  a_unit_label_singular_c varchar(32),
    -- The currency unit label when the amount is > 1
  a_unit_label_plural_c varchar(32),
    -- This is needed to determine the gender in some languages.
  a_special_code_c varchar(32),
    -- Describes what to do with the decimal when representing it as text.  Not to display, display as text or as a numeric
  a_decimal_options_c varchar(24),
    -- The currency decimal label when the amount is 1
  a_decimal_label_singular_c varchar(32),
    -- The currency decimal label when the amount is > 1
  a_decimal_label_plural_c varchar(32),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_currency_language_pk
    PRIMARY KEY (a_currency_code_c,p_language_code_c),
  CONSTRAINT a_currency_language_fk1
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_currency_language_fk2
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT a_currency_language_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_currency_language_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_currency_language_pk0 
   (a_currency_code_c,p_language_code_c),
KEY inx_a_currency_language_fk1_key1 
   (a_currency_code_c),
KEY inx_a_currency_language_fk2_key2 
   (p_language_code_c),
KEY inx_a_currency_language_fkcr_key3 
   (s_created_by_c),
KEY inx_a_currency_language_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- there are several categories that are can use invoicing: catering, hospitality, store and fees
-- GROUP: ar
CREATE TABLE a_ar_category (

    -- categories help to specify certain discounts and group articles etc
  a_ar_category_code_c varchar(100) NOT NULL,
    -- description of this category
  a_ar_description_c varchar(300),
    -- description of this category in the local language
  a_ar_local_description_c varchar(300),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_category_pk
    PRIMARY KEY (a_ar_category_code_c),
  CONSTRAINT a_ar_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ar_category_pk0 
   (a_ar_category_code_c),
KEY inx_a_ar_category_fkcr_key1 
   (s_created_by_c),
KEY inx_a_ar_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines an item that can be sold or a service that can be charged for; this can be used for catering, hospitality, store and fees; it can describe a specific book, or a group of equally priced books
-- GROUP: ar
CREATE TABLE a_ar_article (

    -- code that uniquely identifies the item; can also be a code of a group of equally priced items
  a_ar_article_code_c varchar(100) NOT NULL,
    -- this article belongs to a certain category (catering, hospitality, store, fees)
  a_ar_category_code_c varchar(100) NOT NULL,
    -- this article falls into a special tax/VAT category
  a_tax_type_code_c varchar(100) NOT NULL,
    -- describes whether this describes a specific item, e.g. book, or a group of equally priced items
  a_ar_specific_article_l boolean,
    -- description of this article
  a_ar_description_c varchar(300),
    -- description of this article in the local language
  a_ar_local_description_c varchar(300),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_article_pk
    PRIMARY KEY (a_ar_article_code_c),
  CONSTRAINT a_ar_article_fk1
    FOREIGN KEY (a_ar_category_code_c)
    REFERENCES a_ar_category(a_ar_category_code_c),
  CONSTRAINT a_ar_article_fk2
    FOREIGN KEY (a_tax_type_code_c)
    REFERENCES a_tax_type(a_tax_type_code_c),
  CONSTRAINT a_ar_article_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_article_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ar_article_pk0 
   (a_ar_article_code_c),
KEY inx_a_ar_article_fk1_key1 
   (a_ar_category_code_c),
KEY inx_a_ar_article_fk2_key2 
   (a_tax_type_code_c),
KEY inx_a_ar_article_fkcr_key3 
   (s_created_by_c),
KEY inx_a_ar_article_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- assign a price to an article, which can be updated by time
-- GROUP: ar
CREATE TABLE a_ar_article_price (

    -- code that identifies the item to be sold or service to be charged
  a_ar_article_code_c varchar(100) NOT NULL,
    -- date from which this price is valid
  a_ar_date_valid_from_d date NOT NULL,
    -- the value of the item in base currency
  a_ar_amount_n numeric(24, 10) NOT NULL,
    -- the currency in which the price is given
  a_currency_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ar_article_price_pk
    PRIMARY KEY (a_ar_article_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_article_price_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_article_price_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ar_article_price_fk1
    FOREIGN KEY (a_ar_article_code_c)
    REFERENCES a_ar_article(a_ar_article_code_c),
  CONSTRAINT ar_article_price_fk2
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
UNIQUE KEY inx_ar_article_price_pk0 
   (a_ar_article_code_c,a_ar_date_valid_from_d),
KEY inx_a_ar_article_price_fkcr_key1 
   (s_created_by_c),
KEY inx_a_ar_article_price_fkmd_key2 
   (s_modified_by_c),
KEY inx_ar_article_price_fk1_key3 
   (a_ar_article_code_c),
KEY inx_ar_article_price_fk2_key4 
   (a_currency_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines a discount that depends on other conditions or can just be assigned to an invoice or article
-- GROUP: ar
CREATE TABLE a_ar_discount (

    -- code that identifies the discount
  a_ar_discount_code_c varchar(100) NOT NULL,
    -- date from which this discount is valid
  a_ar_date_valid_from_d date NOT NULL,
    -- this discount has only be created on the fly and should not be reusable elsewhere
  a_ar_adhoc_l boolean DEFAULT 1,
    -- flag that prevents this discount from being used, to avoid too long lists in comboboxes etc
  a_active_l boolean DEFAULT 1,
    -- discount percentage; can be negative for expensive rooms etc
  a_ar_discount_percentage_n numeric(5, 2) DEFAULT 0,
    -- the absolute discount that is substracted from the article price; can be negative as well
  a_ar_discount_absolute_n numeric(24, 10),
    -- the absolute amount that is charged if this discount applies; e.g. 3 books for 5 Pound
  a_ar_absolute_amount_n numeric(24, 10),
    -- the currency in which the absolute discount or amount is given
  a_currency_code_c varchar(16) NOT NULL,
    -- this discount applies for this number of items that are bought at the same time
  a_ar_number_of_items_i integer,
    -- this discount applies for all of the items if at least this number of items is bought at the same time
  a_ar_minimum_number_of_items_i integer,
    -- this discount applies for this number of nights that the individual or group stays; this is needed because 100 people staying for one night do cost more than 50 people staying for 2 nights
  a_ar_number_of_nights_i integer,
    -- this discount applies for all of the nights if the individual or group stays at least for the given amount of nights; this is needed because 100 people staying for one night do cost more than 50 people staying for 2 nights 
  a_ar_minimum_number_of_nights_i integer,
    -- this discount applies when a whole room is booked rather than just a bed
  a_ar_whole_room_l boolean,
    -- this discount applies for a children (e.g. meals)
  a_ar_children_l boolean,
    -- this discount applies when the booking has been done so many days before the stay (using ph_booking.ph_confirmed_d and ph_in_d)
  a_ar_early_booking_days_i integer,
    -- this discount applies when the payment has been received within the given number of days after the invoice has been charged
  a_ar_early_payment_days_i integer,
    -- this discount applies if the article code matches
  a_ar_article_code_c varchar(100),
    -- this discounts applies to partners of this type
  p_partner_type_code_c varchar(20),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ar_discount_pk
    PRIMARY KEY (a_ar_discount_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_discount_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_discount_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ar_discount_fk1
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT ar_discount_fk2
    FOREIGN KEY (p_partner_type_code_c)
    REFERENCES p_type(p_type_code_c),
  CONSTRAINT ar_discount_fk3
    FOREIGN KEY (a_ar_article_code_c)
    REFERENCES a_ar_article(a_ar_article_code_c),
UNIQUE KEY inx_ar_discount_pk0 
   (a_ar_discount_code_c,a_ar_date_valid_from_d),
KEY inx_a_ar_discount_fkcr_key1 
   (s_created_by_c),
KEY inx_a_ar_discount_fkmd_key2 
   (s_modified_by_c),
KEY inx_ar_discount_fk1_key3 
   (a_currency_code_c),
KEY inx_ar_discount_fk2_key4 
   (p_partner_type_code_c),
KEY inx_ar_discount_fk3_key5 
   (a_ar_article_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines which discount applies to which category to limit the options in the UI
-- GROUP: ar
CREATE TABLE a_ar_discount_per_category (

    -- refers to a certain category (catering, hospitality, store, fees)
  a_ar_category_code_c varchar(100) NOT NULL,
    -- code that identifies the discount
  a_ar_discount_code_c varchar(100) NOT NULL,
    -- date is only required for foreign key; this applies to all of that discount code; therefore the primary key of this table does not include the date
  a_ar_date_valid_from_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ar_discount_per_category_pk
    PRIMARY KEY (a_ar_category_code_c,a_ar_discount_code_c),
  CONSTRAINT a_ar_discount_per_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_discount_per_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ar_discount_per_category_fk1
    FOREIGN KEY (a_ar_category_code_c)
    REFERENCES a_ar_category(a_ar_category_code_c),
  CONSTRAINT ar_discount_per_category_fk2
    FOREIGN KEY (a_ar_discount_code_c,a_ar_date_valid_from_d)
    REFERENCES a_ar_discount(a_ar_discount_code_c,a_ar_date_valid_from_d),
UNIQUE KEY inx_ar_discount_per_category_pk0 
   (a_ar_category_code_c,a_ar_discount_code_c),
KEY inx_scount_per_category_fkcr_key1 
   (s_created_by_c),
KEY inx_scount_per_category_fkmd_key2 
   (s_modified_by_c),
KEY inx_iscount_per_category_fk1_key3 
   (a_ar_category_code_c),
KEY inx_iscount_per_category_fk2_key4 
   (a_ar_discount_code_c,a_ar_date_valid_from_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines which discounts should be applied by default during a certain event or time period to articles from a certain category
-- GROUP: ar
CREATE TABLE a_ar_default_discount (

    -- refers to a certain category (catering, hospitality, store, fees)
  a_ar_category_code_c varchar(100) NOT NULL,
    -- code that identifies the discount
  a_ar_discount_code_c varchar(100) NOT NULL,
    -- this clearly specifies which version of the discount is meant
  a_ar_discount_date_valid_from_d date NOT NULL,
    -- this default discount is only applied during this time period
  a_ar_date_valid_from_d date NOT NULL,
    -- this default discount is only applied during this time period; can be null for ongoing default discounts
  a_ar_date_valid_to_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ar_default_discount_pk
    PRIMARY KEY (a_ar_category_code_c,a_ar_discount_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_default_discount_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_default_discount_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ar_default_discount_fk1
    FOREIGN KEY (a_ar_category_code_c)
    REFERENCES a_ar_category(a_ar_category_code_c),
  CONSTRAINT ar_default_discount_fk2
    FOREIGN KEY (a_ar_discount_code_c,a_ar_date_valid_from_d)
    REFERENCES a_ar_discount(a_ar_discount_code_c,a_ar_date_valid_from_d),
UNIQUE KEY inx_ar_default_discount_pk0 
   (a_ar_category_code_c,a_ar_discount_code_c,a_ar_date_valid_from_d),
KEY inx_ar_default_discount_fkcr_key1 
   (s_created_by_c),
KEY inx_ar_default_discount_fkmd_key2 
   (s_modified_by_c),
KEY inx_ar_default_discount_fk1_key3 
   (a_ar_category_code_c),
KEY inx_ar_default_discount_fk2_key4 
   (a_ar_discount_code_c,a_ar_date_valid_from_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the different codes that indicate where an applicant is in the application continuum.
-- GROUP: personnel
CREATE TABLE pt_applicant_status (

    -- This code indicates the status of an applicant.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the applicant status code.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_applicant_status_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_applicant_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_applicant_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_applicant_status_pk0 
   (pt_code_c),
KEY inx_pt_applicant_status_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_applicant_status_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This decribes the type of application a person has submitted, e.g, Short-Term, Long-Term, or 2 years.
-- GROUP: personnel
CREATE TABLE pt_application_type (

    -- Name of the application type, e.g. Short-Term, Long-Term.
  pt_app_type_name_c varchar(32) NOT NULL,
    -- Describes the application type.
  pt_app_type_descr_c varchar(80),
    -- Can this application type be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- This field lists the different forms that are used for the various application types. 
  pt_app_form_type_c varchar(32),
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_application_type_pk
    PRIMARY KEY (pt_app_type_name_c),
  CONSTRAINT pt_application_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_application_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_application_type_pk0 
   (pt_app_type_name_c),
KEY inx_pt_application_type_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_application_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This list the various methods by which a person learns of this organisation. This table can be changed to fit any field's particular awareness programs.
-- GROUP: personnel
CREATE TABLE pt_contact (

    -- Name of the contact, e.g.  Friend, Program, Church.
  pt_contact_name_c varchar(40) NOT NULL,
    -- Describes the contact.
  pt_contact_descr_c varchar(80),
    -- Can this position be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_contact_pk
    PRIMARY KEY (pt_contact_name_c),
  CONSTRAINT pt_contact_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_contact_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_contact_pk0 
   (pt_contact_name_c),
KEY inx_pt_contact_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_contact_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate special situations of applicants. 
-- GROUP: personnel
CREATE TABLE pt_special_applicant (

    -- This code indicates a special status an applicant could have.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the special applicant status code.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_special_applicant_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_special_applicant_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_special_applicant_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_special_applicant_pk0 
   (pt_code_c),
KEY inx_t_special_applicant_fkcr_key1 
   (s_created_by_c),
KEY inx_t_special_applicant_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes indicating the leadership potential of someone.
-- GROUP: personnel
CREATE TABLE pt_leadership_rating (

    -- This code indicates the leadership rating of an applicant.
  pt_code_c varchar(2) NOT NULL,
    -- This describes the Leadership Rating code.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_leadership_rating_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_leadership_rating_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_leadership_rating_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_leadership_rating_pk0 
   (pt_code_c),
KEY inx_t_leadership_rating_fkcr_key1 
   (s_created_by_c),
KEY inx_t_leadership_rating_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes used to indicate where the conferee is arriving or departing.
-- GROUP: personnel
CREATE TABLE pt_arrival_point (

    -- This code indicates the arrival point of the congress attendee.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the arrival point.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_arrival_point_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_arrival_point_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_arrival_point_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_arrival_point_pk0 
   (pt_code_c),
KEY inx_pt_arrival_point_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_arrival_point_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Describes the importance of the country and activity choices. 
-- GROUP: personnel
CREATE TABLE pt_outreach_preference_level (

    -- This code indicates the level of importance of the country and activity choice on outreachs.
  pt_code_c varchar(2) NOT NULL,
    -- This describes the preference level code.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_outreach_preference_level_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_outreach_preference_level_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_outreach_preference_level_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_outreach_preference_level_pk0 
   (pt_code_c),
KEY inx_ch_preference_level_fkcr_key1 
   (s_created_by_c),
KEY inx_ch_preference_level_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate a person's role and/or position at conferences and outreachs.
-- GROUP: personnel
CREATE TABLE pt_congress_code (

    -- This code indicates the role of the event attendee.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the Event Role.
  pt_description_c varchar(80),
    -- Indicates if this is valid pre-Conference Role.
  pt_pre_congress_l boolean DEFAULT 0,
    -- Indicates if this role can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this role was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
  pt_discounted_l boolean DEFAULT 0,
    -- Indicates if this is a valid role during a outreach.
  pt_outreach_l boolean DEFAULT 0,
    -- Indicates if this is valid Conference Role.
  pt_conference_l boolean DEFAULT 0,
    -- Indicates if this role is considered as a normal participant (e.g. in reports)
  pt_participant_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_congress_code_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_congress_code_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_congress_code_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_congress_code_pk0 
   (pt_code_c),
KEY inx_pt_congress_code_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_congress_code_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the mode of travel being used. 
-- GROUP: personnel
CREATE TABLE pt_travel_type (

    -- This code indicates the different types of travel.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the types of travel .
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_travel_type_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_travel_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_travel_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_travel_type_pk0 
   (pt_code_c),
KEY inx_pt_travel_type_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_travel_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the categories of document types (grouping). 
-- GROUP: personnel
CREATE TABLE pm_document_category (

    -- This code indicates the category of a document type.
  pm_code_c varchar(64) NOT NULL,
    -- This describes the document type category.
  pm_description_c varchar(100),
    -- Indicates if document types for this category can be added on the fly
  pm_extendable_l boolean DEFAULT 0,
    -- Indicates if this code can still be assigned?
  pm_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pm_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pm_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_document_category_pk
    PRIMARY KEY (pm_code_c),
  CONSTRAINT pm_document_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_document_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_document_category_pk0 
   (pm_code_c),
KEY inx_m_document_category_fkcr_key1 
   (s_created_by_c),
KEY inx_m_document_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the types of documents for a person. 
-- GROUP: personnel
CREATE TABLE pm_document_type (

    -- This code indicates the type of document for a person.
  pm_doc_code_c varchar(40) NOT NULL,
    -- This code indicates the category of a document type.
  pm_doc_category_c varchar(64) NOT NULL,
    -- This describes the document type.
  pm_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pm_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pm_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pm_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_document_type_pk
    PRIMARY KEY (pm_doc_code_c),
  CONSTRAINT pm_document_type_fk1
    FOREIGN KEY (pm_doc_category_c)
    REFERENCES pm_document_category(pm_code_c),
  CONSTRAINT pm_document_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_document_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_document_type_pk0 
   (pm_doc_code_c),
KEY inx_pm_document_type_fk1_key1 
   (pm_doc_category_c),
KEY inx_pm_document_type_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_document_type_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the type of passport a person holds. 
-- GROUP: personnel
CREATE TABLE pt_passport_type (

    -- This code indicates the type of passport a person holds.
  pt_code_c varchar(32) NOT NULL,
    -- This describes the passport type.
  pt_description_c varchar(80),
    -- Indicates if this code can still be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_passport_type_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_passport_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_passport_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_passport_type_pk0 
   (pt_code_c),
KEY inx_pt_passport_type_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_passport_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the degree to which a language is spoken, e.g. a little, phrases, fluent.
-- GROUP: personnel
CREATE TABLE pt_language_level (

    -- This field is a numeric representation of level of language.
  pt_language_level_i integer DEFAULT 0 NOT NULL,
    -- Describes the level of fluency in a language.
  pt_language_level_descr_c varchar(70) NOT NULL,
    -- Can this contact still be listed?
  pt_unassignable_flag_l boolean DEFAULT 0,
    -- This is the date from which this contact can no longer be assigned.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- Exhaustive explanation of the Language Level.
  pt_language_comment_c varchar(800),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_language_level_pk
    PRIMARY KEY (pt_language_level_i),
  CONSTRAINT pt_language_level_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_language_level_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_language_level_pk0 
   (pt_language_level_i),
KEY inx_pt_language_level_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_language_level_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Defines the areas in which a person may possess an ability, e.g. plays the guitar.
-- GROUP: personnel
CREATE TABLE pt_ability_area (

    -- Name of the area of ability
  pt_ability_area_name_c varchar(32) NOT NULL,
    -- Describes the area of ability.
  pt_ability_area_descr_c varchar(90) NOT NULL,
    -- Describes the area of ability
  pt_requirement_area_descr_c varchar(90) NOT NULL,
    -- Can this ability be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_ability_area_pk
    PRIMARY KEY (pt_ability_area_name_c),
  CONSTRAINT pt_ability_area_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_ability_area_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_ability_area_pk0 
   (pt_ability_area_name_c),
KEY inx_pt_ability_area_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_ability_area_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the degree to which an ability is possessed, e.g. a little. professional.
-- GROUP: personnel
CREATE TABLE pt_ability_level (

    -- This field is a numeric representation of level of ability.
  pt_ability_level_i integer DEFAULT 0 NOT NULL,
    -- Describes the level of ability.
  pt_ability_level_descr_c varchar(90) NOT NULL,
    -- Can this ability level be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_ability_level_pk
    PRIMARY KEY (pt_ability_level_i),
  CONSTRAINT pt_ability_level_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_ability_level_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_ability_level_pk0 
   (pt_ability_level_i),
KEY inx_pt_ability_level_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_ability_level_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This defines the areas in which a person may posess a qualification, e.g. computing or accountancy. 
-- GROUP: personnel
CREATE TABLE pt_qualification_area (

    -- Name of the area of qualification.
  pt_qualification_area_name_c varchar(32) NOT NULL,
    -- Describes the area of qualification.
  pt_qualification_area_descr_c varchar(80) NOT NULL,
    -- Can this qualification be assigned?
  pt_qualification_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_qualification_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_qualification_area_pk
    PRIMARY KEY (pt_qualification_area_name_c),
  CONSTRAINT pt_qualification_area_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_qualification_area_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_qualification_area_pk0 
   (pt_qualification_area_name_c),
KEY inx__qualification_area_fkcr_key1 
   (s_created_by_c),
KEY inx__qualification_area_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the level to which a qualifications is possessed, e.g. Secondary education, Master's Degree.
-- GROUP: personnel
CREATE TABLE pt_qualification_level (

    -- This field is a numeric representation of level of qualification.
  pt_qualification_level_i integer DEFAULT 0 NOT NULL,
    -- Describes the level of qualification.
  pt_qualification_level_descr_c varchar(80) NOT NULL,
    -- Can this qualification level be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_qualification_level_pk
    PRIMARY KEY (pt_qualification_level_i),
  CONSTRAINT pt_qualification_level_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_qualification_level_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_qualification_level_pk0 
   (pt_qualification_level_i),
KEY inx_qualification_level_fkcr_key1 
   (s_created_by_c),
KEY inx_qualification_level_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the category that will be used for a person's skill
-- GROUP: personnel
CREATE TABLE pt_skill_category (

    -- Code for the Skill Category
  pt_code_c varchar(60) NOT NULL,
    -- Describes the Category used for skills
  pt_description_c varchar(160),
    -- Can this category be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_skill_category_pk
    PRIMARY KEY (pt_code_c),
  CONSTRAINT pt_skill_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_skill_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_skill_category_pk0 
   (pt_code_c),
KEY inx_pt_skill_category_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_skill_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the level to which a skill is possessed
-- GROUP: personnel
CREATE TABLE pt_skill_level (

    -- This field is a numeric representation of level of skill.
  pt_level_i integer DEFAULT 0 NOT NULL,
    -- Describes the level of skill.
  pt_description_c varchar(100) NOT NULL,
    -- Can this skill level be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_skill_level_pk
    PRIMARY KEY (pt_level_i),
  CONSTRAINT pt_skill_level_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_skill_level_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_skill_level_pk0 
   (pt_level_i),
KEY inx_pt_skill_level_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_skill_level_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds the categories that can be used for data label values.
-- GROUP: partner
CREATE TABLE p_data_label_lookup_category (

    -- Code for Lookup Category
  p_category_code_c varchar(60) NOT NULL,
    -- Description for Lookup Category
  p_category_desc_c varchar(120),
    -- Indicates if values for this category can be added on the fly
  p_extendable_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_lookup_category_pk
    PRIMARY KEY (p_category_code_c),
  CONSTRAINT p_data_label_lookup_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_lookup_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_data_label_fk2_ref0 
   (p_category_code_c),
UNIQUE KEY inx_ata_label_lookup_category_pk1 
   (p_category_code_c),
KEY inx_bel_lookup_category_fkcr_key2 
   (s_created_by_c),
KEY inx_bel_lookup_category_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table is used to define data labels for individual use in each office.
-- GROUP: partner
CREATE TABLE p_data_label (

    -- A sequence key for data labels.
  p_key_i integer NOT NULL,
    -- Label Text
  p_text_c varchar(100),
    -- Data Label Group
  p_group_c varchar(100),
    -- Description (Help Text) for the label that the user will see
  p_description_c varchar(400),
    -- Data Type (char | integer | float | currency | boolean | date | time | partnerkey | lookup)
  p_data_type_c varchar(40) NOT NULL,
    -- Maximum length of data string if data type is set to character.
  p_char_length_i integer,
    -- Number of decimal places if data type is set to numeric.
  p_num_decimal_places_i integer,
    -- This defines which currency is being used (if data type is currency)
  p_currency_code_c varchar(16),
    -- If a lookup category is selected then the values can be chosen from a drop down box
  p_lookup_category_code_c varchar(60),
    -- If TRUE then a value for this label has to be set
  p_entry_mandatory_l boolean DEFAULT 0,
    -- If TRUE then this data label will be displayed. Gives a chance to hide but keep labels for historical reasons.
  p_displayed_l boolean DEFAULT 1,
    -- Data label will not be displayed any longer from this date on.
  p_not_displayed_from_d date,
    -- if FALSE then the values will be displayed but are not editable.
  p_editable_l boolean DEFAULT 1,
    -- Data label values will not be editable any longer from this date on.
  p_not_editable_from_d date,
    -- Indicates whether or not the data label has restricted access. If it does then the access will be controlled by s_group_data_label
  p_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_pk
    PRIMARY KEY (p_key_i),
  CONSTRAINT p_data_label_fk1
    FOREIGN KEY (p_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT p_data_label_fk2
    FOREIGN KEY (p_lookup_category_code_c)
    REFERENCES p_data_label_lookup_category(p_category_code_c),
  CONSTRAINT p_data_label_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_data_label_pk0 
   (p_key_i),
KEY inx_p_data_label_fk1_key1 
   (p_currency_code_c),
KEY inx_p_data_label_fk2_key2 
   (p_lookup_category_code_c),
KEY inx_p_data_label_fkcr_key3 
   (s_created_by_c),
KEY inx_p_data_label_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table defines where a data label is used and the order the labels appear in.
-- GROUP: partner
CREATE TABLE p_data_label_use (

    -- A sequence key for data labels.
  p_data_label_key_i integer NOT NULL,
    -- Use of Data Label ( Person | Family | Church | Organisation | Bank | Unit | Venue | Personnel | LongTermApp | ShortTermApp )
  p_use_c varchar(40) NOT NULL,
    -- Label Index (for sorting of labels).
  p_idx1_i integer NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_use_pk
    PRIMARY KEY (p_data_label_key_i,p_use_c),
  CONSTRAINT p_data_label_use_fk1
    FOREIGN KEY (p_data_label_key_i)
    REFERENCES p_data_label(p_key_i),
  CONSTRAINT p_data_label_use_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_use_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_data_label_use_pk0 
   (p_data_label_key_i,p_use_c),
KEY inx_p_data_label_use_fk1_key1 
   (p_data_label_key_i),
KEY inx_p_data_label_use_fkcr_key2 
   (s_created_by_c),
KEY inx_p_data_label_use_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds all lookup values that can be used for data label values.
-- GROUP: partner
CREATE TABLE p_data_label_lookup (

    -- Code for Lookup Category
  p_category_code_c varchar(60) NOT NULL,
    -- Code for Lookup Value
  p_value_code_c varchar(80) NOT NULL,
    -- Description for Lookup Value
  p_value_desc_c varchar(120),
    -- Indicates if this value is active (i.e. can be used)
  p_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_lookup_pk
    PRIMARY KEY (p_category_code_c,p_value_code_c),
  CONSTRAINT p_data_label_lookup_fk1
    FOREIGN KEY (p_category_code_c)
    REFERENCES p_data_label_lookup_category(p_category_code_c),
  CONSTRAINT p_data_label_lookup_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_lookup_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_data_label_lookup_pk0 
   (p_category_code_c,p_value_code_c),
KEY inx_p_data_label_lookup_fk1_key1 
   (p_category_code_c),
KEY inx_p_data_label_lookup_fkcr_key2 
   (s_created_by_c),
KEY inx_p_data_label_lookup_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds the statuses that are be used for commitments.
-- GROUP: personnel
CREATE TABLE pm_commitment_status (

    -- Code for Status
  pm_code_c varchar(40) NOT NULL,
    -- Description for Status
  pm_desc_c varchar(100),
    -- Detailed Explanation for the Status
  pm_explanation_c varchar(1000),
    -- Indicates if the person with this status is generally supposed to have access to the worldwide intranet of the organisation
  pm_intranet_access_l boolean DEFAULT 1,
    -- Display Index (for sorting other than alphabetically).
  pm_display_idx1_i integer,
    -- Indicates if this code can still be assigned?
  pm_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  pm_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pm_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_commitment_status_pk
    PRIMARY KEY (pm_code_c),
  CONSTRAINT pm_commitment_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_commitment_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_pm_staff_data_fk6_ref0 
   (pm_code_c),
UNIQUE KEY inx_pm_commitment_status_pk1 
   (pm_code_c),
KEY inx_m_commitment_status_fkcr_key2 
   (s_created_by_c),
KEY inx_m_commitment_status_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is a listing of the different position which exist within our organisation, e.g. Field Leader, Book Keeper, Computer support.
-- GROUP: units
CREATE TABLE pt_position (

    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of this position.
  pt_position_scope_c varchar(24) NOT NULL,
    -- Describes the position.
  pt_position_descr_c varchar(160),
    -- Can this position be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_position_pk
    PRIMARY KEY (pt_position_name_c,pt_position_scope_c),
  CONSTRAINT pt_position_fk1
    FOREIGN KEY (pt_position_scope_c)
    REFERENCES u_unit_type(u_unit_type_code_c),
  CONSTRAINT pt_position_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_position_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx__program_application_fk2_ref0 
   (pt_position_name_c,pt_position_scope_c),
UNIQUE KEY inx_pt_position_pk1 
   (pt_position_name_c,pt_position_scope_c),
KEY inx_pt_position_fk1_key2 
   (pt_position_scope_c),
KEY inx_pt_position_fkcr_key3 
   (s_created_by_c),
KEY inx_pt_position_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Describes whether a person is full-time, part-time, etc.
-- GROUP: units
CREATE TABLE pt_assignment_type (

    -- Indicates the type of assignment .
  pt_assignment_type_code_c varchar(2) NOT NULL,
    -- This describes the one-letter assignment code.
  pt_assignment_code_descr_c varchar(70) NOT NULL,
    -- Can this qualification level be assigned?
  pt_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pt_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pt_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pt_assignment_type_pk
    PRIMARY KEY (pt_assignment_type_code_c),
  CONSTRAINT pt_assignment_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pt_assignment_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pt_assignment_type_pk0 
   (pt_assignment_type_code_c),
KEY inx_pt_assignment_type_fkcr_key1 
   (s_created_by_c),
KEY inx_pt_assignment_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Cost types to be used for conference (extra) charges
-- GROUP: conference
CREATE TABLE pc_cost_type (

    -- Unique name of the cost type
  pc_cost_type_code_c varchar(32) NOT NULL,
    -- Description of the cost type
  pc_cost_type_description_c varchar(80),
    -- Can this cost type be assigned?
  pc_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pc_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pc_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_cost_type_pk
    PRIMARY KEY (pc_cost_type_code_c),
  CONSTRAINT pc_cost_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_cost_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_cost_type_pk0 
   (pc_cost_type_code_c),
KEY inx_pc_cost_type_fkcr_key1 
   (s_created_by_c),
KEY inx_pc_cost_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists types of options that can be used for a conference
-- GROUP: conference
CREATE TABLE pc_conference_option_type (

    -- Unique name of the cost type
  pc_option_type_code_c varchar(32) NOT NULL,
    -- Description of the option type
  pc_option_type_description_c varchar(80),
  pc_option_type_comment_c varchar(512),
    -- Can this option type be assigned?
  pc_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pc_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pc_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_conference_option_type_pk
    PRIMARY KEY (pc_option_type_code_c),
  CONSTRAINT pc_conference_option_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_conference_option_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_conference_option_type_pk0 
   (pc_option_type_code_c),
KEY inx_ference_option_type_fkcr_key1 
   (s_created_by_c),
KEY inx_ference_option_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists possible criterias that must be met for discounts to be applied
-- GROUP: conference
CREATE TABLE pc_discount_criteria (

    -- Unique name of the discount criteria
  pc_discount_criteria_code_c varchar(16) NOT NULL,
    -- Description of the discount criteria
  pc_discount_criteria_desc_c varchar(80),
    -- Can this discount criteria be assigned?
  pc_unassignable_flag_l boolean DEFAULT 0 NOT NULL,
    -- This is the date the record was last updated.
  pc_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  pc_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_discount_criteria_pk
    PRIMARY KEY (pc_discount_criteria_code_c),
  CONSTRAINT pc_discount_criteria_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_discount_criteria_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_discount_criteria_pk0 
   (pc_discount_criteria_code_c),
KEY inx_c_discount_criteria_fkcr_key1 
   (s_created_by_c),
KEY inx_c_discount_criteria_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains type of attributes that can be assigned to a room
-- GROUP: hospitality
CREATE TABLE pc_room_attribute_type (

  pc_code_c varchar(40) NOT NULL,
  pc_desc_c varchar(100),
  pc_valid_l boolean DEFAULT 1,
  pc_deletable_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_room_attribute_type_pk
    PRIMARY KEY (pc_code_c),
  CONSTRAINT pc_room_attribute_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_room_attribute_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_room_attribute_type_pk0 
   (pc_code_c),
KEY inx_room_attribute_type_fkcr_key1 
   (s_created_by_c),
KEY inx_room_attribute_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Categories for Area of Interest
-- GROUP: partner
CREATE TABLE p_interest_category (

    -- Unique key for the table
  p_category_c varchar(20) NOT NULL,
  p_description_c varchar(100),
    -- Description of all the interest intensity levels.  Leave empty if levels not needed.
  p_level_descriptions_c varchar(512),
  p_level_range_low_i integer,
  p_level_range_high_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_interest_category_pk
    PRIMARY KEY (p_category_c),
  CONSTRAINT p_interest_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_interest_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_interest_category_pk0 
   (p_category_c),
KEY inx_p_interest_category_fkcr_key1 
   (s_created_by_c),
KEY inx_p_interest_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Area of Interest
-- GROUP: partner
CREATE TABLE p_interest (

    -- Unique key for the table
  p_interest_c varchar(20) NOT NULL,
    -- Interest category
  p_category_c varchar(20),
  p_description_c varchar(100),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_interest_pk
    PRIMARY KEY (p_interest_c),
  CONSTRAINT p_interest_fk1
    FOREIGN KEY (p_category_c)
    REFERENCES p_interest_category(p_category_c),
  CONSTRAINT p_interest_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_interest_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_interest_pk0 
   (p_interest_c),
KEY inx_p_interest_fk1_key1 
   (p_category_c),
KEY inx_p_interest_fkcr_key2 
   (s_created_by_c),
KEY inx_p_interest_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains the codes that indicate the categories of reminders
-- GROUP: partner
CREATE TABLE p_reminder_category (

    -- This code indicates the category of a reminder
  p_code_c varchar(40) NOT NULL,
    -- This describes the reminder category.
  p_description_c varchar(100),
    -- Indicates if this code can still be assigned?
  p_unassignable_flag_l boolean DEFAULT 0,
    -- Date from which this code was made unassignable.
  p_unassignable_date_d date,
    -- Indicates if a record can be deleted.
  p_deletable_flag_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_reminder_category_pk
    PRIMARY KEY (p_code_c),
  CONSTRAINT p_reminder_category_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_reminder_category_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_reminder_category_pk0 
   (p_code_c),
KEY inx_p_reminder_category_fkcr_key1 
   (s_created_by_c),
KEY inx_p_reminder_category_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Refers to a process through which a Partner may go (eg. application, different status of donor - small->medium->big, etc)
-- GROUP: partner
CREATE TABLE p_process (

    -- Code for the process
  p_process_code_c varchar(60) NOT NULL,
    -- Description of the process
  p_process_descr_c varchar(400),
    -- Comma separated list of Partner Classes for which this process is valid. Null implies all Classes
  p_process_partner_classes_c varchar(400),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_process_pk
    PRIMARY KEY (p_process_code_c),
  CONSTRAINT p_process_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_process_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_process_pk0 
   (p_process_code_c),
KEY inx_p_process_fkcr_key1 
   (s_created_by_c),
KEY inx_p_process_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A particular state within a process which can apply to a Partner. Each state will probably need to have an Idle state to indicate when no state applies.
-- GROUP: partner
CREATE TABLE p_state (

    -- Process that this state belongs to
  p_process_code_c varchar(60) NOT NULL,
    -- Code for the state
  p_state_code_c varchar(60) NOT NULL,
    -- Description of the state
  p_state_descr_c varchar(400),
    -- Is this a currently active state?
  p_active_l boolean,
    -- Is this a system defined state (as opposed to a user defined one)?
  p_system_state_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_state_pk
    PRIMARY KEY (p_process_code_c,p_state_code_c),
  CONSTRAINT p_state_fk1
    FOREIGN KEY (p_process_code_c)
    REFERENCES p_process(p_process_code_c),
  CONSTRAINT p_state_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_state_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_state_pk0 
   (p_process_code_c,p_state_code_c),
KEY inx_p_state_fk1_key1 
   (p_process_code_c),
KEY inx_p_state_fkcr_key2 
   (s_created_by_c),
KEY inx_p_state_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A particular action within a process which can be performed on a Partner
-- GROUP: partner
CREATE TABLE p_action (

    -- Process that this action is part of
  p_process_code_c varchar(60) NOT NULL,
    -- Code for the action
  p_action_code_c varchar(60) NOT NULL,
    -- Description of the action
  p_action_descr_c varchar(400),
    -- Is this action currently in progress?
  p_active_l boolean,
    -- Is this a system defined action (as opposed to a user defined one)?
  p_system_action_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_action_pk
    PRIMARY KEY (p_process_code_c,p_action_code_c),
  CONSTRAINT p_action_fk1
    FOREIGN KEY (p_process_code_c)
    REFERENCES p_process(p_process_code_c),
  CONSTRAINT p_action_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_action_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_action_pk0 
   (p_process_code_c,p_action_code_c),
KEY inx_p_action_fk1_key1 
   (p_process_code_c),
KEY inx_p_action_fkcr_key2 
   (s_created_by_c),
KEY inx_p_action_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lookup table for First Contact Codes (ie. how did person first hear about us)
-- GROUP: partner
CREATE TABLE p_first_contact (

  p_first_contact_code_c varchar(60) NOT NULL,
  p_first_contact_descr_c varchar(400),
    -- Is this contact code still active (ie. should it be shown on picklists)?
  p_active_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_first_contact_pk
    PRIMARY KEY (p_first_contact_code_c),
  CONSTRAINT p_first_contact_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_first_contact_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_partner_fk8_ref0 
   (p_first_contact_code_c),
UNIQUE KEY inx_p_first_contact_pk1 
   (p_first_contact_code_c),
KEY inx_p_first_contact_fkcr_key2 
   (s_created_by_c),
KEY inx_p_first_contact_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the main table of the partner system.  Partners can be individuals, families, organisations (churches, businesses), fields, cost centers, and others.  These represent various classes of partners.
-- GROUP: partner
CREATE TABLE p_partner (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This defines what type of partner this is. The classes that may be assigned are PERSON, FAMILY, CHURCH, ORGANISATION, UNIT, VENUE and BANK.
  p_partner_class_c varchar(24),
    -- This is a sub-class of the partner class.
  p_addressee_type_code_c varchar(24) DEFAULT 'DEFAULT',
    -- Name of the person or organisation.  If a person, more name info is stored in p_person.
  p_partner_short_name_c varchar(160),
    -- An alternative name for a partner - mainly for entering the local language equivalent.
  p_partner_short_name_loc_c varchar(160),
    -- Partner name how it should appear as printed version (to be used if mechanism to build short name from first and last name does not work e.g. in case of different surnames for husband and wife)
  p_printed_name_c varchar(160),
    -- Identifies the preferred language of the partner.
  p_language_code_c varchar(20) DEFAULT '99',
    -- Important information about this partner that users need to be aware of.
  p_key_information_c varchar(500),
    -- Additional information about the partner that is important to store in the database.
  p_comment_c longtext,
    -- This code identifies the method of aquisition.
  p_acquisition_code_c varchar(16),
    -- This code describes the status of a partner. Eg,  Active, Deceased etc
  p_status_code_c varchar(16),
    -- This is the date the status of the partner was last updated.
  p_status_change_d date,
    -- Why was the status changed?
  p_status_change_reason_c varchar(400),
    -- Yes if this partner has been ""deleted.""
  p_deleted_partner_l boolean DEFAULT 0,
    -- This is the finance details comment.
  p_finance_comment_c varchar(510),
    -- How often the partner receives a receipt letter.
  p_receipt_letter_frequency_c varchar(24),
    -- Flags whether each gift given by a user is receipted
  p_receipt_each_gift_l boolean DEFAULT 1,
    -- Flag whether to include this partner when running the Recipient Gift Email report
  p_email_gift_statement_l boolean DEFAULT 0,
    -- Yes if the donor wants to remain anonymous.  Names of anonymous donors will not appear on recipient gift statements.
  p_anonymous_donor_l boolean DEFAULT 0,
    -- Yes if not ok to solicit the partner for funds.
  p_no_solicitations_l boolean DEFAULT 0,
    -- Inicates if this partner has been assigned as a child unit to another unit. 
  p_child_indicator_l boolean DEFAULT 0,
    -- Restricts use of the partner record to the user in p_user_id_c if 2 or the group in p_group_id_c if 1.
  p_restricted_i integer DEFAULT 0,
    -- The Petra user that the partner record is restricted to if p_restricted_i is 2.
  p_user_id_c varchar(40),
    -- The group of Petra users that the partner record is restricted to if p_restricted_i is 1.
  p_group_id_c varchar(40),
  p_previous_name_c varchar(512),
    -- How did this Partner first hear about us?
  p_first_contact_code_c varchar(60),
    -- How did this Partner first hear about us (freetext)?
  p_first_contact_freeform_c varchar(400),
    -- Intranet ID. Needed for making the link to the International Website, e.g. for using the Online Address Book.
  p_intranet_id_c varchar(200),
    -- Timezone that applies to the partner (address does not necessarily determine that). This refers to data in the International Address Book.
  p_timezone_c varchar(100),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_partner_fk1
    FOREIGN KEY (p_acquisition_code_c)
    REFERENCES p_acquisition(p_acquisition_code_c),
  CONSTRAINT p_partner_fk2
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT p_partner_fk3
    FOREIGN KEY (p_status_code_c)
    REFERENCES p_partner_status(p_status_code_c),
  CONSTRAINT p_partner_fk4
    FOREIGN KEY (p_addressee_type_code_c)
    REFERENCES p_addressee_type(p_addressee_type_code_c),
  CONSTRAINT p_partner_fk7
    FOREIGN KEY (p_partner_class_c)
    REFERENCES p_partner_classes(p_partner_class_c),
  CONSTRAINT p_partner_fk8
    FOREIGN KEY (p_first_contact_code_c)
    REFERENCES p_first_contact(p_first_contact_code_c),
  CONSTRAINT p_partner_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_s_user_fk1_ref0 
   (p_partner_key_n),
UNIQUE KEY inx_p_partner_pk1 
   (p_partner_key_n),
KEY inx_p_partner_fk1_key2 
   (p_acquisition_code_c),
KEY inx_p_partner_fk2_key3 
   (p_language_code_c),
KEY inx_p_partner_fk3_key4 
   (p_status_code_c),
KEY inx_p_partner_fk4_key5 
   (p_addressee_type_code_c),
KEY inx_p_partner_fk7_key6 
   (p_partner_class_c),
KEY inx_p_partner_fk8_key7 
   (p_first_contact_code_c),
KEY inx_p_partner_fkcr_key8 
   (s_created_by_c),
KEY inx_p_partner_fkmd_key9 
   (s_modified_by_c),
KEY p_short_name_k0 
   (p_partner_short_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The partners a user has been working with recently
-- GROUP: partner
CREATE TABLE p_recent_partners (

    -- When was this partner edited by this user?
  p_when_d date,
  p_when_t integer,
    -- Who is this for
  s_user_id_c varchar(20) NOT NULL,
    -- What partner is it about?
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_recent_partners_pk
    PRIMARY KEY (s_user_id_c,p_partner_key_n),
  CONSTRAINT p_recent_partners_fk1
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_recent_partners_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_recent_partners_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_recent_partners_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_recent_partners_pk0 
   (s_user_id_c,p_partner_key_n),
KEY inx_p_recent_partners_fk1_key1 
   (s_user_id_c),
KEY inx_p_recent_partners_fk2_key2 
   (p_partner_key_n),
KEY inx_p_recent_partners_fkcr_key3 
   (s_created_by_c),
KEY inx_p_recent_partners_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links partners with locations (addresses) and has specific info about the link (e.g. date effective)
-- GROUP: partner
CREATE TABLE p_partner_location (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the key that tell what site created the linked location
  p_site_key_n bigint DEFAULT 0 NOT NULL,
  p_location_key_i integer DEFAULT 0 NOT NULL,
  p_date_effective_d date,
  p_date_good_until_d date,
  p_location_type_c varchar(24),
  p_send_mail_l boolean DEFAULT 0,
  p_location_detail_comment_c varchar(512),
    -- Indicates whether or not the partner location has restricted access. If it does then the access will be controlled by s_group_partner_location.
  p_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_location_pk
    PRIMARY KEY (p_partner_key_n,p_site_key_n,p_location_key_i),
  CONSTRAINT p_partner_location_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_location_fk2
    FOREIGN KEY (p_site_key_n,p_location_key_i)
    REFERENCES p_location(p_site_key_n,p_location_key_i),
  CONSTRAINT p_partner_location_fk3
    FOREIGN KEY (p_location_type_c)
    REFERENCES p_location_type(p_code_c),
  CONSTRAINT p_partner_location_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_location_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_location_pk0 
   (p_partner_key_n,p_site_key_n,p_location_key_i),
KEY inx_p_partner_location_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_location_fk2_key2 
   (p_site_key_n,p_location_key_i),
KEY inx_p_partner_location_fk3_key3 
   (p_location_type_c),
KEY inx_p_partner_location_fkcr_key4 
   (s_created_by_c),
KEY inx_p_partner_location_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Mainly introduced for Partner Contact Details like email addresses and (mobile) phone numbers, but can be used for any attribute we might want to save for a Partner.
-- GROUP: partner
CREATE TABLE p_partner_attribute (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- The Partner Attribute Type that the Partner Attribute that is assigned here has got.
  p_attribute_type_c varchar(60) NOT NULL,
    -- A normal sequence so that each partner can have more than one value for any particular attribute type.
  p_sequence_i integer DEFAULT 0 NOT NULL,
    -- Allows for manual ordering of Partner Attributes within a Partner Attribute Type by the user (e.g. re-ordering of phone numbers within the 'Phone Number' Partner Attribute Type).
  p_index_i integer NOT NULL,
    -- The value of the attribute.
  p_value_c varchar(400) NOT NULL,
    -- The country of the value of the attribute (only relevant for Telephone and Fax Numbers).
  p_value_country_c varchar(8),
    -- Any comment to explain this attribute value, or some additional info that may be required.
  p_comment_c varchar(400),
    -- Flag to indicate whether this Partner Attribute is the primary one among several that have the same Partner Attribute Type.
  p_primary_l boolean DEFAULT 0 NOT NULL,
    -- Flag to indicate whether this Partner Attribute is for use within The Organisation.
  p_within_organisation_l boolean DEFAULT 0 NOT NULL,
    -- Flag to indicate whether this Partner Attribute is specialised. For Partner Contact Details this flag is set for work-related Partner Attribute Types. The label is set in p_partner_attribute_type.p_special_label_c, eg. Business
  p_specialised_l boolean DEFAULT 0 NOT NULL,
    -- Flag to indicate whether this Partner Attribute is confidential.
  p_confidential_l boolean DEFAULT 0 NOT NULL,
    -- Flag to indicate whether this Partner Attribute is current.
  p_current_l boolean DEFAULT 1 NOT NULL,
    -- When this Partner Attribute value is no longer current from.
  p_no_longer_current_from_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_attribute_pk
    PRIMARY KEY (p_partner_key_n,p_attribute_type_c,p_sequence_i),
  CONSTRAINT p_partner_attribute_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_attribute_fk2
    FOREIGN KEY (p_attribute_type_c)
    REFERENCES p_partner_attribute_type(p_attribute_type_c),
  CONSTRAINT p_partner_attribute_fk3
    FOREIGN KEY (p_value_country_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_partner_attribute_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_attribute_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_attribute_pk0 
   (p_partner_key_n,p_attribute_type_c,p_sequence_i),
KEY inx_p_partner_attribute_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_attribute_fk2_key2 
   (p_attribute_type_c),
KEY inx_p_partner_attribute_fk3_key3 
   (p_value_country_c),
KEY inx_p_partner_attribute_fkcr_key4 
   (s_created_by_c),
KEY inx_p_partner_attribute_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of a unit.  This is an organizational unit such as an om field, department, local cost center, etc.
-- GROUP: partner
CREATE TABLE p_unit (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
  p_unit_name_c varchar(160),
  p_description_c varchar(1000),
  u_unit_type_code_c varchar(24),
    -- Indicates the minimum number of staff required. (Computed from um_jobs.)
  um_minimum_i integer DEFAULT 0,
    -- Indicates the maximum number of staff required. (Computed from um_jobs.)
  um_maximum_i integer DEFAULT 0,
    -- Indicates the present number on staff. (Computed from um_jobs.)
  um_present_i integer DEFAULT 0,
    -- Number of part-timers acceptable. (Computed from um_jobs.)
  um_part_timers_i integer DEFAULT 0,
    -- todo
  p_outreach_code_c varchar(26),
    -- This is the cost of the outreach/Event
  p_outreach_cost_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is the currency that is used for the outreach cost.
  p_outreach_cost_currency_code_c varchar(16),
    -- This is a code which identifies a country. It is the ISO code
  p_country_code_c varchar(8) DEFAULT '99',
    -- The partner key of the office that will be this field's primary office..
  p_primary_office_n bigint DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_unit_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_unit_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_unit_fk2
    FOREIGN KEY (u_unit_type_code_c)
    REFERENCES u_unit_type(u_unit_type_code_c),
  CONSTRAINT p_unit_fk3
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_unit_fk4
    FOREIGN KEY (p_outreach_cost_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT p_unit_fk7
    FOREIGN KEY (p_primary_office_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_unit_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_unit_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_unit_pk0 
   (p_partner_key_n),
KEY inx_p_unit_fk2_key1 
   (u_unit_type_code_c),
KEY inx_p_unit_fk3_key2 
   (p_country_code_c),
KEY inx_p_unit_fk4_key3 
   (p_outreach_cost_currency_code_c),
KEY inx_p_unit_fk7_key4 
   (p_primary_office_n),
KEY inx_p_unit_fkcr_key5 
   (s_created_by_c),
KEY inx_p_unit_fkmd_key6 
   (s_modified_by_c),
KEY p_outreach_k0 
   (p_outreach_code_c,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This contains parent/child relationships.
-- GROUP: partner
CREATE TABLE um_unit_structure (

    -- This is an alias of the partner key. This is used to identify parent-child relationships. It consists of the fund id followed by a computer generated six digit number.
  um_parent_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- This is an alias of the partner key. This is used to identify parent-child relationships. It consists of the fund id followed by a computer generated six digit number.
  um_child_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_unit_structure_pk
    PRIMARY KEY (um_parent_unit_key_n,um_child_unit_key_n),
  CONSTRAINT um_unit_structure_fk1
    FOREIGN KEY (um_parent_unit_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_structure_fk2
    FOREIGN KEY (um_child_unit_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_structure_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_unit_structure_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_unit_structure_pk0 
   (um_parent_unit_key_n,um_child_unit_key_n),
KEY inx_um_unit_structure_fk1_key1 
   (um_parent_unit_key_n),
KEY inx_um_unit_structure_fk2_key2 
   (um_child_unit_key_n),
KEY inx_um_unit_structure_fkcr_key3 
   (s_created_by_c),
KEY inx_um_unit_structure_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains details about a family in Partnership with us.  May have P_Person records linked to it.
-- GROUP: partner
CREATE TABLE p_family (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Flag is set if there are family members with their own records in the p_person table. IMPORTANT: Don't rely on this flag anymore but determine this state on-the-fly! Reason: The value of this field isn't maintained reliably any more and the field will be removed in a future version of the DB!
  p_family_members_l boolean DEFAULT 0,
    -- How the family is to be addressed
  p_title_c varchar(64),
  p_first_name_c varchar(96),
  p_family_name_c varchar(120),
    -- Flag is set if there are different surnames entered for husband and wife
  p_different_surnames_l boolean DEFAULT 0,
  p_marital_status_c varchar(4) DEFAULT 'U',
  p_marital_status_since_d date,
  p_marital_status_comment_c varchar(512),
    -- This is the date the person was born
  p_date_of_birth_d date DEFAULT NULL,
  p_gender_c varchar(16) DEFAULT 'Unknown',
    -- A photo of the person, encoded with Base64, and prefixed with the file type
  p_photo_b longtext DEFAULT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_family_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_family_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_family_fk2
    FOREIGN KEY (p_marital_status_c)
    REFERENCES pt_marital_status(pt_code_c),
  CONSTRAINT p_family_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_family_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_family_pk0 
   (p_partner_key_n),
KEY inx_p_family_fk2_key1 
   (p_marital_status_c),
KEY inx_p_family_fkcr_key2 
   (s_created_by_c),
KEY inx_p_family_fkmd_key3 
   (s_modified_by_c),
KEY fam_family_name_k0 
   (p_family_name_c),
KEY fam_first_name_k1 
   (p_first_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- DEPRECATED. USE P_FAMILY INSTEAD. Details of a person. A person must also have a related FAMILY class p_partner record.
-- GROUP: partner
CREATE TABLE p_person (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_title_c varchar(64),
  p_first_name_c varchar(64),
  p_prefered_name_c varchar(64),
  p_middle_name_1_c varchar(64),
  p_middle_name_2_c varchar(64),
  p_middle_name_3_c varchar(64),
  p_family_name_c varchar(64),
  p_decorations_c varchar(64),
    -- This is the date the person was born
  p_date_of_birth_d date,
  p_gender_c varchar(16) DEFAULT 'Unknown',
  p_marital_status_c varchar(4) DEFAULT 'U',
  p_occupation_code_c varchar(32),
    -- A cross reference to the family record of this person. It should be set to ? (not 0 because such a record does not exist!) when there is no family record.
  p_family_key_n bigint DEFAULT 0,
    -- This field indicates the family id of the individual. ID's 0 and 1 are used for parents; 2, 3, 4 ... 9 are used for children.
  p_family_id_i integer DEFAULT 0 NOT NULL,
    -- A person's academic title such as BSc(Hons) or Prof. (eg. Herr Prof. Klaus Shmitt)
  p_academic_title_c varchar(48),
  p_marital_status_since_d date,
  p_marital_status_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_person_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_person_uq
    UNIQUE (p_family_key_n,p_family_id_i),
  CONSTRAINT p_person_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_person_fk2
    FOREIGN KEY (p_family_key_n)
    REFERENCES p_family(p_partner_key_n),
  CONSTRAINT p_person_fk3
    FOREIGN KEY (p_marital_status_c)
    REFERENCES pt_marital_status(pt_code_c),
  CONSTRAINT p_person_fk4
    FOREIGN KEY (p_occupation_code_c)
    REFERENCES p_occupation(p_occupation_code_c),
  CONSTRAINT p_person_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_person_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_person_pk0 
   (p_partner_key_n),
KEY inx_p_person_fk2_key1 
   (p_family_key_n),
KEY inx_p_person_fk3_key2 
   (p_marital_status_c),
KEY inx_p_person_fk4_key3 
   (p_occupation_code_c),
KEY inx_p_person_fkcr_key4 
   (s_created_by_c),
KEY inx_p_person_fkmd_key5 
   (s_modified_by_c),
UNIQUE KEY inx_p_person_uq0 
   (p_family_key_n,p_family_id_i),
KEY pers_family_name_k1 
   (p_family_name_c),
KEY pers_first_name_k2 
   (p_first_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Specific information about a church which is a partner
-- GROUP: partner
CREATE TABLE p_church (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_church_name_c varchar(160),
    -- Number of people in regular attendance.
  p_approximate_size_i integer DEFAULT 0,
  p_denomination_code_c varchar(16) DEFAULT 'UNKNOWN',
  p_accomodation_l boolean DEFAULT 0,
  p_prayer_group_l boolean DEFAULT 0,
    -- Paper (Digital?) Map of church is on file at Site
  p_map_on_file_l boolean DEFAULT 0,
  p_accomodation_type_c varchar(16) DEFAULT 'OTHER',
  p_accomodation_size_i integer DEFAULT 0,
    -- Generally the contact person for the unit who will be addressed in any correspondence
  p_contact_partner_key_n bigint DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_church_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_church_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_church_fk2
    FOREIGN KEY (p_denomination_code_c)
    REFERENCES p_denomination(p_denomination_code_c),
  CONSTRAINT p_church_fk3
    FOREIGN KEY (p_contact_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_church_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_church_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_church_pk0 
   (p_partner_key_n),
KEY inx_p_church_fk2_key1 
   (p_denomination_code_c),
KEY inx_p_church_fk3_key2 
   (p_contact_partner_key_n),
KEY inx_p_church_fkcr_key3 
   (s_created_by_c),
KEY inx_p_church_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of an organisation
-- GROUP: partner
CREATE TABLE p_organisation (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_organisation_name_c varchar(160),
  p_business_code_c varchar(16) DEFAULT 'UNKNOWN',
  p_religious_l boolean DEFAULT 0,
    -- Generally the contact person for the unit who will be addressed in any correspondence
  p_contact_partner_key_n bigint DEFAULT 0,
  p_foundation_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_organisation_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_organisation_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_organisation_fk2
    FOREIGN KEY (p_business_code_c)
    REFERENCES p_business(p_business_code_c),
  CONSTRAINT p_organisation_fk3
    FOREIGN KEY (p_contact_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_organisation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_organisation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_organisation_pk0 
   (p_partner_key_n),
KEY inx_p_organisation_fk2_key1 
   (p_business_code_c),
KEY inx_p_organisation_fk3_key2 
   (p_contact_partner_key_n),
KEY inx_p_organisation_fkcr_key3 
   (s_created_by_c),
KEY inx_p_organisation_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of an bank
-- GROUP: partner
CREATE TABLE p_bank (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
  p_branch_name_c varchar(160),
    -- Generally the contact person for the unit who will be addressed in any correspondence
  p_contact_partner_key_n bigint,
    -- The bank code/branch code/sort code (synonymous country-specific terms) for identifying a bank branch with a number/code.
  p_branch_code_c varchar(20),
    -- BIC (Bank Identifier Code)/SWIFT code. The Bank Identifier Code is a unique address which, in telecommunication messages, identifies precisely the financial institutions involved in financial transactions. BICs either have 8 or 11 characters.
  p_bic_c varchar(22),
    -- The format file to be used for electronic payment
  p_ep_format_file_c varchar(96),
    -- Is this a credit card type (eg. VISA)?
  p_credit_card_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_bank_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_bank_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_bank_fk2
    FOREIGN KEY (p_contact_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_bank_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_bank_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_bank_pk0 
   (p_partner_key_n),
KEY inx_p_bank_fk2_key1 
   (p_contact_partner_key_n),
KEY inx_p_bank_fkcr_key2 
   (s_created_by_c),
KEY inx_p_bank_fkmd_key3 
   (s_modified_by_c),
KEY inx_p_bank_branch0 
   (p_branch_code_c),
KEY inx_p_bank_bic1 
   (p_bic_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of a venue
-- GROUP: partner
CREATE TABLE p_venue (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
  p_venue_name_c varchar(160),
  p_venue_code_c varchar(16) NOT NULL,
    -- This is the currency that is used for the venue.
  a_currency_code_c varchar(16),
    -- Generally the contact person for the unit who will be addressed in any correspondence
  p_contact_partner_key_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_venue_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_venue_uk
    UNIQUE (p_venue_code_c),
  CONSTRAINT p_venue_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_venue_fk2
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT p_venue_fk3
    FOREIGN KEY (p_contact_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_venue_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_venue_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_venue_pk0 
   (p_partner_key_n),
KEY inx_p_venue_fk2_key1 
   (a_currency_code_c),
KEY inx_p_venue_fk3_key2 
   (p_contact_partner_key_n),
KEY inx_p_venue_fkcr_key3 
   (s_created_by_c),
KEY inx_p_venue_fkmd_key4 
   (s_modified_by_c),
UNIQUE KEY inx_p_venue_uk0 
   (p_venue_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Any bank details for a partner can be stored in this table
-- GROUP: partner
CREATE TABLE p_banking_details (

    -- A sequence key for the banking details.
  p_banking_details_key_i integer NOT NULL,
    -- The type of banking: BANK ACCOUNT, CREDIT CARD, etc.
  p_banking_type_i integer NOT NULL,
    -- The full name used for authorizing a transaction with this banking entity.
  p_account_name_c varchar(160),
  p_title_c varchar(64),
  p_first_name_c varchar(64),
  p_middle_name_c varchar(64),
  p_last_name_c varchar(64),
    -- Link to p_bank to see what details the bank has.
  p_bank_key_n bigint NOT NULL,
    -- The account number in the bank
  p_bank_account_number_c varchar(40),
    -- The IBAN (International Bank Account Number). IBAN is a standardised international format for entering account details which consists of the country code, the local bank code, the (payee) account number and a control number.
  p_iban_c varchar(128),
    -- Credit Card Security Code
  p_security_code_c varchar(24),
    -- When the credit card is valid from
  p_valid_from_date_d date,
    -- When this expires
  p_expiry_date_d date,
    -- This is the finance details comment.
  p_comment_c varchar(510),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_banking_details_pk
    PRIMARY KEY (p_banking_details_key_i),
  CONSTRAINT p_banking_details_fk3
    FOREIGN KEY (p_banking_type_i)
    REFERENCES p_banking_type(p_id_i),
  CONSTRAINT p_banking_details_fk4
    FOREIGN KEY (p_bank_key_n)
    REFERENCES p_bank(p_partner_key_n),
  CONSTRAINT p_banking_details_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_banking_details_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_banking_details_pk0 
   (p_banking_details_key_i),
KEY inx_p_banking_details_fk3_key1 
   (p_banking_type_i),
KEY inx_p_banking_details_fk4_key2 
   (p_bank_key_n),
KEY inx_p_banking_details_fkcr_key3 
   (s_created_by_c),
KEY inx_p_banking_details_fkmd_key4 
   (s_modified_by_c),
KEY inx_p_banking_details_name0 
   (p_account_name_c),
KEY inx_p_banking_details_account1 
   (p_bank_account_number_c),
KEY inx_p_banking_details_iban2 
   (p_iban_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links p_partner table with p_banking_details table for many to many relationship
-- GROUP: partner
CREATE TABLE p_partner_banking_details (

    -- The partner to link with
  p_partner_key_n bigint NOT NULL,
    -- The banking details to link with
  p_banking_details_key_i integer NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_banking_link_pk
    PRIMARY KEY (p_partner_key_n,p_banking_details_key_i),
  CONSTRAINT p_partner_banking_details_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_banking_details_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_banking_link_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_banking_link_fk2
    FOREIGN KEY (p_banking_details_key_i)
    REFERENCES p_banking_details(p_banking_details_key_i),
UNIQUE KEY inx_p_partner_banking_link_pk0 
   (p_partner_key_n,p_banking_details_key_i),
KEY inx_ner_banking_details_fkcr_key1 
   (s_created_by_c),
KEY inx_ner_banking_details_fkmd_key2 
   (s_modified_by_c),
KEY inx_partner_banking_link_fk1_key3 
   (p_partner_key_n),
KEY inx_partner_banking_link_fk2_key4 
   (p_banking_details_key_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links p_partner - p_banking_details combination to a usage type
-- GROUP: partner
CREATE TABLE p_banking_details_usage (

    -- The partner to link with
  p_partner_key_n bigint NOT NULL,
    -- The banking details to link with
  p_banking_details_key_i integer NOT NULL,
    -- Reference to the usage type.
  p_type_c varchar(32) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_banking_details_usage_pk
    PRIMARY KEY (p_partner_key_n,p_banking_details_key_i,p_type_c),
  CONSTRAINT p_banking_details_usage_fk1
    FOREIGN KEY (p_partner_key_n,p_banking_details_key_i)
    REFERENCES p_partner_banking_details(p_partner_key_n,p_banking_details_key_i),
  CONSTRAINT p_banking_details_usage_fk2
    FOREIGN KEY (p_type_c)
    REFERENCES p_banking_details_usage_type(p_type_c),
  CONSTRAINT p_banking_details_usage_fk3
    FOREIGN KEY (p_banking_details_key_i)
    REFERENCES p_banking_details(p_banking_details_key_i),
  CONSTRAINT p_banking_details_usage_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_banking_details_usage_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_banking_details_usage_pk0 
   (p_partner_key_n,p_banking_details_key_i,p_type_c),
KEY inx_anking_details_usage_fk1_key1 
   (p_partner_key_n,p_banking_details_key_i),
KEY inx_anking_details_usage_fk2_key2 
   (p_type_c),
KEY inx_anking_details_usage_fk3_key3 
   (p_banking_details_key_i),
KEY inx_nking_details_usage_fkcr_key4 
   (s_created_by_c),
KEY inx_nking_details_usage_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Specifies the percentage of incoming gifts to this recipient that can be considered tax deductible
-- GROUP: partner
CREATE TABLE p_partner_tax_deductible_pct (

    -- Partner Key of recipient to which this percentage applies
  p_partner_key_n bigint NOT NULL,
    -- The date from which this percentage is valid
  p_date_valid_from_d date NOT NULL,
    -- The percentage of incoming gifts to this recipient that should be considered tax deductible
  p_percentage_tax_deductible_n numeric(5, 2) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_tax_deductible_pct_pk
    PRIMARY KEY (p_partner_key_n,p_date_valid_from_d),
  CONSTRAINT p_partner_tax_deductible_pct_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_tax_deductible_pct_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_tax_deductible_pct_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_artner_tax_deductible_pct_pk0 
   (p_partner_key_n,p_date_valid_from_d),
KEY inx_r_tax_deductible_pct_fk1_key1 
   (p_partner_key_n),
KEY inx__tax_deductible_pct_fkcr_key2 
   (s_created_by_c),
KEY inx__tax_deductible_pct_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Types assigned to each partner.  Also known as special types.
-- GROUP: partner
CREATE TABLE p_partner_type (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This code identifies the type
  p_type_code_c varchar(20) NOT NULL,
    -- The date the special type is valid from. Can be NULL if there is no relevant start date.
  p_valid_from_d date,
    -- The date the special type is valid to. Can be NULL if there is no end date.
  p_valid_until_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_type_pk
    PRIMARY KEY (p_partner_key_n,p_type_code_c),
  CONSTRAINT p_partner_type_fk3
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_type_fk4
    FOREIGN KEY (p_type_code_c)
    REFERENCES p_type(p_type_code_c),
  CONSTRAINT p_partner_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_type_pk0 
   (p_partner_key_n,p_type_code_c),
KEY inx_p_partner_type_fk3_key1 
   (p_partner_key_n),
KEY inx_p_partner_type_fk4_key2 
   (p_type_code_c),
KEY inx_p_partner_type_fkcr_key3 
   (s_created_by_c),
KEY inx_p_partner_type_fkmd_key4 
   (s_modified_by_c),
UNIQUE KEY p_partner_type_k0 
   (p_type_code_c,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Relationships between pairs of partners.  Among other relationships, this also relates the FAMILY class partners to the PERSON class partners to indicate members of a family.
-- GROUP: partner
CREATE TABLE p_partner_relationship (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This code identifies the relation
  p_relation_name_c varchar(20) NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_relation_key_n bigint DEFAULT 0 NOT NULL,
  p_comment_c varchar(2000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_relationship_pk
    PRIMARY KEY (p_partner_key_n,p_relation_name_c,p_relation_key_n),
  CONSTRAINT p_partner_relationship_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_relationship_fk2
    FOREIGN KEY (p_relation_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_relationship_fk3
    FOREIGN KEY (p_relation_name_c)
    REFERENCES p_relation(p_relation_name_c),
  CONSTRAINT p_partner_relationship_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_relationship_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_relationship_pk0 
   (p_partner_key_n,p_relation_name_c,p_relation_key_n),
KEY inx_partner_relationship_fk1_key1 
   (p_partner_key_n),
KEY inx_partner_relationship_fk2_key2 
   (p_relation_key_n),
KEY inx_partner_relationship_fk3_key3 
   (p_relation_name_c),
KEY inx_artner_relationship_fkcr_key4 
   (s_created_by_c),
KEY inx_artner_relationship_fkmd_key5 
   (s_modified_by_c),
KEY p_partner_relationship_k0 
   (p_relation_name_c,p_partner_key_n,p_relation_key_n),
KEY p_partner_key_k1 
   (p_partner_key_n,p_relation_key_n),
KEY p_relation_key_k2 
   (p_relation_key_n,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Used to keep track of partner keys
-- GROUP: partner
CREATE TABLE p_partner_ledger (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_last_partner_id_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_ledger_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_partner_ledger_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_partner_ledger_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_ledger_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_ledger_pk0 
   (p_partner_key_n),
KEY inx_p_partner_ledger_fkcr_key1 
   (s_created_by_c),
KEY inx_p_partner_ledger_fkmd_key2 
   (s_modified_by_c),
UNIQUE KEY p_partner_ledger_k0 
   (p_partner_key_n,p_last_partner_id_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains the list of partners in each mailing extract
-- GROUP: mailroom
CREATE TABLE m_extract (

    -- Identifier for the extract
  m_extract_id_i integer DEFAULT 0 NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the key that tell what site created the linked location
  p_site_key_n bigint DEFAULT 0 NOT NULL,
  p_location_key_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT m_extract_pk
    PRIMARY KEY (m_extract_id_i,p_partner_key_n,p_site_key_n),
  CONSTRAINT m_extract_fk1
    FOREIGN KEY (m_extract_id_i)
    REFERENCES m_extract_master(m_extract_id_i),
  CONSTRAINT m_extract_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT m_extract_fk3
    FOREIGN KEY (p_site_key_n,p_location_key_i)
    REFERENCES p_location(p_site_key_n,p_location_key_i),
  CONSTRAINT m_extract_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT m_extract_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_m_extract_pk0 
   (m_extract_id_i,p_partner_key_n,p_site_key_n),
KEY inx_m_extract_fk1_key1 
   (m_extract_id_i),
KEY inx_m_extract_fk2_key2 
   (p_partner_key_n),
KEY inx_m_extract_fk3_key3 
   (p_site_key_n,p_location_key_i),
KEY inx_m_extract_fkcr_key4 
   (s_created_by_c),
KEY inx_m_extract_fkmd_key5 
   (s_modified_by_c),
UNIQUE KEY m_extract_k0 
   (m_extract_id_i,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Specific greetings from a user to a partner
-- GROUP: mailroom
CREATE TABLE p_customised_greeting (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the system user id. Each user of the system is allocated one
  s_user_id_c varchar(20) NOT NULL,
  p_customised_greeting_text_c varchar(64),
  p_customised_closing_text_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_customised_greeting_pk
    PRIMARY KEY (p_partner_key_n,s_user_id_c),
  CONSTRAINT p_customised_greeting_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_customised_greeting_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_customised_greeting_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_customised_greeting_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_customised_greeting_pk0 
   (p_partner_key_n,s_user_id_c),
KEY inx__customised_greeting_fk1_key1 
   (p_partner_key_n),
KEY inx__customised_greeting_fk2_key2 
   (s_user_id_c),
KEY inx_customised_greeting_fkcr_key3 
   (s_created_by_c),
KEY inx_customised_greeting_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of which partners receive which publications.
-- GROUP: mailroom
CREATE TABLE p_subscription (

    -- The is the key to the publication table
  p_publication_code_c varchar(20) NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_publication_copies_i integer DEFAULT 1,
  p_reason_subs_given_code_c varchar(24) NOT NULL,
  p_reason_subs_cancelled_code_c varchar(24),
    -- Date the subscription expires
  p_expiry_date_d date,
    -- Provisional date on which the subscription may expire
  p_provisional_expiry_date_d date,
  p_gratis_subscription_l boolean DEFAULT 1 NOT NULL,
  p_date_notice_sent_d date,
  p_date_cancelled_d date,
  p_start_date_d date NOT NULL,
  p_number_issues_received_i integer DEFAULT 0 NOT NULL,
    -- The number of issues sent after a subscription has ceased
  p_number_complimentary_i integer DEFAULT 1 NOT NULL,
  p_subscription_renewal_date_d date,
  p_subscription_status_c varchar(24) DEFAULT 'PERMANENT',
  p_first_issue_d date,
  p_last_issue_d date,
  p_gift_from_key_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_subscription_pk
    PRIMARY KEY (p_publication_code_c,p_partner_key_n),
  CONSTRAINT p_subscription_fk1
    FOREIGN KEY (p_publication_code_c)
    REFERENCES p_publication(p_publication_code_c),
  CONSTRAINT p_subscription_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_subscription_fk3
    FOREIGN KEY (p_gift_from_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_subscription_fk4
    FOREIGN KEY (p_reason_subs_given_code_c)
    REFERENCES p_reason_subscription_given(p_code_c),
  CONSTRAINT p_subscription_fk5
    FOREIGN KEY (p_reason_subs_cancelled_code_c)
    REFERENCES p_reason_subscription_cancelled(p_code_c),
  CONSTRAINT p_subscription_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_subscription_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_subscription_pk0 
   (p_publication_code_c,p_partner_key_n),
KEY inx_p_subscription_fk1_key1 
   (p_publication_code_c),
KEY inx_p_subscription_fk2_key2 
   (p_partner_key_n),
KEY inx_p_subscription_fk3_key3 
   (p_gift_from_key_n),
KEY inx_p_subscription_fk4_key4 
   (p_reason_subs_given_code_c),
KEY inx_p_subscription_fk5_key5 
   (p_reason_subs_cancelled_code_c),
KEY inx_p_subscription_fkcr_key6 
   (s_created_by_c),
KEY inx_p_subscription_fkmd_key7 
   (s_modified_by_c),
KEY p_expiry_k0 
   (p_expiry_date_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- membership of a partner
-- GROUP: partner
CREATE TABLE p_partner_membership (

    -- A sequence key for the memberships.
  p_partner_membership_key_i integer NOT NULL,
    -- The is the key to the membership table
  p_membership_code_c varchar(20) NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Date the membership expires
  p_expiry_date_d date,
  p_date_cancelled_d date,
  p_start_date_d date NOT NULL,
  p_membership_status_c varchar(24) DEFAULT 'ACTIVE',
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_membership_pk
    PRIMARY KEY (p_partner_membership_key_i),
  CONSTRAINT p_partner_membership_uk
    UNIQUE (p_membership_code_c,p_partner_key_n,p_start_date_d),
  CONSTRAINT p_partner_membership_fk1
    FOREIGN KEY (p_membership_code_c)
    REFERENCES p_membership(p_membership_code_c),
  CONSTRAINT p_partner_membership_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_membership_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_membership_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_membership_pk0 
   (p_partner_membership_key_i),
UNIQUE KEY inx_p_partner_membership_uk1 
   (p_membership_code_c,p_partner_key_n,p_start_date_d),
KEY inx_p_partner_membership_fk1_key2 
   (p_membership_code_c),
KEY inx_p_partner_membership_fk2_key3 
   (p_partner_key_n),
KEY inx__partner_membership_fkcr_key4 
   (s_created_by_c),
KEY inx__partner_membership_fkmd_key5 
   (s_modified_by_c),
KEY p_membership_expiry_k0 
   (p_expiry_date_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- There is a record for each payment of a membership, or hours served
-- GROUP: partner
CREATE TABLE p_partner_membership_paid (

    -- The is the key to the membership table
  p_membership_code_c varchar(20) NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Date of the period that this payment was for
  p_period_date_d date,
    -- Date when this payment was made or the hours were served
  p_date_payment_d date NOT NULL,
    -- This is the amount that was paid.
  p_amount_paid_n numeric(24, 10) DEFAULT 0,
    -- This is the number of hours that were served.
  p_hours_served_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_membership_paid_pk
    PRIMARY KEY (p_membership_code_c,p_partner_key_n,p_date_payment_d),
  CONSTRAINT p_partner_membership_paid_fk1
    FOREIGN KEY (p_membership_code_c)
    REFERENCES p_membership(p_membership_code_c),
  CONSTRAINT p_partner_membership_paid_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_membership_paid_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_membership_paid_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_membership_paid_pk0 
   (p_membership_code_c,p_partner_key_n,p_date_payment_d),
KEY inx_tner_membership_paid_fk1_key1 
   (p_membership_code_c),
KEY inx_tner_membership_paid_fk2_key2 
   (p_partner_key_n),
KEY inx_ner_membership_paid_fkcr_key3 
   (s_created_by_c),
KEY inx_ner_membership_paid_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link between Partners and Contacts
-- GROUP: mailroom
CREATE TABLE p_partner_contact (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
    -- This is the contact log id.  
  p_contact_log_id_i bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_contact_pk
    PRIMARY KEY (p_partner_key_n,p_contact_log_id_i),
  CONSTRAINT p_partner_contact_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_contact_fk2
    FOREIGN KEY (p_contact_log_id_i)
    REFERENCES p_contact_log(p_contact_log_id_i),
  CONSTRAINT p_partner_contact_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_contact_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_partner_contact_fk1_key0 
   (p_partner_key_n),
KEY inx_p_partner_contact_fk2_key1 
   (p_contact_log_id_i),
KEY inx_p_partner_contact_fkcr_key2 
   (s_created_by_c),
KEY inx_p_partner_contact_fkmd_key3 
   (s_modified_by_c),
UNIQUE KEY inx_p_partner_contact_uk0 
   (p_partner_key_n,p_contact_log_id_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details about each account code within a ledger. Also holds information on the summary account structure for reporting.
-- GROUP: account
CREATE TABLE a_account (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- Income, Expense, Asset, Liability, Equity.  Suspense accounts are in a_suspense_account.
  a_account_type_c varchar(20),
    -- This is a long description and is 80 characters long.
  a_account_code_long_desc_c varchar(160),
    -- This is a short description which is 32 charcters long
  a_account_code_short_desc_c varchar(64),
    -- This is a short description which is 32 charcters long
  a_eng_account_code_short_desc_c varchar(64),
    -- This is a long description and is 80 characters long.
  a_eng_account_code_long_desc_c varchar(160),
    -- Defines if the the transcation is a debit or credit transaction
  a_debit_credit_indicator_l boolean DEFAULT 1,
    -- Defines whether the acount is active or not
  a_account_active_flag_l boolean DEFAULT 1,
    -- Yes if the account has any analysis attributes.
  a_analysis_attribute_flag_l boolean DEFAULT 0,
  a_standard_account_flag_l boolean DEFAULT 0,
  a_consolidation_account_flag_l boolean DEFAULT 0,
  a_intercompany_account_flag_l boolean DEFAULT 0,
    -- The type of budget.  See the a_budget_type table.
  a_budget_type_code_c varchar(16),
  a_posting_status_l boolean DEFAULT 1,
  a_system_account_flag_l boolean DEFAULT 0,
  a_budget_control_flag_l boolean DEFAULT 0,
    -- Which type of cost centres may be combined with this account.
  a_valid_cc_combo_c varchar(16) DEFAULT 'All',
  a_foreign_currency_flag_l boolean DEFAULT 0,
    -- This defines which currency is being used
  a_foreign_currency_code_c varchar(16),
    -- Link to banking details to use for this account - only really used if this account is a bank account
  p_banking_details_key_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_account_pk
    PRIMARY KEY (a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_account_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_account_fk2
    FOREIGN KEY (a_budget_type_code_c)
    REFERENCES a_budget_type(a_budget_type_code_c),
  CONSTRAINT a_account_fk3
    FOREIGN KEY (a_foreign_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_account_fk4
    FOREIGN KEY (p_banking_details_key_i)
    REFERENCES p_banking_details(p_banking_details_key_i),
  CONSTRAINT a_account_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_account_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_account_fk3_ref0 
   (a_ledger_number_i,a_account_code_c),
UNIQUE KEY inx_a_account_pk1 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_account_fk1_key2 
   (a_ledger_number_i),
KEY inx_a_account_fk2_key3 
   (a_budget_type_code_c),
KEY inx_a_account_fk3_key4 
   (a_foreign_currency_code_c),
KEY inx_a_account_fk4_key5 
   (p_banking_details_key_i),
KEY inx_a_account_fkcr_key6 
   (s_created_by_c),
KEY inx_a_account_fkmd_key7 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of recently imported bank statements
-- GROUP: account
CREATE TABLE a_ep_statement (

    -- auto generated unique identifier for bank statements
  a_statement_key_i integer NOT NULL,
    -- this is the bank account that this statement is for; this is necessary because you can have several accounts at the same bank
  a_bank_account_key_i integer,
    -- The four digit ledger number of the gift.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The bank account for creating GL transactions against
  a_bank_account_code_c varchar(16),
    -- The date of this statement
  a_date_d date NOT NULL,
    -- This is the ID that the bank uses for this statement; it can be used to find the paper version of the bank statement
  a_id_from_bank_c varchar(40),
    -- This is the name of the file that this statement was read from; this can be used to prevent duplicate import of bank statements
  a_filename_c varchar(40),
    -- The start balance of the bank account at the beginning of the statement
  a_start_balance_n numeric(24, 10),
    -- The end balance of the bank account after the statement
  a_end_balance_n numeric(24, 10),
    -- This defines the currency of the transactions on this statement
  a_currency_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_statement_pk
    PRIMARY KEY (a_statement_key_i),
  CONSTRAINT a_ep_statement_fk1
    FOREIGN KEY (a_bank_account_key_i)
    REFERENCES p_banking_details(p_banking_details_key_i),
  CONSTRAINT a_ep_statement_fk2
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ep_statement_fk3
    FOREIGN KEY (a_ledger_number_i,a_bank_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ep_statement_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_statement_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_statement_pk0 
   (a_statement_key_i),
KEY inx_a_ep_statement_fk1_key1 
   (a_bank_account_key_i),
KEY inx_a_ep_statement_fk2_key2 
   (a_currency_code_c),
KEY inx_a_ep_statement_fk3_key3 
   (a_ledger_number_i,a_bank_account_code_c),
KEY inx_a_ep_statement_fkcr_key4 
   (s_created_by_c),
KEY inx_a_ep_statement_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Properties and optional values for an account can be saved in this table.
-- GROUP: account
CREATE TABLE a_account_property (

  a_ledger_number_i integer NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- Code for the property
  a_property_code_c varchar(40) NOT NULL,
    -- value of this property
  a_property_value_c varchar(200) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_account_property_pk
    PRIMARY KEY (a_ledger_number_i,a_account_code_c,a_property_code_c,a_property_value_c),
  CONSTRAINT a_account_property_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_account_property_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_account_property_fk3
    FOREIGN KEY (a_property_code_c)
    REFERENCES a_account_property_code(a_property_code_c),
  CONSTRAINT a_account_property_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_account_property_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_account_property_pk0 
   (a_ledger_number_i,a_account_code_c,a_property_code_c,a_property_value_c),
KEY inx_a_account_property_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_account_property_fk2_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_account_property_fk3_key3 
   (a_property_code_c),
KEY inx_a_account_property_fkcr_key4 
   (s_created_by_c),
KEY inx_a_account_property_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- hierarchy of accounts; what is the root account of the hierarchy
-- GROUP: account
CREATE TABLE a_account_hierarchy (

  a_ledger_number_i integer NOT NULL,
    -- The code for the hierarchy
  a_account_hierarchy_code_c varchar(16) NOT NULL,
    -- The root account of the hierarchy, which is at the top of the hierarchy
  a_root_account_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_account_hierarchy_pk
    PRIMARY KEY (a_ledger_number_i,a_account_hierarchy_code_c),
  CONSTRAINT a_account_hierarchy_fk1
    FOREIGN KEY (a_ledger_number_i,a_root_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_account_hierarchy_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_account_hierarchy_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_account_hierarchy_pk0 
   (a_ledger_number_i,a_account_hierarchy_code_c),
KEY inx_a_account_hierarchy_fk1_key1 
   (a_ledger_number_i,a_root_account_code_c),
KEY inx_a_account_hierarchy_fkcr_key2 
   (s_created_by_c),
KEY inx_a_account_hierarchy_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- the elements of the hierarchy of accounts; which account is reporting to which other account
-- GROUP: account
CREATE TABLE a_account_hierarchy_detail (

  a_ledger_number_i integer NOT NULL,
    -- The code for the hierarchy
  a_account_hierarchy_code_c varchar(16) NOT NULL,
    -- The reporting account
  a_reporting_account_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_to_report_to_c varchar(16) NOT NULL,
    -- Order to display the account or heading on the Balance Sheet & Income Statement report.
  a_report_order_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_account_hierarchy_detail_pk
    PRIMARY KEY (a_ledger_number_i,a_account_hierarchy_code_c,a_reporting_account_code_c),
  CONSTRAINT a_account_hierarchy_detail_fk1
    FOREIGN KEY (a_ledger_number_i,a_account_hierarchy_code_c)
    REFERENCES a_account_hierarchy(a_ledger_number_i,a_account_hierarchy_code_c),
  CONSTRAINT a_account_hierarchy_detail_fk2
    FOREIGN KEY (a_ledger_number_i,a_reporting_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_account_hierarchy_detail_fk3
    FOREIGN KEY (a_ledger_number_i,a_account_code_to_report_to_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_account_hierarchy_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_account_hierarchy_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__account_hierarchy_detail_pk0 
   (a_ledger_number_i,a_account_hierarchy_code_c,a_reporting_account_code_c),
KEY inx_unt_hierarchy_detail_fk1_key1 
   (a_ledger_number_i,a_account_hierarchy_code_c),
KEY inx_unt_hierarchy_detail_fk2_key2 
   (a_ledger_number_i,a_reporting_account_code_c),
KEY inx_unt_hierarchy_detail_fk3_key3 
   (a_ledger_number_i,a_account_code_to_report_to_c),
KEY inx_nt_hierarchy_detail_fkcr_key4 
   (s_created_by_c),
KEY inx_nt_hierarchy_detail_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- List of foreign ledgers (eg, other fields) which the local ledger may send transctions to.
-- GROUP: account
CREATE TABLE a_valid_ledger_number (

    -- This is used as a key field in most of the accounting system files .It is created from the first 4 digits of a partner key of type ""ledger"".
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- The ledger through which inter ledger transactions are routed for processing.
  a_ilt_processing_centre_n bigint DEFAULT 0,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner but reflected as a character rather than a numeric
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_valid_ledger_number_pk
    PRIMARY KEY (a_ledger_number_i,p_partner_key_n),
  CONSTRAINT a_valid_ledger_number_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_valid_ledger_number_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_valid_ledger_number_fk3
    FOREIGN KEY (a_ilt_processing_centre_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_valid_ledger_number_fk4
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_valid_ledger_number_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_valid_ledger_number_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_valid_ledger_number_pk0 
   (a_ledger_number_i,p_partner_key_n),
KEY inx__valid_ledger_number_fk1_key1 
   (a_ledger_number_i),
KEY inx__valid_ledger_number_fk2_key2 
   (p_partner_key_n),
KEY inx__valid_ledger_number_fk3_key3 
   (a_ilt_processing_centre_n),
KEY inx__valid_ledger_number_fk4_key4 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_valid_ledger_number_fkcr_key5 
   (s_created_by_c),
KEY inx_valid_ledger_number_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Budget information for cost centre-account combinations.
-- GROUP: account
CREATE TABLE a_budget (

  a_budget_sequence_i integer NOT NULL,
    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The year that this budget applies to
  a_year_i integer DEFAULT 0 NOT NULL,
    -- A budget can have several revisions per year
  a_revision_i integer DEFAULT 0 NOT NULL,
    -- This identifies which cost centre an account is applied to
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- See a_budget_type table.  Indicates the method used for creating the budget.
  a_budget_type_code_c varchar(16) NOT NULL,
    -- Has the budget been ""posted"" to the general ledger master and account files.
  a_budget_status_l boolean DEFAULT 1,
    -- A comment for this specific budget item and revision
  a_comment_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_budget_pk
    PRIMARY KEY (a_budget_sequence_i),
  CONSTRAINT a_budget_uc1
    UNIQUE (a_ledger_number_i,a_year_i,a_revision_i,a_cost_centre_code_c,a_account_code_c),
  CONSTRAINT a_budget_fk1
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_budget_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_budget_fk3
    FOREIGN KEY (a_budget_type_code_c)
    REFERENCES a_budget_type(a_budget_type_code_c),
  CONSTRAINT a_budget_fk4
    FOREIGN KEY (a_ledger_number_i,a_year_i,a_revision_i)
    REFERENCES a_budget_revision(a_ledger_number_i,a_year_i,a_revision_i),
  CONSTRAINT a_budget_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_budget_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_budget_pk0 
   (a_budget_sequence_i),
UNIQUE KEY inx_a_budget_uc11 
   (a_ledger_number_i,a_year_i,a_revision_i,a_cost_centre_code_c,a_account_code_c),
KEY inx_a_budget_fk1_key2 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_budget_fk2_key3 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_budget_fk3_key4 
   (a_budget_type_code_c),
KEY inx_a_budget_fk4_key5 
   (a_ledger_number_i,a_year_i,a_revision_i),
KEY inx_a_budget_fkcr_key6 
   (s_created_by_c),
KEY inx_a_budget_fkmd_key7 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This is the budget data for one period (e.g. a month)
-- GROUP: account
CREATE TABLE a_budget_period (

  a_budget_sequence_i integer NOT NULL,
  a_period_number_i integer DEFAULT 0 NOT NULL,
    -- Budget amount in base currency
  a_budget_base_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_budget_period_pk
    PRIMARY KEY (a_budget_sequence_i,a_period_number_i),
  CONSTRAINT a_budget_period_fk1
    FOREIGN KEY (a_budget_sequence_i)
    REFERENCES a_budget(a_budget_sequence_i),
  CONSTRAINT a_budget_period_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_budget_period_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_budget_period_pk0 
   (a_budget_sequence_i,a_period_number_i),
KEY inx_a_budget_period_fk1_key1 
   (a_budget_sequence_i),
KEY inx_a_budget_period_fkcr_key2 
   (s_created_by_c),
KEY inx_a_budget_period_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Indicates which accounts use analysis attributes and what attribute types may be used with the account.   Analysis attributes are user-definable extra information to be stored against an account.
-- GROUP: account
CREATE TABLE a_analysis_attribute (

    -- The number of the ledger in which the analysis attribute is used.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- A code indicating the type of analysis.  Valid analysis types are stored in the a_analysis_type table.
  a_analysis_type_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction analysis information must be stored against.
  a_account_code_c varchar(16) NOT NULL,
    -- Cost Centre to which the attribute relates
  a_cost_centre_code_c varchar(24),
    -- Analysis attributes cannot be deleted, because they are needed for existing transaction analysis attributes. But they can be deactivated.
  a_active_l boolean DEFAULT 1,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_analysis_attribute_pk
    PRIMARY KEY (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
  CONSTRAINT a_analysis_attribute_fk1
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_analysis_attribute_fk2
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c)
    REFERENCES a_analysis_type(a_ledger_number_i,a_analysis_type_code_c),
  CONSTRAINT a_analysis_attribute_fk3
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_analysis_attribute_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_analysis_attribute_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_analysis_attribute_pk0 
   (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
KEY inx_a_analysis_attribute_fk1_key1 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_analysis_attribute_fk2_key2 
   (a_ledger_number_i,a_analysis_type_code_c),
KEY inx_a_analysis_attribute_fk3_key3 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx__analysis_attribute_fkcr_key4 
   (s_created_by_c),
KEY inx__analysis_attribute_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Where Petra supports it a cross reference between a file and destination can be established for automatic distribution.
-- GROUP: account
CREATE TABLE a_email_destination (

    -- A code to describe the file which is to be distributed via email.
  a_file_code_c varchar(40) NOT NULL,
    -- Foriegn Cost Centre Code.
  a_conditional_value_c varchar(40) NOT NULL,
    -- HOSA partner.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_email_address_c varchar(500),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_email_destination_pk
    PRIMARY KEY (a_file_code_c,a_conditional_value_c,p_partner_key_n),
  CONSTRAINT a_email_destination_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_email_destination_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_email_destination_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_email_destination_pk0 
   (a_file_code_c,a_conditional_value_c,p_partner_key_n),
KEY inx_a_email_destination_fk1_key1 
   (p_partner_key_n),
KEY inx_a_email_destination_fkcr_key2 
   (s_created_by_c),
KEY inx_a_email_destination_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Fees owed to another ledger. (e.g. admin grant)
-- GROUP: account
CREATE TABLE a_fees_payable (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a specific fee.
  a_fee_code_c varchar(16) NOT NULL,
  a_charge_option_c varchar(40) NOT NULL,
  a_charge_percentage_n numeric(5, 2) DEFAULT 0,
    -- This is a the max or min amount charged dependent on the charge option.  The value is the number of currency units.
  a_charge_amount_n numeric(24, 10) DEFAULT 0,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner but reflected as a character rather than a numeric
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- explain what this fee is for.
  a_fee_description_c varchar(48),
    -- This identifies the account for the DR side of the financial transaction.
  a_dr_account_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_fees_payable_pk
    PRIMARY KEY (a_ledger_number_i,a_fee_code_c),
  CONSTRAINT a_fees_payable_fk1
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_fees_payable_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_fees_payable_fk3
    FOREIGN KEY (a_ledger_number_i,a_dr_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_fees_payable_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_fees_payable_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_fees_payable_pk0 
   (a_ledger_number_i,a_fee_code_c),
KEY inx_a_fees_payable_fk1_key1 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_fees_payable_fk2_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_fees_payable_fk3_key3 
   (a_ledger_number_i,a_dr_account_code_c),
KEY inx_a_fees_payable_fkcr_key4 
   (s_created_by_c),
KEY inx_a_fees_payable_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Charges to collect from other ledgers. (e.g. office admin fee)
-- GROUP: account
CREATE TABLE a_fees_receivable (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a specific fee.
  a_fee_code_c varchar(16) NOT NULL,
  a_charge_option_c varchar(40) NOT NULL,
  a_charge_percentage_n numeric(5, 2) DEFAULT 0,
    -- This is a the max or min amount charged dependent on the charge option in number of currency units
  a_charge_amount_n numeric(24, 10) DEFAULT 0,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner but reflected as a character rather than a numeric
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- Description of what this admin charge is for.
  a_fee_description_c varchar(48),
    -- This identifies the account for the DR side of the financial transaction.
  a_dr_account_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_fees_receivable_pk
    PRIMARY KEY (a_ledger_number_i,a_fee_code_c),
  CONSTRAINT a_fees_receivable_fk1
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_fees_receivable_fk2
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_fees_receivable_fk3
    FOREIGN KEY (a_ledger_number_i,a_dr_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_fees_receivable_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_fees_receivable_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_fees_receivable_pk0 
   (a_ledger_number_i,a_fee_code_c),
KEY inx_a_fees_receivable_fk1_key1 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_fees_receivable_fk2_key2 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_fees_receivable_fk3_key3 
   (a_ledger_number_i,a_dr_account_code_c),
KEY inx_a_fees_receivable_fkcr_key4 
   (s_created_by_c),
KEY inx_a_fees_receivable_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- All balances on accounts, including summary information is stored here.
-- GROUP: account
CREATE TABLE a_general_ledger_master (

  a_glm_sequence_i integer NOT NULL,
    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The year is a number starting from 0 (the year of the installation and first accountings).
  a_year_i integer NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- This identifies which cost centre an account is applied to
  a_cost_centre_code_c varchar(24) NOT NULL,
  a_ytd_actual_base_n numeric(24, 10) DEFAULT 0,
    -- Special period for the year end close journal.
  a_closing_period_actual_base_n numeric(24, 10) DEFAULT 0,
    -- Starting Balance in the ledger currency
  a_start_balance_base_n numeric(24, 10) DEFAULT 0,
    -- Total for the current year, 2nd (int'l) base currency.
  a_ytd_actual_intl_n numeric(24, 10) DEFAULT 0,
    -- Special period for the year end close journal.
  a_closing_period_actual_intl_n numeric(24, 10) DEFAULT 0,
    -- Starting Balance (Int'l)
  a_start_balance_intl_n numeric(24, 10) DEFAULT 0,
  a_ytd_actual_foreign_n numeric(24, 10),
  a_start_balance_foreign_n numeric(24, 10),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_general_ledger_master_pk
    PRIMARY KEY (a_glm_sequence_i),
  CONSTRAINT a_general_ledger_master_uc1
    UNIQUE (a_ledger_number_i,a_year_i,a_account_code_c,a_cost_centre_code_c),
  CONSTRAINT a_general_ledger_master_fk1
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_general_ledger_master_fk2
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_general_ledger_master_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_general_ledger_master_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_general_ledger_master_pk0 
   (a_glm_sequence_i),
KEY inx_eneral_ledger_master_fk1_key1 
   (a_ledger_number_i,a_account_code_c),
KEY inx_eneral_ledger_master_fk2_key2 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_neral_ledger_master_fkcr_key3 
   (s_created_by_c),
KEY inx_neral_ledger_master_fkmd_key4 
   (s_modified_by_c),
UNIQUE KEY inx_a_general_ledger_master_uc10 
   (a_ledger_number_i,a_year_i,a_account_code_c,a_cost_centre_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The General Ledger Master data for one period (e.g. a month)
-- GROUP: account
CREATE TABLE a_general_ledger_master_period (

  a_glm_sequence_i integer NOT NULL,
  a_period_number_i integer NOT NULL,
    -- This is a number of ledger currency units
  a_actual_base_n numeric(24, 10) DEFAULT 0,
    -- This is a number of ledger currency units
  a_budget_base_n numeric(24, 10) DEFAULT 0,
    -- Not supported in Openpetra anymore ...
  a_actual_intl_n numeric(24, 10) DEFAULT 0,
    -- Not supported in Openpetra anymore ...
  a_budget_intl_n numeric(24, 10) DEFAULT 0,
  a_actual_foreign_n numeric(24, 10),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_glm_period_pk
    PRIMARY KEY (a_glm_sequence_i,a_period_number_i),
  CONSTRAINT a_general_ledger_master_period_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_general_ledger_master_period_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_glm_period_fk1
    FOREIGN KEY (a_glm_sequence_i)
    REFERENCES a_general_ledger_master(a_glm_sequence_i),
UNIQUE KEY inx_a_glm_period_pk0 
   (a_glm_sequence_i,a_period_number_i),
KEY inx_edger_master_period_fkcr_key1 
   (s_created_by_c),
KEY inx_edger_master_period_fkmd_key2 
   (s_modified_by_c),
KEY inx_a_glm_period_fk1_key3 
   (a_glm_sequence_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Used as a subdvision of motivation group. Details of the reason money has been received, where it is going (cost centre and account), and fees to be charged on it.
-- GROUP: gift
CREATE TABLE a_motivation_detail (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16) NOT NULL,
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16) NOT NULL,
    -- This is a long description and is 80 characters long.
  a_motivation_detail_audience_c varchar(160),
    -- This is a long description and is 80 characters long.
  a_motivation_detail_desc_c varchar(160) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- This identifies which cost centre an account is applied to
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- Defines whether the motivation code is still in use
  a_motivation_status_l boolean DEFAULT 1 NOT NULL,
    -- This is a number of currency units
  a_mailing_cost_n numeric(24, 10) DEFAULT 0,
    -- Used to get a yes no response from the user
  a_bulk_rate_flag_l boolean DEFAULT 0 NOT NULL,
    -- This defines what should happen next
  a_next_response_status_c varchar(24),
    -- Used to get a yes no response from the user
  a_activate_partner_flag_l boolean DEFAULT 0 NOT NULL,
    -- The number of items sent out in a mailing
  a_number_sent_i integer DEFAULT 0,
    -- The number of items returned from a mailing
  a_number_of_responses_i integer DEFAULT 0,
    -- The target number of items returned from a mailing
  a_target_number_of_responses_i integer DEFAULT 0,
    -- This is a number of currency units
  a_target_amount_n numeric(24, 10) DEFAULT 0,
    -- This is a number of currency units
  a_amount_received_n numeric(24, 10) DEFAULT 0,
    -- This is the partner key assigned to each partner. It consists of the ledger id followed by a computer generated six digit number.
  p_recipient_key_n bigint DEFAULT 0 NOT NULL,
    -- A flag to automatically populate the description in the gift comment
  a_autopopdesc_l boolean DEFAULT 0,
    -- Whether receipts should be printed
  a_receipt_l boolean DEFAULT 1,
    -- Don't include these gifts in reporting
  a_dont_report_l boolean DEFAULT 0,
    -- Whether this gift is tax deductible
  a_tax_deductible_l boolean DEFAULT 1,
    -- This is a long description and is 80 characters long in the local language.
  a_motivation_detail_desc_local_c varchar(160),
    -- A short code for the motivation which can then be used on receipts
  a_short_code_c varchar(8),
    -- Indicates whether or not the motivation has restricted access. If it does then the access will be controlled by s_group_motivation
  a_restricted_l boolean DEFAULT 0,
    -- Whether or not gifts with this motivation should be exported to the worldwide Intranet (to help distinguish non-gifts like sales)
  a_export_to_intranet_l boolean DEFAULT 1,
    -- Which column should these gifts be reported in?
  a_report_column_c varchar(40),
    -- Account Code to be used for the Tax deductible portion of this Gift
  a_tax_deductible_account_code_c varchar(16),
    -- Indicates whether or not the motivation is for a paying a sponsorship. The recipient key might be used.
  a_sponsorship_l boolean DEFAULT 0 NOT NULL,
    -- Indicates whether or not the motivation is for paying a membership. The recipient key should be used.
  a_membership_l boolean DEFAULT 0 NOT NULL,
    -- Indicates whether or not the motivation is for supporting a worker. The recipient key must be used.
  a_worker_support_l boolean DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_motivation_detail_pk
    PRIMARY KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_motivation_detail_fk1
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c)
    REFERENCES a_motivation_group(a_ledger_number_i,a_motivation_group_code_c),
  CONSTRAINT a_motivation_detail_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_motivation_detail_fk3
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_motivation_detail_fk4
    FOREIGN KEY (p_recipient_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_motivation_detail_fk5
    FOREIGN KEY (a_ledger_number_i,a_tax_deductible_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_motivation_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_motivation_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_ep_account_fk4_ref0 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
UNIQUE KEY inx_a_motivation_detail_pk1 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_a_motivation_detail_fk1_key2 
   (a_ledger_number_i,a_motivation_group_code_c),
KEY inx_a_motivation_detail_fk2_key3 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_motivation_detail_fk3_key4 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_motivation_detail_fk4_key5 
   (p_recipient_key_n),
KEY inx_a_motivation_detail_fk5_key6 
   (a_ledger_number_i,a_tax_deductible_account_code_c),
KEY inx_a_motivation_detail_fkcr_key7 
   (s_created_by_c),
KEY inx_a_motivation_detail_fkmd_key8 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This contains the settings for one specific bank account
-- GROUP: account
CREATE TABLE a_ep_account (

    -- The bank account whose settings are defined here
  a_banking_details_key_i integer NOT NULL,
    -- The bank account has to be assigned to a ledger
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The GL account for the bank account
  a_account_code_c varchar(16) NOT NULL,
    -- This tells the plugin where to find the statement files for this bank account
  a_importfile_path_c varchar(200),
    -- This tells the plugin where to write any generated files for this bank account
  a_exportfile_path_c varchar(200),
    -- This can be either the name of an executable or a DLL that is able to process the country (or bank) specific bank statements
  a_plugin_filename_c varchar(200),
    -- Other parameters for the plugin can be stored here
  a_plugin_parameters_c varchar(500),
    -- if this is true, all gifts to this bank account get this flag set
  a_confidential_gift_flag_l boolean DEFAULT 0,
    -- Whether the gifts to this bank account are tax deductible
  a_tax_deductible_l boolean DEFAULT 1,
    -- This defines a default motivation group.
  a_motivation_group_code_c varchar(16),
    -- This defines a default motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_account_pk
    PRIMARY KEY (a_banking_details_key_i),
  CONSTRAINT a_ep_account_fk1
    FOREIGN KEY (a_banking_details_key_i)
    REFERENCES p_banking_details(p_banking_details_key_i),
  CONSTRAINT a_ep_account_fk2
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ep_account_fk3
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ep_account_fk4
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_ep_account_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_account_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_account_pk0 
   (a_banking_details_key_i),
KEY inx_a_ep_account_fk2_key1 
   (a_ledger_number_i),
KEY inx_a_ep_account_fk3_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_ep_account_fk4_key3 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_a_ep_account_fkcr_key4 
   (s_created_by_c),
KEY inx_a_ep_account_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- the matches that can be used to identify recurring gift or GL transactions
-- GROUP: account
CREATE TABLE a_ep_match (

    -- this is a sequence to easily identify which transaction has been matched and how
  a_ep_match_key_i integer NOT NULL,
    -- this is a separated list of all the recurring details of a_ep_transaction (ie. name, bank account, sort code, IBAN, amount, description)
  a_match_text_c varchar(200) NOT NULL,
    -- the match can be applied to split gifts as well
  a_detail_i integer DEFAULT 0 NOT NULL,
    -- What to do with this match: gift, GL, or discard
  a_action_c varchar(40) NOT NULL,
    -- The date when this match was recently applied; useful for purging old entries
  a_recent_match_d date NOT NULL,
    -- The four digit ledger number of the gift.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The partner key of the commitment field (the unit) of the recipient of the gift.  This is not the ledger number but rather the partner key of the unit associated with the ledger.
  a_recipient_ledger_number_n bigint DEFAULT 0,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16),
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16),
    -- Used to decide whose reports will see this comment
  a_comment_one_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_one_c varchar(160),
    -- Defines whether the donor wishes the recipient to know who gave the gift
  a_confidential_gift_flag_l boolean DEFAULT 0 NOT NULL,
    -- Whether this gift is tax deductaible
  a_tax_deductible_l boolean DEFAULT 1,
    -- The partner key of the recipient of the gift.
  p_recipient_key_n bigint DEFAULT 0 NOT NULL,
    -- To determine whether an admin fee on the transaction should be overwritten if it normally has a charge associated with it. Used for both local and ilt transaction.
  a_charge_flag_l boolean DEFAULT 1,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner.
  a_cost_centre_code_c varchar(24),
    -- Mailing Code of the mailing that the gift was a response to.
  p_mailing_code_c varchar(50),
    -- Used to decide whose reports will see this comment
  a_comment_two_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_two_c varchar(160),
    -- Used to decide whose reports will see this comment
  a_comment_three_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_three_c varchar(160),
    -- This is a number of currency units in the entered Currency
  a_gift_transaction_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_home_admin_charges_flag_l boolean DEFAULT 1 NOT NULL,
    -- Used to get a yes no response from the user
  a_ilt_admin_charges_flag_l boolean DEFAULT 1 NOT NULL,
  a_receipt_letter_code_c varchar(20),
    -- Defines how a gift is given.
  a_method_of_giving_code_c varchar(24),
    -- This is how the partner paid. Eg cash, Cheque etc
  a_method_of_payment_code_c varchar(16),
    -- This is the partner key of the donor.
  p_donor_key_n bigint DEFAULT 0 NOT NULL,
    -- NOT USED AT ALL
  a_admin_charge_l boolean DEFAULT 0,
  a_narrative_c varchar(240),
    -- Reference number/code for the transaction
  a_reference_c varchar(20),
    -- short name of the donor; will be used for generating export files
  p_donor_short_name_c varchar(500),
    -- short name of recipient
  p_recipient_short_name_c varchar(500),
    -- Indicates whether or not the gift has restricted access. If it does then the access will be controlled by s_group_gift
  a_restricted_l boolean DEFAULT 0,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16),
    -- Key ministry to which this transaction applies (just for fund transfers)
  a_key_ministry_key_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_match_pk
    PRIMARY KEY (a_ep_match_key_i),
  CONSTRAINT a_ep_match_uk
    UNIQUE (a_match_text_c,a_detail_i,a_ledger_number_i),
  CONSTRAINT a_ep_match_fk1
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_ep_match_fk2
    FOREIGN KEY (p_recipient_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_ep_match_fk3
    FOREIGN KEY (p_mailing_code_c)
    REFERENCES p_mailing(p_mailing_code_c),
  CONSTRAINT a_ep_match_fk4
    FOREIGN KEY (a_recipient_ledger_number_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_ep_match_fk5
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_ep_match_fk6
    FOREIGN KEY (a_method_of_giving_code_c)
    REFERENCES a_method_of_giving(a_method_of_giving_code_c),
  CONSTRAINT a_ep_match_fk7
    FOREIGN KEY (a_method_of_payment_code_c)
    REFERENCES a_method_of_payment(a_method_of_payment_code_c),
  CONSTRAINT a_ep_match_fk8
    FOREIGN KEY (p_donor_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_ep_match_fk9
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ep_match_fk10
    FOREIGN KEY (a_key_ministry_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT a_ep_match_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_match_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_match_pk0 
   (a_ep_match_key_i),
UNIQUE KEY inx_a_ep_match_uk1 
   (a_match_text_c,a_detail_i,a_ledger_number_i),
KEY inx_a_ep_match_fk1_key2 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_a_ep_match_fk2_key3 
   (p_recipient_key_n),
KEY inx_a_ep_match_fk3_key4 
   (p_mailing_code_c),
KEY inx_a_ep_match_fk4_key5 
   (a_recipient_ledger_number_n),
KEY inx_a_ep_match_fk5_key6 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_ep_match_fk6_key7 
   (a_method_of_giving_code_c),
KEY inx_a_ep_match_fk7_key8 
   (a_method_of_payment_code_c),
KEY inx_a_ep_match_fk8_key9 
   (p_donor_key_n),
KEY inx_a_ep_match_fk9_key10 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_ep_match_fk10_key11 
   (a_key_ministry_key_n),
KEY inx_a_ep_match_fkcr_key12 
   (s_created_by_c),
KEY inx_a_ep_match_fkmd_key13 
   (s_modified_by_c),
KEY a_match_text_k0 
   (a_match_text_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- the transactions from the recently imported bank statements; they should help to identify the other party of the transaction (donor, etc) and the purpose of the transaction
-- GROUP: account
CREATE TABLE a_ep_transaction (

    -- this transaction belongs to that statement
  a_statement_key_i integer NOT NULL,
    -- to complete the primary key
  a_order_i integer NOT NULL,
    -- a transaction can be split in order to support mixed GL and Gift records
  a_detail_key_i integer DEFAULT -1 NOT NULL,
    -- can be different from order, since the paper statements can have different order than the electronic statement
  a_number_on_paper_statement_i integer DEFAULT -1,
    -- this is a calculated text that uniquely identifies this transaction so that it can be recognised next month. TODO: should have a link table a_ep_match between a_ep_transaction and a_ep_match_detail
  a_match_text_c varchar(200),
    -- This can be a summary of title, first name, last name etc. of the other party
  a_account_name_c varchar(160),
  a_title_c varchar(64),
  a_first_name_c varchar(64),
  a_middle_name_c varchar(64),
    -- the name of the other party
  a_last_name_c varchar(64),
    -- The bank code/branch code/sort code of the other party.
  p_branch_code_c varchar(20),
    -- BIC (Bank Identifier Code)/SWIFT code of the other party
  p_bic_c varchar(22),
    -- The account number in the bank of the other party
  a_bank_account_number_c varchar(40),
    -- The IBAN (International Bank Account Number) of the other party.
  a_iban_c varchar(128),
    -- This can be recurring income, recurring payment, income, payment, direct debit, etc.
  a_transaction_type_code_c varchar(40),
    -- The amount in the currency of the bank account
  a_transaction_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This description was given when the transfer was initiated
  a_description_c varchar(512),
    -- The date when this transaction became valid or available
  a_date_effective_d date NOT NULL,
    -- set this value to the match (either new match or set automatically)
  a_ep_match_key_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_transaction_pk
    PRIMARY KEY (a_statement_key_i,a_order_i,a_detail_key_i),
  CONSTRAINT a_ep_transaction_fk1
    FOREIGN KEY (a_statement_key_i)
    REFERENCES a_ep_statement(a_statement_key_i),
  CONSTRAINT a_ep_transaction_fk2
    FOREIGN KEY (a_ep_match_key_i)
    REFERENCES a_ep_match(a_ep_match_key_i),
  CONSTRAINT a_ep_transaction_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_transaction_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_transaction_pk0 
   (a_statement_key_i,a_order_i,a_detail_key_i),
KEY inx_a_ep_transaction_fk1_key1 
   (a_statement_key_i),
KEY inx_a_ep_transaction_fk2_key2 
   (a_ep_match_key_i),
KEY inx_a_ep_transaction_fkcr_key3 
   (s_created_by_c),
KEY inx_a_ep_transaction_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- motivation details can have several fees
-- GROUP: gift
CREATE TABLE a_motivation_detail_fee (

  a_ledger_number_i integer NOT NULL,
  a_motivation_group_code_c varchar(16) NOT NULL,
  a_motivation_detail_code_c varchar(16) NOT NULL,
  a_fee_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_motivation_detail_fee_pk
    PRIMARY KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c,a_fee_code_c),
  CONSTRAINT a_motivation_detail_fee_fk1
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_motivation_detail_fee_fk2
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_motivation_detail_fee_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_motivation_detail_fee_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_motivation_detail_fee_pk0 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c,a_fee_code_c),
KEY inx_otivation_detail_fee_fk1_key1 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_otivation_detail_fee_fk2_key2 
   (a_ledger_number_i),
KEY inx_tivation_detail_fee_fkcr_key3 
   (s_created_by_c),
KEY inx_tivation_detail_fee_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Various ways a transaction may originate, depending on the subsystem.
-- GROUP: account
CREATE TABLE a_transaction_type (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24) NOT NULL,
  a_transaction_type_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_debit_account_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_credit_account_code_c varchar(16) NOT NULL,
    -- Identifies a journal within a batch
  a_last_journal_i integer DEFAULT 0 NOT NULL,
    -- Identifies a journal within a batch
  a_last_recurring_journal_i integer DEFAULT 0 NOT NULL,
    -- This is a short description which is 32 charcters long
  a_transaction_type_description_c varchar(64) NOT NULL,
  a_balancing_account_code_c varchar(16),
    -- Is this transaction type a special transaction type or not?
  a_special_transaction_type_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_transaction_type_pk
    PRIMARY KEY (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
  CONSTRAINT a_transaction_type_fk1
    FOREIGN KEY (a_ledger_number_i,a_debit_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_transaction_type_fk2
    FOREIGN KEY (a_ledger_number_i,a_credit_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_transaction_type_fk3
    FOREIGN KEY (a_sub_system_code_c)
    REFERENCES a_sub_system(a_sub_system_code_c),
  CONSTRAINT a_transaction_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_transaction_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_transaction_type_pk0 
   (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
KEY inx_a_transaction_type_fk1_key1 
   (a_ledger_number_i,a_debit_account_code_c),
KEY inx_a_transaction_type_fk2_key2 
   (a_ledger_number_i,a_credit_account_code_c),
KEY inx_a_transaction_type_fk3_key3 
   (a_sub_system_code_c),
KEY inx_a_transaction_type_fkcr_key4 
   (s_created_by_c),
KEY inx_a_transaction_type_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Templates of general ledger journals which are copied into the ledger with recurring general ledger batches.
-- GROUP: account
CREATE TABLE a_recurring_journal (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a journal within a batch
  a_journal_number_i integer DEFAULT 0 NOT NULL,
    -- This is a long description and is 80 characters long.
  a_journal_description_c varchar(160) NOT NULL,
    -- identifies the status of a batch
  a_journal_status_c varchar(40) DEFAULT 'Unposted' NOT NULL,
    -- This is a number of currency units
  a_journal_debit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is a number of currency units
  a_journal_credit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This defines which accounting period is being used
  a_journal_period_i integer DEFAULT 0,
    -- Date the  batch comes into effect
  a_date_effective_d date NOT NULL,
  a_transaction_type_code_c varchar(16) NOT NULL,
    -- This is how the partner paid. Eg cash, Cheque etc
  a_method_of_payment_code_c varchar(16),
  a_last_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24) NOT NULL,
    -- The rate of exchange
  a_exchange_rate_to_base_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This defines which currency is being used
  a_transaction_currency_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_journal_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_recurring_journal_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_recurring_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_journal_fk2
    FOREIGN KEY (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c)
    REFERENCES a_transaction_type(a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
  CONSTRAINT a_recurring_journal_fk3
    FOREIGN KEY (a_method_of_payment_code_c)
    REFERENCES a_method_of_payment(a_method_of_payment_code_c),
  CONSTRAINT a_recurring_journal_fk4
    FOREIGN KEY (a_transaction_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_recurring_journal_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_journal_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_journal_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_a_recurring_journal_fk1_key1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_recurring_journal_fk2_key2 
   (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
KEY inx_a_recurring_journal_fk3_key3 
   (a_method_of_payment_code_c),
KEY inx_a_recurring_journal_fk4_key4 
   (a_transaction_currency_c),
KEY inx_a_recurring_journal_fkcr_key5 
   (s_created_by_c),
KEY inx_a_recurring_journal_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Templates of general ledger transactions which are copied into the ledger with general ledger batches.
-- GROUP: account
CREATE TABLE a_recurring_transaction (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Journal  number that transaction belongs to
  a_journal_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- This identifies which cost centre an account is applied to
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- Date the transaction took place
  a_transaction_date_d date NOT NULL,
    -- This defines which currency is being used
  a_transaction_currency_c varchar(16),
    -- This is a number of currency units
  a_transaction_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This defines which currency is being used
  a_base_currency_c varchar(16),
    -- The rate of exchange
  a_exchange_rate_to_base_n numeric(24, 10) DEFAULT 0,
    -- This is a number of currency units
  a_amount_in_base_currency_n numeric(24, 10) DEFAULT 0 NOT NULL,
  a_analysis_indicator_l boolean DEFAULT 0 NOT NULL,
  a_method_of_payment_code_c varchar(16),
    -- This defines which accounting period is being used
  a_period_number_i integer DEFAULT 0,
    -- Shows if the transaction has been reconciled or not
  a_reconciled_flag_l boolean DEFAULT 0 NOT NULL,
    -- Defines a sub system of accounts
  a_sub_system_code_c varchar(24),
  a_transaction_type_code_c varchar(16),
  a_narrative_c varchar(240),
  a_reference_c varchar(16) NOT NULL,
  a_date_of_entry_d date,
  s_user_id_c varchar(20),
  a_debit_credit_indicator_l boolean DEFAULT 1 NOT NULL,
    -- Has a transaction been posted yet
  a_transaction_status_l boolean DEFAULT 0,
    -- The header (eg, cashbook #) that the transaction is associated with.
  a_header_number_i integer DEFAULT 0,
    -- The detail (within the header) that the transaction is associated with.
  a_detail_number_i integer DEFAULT 0,
  a_sub_type_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_transaction_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
  CONSTRAINT a_recurring_transaction_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i)
    REFERENCES a_recurring_journal(a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_recurring_transaction_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_recurring_transaction_fk3
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_recurring_transaction_fk4
    FOREIGN KEY (a_transaction_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_recurring_transaction_fk5
    FOREIGN KEY (a_base_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_recurring_transaction_fk6
    FOREIGN KEY (a_method_of_payment_code_c)
    REFERENCES a_method_of_payment(a_method_of_payment_code_c),
  CONSTRAINT a_recurring_transaction_fk7
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_transaction_fk8
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_recurring_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_transaction_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_transaction_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_transaction_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
KEY inx_ecurring_transaction_fk1_key1 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_ecurring_transaction_fk2_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_ecurring_transaction_fk3_key3 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_ecurring_transaction_fk4_key4 
   (a_transaction_currency_c),
KEY inx_ecurring_transaction_fk5_key5 
   (a_base_currency_c),
KEY inx_ecurring_transaction_fk6_key6 
   (a_method_of_payment_code_c),
KEY inx_ecurring_transaction_fk7_key7 
   (s_user_id_c),
KEY inx_ecurring_transaction_fk8_key8 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_curring_transaction_fkcr_key9 
   (s_created_by_c),
KEY inx_curring_transaction_fkmd_key10 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Detailed analysis information stored along with the recurring transaction template.
-- GROUP: account
CREATE TABLE a_recurring_trans_anal_attrib (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a journal within a batch
  a_journal_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- Cost Centre to which the attribute relates
  a_cost_centre_code_c varchar(24),
  a_analysis_type_code_c varchar(16) NOT NULL,
  a_analysis_attribute_value_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_trans_anal_attr_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i,a_analysis_type_code_c),
  CONSTRAINT a_recurring_trans_anal_attr_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i)
    REFERENCES a_recurring_transaction(a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
  CONSTRAINT a_recurring_trans_anal_attr_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_recurring_trans_anal_attr_fk3
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c)
    REFERENCES a_analysis_attribute(a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
  CONSTRAINT a_recurring_trans_anal_attr_fk4
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c)
    REFERENCES a_freeform_analysis(a_ledger_number_i,a_analysis_type_code_c,a_analysis_value_c),
  CONSTRAINT a_recurring_trans_anal_attr_fk5
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_recurring_trans_anal_attr_fk6
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_recurring_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_trans_anal_attr_fk7
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i)
    REFERENCES a_recurring_journal(a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_recurring_trans_anal_attrib_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_trans_anal_attrib_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_recurring_trans_anal_attr_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i,a_analysis_type_code_c),
KEY inx_ring_trans_anal_attr_fk1_key1 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
KEY inx_ring_trans_anal_attr_fk2_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_ring_trans_anal_attr_fk3_key3 
   (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
KEY inx_ring_trans_anal_attr_fk4_key4 
   (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c),
KEY inx_ring_trans_anal_attr_fk5_key5 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_ring_trans_anal_attr_fk6_key6 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_ring_trans_anal_attr_fk7_key7 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_g_trans_anal_attrib_fkcr_key8 
   (s_created_by_c),
KEY inx_g_trans_anal_attrib_fkmd_key9 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Templates of gift batches which can be copied into the gift system.
-- GROUP: gift
CREATE TABLE a_recurring_gift_batch (

    -- ledger number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Gift batch number
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- gift batch description
  a_batch_description_c varchar(80),
    -- hash total for the gift batch
  a_hash_total_n numeric(24, 10) DEFAULT 0,
    -- total for the gift batch
  a_batch_total_n numeric(24, 10) DEFAULT 0,
    -- bank account code which this batch is for
  a_bank_account_code_c varchar(16) NOT NULL,
    -- last gift number of the batch
  a_last_gift_number_i integer DEFAULT 0,
    -- This defines which currency is being used
  a_currency_code_c varchar(16) NOT NULL,
    -- This identifies which cost centre is applied to the bank
  a_bank_cost_centre_c varchar(24),
    -- What type of gift is this? a gift or a gift in kind generally
  a_gift_type_c varchar(16) DEFAULT 'Gift' NOT NULL,
    -- This is how the partner paid. EgCash, Cheque etc
  a_method_of_payment_code_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_gift_batch_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_gift_batch_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_recurring_gift_batch_fk2
    FOREIGN KEY (a_ledger_number_i,a_bank_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_recurring_gift_batch_fk3
    FOREIGN KEY (a_ledger_number_i,a_bank_cost_centre_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_recurring_gift_batch_fk4
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_recurring_gift_batch_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_gift_batch_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_gift_batch_pk0 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_recurring_gift_batch_fk1_key1 
   (a_ledger_number_i),
KEY inx_recurring_gift_batch_fk2_key2 
   (a_ledger_number_i,a_bank_account_code_c),
KEY inx_recurring_gift_batch_fk3_key3 
   (a_ledger_number_i,a_bank_cost_centre_c),
KEY inx_recurring_gift_batch_fk4_key4 
   (a_currency_code_c),
KEY inx_ecurring_gift_batch_fkcr_key5 
   (s_created_by_c),
KEY inx_ecurring_gift_batch_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Templates of donor gift information which can be copied into the gift system with recurring gift batches.
-- GROUP: gift
CREATE TABLE a_recurring_gift (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_gift_transaction_number_i integer DEFAULT 0 NOT NULL,
  a_receipt_letter_code_c varchar(16),
    -- Defines how a gift is given
  a_method_of_giving_code_c varchar(24),
    -- This is how the partner paid. Eg cash, Cheque etc
  a_method_of_payment_code_c varchar(16),
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_donor_key_n bigint DEFAULT 0 NOT NULL,
    -- Identifies the last gift detail entered
  a_last_detail_number_i integer DEFAULT 0 NOT NULL,
    -- Reference number/code for the transaction
  a_reference_c varchar(16),
    -- Bank or credit card account to use for making this gift transaction.
  p_banking_details_key_i integer DEFAULT 0 NOT NULL,
    -- This reference is a unique string that reflects the customer or contract and the date of the SEPA mandate
  a_sepa_mandate_reference_c varchar(70),
    -- The date the SEPA Mandate was given
  a_sepa_mandate_given_d date,
    -- Status of the credit card transaction
  a_charge_status_c varchar(20),
    -- The last date that a successfull direct debit or credit card charge occurred for this gift
  a_last_debit_d date,
    -- The day of the month to make the recurring gift
  a_debit_day_i integer DEFAULT 0,
    -- Whether the recurring gift should be made
  a_active_l boolean DEFAULT 1 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_gift_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT a_recurring_gift_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_recurring_gift_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_gift_fk2
    FOREIGN KEY (a_method_of_giving_code_c)
    REFERENCES a_method_of_giving(a_method_of_giving_code_c),
  CONSTRAINT a_recurring_gift_fk3
    FOREIGN KEY (a_method_of_payment_code_c)
    REFERENCES a_method_of_payment(a_method_of_payment_code_c),
  CONSTRAINT a_recurring_gift_fk4
    FOREIGN KEY (p_donor_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_recurring_gift_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_gift_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_gift_pk0 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_a_recurring_gift_fk1_key1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_recurring_gift_fk2_key2 
   (a_method_of_giving_code_c),
KEY inx_a_recurring_gift_fk3_key3 
   (a_method_of_payment_code_c),
KEY inx_a_recurring_gift_fk4_key4 
   (p_donor_key_n),
KEY inx_a_recurring_gift_fkcr_key5 
   (s_created_by_c),
KEY inx_a_recurring_gift_fkmd_key6 
   (s_modified_by_c),
KEY a_donor_k0 
   (a_ledger_number_i,p_donor_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Store recipient information for the recurring gift.
-- GROUP: gift
CREATE TABLE a_recurring_gift_detail (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Number of the gift batch containing this detail.
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a gift transaction within a gift batch.
  a_gift_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a gift
  a_detail_number_i integer DEFAULT 0 NOT NULL,
    -- This is used as a key field in most of the accounting system files
  a_recipient_ledger_number_n bigint DEFAULT 0,
    -- This is the amount in transaction currency. This field should be renamed to a_gift_transaction_amount_n, to be in analogy to a_gift_detail.a_gift_transaction_amount_n
  a_gift_amount_n numeric(24, 10) DEFAULT 0,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16) NOT NULL,
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16) NOT NULL,
    -- Used to decide whose reports will see this comment
  a_comment_one_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_one_c varchar(160),
    -- Defines whether the donor wishes the recipient to know who gave the gift
  a_confidential_gift_flag_l boolean DEFAULT 0 NOT NULL,
    -- Whether this gift is tax deductible
  a_tax_deductible_l boolean DEFAULT 1,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_recipient_key_n bigint DEFAULT 0 NOT NULL,
    -- To determine whether an admin fee on the transaction should be overwritten if it normally has a charge associated with it. Used for both local and ilt transaction.
  a_charge_flag_l boolean DEFAULT 1,
    -- Mailing Code
  p_mailing_code_c varchar(50),
    -- Used to decide whose reports will see this comment
  a_comment_two_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_two_c varchar(160),
    -- Used to decide whose reports will see this comment
  a_comment_three_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_three_c varchar(160),
    -- Date that donor wants to begin giving this recurring donation
  a_start_donations_d date,
    -- Date that donor wants to stop giving this recurring donation
  a_end_donations_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_recurring_gift_detail_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
  CONSTRAINT a_recurring_gift_detail_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i)
    REFERENCES a_recurring_gift(a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT a_recurring_gift_detail_fk2
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_recurring_gift_detail_fk3
    FOREIGN KEY (p_recipient_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_recurring_gift_detail_fk4
    FOREIGN KEY (p_mailing_code_c)
    REFERENCES p_mailing(p_mailing_code_c),
  CONSTRAINT a_recurring_gift_detail_fk5
    FOREIGN KEY (a_recipient_ledger_number_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_recurring_gift_detail_fk6
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_recurring_gift_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_recurring_gift_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_recurring_gift_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_recurring_gift_detail_pk0 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
KEY inx_ecurring_gift_detail_fk1_key1 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_ecurring_gift_detail_fk2_key2 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_ecurring_gift_detail_fk3_key3 
   (p_recipient_key_n),
KEY inx_ecurring_gift_detail_fk4_key4 
   (p_mailing_code_c),
KEY inx_ecurring_gift_detail_fk5_key5 
   (a_recipient_ledger_number_n),
KEY inx_ecurring_gift_detail_fk6_key6 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_curring_gift_detail_fkcr_key7 
   (s_created_by_c),
KEY inx_curring_gift_detail_fkmd_key8 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Information describing groups (batches) of gifts.
-- GROUP: gift
CREATE TABLE a_gift_batch (

    -- ledger number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Gift batch number
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- gift batch description
  a_batch_description_c varchar(80) NOT NULL,
    -- date of user entry or last modification.
  s_modification_date_d date,
    -- hash total for the gift batch
  a_hash_total_n numeric(24, 10) DEFAULT 0,
    -- total for the gift batch
  a_batch_total_n numeric(24, 10) DEFAULT 0,
    -- bank account code which this batch is for
  a_bank_account_code_c varchar(16) NOT NULL,
    -- last gift number of the batch
  a_last_gift_number_i integer DEFAULT 0,
    -- Status of a gift batch: Unposted, Posted, Cancelled.
  a_batch_status_c varchar(16) DEFAULT 'Unposted',
    -- The accounting period that the batch belongs to.  Must be <= 20.
  a_batch_period_i integer DEFAULT 0 NOT NULL,
    -- The financial year that the batch belongs to.
  a_batch_year_i integer NOT NULL,
    -- Effective date when posted to the general ledger.
  a_gl_effective_date_d date NOT NULL,
    -- This defines which currency is being used
  a_currency_code_c varchar(16) NOT NULL,
    -- The rate of exchange
  a_exchange_rate_to_base_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This identifies which cost centre is applied to the bank
  a_bank_cost_centre_c varchar(24) NOT NULL,
    -- What type of gift is this? a gift or a gift in kind generally
  a_gift_type_c varchar(16) DEFAULT 'Gift' NOT NULL,
    -- This is how the partner paid. EgCash, Cheque etc
  a_method_of_payment_code_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_gift_batch_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_gift_batch_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_gift_batch_fk2
    FOREIGN KEY (a_ledger_number_i,a_bank_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_gift_batch_fk3
    FOREIGN KEY (a_ledger_number_i,a_bank_cost_centre_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_gift_batch_fk4
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_gift_batch_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_gift_batch_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_processed_fee_fk4_ref0 
   (a_ledger_number_i,a_batch_number_i),
UNIQUE KEY inx_a_gift_batch_pk1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_gift_batch_fk1_key2 
   (a_ledger_number_i),
KEY inx_a_gift_batch_fk2_key3 
   (a_ledger_number_i,a_bank_account_code_c),
KEY inx_a_gift_batch_fk3_key4 
   (a_ledger_number_i,a_bank_cost_centre_c),
KEY inx_a_gift_batch_fk4_key5 
   (a_currency_code_c),
KEY inx_a_gift_batch_fkcr_key6 
   (s_created_by_c),
KEY inx_a_gift_batch_fkmd_key7 
   (s_modified_by_c),
KEY date_effective0 
   (a_ledger_number_i,a_batch_status_c,a_gl_effective_date_d),
KEY period_effective1 
   (a_ledger_number_i,a_batch_status_c,a_batch_year_i,a_batch_period_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Information on the donor's giving. Points to the gift detail records.
-- GROUP: gift
CREATE TABLE a_gift (

    -- This is used as a key field in most of the accounting system files.  The four digit ledger number of the gift.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_gift_transaction_number_i integer DEFAULT 0 NOT NULL,
  a_gift_status_c varchar(24),
  a_date_entered_d date NOT NULL,
    -- Used to get a yes no response from the user
  a_home_admin_charges_flag_l boolean DEFAULT 1 NOT NULL,
    -- Used to get a yes no response from the user
  a_ilt_admin_charges_flag_l boolean DEFAULT 1 NOT NULL,
    -- Indicates that this gift is linked to the previous gift transaction number. For adjustments this links the new transaction to the reversal one.  Set by the system.
  a_link_to_previous_gift_l boolean DEFAULT 0 NOT NULL,
    -- Indicates whether this gift should be included on receipts. For adjustments this field can be set to No to suppress printing.
  a_print_receipt_l boolean DEFAULT 1 NOT NULL,
  a_receipt_letter_code_c varchar(16),
    -- Defines how a gift is given.
  a_method_of_giving_code_c varchar(24),
    -- This is how the partner paid. Eg cash, Cheque etc
  a_method_of_payment_code_c varchar(16),
    -- This is the partner key of the donor.
  p_donor_key_n bigint DEFAULT 0 NOT NULL,
    -- NOT USED AT ALL
  a_admin_charge_l boolean DEFAULT 0,
    -- Gift Receipt Number
  a_receipt_number_i integer DEFAULT 0,
    -- Identifies the last gift detail entered
  a_last_detail_number_i integer DEFAULT 0 NOT NULL,
    -- Reference number/code for the transaction
  a_reference_c varchar(20),
    -- Flag to indicate Donors first gift
  a_first_time_gift_l boolean DEFAULT 0,
    -- Indicates whether or not the receipt has been printed for this gift
  a_receipt_printed_l boolean DEFAULT 0 NOT NULL,
    -- Indicates whether or not the gift has restricted access. If it does then the access will be controlled by s_group_gift
  a_restricted_l boolean DEFAULT 0,
    -- Bank or credit card account used for making this gift transaction.
  p_banking_details_key_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_gift_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT a_gift_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_gift_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_gift_fk2
    FOREIGN KEY (a_method_of_giving_code_c)
    REFERENCES a_method_of_giving(a_method_of_giving_code_c),
  CONSTRAINT a_gift_fk3
    FOREIGN KEY (a_method_of_payment_code_c)
    REFERENCES a_method_of_payment(a_method_of_payment_code_c),
  CONSTRAINT a_gift_fk4
    FOREIGN KEY (p_donor_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_gift_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_gift_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_gift_pk0 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_a_gift_fk1_key1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_gift_fk2_key2 
   (a_method_of_giving_code_c),
KEY inx_a_gift_fk3_key3 
   (a_method_of_payment_code_c),
KEY inx_a_gift_fk4_key4 
   (p_donor_key_n),
KEY inx_a_gift_fkcr_key5 
   (s_created_by_c),
KEY inx_a_gift_fkmd_key6 
   (s_modified_by_c),
KEY a_date_entered0 
   (a_ledger_number_i,a_date_entered_d),
KEY a_donor1 
   (p_donor_key_n,a_ledger_number_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The gift recipient information for a gift.  A single gift can be split among more than one recipient.  A gift detail record is created for each recipient.
-- GROUP: gift
CREATE TABLE a_gift_detail (

    -- The four digit ledger number of the gift.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Number of the gift batch containing this detail.
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a gift transaction within a gift batch.
  a_gift_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a gift detail within a gift transaction.   When a donor gives a donation to multiple recipients (a split gift), a gift detail record is created for each recipient.
  a_detail_number_i integer DEFAULT 0 NOT NULL,
    -- The partner key of the commitment field (the unit) of the recipient of the gift.  This is not the ledger number but rather the partner key of the unit associated with the ledger.
  a_recipient_ledger_number_n bigint DEFAULT 0 NOT NULL,
    -- This is a number of currency units of the ledger base currency.
  a_gift_amount_n numeric(24, 10) DEFAULT 0,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16) NOT NULL,
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16) NOT NULL,
    -- Used to decide whose reports will see this comment
  a_comment_one_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_one_c varchar(160),
    -- Defines whether the donor wishes the recipient to know who gave the gift
  a_confidential_gift_flag_l boolean DEFAULT 0 NOT NULL,
    -- Whether this gift is tax deductible
  a_tax_deductible_l boolean DEFAULT 1,
    -- The partner key of the recipient of the gift.
  p_recipient_key_n bigint DEFAULT 0 NOT NULL,
    -- To determine whether an admin fee on the transaction should be overwritten if it normally has a charge associated with it. Used for both local and ilt transaction.
  a_charge_flag_l boolean DEFAULT 1,
    -- This identifies which cost centre an account is applied to. A cost centre can be a partner.
  a_cost_centre_code_c varchar(24),
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16),
    -- This is a number of currency units in the International Currency
  a_gift_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates whether this gift detail has a matching inverse detail record because a modification was made
  a_modified_detail_l boolean DEFAULT 0,
    -- Stores the concatenated primary key fields of the originating gift detail, of which this gift detail is the inverse.
  a_modified_detail_key_c varchar(48),
    -- Indicates whether this gift detail's gift destination can be changed. Used for gift adjustments with family recipients.
  a_fixed_gift_destination_l boolean DEFAULT 0,
    -- This is a number of currency units in the entered Currency
  a_gift_transaction_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- identifies the ICH process number
  a_ich_number_i integer DEFAULT 0 NOT NULL,
    -- Mailing Code of the mailing that the gift was a response to.
  p_mailing_code_c varchar(50),
    -- Used to decide whose reports will see this comment
  a_comment_two_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_two_c varchar(160),
    -- Used to decide whose reports will see this comment
  a_comment_three_type_c varchar(24),
    -- This is a long description and is 80 characters long.
  a_gift_comment_three_c varchar(160),
    -- Account Code to be used for the Tax deductible portion of this Gift
  a_tax_deductible_account_code_c varchar(16),
    -- Percentage of gift amount that is tax-deductible
  a_tax_deductible_pct_n numeric(5, 2),
    -- Tax deductible portion of gift
  a_tax_deductible_amount_n numeric(24, 10) DEFAULT 0,
    -- Tax deductible portion of gift (Base Currency)
  a_tax_deductible_amount_base_n numeric(24, 10) DEFAULT 0,
    -- Tax deductible portion of gift (Intl Currency)
  a_tax_deductible_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- Non tax-deductible portion of gift
  a_non_deductible_amount_n numeric(24, 10) DEFAULT 0,
    -- Non tax-deductible portion of gift (Base Currency)
  a_non_deductible_amount_base_n numeric(24, 10) DEFAULT 0,
    -- Non tax-deductible portion of gift (Intl Currency)
  a_non_deductible_amount_intl_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_gift_detail_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
  CONSTRAINT a_gift_detail_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i)
    REFERENCES a_gift(a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT a_gift_detail_fk2
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT a_gift_detail_fk3
    FOREIGN KEY (p_recipient_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_gift_detail_fk4
    FOREIGN KEY (p_mailing_code_c)
    REFERENCES p_mailing(p_mailing_code_c),
  CONSTRAINT a_gift_detail_fk5
    FOREIGN KEY (a_recipient_ledger_number_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_gift_detail_fk6
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a__gift_detail_fk7
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_gift_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_gift_detail_fk8
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_gift_detail_fk9
    FOREIGN KEY (a_ledger_number_i,a_tax_deductible_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_gift_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_gift_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_a_processed_fee_fk3_ref0 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
UNIQUE KEY inx_a_gift_detail_pk1 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
KEY inx_a_gift_detail_fk1_key2 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_a_gift_detail_fk2_key3 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_a_gift_detail_fk3_key4 
   (p_recipient_key_n),
KEY inx_a_gift_detail_fk4_key5 
   (p_mailing_code_c),
KEY inx_a_gift_detail_fk5_key6 
   (a_recipient_ledger_number_n),
KEY inx_a_gift_detail_fk6_key7 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a__gift_detail_fk7_key8 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_gift_detail_fk8_key9 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_gift_detail_fk9_key10 
   (a_ledger_number_i,a_tax_deductible_account_code_c),
KEY inx_a_gift_detail_fkcr_key11 
   (s_created_by_c),
KEY inx_a_gift_detail_fkmd_key12 
   (s_modified_by_c),
KEY a_recipient_ledger_number_i0 
   (a_ledger_number_i,a_recipient_ledger_number_n,p_recipient_key_n),
KEY p_recipient_key1 
   (a_ledger_number_i,p_recipient_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores administrative fees and grants which have been calculated on gifts.
-- GROUP: gift
CREATE TABLE a_processed_fee (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Number of the gift batch containing this detail.
  a_batch_number_i integer NOT NULL,
    -- Identifies a gift transaction within a gift batch.
  a_gift_transaction_number_i integer NOT NULL,
    -- Identifies a gift detail within a gift transaction.   When a donor gives a donation to multiple recipients (a split gift), a gift detail record is created for each recipient.
  a_detail_number_i integer NOT NULL,
    -- the fee which the calculated amounts are stored against.
  a_fee_code_c varchar(16) NOT NULL,
    -- Cost centre code which the fee will be charged against.
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- Period number
  a_period_number_i integer DEFAULT 0 NOT NULL,
    -- Total Amount of the fee for the given period.
  a_periodic_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Date ""admin fee calculations"" have been run to fee total has been created as a transaction in the general ledger.
  a_processed_date_d date,
    -- System generated time stamp.
  s_timestamp_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_processed_fee_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i,a_fee_code_c),
  CONSTRAINT a_processed_fee_fk1
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_processed_fee_fk2
    FOREIGN KEY (a_ledger_number_i,a_period_number_i)
    REFERENCES a_accounting_period(a_ledger_number_i,a_accounting_period_number_i),
  CONSTRAINT a_processed_fee_fk3
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i)
    REFERENCES a_gift_detail(a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
  CONSTRAINT a_processed_fee_fk4
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_gift_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_processed_fee_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_processed_fee_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_processed_fee_pk0 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i,a_fee_code_c),
KEY inx_a_processed_fee_fk1_key1 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_processed_fee_fk2_key2 
   (a_ledger_number_i,a_period_number_i),
KEY inx_a_processed_fee_fk3_key3 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i),
KEY inx_a_processed_fee_fk4_key4 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_processed_fee_fkcr_key5 
   (s_created_by_c),
KEY inx_a_processed_fee_fkmd_key6 
   (s_modified_by_c),
KEY a_processed_fee_k0 
   (a_ledger_number_i,a_period_number_i,a_fee_code_c,a_cost_centre_code_c,a_processed_date_d,s_timestamp_i),
KEY a_processed_fee_k21 
   (a_ledger_number_i,a_fee_code_c,a_batch_number_i,a_gift_transaction_number_i,a_detail_number_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Holds details of each general ledger journal, which contains a group of transactions.
-- GROUP: account
CREATE TABLE a_journal (

    -- The journal belongs to this ledger.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch the journal belongs to.
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a journal within a batch
  a_journal_number_i integer NOT NULL,
    -- This is a long description and is 80 characters long.
  a_journal_description_c varchar(160) NOT NULL,
    -- This is a number of currency units in the currency of the transaction.
  a_journal_debit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is the number of currency units in the currency of the transaction.
  a_journal_credit_total_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This defines which accounting period is being used
  a_journal_period_i integer DEFAULT 0 NOT NULL,
    -- Date the journal comes into effect.
  a_date_effective_d date NOT NULL,
    -- Which type of journal is this?  Eg. STD, ALLOC, REALLOC, REVAL.
  a_transaction_type_code_c varchar(16),
    -- The number of the last transaction within the journal.
  a_last_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- Indicates the sub system of accounts that the journal belongs to.  Eg. GL, AP, GR
  a_sub_system_code_c varchar(24) NOT NULL,
    -- Has a journal been posted yet
  a_journal_status_c varchar(24) DEFAULT 'Unposted',
    -- This defines which currency is being used
  a_transaction_currency_c varchar(16) NOT NULL,
  a_base_currency_c varchar(16),
    -- The rate of exchange from the transaction currency (in a_transaction_currency_c) to the ledger base currency.
  a_exchange_rate_to_base_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- The time component of the exchange rate.
  a_exchange_rate_time_i integer DEFAULT 0 NOT NULL,
    -- Date the journal was created.
  a_date_of_entry_d date NOT NULL,
    -- Indicates whether or not a journal has been reversed.
  a_reversed_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_journal_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_journal_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_journal_fk3
    FOREIGN KEY (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c)
    REFERENCES a_transaction_type(a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
  CONSTRAINT a_journal_fk4
    FOREIGN KEY (a_transaction_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_journal_fk5
    FOREIGN KEY (a_base_currency_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_journal_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_journal_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_journal_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_a_journal_fk1_key1 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_journal_fk3_key2 
   (a_ledger_number_i,a_sub_system_code_c,a_transaction_type_code_c),
KEY inx_a_journal_fk4_key3 
   (a_transaction_currency_c),
KEY inx_a_journal_fk5_key4 
   (a_base_currency_c),
KEY inx_a_journal_fkcr_key5 
   (s_created_by_c),
KEY inx_a_journal_fkmd_key6 
   (s_modified_by_c),
KEY a_date_effective_k0 
   (a_ledger_number_i,a_date_effective_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Detailed information for each debit and credit in a general ledger journal.
-- GROUP: account
CREATE TABLE a_transaction (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
  a_journal_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction must be stored against [NOT USED]
  a_primary_account_code_c varchar(16),
    -- This identifies which cost centre an account is applied to
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- This identifies which cost centre an account is applied to [NOT USED]
  a_primary_cost_centre_code_c varchar(24),
    -- Date the transaction took place
  a_transaction_date_d date NOT NULL,
    -- This is a number of currency units
  a_transaction_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- This is a number of currency units
  a_amount_in_base_currency_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- Used to get a yes no response from the user
  a_analysis_indicator_l boolean DEFAULT 0 NOT NULL,
    -- shows if the transaction has been reconciled or not
  a_reconciled_status_l boolean DEFAULT 0 NOT NULL,
  a_narrative_c varchar(500),
  a_debit_credit_indicator_l boolean DEFAULT 1 NOT NULL,
    -- Has a transaction been posted yet
  a_transaction_status_l boolean DEFAULT 0,
    -- The header (eg, cashbook #) that the transaction is associated with. [NOT USED]
  a_header_number_i integer DEFAULT 0,
    -- The detail (within the header) that the transaction is associated with. [NOT USED]
  a_detail_number_i integer DEFAULT 0,
  a_sub_type_c varchar(16),
    -- Indicates whether the ILT transaction has been transferred to transaction for ILT file.
  a_to_ilt_flag_l boolean DEFAULT 0,
    -- To flag a transaction as having come from a source ledger and been processed in an ilt processing centre
  a_source_flag_l boolean DEFAULT 0,
    -- Reference number/code for the transaction
  a_reference_c varchar(100) NOT NULL,
    -- Transaction key which initiated an ILT transaction
  a_source_reference_c varchar(100),
    -- Was this transaction generated automatically by the system?
  a_system_generated_l boolean DEFAULT 0,
    -- The transaction amount in the second base currency.
  a_amount_in_intl_currency_n numeric(24, 10) DEFAULT 0,
    -- identifies the ICH process number
  a_ich_number_i integer DEFAULT 0 NOT NULL,
    -- Key ministry to which this transaction applies (just for fund transfers)
  a_key_ministry_key_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_transaction_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
  CONSTRAINT a_transaction_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i)
    REFERENCES a_journal(a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_transaction_fk2
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_transaction_fk3
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_transaction_fk4
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_transaction_fk5
    FOREIGN KEY (a_key_ministry_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT a_transaction_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_transaction_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_transaction_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
KEY inx_a_transaction_fk1_key1 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_a_transaction_fk2_key2 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_transaction_fk3_key3 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_transaction_fk4_key4 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_transaction_fk5_key5 
   (a_key_ministry_key_n),
KEY inx_a_transaction_fkcr_key6 
   (s_created_by_c),
KEY inx_a_transaction_fkmd_key7 
   (s_modified_by_c),
KEY a_account_k0 
   (a_ledger_number_i,a_account_code_c,a_cost_centre_code_c,a_transaction_date_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Analysis information entered by the user for a general ledger transaction.
-- GROUP: account
CREATE TABLE a_trans_anal_attrib (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a journal within a batch
  a_journal_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
    -- Cost Centre to which the attribute relates
  a_cost_centre_code_c varchar(24),
  a_analysis_type_code_c varchar(16) NOT NULL,
  a_analysis_attribute_value_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_trans_anal_attrib_pk
    PRIMARY KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i,a_analysis_type_code_c),
  CONSTRAINT a_trans_anal_attrib_fk1
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i)
    REFERENCES a_transaction(a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
  CONSTRAINT a_trans_anal_attrib_fk2
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_trans_anal_attrib_fk3
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c)
    REFERENCES a_analysis_attribute(a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
  CONSTRAINT a_trans_anal_attrib_fk4
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c)
    REFERENCES a_freeform_analysis(a_ledger_number_i,a_analysis_type_code_c,a_analysis_value_c),
  CONSTRAINT a_trans_anal_attrib_fk5
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_trans_anal_attrib_fk6
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i)
    REFERENCES a_batch(a_ledger_number_i,a_batch_number_i),
  CONSTRAINT a_trans_anal_attrib_fk7
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_journal_number_i)
    REFERENCES a_journal(a_ledger_number_i,a_batch_number_i,a_journal_number_i),
  CONSTRAINT a_trans_anal_attrib_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_trans_anal_attrib_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_trans_anal_attrib_pk0 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i,a_analysis_type_code_c),
KEY inx_a_trans_anal_attrib_fk1_key1 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i,a_transaction_number_i),
KEY inx_a_trans_anal_attrib_fk2_key2 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_trans_anal_attrib_fk3_key3 
   (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
KEY inx_a_trans_anal_attrib_fk4_key4 
   (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c),
KEY inx_a_trans_anal_attrib_fk5_key5 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_trans_anal_attrib_fk6_key6 
   (a_ledger_number_i,a_batch_number_i),
KEY inx_a_trans_anal_attrib_fk7_key7 
   (a_ledger_number_i,a_batch_number_i,a_journal_number_i),
KEY inx_a_trans_anal_attrib_fkcr_key8 
   (s_created_by_c),
KEY inx_a_trans_anal_attrib_fkmd_key9 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists the suspense accounts of each ledger.
-- GROUP: account
CREATE TABLE a_suspense_account (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This identifies an account which will be treated as a suspense account.
  a_suspense_account_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_suspense_account_pk
    PRIMARY KEY (a_ledger_number_i,a_suspense_account_code_c),
  CONSTRAINT a_suspence_account_fk1
    FOREIGN KEY (a_ledger_number_i,a_suspense_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_suspense_account_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_suspense_account_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_suspense_account_pk0 
   (a_ledger_number_i,a_suspense_account_code_c),
KEY inx_a_suspense_account_fkcr_key1 
   (s_created_by_c),
KEY inx_a_suspense_account_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table defines the concept of a supplier in the AP system and is the centre of the AP system.
-- GROUP: ap
CREATE TABLE a_ap_supplier (

    -- Reference to the partner key for this supplier
  p_partner_key_n bigint NOT NULL,
    -- Number of months to display invoices and credit notes
  a_preferred_screen_display_i integer,
    -- Reference to default bank account to use to pay supplier with.
  a_default_bank_account_c varchar(16),
    -- The default type of payment to use when paying this supplier.
  a_payment_type_c varchar(24),
    -- The currency code to use for this supplier.
  a_currency_code_c varchar(16) NOT NULL,
    -- The default AP Account to use when paying this supplier.
  a_default_ap_account_c varchar(16),
    -- Default credit terms to use for invoices from this supplier.
  a_default_credit_terms_i integer,
    -- Default percentage discount to receive for early payments.
  a_default_discount_percentage_n numeric(24, 10),
    -- Default number of days in which the discount percentage has effect.
  a_default_discount_days_i integer,
    -- What type of supplier this is - normal, credit card, maybe something else.
  a_supplier_type_c varchar(24),
    -- Reference to the default expense Account to use for invoice details.
  a_default_exp_account_c varchar(16),
    -- Reference to the default cost centre to use for invoice details.
  a_default_cost_centre_c varchar(16),
    -- This reference identifies us to the supplier
  a_our_reference_c varchar(100),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_supplier_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT a_ap_supplier_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_ap_supplier_fk2
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ap_supplier_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_supplier_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_supplier_pk0 
   (p_partner_key_n),
KEY inx_a_ap_supplier_fk2_key1 
   (a_currency_code_c),
KEY inx_a_ap_supplier_fkcr_key2 
   (s_created_by_c),
KEY inx_a_ap_supplier_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Either an invoice or a credit note in the Accounts Payable system.
-- GROUP: ap
CREATE TABLE a_ap_document (

    -- Unique key for this record
  a_ap_document_id_i integer DEFAULT 0 NOT NULL,
    -- Reference to the ledger for this invoice.
  a_ledger_number_i integer NOT NULL,
    -- A unique key (together with the ledger number) to identify this document.
  a_ap_number_i integer NOT NULL,
    -- Reference to the supplier that sent this invoice.
  p_partner_key_n bigint NOT NULL,
    -- A flag to indicate if this document is an invoice or a credit note.
  a_credit_note_flag_l boolean DEFAULT 0 NOT NULL,
    -- The code given on the document itself (be it invoice or credit note). This will have to be unique for each supplier.
  a_document_code_c varchar(30),
    -- Some kind of other reference needed.
  a_reference_c varchar(100),
    -- The date when this document was issued.
  a_date_issued_d date NOT NULL,
    -- The date when this document was entered into the system.
  a_date_entered_d date NOT NULL,
    -- Credit terms allowed for this invoice.
  a_credit_terms_i integer DEFAULT 0,
    -- The total amount of money that this document is worth.
  a_total_amount_n numeric(24, 10) DEFAULT 0 NOT NULL,
    -- the currency of the document
  a_currency_code_c varchar(16) NOT NULL,
    -- The exchange rate to the base currency at the time that the document was issued.
  a_exchange_rate_to_base_n numeric(24, 10),
    -- The percentage discount you get for early payment of this document in the case that it is an invoice.
  a_discount_percentage_n numeric(24, 10),
    -- The number of days that the discount is valid for (0 for none).
  a_discount_days_i integer,
    -- Reference to the AP Account to debit/credit when posting/paying the document.
  a_ap_account_c varchar(16),
    -- The number of the last item for this document. This is used simply to quickly get the next number if items are added.
  a_last_detail_number_i integer DEFAULT 0 NOT NULL,
    -- The current status of the invoice. The value can (for now) be one of: OPEN, APPROVED, POSTED, PARTPAID, or PAID.
  a_document_status_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_document_pk
    PRIMARY KEY (a_ap_document_id_i),
  CONSTRAINT a_ap_document_uk
    UNIQUE (a_ledger_number_i,a_ap_number_i),
  CONSTRAINT a_ap_document_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ap_document_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES a_ap_supplier(p_partner_key_n),
  CONSTRAINT a_ap_document_fk3
    FOREIGN KEY (a_ledger_number_i,a_ap_account_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ap_document_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_document_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_document_pk0 
   (a_ap_document_id_i),
UNIQUE KEY inx_a_ap_document_uk1 
   (a_ledger_number_i,a_ap_number_i),
KEY inx_a_ap_document_fk1_key2 
   (a_ledger_number_i),
KEY inx_a_ap_document_fk2_key3 
   (p_partner_key_n),
KEY inx_a_ap_document_fk3_key4 
   (a_ledger_number_i,a_ap_account_c),
KEY inx_a_ap_document_fkcr_key5 
   (s_created_by_c),
KEY inx_a_ap_document_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table receives a new entry when a credit note is applied to an invoice. Since the invoices and credit notes share the same table, we need a way to link the two, and this is the role of this table.
-- GROUP: ap
CREATE TABLE a_crdt_note_invoice_link (

    -- Reference to the ledger number.
  a_ledger_number_i integer NOT NULL,
    -- Reference to the credit note.
  a_credit_note_document_id_i integer NOT NULL,
    -- Reference to the invoice.
  a_invoice_document_id_i integer NOT NULL,
  a_applied_date_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_crdt_note_invoice_link_pk
    PRIMARY KEY (a_credit_note_document_id_i,a_invoice_document_id_i),
  CONSTRAINT a_crdt_note_invoice_link_fk1
    FOREIGN KEY (a_credit_note_document_id_i)
    REFERENCES a_ap_document(a_ap_document_id_i),
  CONSTRAINT a_crdt_note_invoice_link_fk2
    FOREIGN KEY (a_invoice_document_id_i)
    REFERENCES a_ap_document(a_ap_document_id_i),
  CONSTRAINT a_crdt_note_invoice_link_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_crdt_note_invoice_link_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_crdt_note_invoice_link_pk0 
   (a_credit_note_document_id_i,a_invoice_document_id_i),
KEY inx_dt_note_invoice_link_fk1_key1 
   (a_credit_note_document_id_i),
KEY inx_dt_note_invoice_link_fk2_key2 
   (a_invoice_document_id_i),
KEY inx_t_note_invoice_link_fkcr_key3 
   (s_created_by_c),
KEY inx_t_note_invoice_link_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- An invoice or credit note consists out of several items, or details. This table contains all these details.
-- GROUP: ap
CREATE TABLE a_ap_document_detail (

    -- Reference to the ledger
  a_ledger_number_i integer NOT NULL,
    -- Reference to the document
  a_ap_document_id_i integer NOT NULL,
    -- A unique number for this detail for its document.
  a_detail_number_i integer NOT NULL,
    -- Indicates if this detail has been approved or not.
  a_detail_approved_l boolean DEFAULT 0 NOT NULL,
    -- Reference to the cost centre to use for this detail.
  a_cost_centre_code_c varchar(24),
    -- Reference to the account to use for this detail
  a_account_code_c varchar(16),
    -- Some other reference to the item.
  a_item_ref_c varchar(100),
    -- A narrative about what this is.
  a_narrative_c varchar(200),
    -- The amount of money this detail is worth.
  a_amount_n numeric(24, 10),
    -- The date when this detail was approved.
  a_approval_date_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_document_detail_pk
    PRIMARY KEY (a_ap_document_id_i,a_detail_number_i),
  CONSTRAINT a_ap_document_detail_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ap_document_detail_fk2
    FOREIGN KEY (a_ap_document_id_i)
    REFERENCES a_ap_document(a_ap_document_id_i),
  CONSTRAINT a_ap_document_detail_fk3
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT a_ap_document_detail_fk4
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ap_document_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_document_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_document_detail_pk0 
   (a_ap_document_id_i,a_detail_number_i),
KEY inx_a_ap_document_detail_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_ap_document_detail_fk2_key2 
   (a_ap_document_id_i),
KEY inx_a_ap_document_detail_fk3_key3 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_a_ap_document_detail_fk4_key4 
   (a_ledger_number_i,a_account_code_c),
KEY inx__ap_document_detail_fkcr_key5 
   (s_created_by_c),
KEY inx__ap_document_detail_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Records all payments that have been made against an accounts payable detail.
-- GROUP: ap
CREATE TABLE a_ap_payment (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Unique number to identify each payment batch.
  a_payment_number_i integer DEFAULT 0 NOT NULL,
    -- The amount of money that was paid
  a_amount_n numeric(24, 10),
    -- the currency of the document
  a_currency_code_c varchar(16) NOT NULL,
    -- The exchange rate to the base currency at the time of payment.
  a_exchange_rate_to_base_n numeric(24, 10),
    -- Date that the payment for an accounts payable was made.
  a_payment_date_d date,
    -- This is the system user id of the person who made the payment.
  s_user_id_c varchar(20),
    -- Method that was used to make the payment - cheque, cash, ep, credit card, etc.
  a_method_of_payment_c varchar(20),
    -- The source or reference for the accounts payable payment.  This could be a cheque number.
  a_reference_c varchar(100),
    -- Bank account from which to make the payment
  a_bank_account_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_payment_pk
    PRIMARY KEY (a_ledger_number_i,a_payment_number_i),
  CONSTRAINT a_ap_payment_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ap_payment_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_payment_fk3
    FOREIGN KEY (a_ledger_number_i,a_bank_account_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ap_payment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_payment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_payment_pk0 
   (a_ledger_number_i,a_payment_number_i),
KEY inx_a_ap_payment_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_ap_payment_fk2_key2 
   (s_user_id_c),
KEY inx_a_ap_payment_fk3_key3 
   (a_ledger_number_i,a_bank_account_c),
KEY inx_a_ap_payment_fkcr_key4 
   (s_created_by_c),
KEY inx_a_ap_payment_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table links the different payments to actual invoices and credit notes.
-- GROUP: ap
CREATE TABLE a_ap_document_payment (

    -- The ledger that the attribute value is associated with.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- AP document ref
  a_ap_document_id_i integer DEFAULT 0 NOT NULL,
    -- Unique number to identify each payment batch.
  a_payment_number_i integer DEFAULT 0 NOT NULL,
    -- The amount of money that was paid
  a_amount_n numeric(24, 10),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_document_payment_pk
    PRIMARY KEY (a_ap_document_id_i,a_payment_number_i),
  CONSTRAINT a_ap_document_payment_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ap_document_payment_fk2
    FOREIGN KEY (a_ap_document_id_i)
    REFERENCES a_ap_document(a_ap_document_id_i),
  CONSTRAINT a_ap_document_payment_fk3
    FOREIGN KEY (a_ledger_number_i,a_payment_number_i)
    REFERENCES a_ap_payment(a_ledger_number_i,a_payment_number_i),
  CONSTRAINT a_ap_document_payment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_document_payment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_document_payment_pk0 
   (a_ap_document_id_i,a_payment_number_i),
KEY inx__ap_document_payment_fk1_key1 
   (a_ledger_number_i),
KEY inx__ap_document_payment_fk2_key2 
   (a_ap_document_id_i),
KEY inx__ap_document_payment_fk3_key3 
   (a_ledger_number_i,a_payment_number_i),
KEY inx_ap_document_payment_fkcr_key4 
   (s_created_by_c),
KEY inx_ap_document_payment_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table acts as a queue for electronic payments. If an invoice is paid electronically, the payment is added to this table. A EP program will go through this table paying all entries to GL and moving them to the a_ap_payment table.
-- GROUP: ap
CREATE TABLE a_ep_payment (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Unique number to identify each payment batch.
  a_payment_number_i integer DEFAULT 0 NOT NULL,
    -- The amount of money that was paid
  a_amount_n numeric(24, 10),
    -- This is the system user id of the person who made the payment.
  s_user_id_c varchar(20),
    -- The source or reference for the accounts payable payment.  This could be a cheque number.
  a_reference_c varchar(100),
    -- Bank account from which to make the payment
  a_bank_account_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_payment_pk
    PRIMARY KEY (a_ledger_number_i,a_payment_number_i),
  CONSTRAINT a_ep_payment_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ep_payment_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_payment_fk3
    FOREIGN KEY (a_ledger_number_i,a_bank_account_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ep_payment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_payment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_payment_pk0 
   (a_ledger_number_i,a_payment_number_i),
KEY inx_a_ep_payment_fk1_key1 
   (a_ledger_number_i),
KEY inx_a_ep_payment_fk2_key2 
   (s_user_id_c),
KEY inx_a_ep_payment_fk3_key3 
   (a_ledger_number_i,a_bank_account_c),
KEY inx_a_ep_payment_fkcr_key4 
   (s_created_by_c),
KEY inx_a_ep_payment_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table links the different EP payments to actual invoices and credit notes.
-- GROUP: ap
CREATE TABLE a_ep_document_payment (

    -- The ledger that the attribute value is associated with.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- AP Document ref
  a_ap_document_id_i integer DEFAULT 0 NOT NULL,
    -- Unique number to identify each payment batch.
  a_payment_number_i integer DEFAULT 0 NOT NULL,
    -- The amount of money that was paid
  a_amount_n numeric(24, 10),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ep_document_payment_pk
    PRIMARY KEY (a_ap_document_id_i),
  CONSTRAINT a_ep_document_payment_fk1
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT a_ep_document_payment_fk2
    FOREIGN KEY (a_ap_document_id_i)
    REFERENCES a_ap_document(a_ap_document_id_i),
  CONSTRAINT a_ep_document_payment_fk3
    FOREIGN KEY (a_ledger_number_i,a_payment_number_i)
    REFERENCES a_ep_payment(a_ledger_number_i,a_payment_number_i),
  CONSTRAINT a_ep_document_payment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ep_document_payment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ep_document_payment_pk0 
   (a_ap_document_id_i),
KEY inx__ep_document_payment_fk1_key1 
   (a_ledger_number_i),
KEY inx__ep_document_payment_fk3_key2 
   (a_ledger_number_i,a_payment_number_i),
KEY inx_ep_document_payment_fkcr_key3 
   (s_created_by_c),
KEY inx_ep_document_payment_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Analysis Attributes applied to an AP for posting to the GL.
-- GROUP: ap
CREATE TABLE a_ap_anal_attrib (

    -- The ledger that the attribute value is associated with.
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Accounts Payable Sequence Number
  a_ap_document_id_i integer DEFAULT 0 NOT NULL,
    -- The detail number within the invoice/ accounts payable.
  a_detail_number_i integer DEFAULT 0 NOT NULL,
  a_analysis_type_code_c varchar(16) NOT NULL,
    -- This identifies the account the financial transaction must be stored against
  a_account_code_c varchar(16) NOT NULL,
  a_analysis_attribute_value_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ap_anal_attrib_pk
    PRIMARY KEY (a_ap_document_id_i,a_detail_number_i,a_analysis_type_code_c),
  CONSTRAINT a_ap_anal_attrib_fk1
    FOREIGN KEY (a_ap_document_id_i,a_detail_number_i)
    REFERENCES a_ap_document_detail(a_ap_document_id_i,a_detail_number_i),
  CONSTRAINT a_ap_anal_attrib_fk2
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c)
    REFERENCES a_analysis_attribute(a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
  CONSTRAINT a_ap_anal_attrib_fk3
    FOREIGN KEY (a_ledger_number_i,a_account_code_c)
    REFERENCES a_account(a_ledger_number_i,a_account_code_c),
  CONSTRAINT a_ap_anal_attrib_fk4
    FOREIGN KEY (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c)
    REFERENCES a_freeform_analysis(a_ledger_number_i,a_analysis_type_code_c,a_analysis_value_c),
  CONSTRAINT a_ap_anal_attrib_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ap_anal_attrib_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ap_anal_attrib_pk0 
   (a_ap_document_id_i,a_detail_number_i,a_analysis_type_code_c),
KEY inx_a_ap_anal_attrib_fk1_key1 
   (a_ap_document_id_i,a_detail_number_i),
KEY inx_a_ap_anal_attrib_fk2_key2 
   (a_ledger_number_i,a_analysis_type_code_c,a_account_code_c),
KEY inx_a_ap_anal_attrib_fk3_key3 
   (a_ledger_number_i,a_account_code_c),
KEY inx_a_ap_anal_attrib_fk4_key4 
   (a_ledger_number_i,a_analysis_type_code_c,a_analysis_attribute_value_c),
KEY inx_a_ap_anal_attrib_fkcr_key5 
   (s_created_by_c),
KEY inx_a_ap_anal_attrib_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- the invoice (which is also an offer at a certain stage)
-- GROUP: ar
CREATE TABLE a_ar_invoice (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Key to uniquely identify invoice
  a_key_i integer NOT NULL,
    -- an invoice can have these states: OFFER, CHARGED, PARTIALLYPAID, PAID
  a_status_c varchar(32) NOT NULL,
    -- This is the partner who has to pay the bill; can be null for cash payments; could also be another field
  p_partner_key_n bigint,
    -- this is the date when the invoice was charged
  a_date_effective_d date,
    -- refers to the offer that was created for this invoice; it is basically an archived copy of the invoice, and the invoice might actually be different from the offer (e.g. hospitality: different number of people, etc.); table ph_booking always refers to the invoice, and the invoice refers to the offer; there is no requirement for an offer to exist, it can be null
  a_offer_i integer,
    -- this defines whether no tax is applied to this invoice (NONE), or if a SPECIAL tax is applied, or if the DEFAULT tax defined for each article; this should work around issues of selling to businesses or customers abroad
  a_taxing_c varchar(20) DEFAULT 'DEFAULT' NOT NULL,
    -- if a_taxing_c has the value SPECIAL, then this tax applies (defined by tax type code, tax rate code, and date valid from
  a_special_tax_type_code_c varchar(16),
    -- this describes whether it is e.g. the standard, reduced or zero rate of VAT
  a_special_tax_rate_code_c varchar(16),
    -- this describes when this particular percentage rate has become valid by law
  a_special_tax_valid_from_d date,
    -- The total amount of money that this invoice is worth; this includes all discounts, even the early payment discount; if the early payment discount does not apply anymore at the time of payment, this total amount needs to be updated
  a_total_amount_n numeric(24, 10),
    -- the currency of the total amount
  a_currency_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_invoice_pk
    PRIMARY KEY (a_ledger_number_i,a_key_i),
  CONSTRAINT a_ar_invoice_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT a_ar_invoice_fk2
    FOREIGN KEY (a_ledger_number_i,a_special_tax_type_code_c,a_special_tax_rate_code_c,a_special_tax_valid_from_d)
    REFERENCES a_tax_table(a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d),
  CONSTRAINT a_ar_invoice_fk3
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ar_invoice_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_invoice_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ar_invoice_pk0 
   (a_ledger_number_i,a_key_i),
KEY inx_a_ar_invoice_fk1_key1 
   (p_partner_key_n),
KEY inx_a_ar_invoice_fk2_key2 
   (a_ledger_number_i,a_special_tax_type_code_c,a_special_tax_rate_code_c,a_special_tax_valid_from_d),
KEY inx_a_ar_invoice_fk3_key3 
   (a_currency_code_c),
KEY inx_a_ar_invoice_fkcr_key4 
   (s_created_by_c),
KEY inx_a_ar_invoice_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- an invoice consists of one or more details
-- GROUP: ar
CREATE TABLE a_ar_invoice_detail (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- this invoice detail belongs to the invoice with this key
  a_invoice_key_i integer NOT NULL,
    -- A unique number for this detail for its invoice.
  a_detail_number_i integer NOT NULL,
    -- code that uniquely identifies the item; can also be a code of a group of equally priced items
  a_ar_article_code_c varchar(100) NOT NULL,
    -- Reference for this invoice detail; for a non specific article that could give more details (e.g. which book of type small book)
  a_ar_reference_c varchar(100),
    -- defines the number of the article items that is bought
  a_ar_number_of_item_i integer NOT NULL,
    -- Reference to the price that should be used for this article in this invoice, by date; without discounts, just for single item
  a_ar_article_price_d date NOT NULL,
    -- The total amount of money that this invoice detail is worth; includes the discounts
  a_calculated_amount_n numeric(24, 10),
    -- the currency of the total amount
  a_currency_code_c varchar(16) NOT NULL,
    -- The tax type is always the same, e.g. VAT
  a_tax_type_code_c varchar(16) NOT NULL,
    -- this describes whether it is e.g. the standard, reduced or zero rate of VAT
  a_tax_rate_code_c varchar(16) NOT NULL,
    -- we need to be able to pick a rate that was valid in the past or will be valid in the future
  a_tax_valid_from_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_invoice_detail_pk
    PRIMARY KEY (a_ledger_number_i,a_invoice_key_i,a_detail_number_i),
  CONSTRAINT a_ar_invoice_detail_fk1
    FOREIGN KEY (a_ledger_number_i,a_invoice_key_i)
    REFERENCES a_ar_invoice(a_ledger_number_i,a_key_i),
  CONSTRAINT a_ar_invoice_detail_fk2
    FOREIGN KEY (a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d)
    REFERENCES a_tax_table(a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d),
  CONSTRAINT a_ar_invoice_detail_fk3
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT a_ar_invoice_detail_fk4
    FOREIGN KEY (a_ar_article_code_c)
    REFERENCES a_ar_article(a_ar_article_code_c),
  CONSTRAINT a_ar_invoice_detail_fk5
    FOREIGN KEY (a_ar_article_code_c,a_ar_article_price_d)
    REFERENCES a_ar_article_price(a_ar_article_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_invoice_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_invoice_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ar_invoice_detail_pk0 
   (a_ledger_number_i,a_invoice_key_i,a_detail_number_i),
KEY inx_a_ar_invoice_detail_fk1_key1 
   (a_ledger_number_i,a_invoice_key_i),
KEY inx_a_ar_invoice_detail_fk2_key2 
   (a_ledger_number_i,a_tax_type_code_c,a_tax_rate_code_c,a_tax_valid_from_d),
KEY inx_a_ar_invoice_detail_fk3_key3 
   (a_currency_code_c),
KEY inx_a_ar_invoice_detail_fk4_key4 
   (a_ar_article_code_c),
KEY inx_a_ar_invoice_detail_fk5_key5 
   (a_ar_article_code_c,a_ar_article_price_d),
KEY inx_a_ar_invoice_detail_fkcr_key6 
   (s_created_by_c),
KEY inx_a_ar_invoice_detail_fkmd_key7 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines which discounts apply directly to the invoice rather than the invoice items; this can depend on the customer etc
-- GROUP: ar
CREATE TABLE a_ar_invoice_discount (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Key to uniquely identify invoice
  a_invoice_key_i integer NOT NULL,
    -- code that identifies the discount
  a_ar_discount_code_c varchar(100) NOT NULL,
    -- this clearly specifies which version of the discount is meant
  a_ar_discount_date_valid_from_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_invoice_discount_pk
    PRIMARY KEY (a_ledger_number_i,a_invoice_key_i,a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
  CONSTRAINT a_ar_invoice_discount_fk1
    FOREIGN KEY (a_ledger_number_i,a_invoice_key_i)
    REFERENCES a_ar_invoice(a_ledger_number_i,a_key_i),
  CONSTRAINT a_ar_invoice_discount_fk2
    FOREIGN KEY (a_ar_discount_code_c,a_ar_discount_date_valid_from_d)
    REFERENCES a_ar_discount(a_ar_discount_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_invoice_discount_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_invoice_discount_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_ar_invoice_discount_pk0 
   (a_ledger_number_i,a_invoice_key_i,a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
KEY inx__ar_invoice_discount_fk1_key1 
   (a_ledger_number_i,a_invoice_key_i),
KEY inx__ar_invoice_discount_fk2_key2 
   (a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
KEY inx_ar_invoice_discount_fkcr_key3 
   (s_created_by_c),
KEY inx_ar_invoice_discount_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- defines which discounts apply one invoice item
-- GROUP: ar
CREATE TABLE a_ar_invoice_detail_discount (

    -- This is used as a key field in most of the accounting system files
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Key to uniquely identify invoice
  a_invoice_key_i integer NOT NULL,
    -- A unique number for this detail for its invoice.
  a_detail_number_i integer NOT NULL,
    -- code that identifies the discount
  a_ar_discount_code_c varchar(100) NOT NULL,
    -- this clearly specifies which version of the discount is meant
  a_ar_discount_date_valid_from_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT a_ar_invoice_detail_discount_pk
    PRIMARY KEY (a_ledger_number_i,a_invoice_key_i,a_detail_number_i,a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
  CONSTRAINT a_ar_invoice_detail_discount_fk1
    FOREIGN KEY (a_ledger_number_i,a_invoice_key_i,a_detail_number_i)
    REFERENCES a_ar_invoice_detail(a_ledger_number_i,a_invoice_key_i,a_detail_number_i),
  CONSTRAINT a_ar_invoice_detail_discount_fk2
    FOREIGN KEY (a_ar_discount_code_c,a_ar_discount_date_valid_from_d)
    REFERENCES a_ar_discount(a_ar_discount_code_c,a_ar_date_valid_from_d),
  CONSTRAINT a_ar_invoice_detail_discount_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT a_ar_invoice_detail_discount_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_r_invoice_detail_discount_pk0 
   (a_ledger_number_i,a_invoice_key_i,a_detail_number_i,a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
KEY inx_oice_detail_discount_fk1_key1 
   (a_ledger_number_i,a_invoice_key_i,a_detail_number_i),
KEY inx_oice_detail_discount_fk2_key2 
   (a_ar_discount_code_c,a_ar_discount_date_valid_from_d),
KEY inx_ice_detail_discount_fkcr_key3 
   (s_created_by_c),
KEY inx_ice_detail_discount_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The first part of any application which contains data common to either short term or long term applications. 
-- GROUP: personnel
CREATE TABLE pm_general_application (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer DEFAULT 0 NOT NULL,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint DEFAULT 0 NOT NULL,
    -- Date of application.
  pm_gen_app_date_d date NOT NULL,
    -- Describes what the application is for, eg. conference, year program.
  pt_app_type_name_c varchar(32) NOT NULL,
    -- TODO: this field is a combination of registration office and application number. might not be needed???
  pm_old_link_c varchar(32) NOT NULL,
    -- This is the possible field or team of service.
  pm_gen_app_poss_srv_unit_key_n bigint,
    -- This field will not appear on the screen but will be updated when someone chooses to delete a record. Rather that an actual deletion, the record will be 'marked' for deletion after an agreed upon interval.
  pm_gen_app_delete_flag_l boolean DEFAULT 0,
    -- Describes the applicant, eg. volunteer, staff, speaker.
  pm_gen_applicant_type_c varchar(30) NOT NULL,
    -- Indicates the status of the application.
  pm_gen_application_status_c varchar(32),
    -- Indicates if the application is closed.
  pm_closed_l boolean DEFAULT 0,
    -- This identifies the user that closed the application
  pm_closed_by_c varchar(20),
    -- This is the date the application was closed.
  pm_date_closed_d date,
    -- Indicates if the application is on hold.
  pm_gen_application_on_hold_l boolean DEFAULT 0,
    -- Comment on why the application is on hold.
  pm_gen_application_hold_reason_c varchar(100),
    -- Indicates if the application process has been cancelled.
  pm_gen_cancelled_app_l boolean DEFAULT 0,
    -- Comment on why the application is on hold.
  pm_gen_app_cancel_reason_c varchar(54),
    -- This is the date the application was cancelled.
  pm_gen_app_cancelled_d date,
    -- Describes if the applicant has been accepted by the serving field.
  pm_gen_app_srv_fld_accept_l boolean DEFAULT 0,
    -- This is the date the receiving field accepted the applicant.
  pm_gen_app_recvg_fld_accept_d date,
    -- Describes if the applicant has been accepted by the sending field.
  pm_gen_app_send_fld_accept_l boolean DEFAULT 0,
    -- This is the date the sending field accepted the applicant.
  pm_gen_app_send_fld_accept_d date,
    -- This field indicates how they were influenced to apply with us.
  pm_gen_contact1_c varchar(40),
    -- This field indicates how they were influenced to apply with us.
  pm_gen_contact2_c varchar(40),
    -- Indicates the date the record was last updated.
  pm_gen_app_update_d date,
    -- Sometimes there are extra comments of preferences that are related to an application.  These can be entered here.
  pm_comment_c varchar(2000),
    -- This is the currency that is used for amounts listed in this application
  pm_gen_app_currency_code_c varchar(16),
    -- This is the placement person handling this application.
  pm_placement_partner_key_n bigint,
    -- stores the plain data received from the browser in JSON format
  pm_raw_application_data_c longtext,
    -- the person partner key of the applicant in the local database of the registration office. no foreign key since the person partner is in another database
  p_local_partner_key_n bigint,
    -- true if the applicant has already been imported into the local database of the registration office.
  p_imported_local_petra_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_general_application_pk
    PRIMARY KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_general_application_nk
    UNIQUE (p_partner_key_n,pm_gen_app_date_d,pt_app_type_name_c,pm_old_link_c),
  CONSTRAINT pm_general_application_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_general_application_fk2
    FOREIGN KEY (pm_registration_office_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_general_application_fk3
    FOREIGN KEY (pm_gen_app_poss_srv_unit_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_general_application_fk4
    FOREIGN KEY (pm_gen_application_status_c)
    REFERENCES pt_applicant_status(pt_code_c),
  CONSTRAINT pm_general_application_fk5
    FOREIGN KEY (pt_app_type_name_c)
    REFERENCES pt_application_type(pt_app_type_name_c),
  CONSTRAINT pm_general_application_fk6
    FOREIGN KEY (pm_gen_contact1_c)
    REFERENCES pt_contact(pt_contact_name_c),
  CONSTRAINT pm_general_application_fk7
    FOREIGN KEY (pm_gen_contact2_c)
    REFERENCES pt_contact(pt_contact_name_c),
  CONSTRAINT pm_general_application_fk8
    FOREIGN KEY (pm_gen_app_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT pm_general_application_fk9
    FOREIGN KEY (pm_placement_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_general_application_fk10
    FOREIGN KEY (pm_closed_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_general_application_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_general_application_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_general_application_pk0 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
KEY inx__general_application_fk1_key1 
   (p_partner_key_n),
KEY inx__general_application_fk2_key2 
   (pm_registration_office_n),
KEY inx__general_application_fk3_key3 
   (pm_gen_app_poss_srv_unit_key_n),
KEY inx__general_application_fk4_key4 
   (pm_gen_application_status_c),
KEY inx__general_application_fk5_key5 
   (pt_app_type_name_c),
KEY inx__general_application_fk6_key6 
   (pm_gen_contact1_c),
KEY inx__general_application_fk7_key7 
   (pm_gen_contact2_c),
KEY inx__general_application_fk8_key8 
   (pm_gen_app_currency_code_c),
KEY inx__general_application_fk9_key9 
   (pm_placement_partner_key_n),
KEY inx_general_application_fk10_key10 
   (pm_closed_by_c),
KEY inx_general_application_fkcr_key11 
   (s_created_by_c),
KEY inx_general_application_fkmd_key12 
   (s_modified_by_c),
UNIQUE KEY inx_pm_general_application_nk0 
   (p_partner_key_n,pm_gen_app_date_d,pt_app_type_name_c,pm_old_link_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Keeps a history of the application status for short and long term applications.
-- GROUP: personnel
CREATE TABLE pm_application_status_history (

    -- Key to make editing these records easier
  pm_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer DEFAULT 0,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint NOT NULL,
    -- Indicates the status of the application.
  pm_status_code_c varchar(32),
    -- Effective Date of the chosen Application Status
  pm_status_date_effective_d date,
    -- Gives further comments about application status.
  pm_comment_c varchar(500),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_application_status_history_pk
    PRIMARY KEY (pm_key_n),
  CONSTRAINT pm_application_status_history_fk1
    FOREIGN KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n)
    REFERENCES pm_general_application(p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_application_status_history_fk2
    FOREIGN KEY (pm_status_code_c)
    REFERENCES pt_applicant_status(pt_code_c),
  CONSTRAINT pm_application_status_history_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_application_status_history_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pplication_status_history_pk0 
   (pm_key_n),
KEY inx_ation_status_history_fk1_key1 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
KEY inx_ation_status_history_fk2_key2 
   (pm_status_code_c),
KEY inx_tion_status_history_fkcr_key3 
   (s_created_by_c),
KEY inx_tion_status_history_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Table for short term applications.
-- GROUP: personnel
CREATE TABLE pm_short_term_application (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer DEFAULT 0 NOT NULL,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint DEFAULT 0 NOT NULL,
    -- Date of application
  pm_st_app_date_d date NOT NULL,
  pm_st_application_type_c varchar(32) NOT NULL,
    -- TODO: this field is a combination of registration office and application number. might not be needed???
  pm_st_basic_outreach_id_c varchar(32) NOT NULL,
    -- This field will not appear on the screen but will be updated when someone chooses to delete a record. Rather that an actual deletion, the record will be 'marked' for deletion after an agreed upon interval.
  pm_st_basic_delete_flag_l boolean DEFAULT 0,
    -- Indicates if the application is on hold.
  pm_st_application_on_hold_l boolean DEFAULT 0,
    -- Comment on why the application is on hold.
  pm_st_application_hold_reason_c varchar(40),
    -- Indicates the outreach code of the confirmed option.
  pm_confirmed_option_code_c varchar(26),
    -- Indicates if for outreach only.
  pm_st_outreach_only_flag_l boolean DEFAULT 0,
    -- Indicates the confirmed outreach option.
  pm_st_confirmed_option_n bigint,
    -- Indicates the current field.
  pm_st_current_field_n bigint,
    -- Indicates the status of given arrival details (not known, being planned,...).
  pm_arrival_details_status_c varchar(24) DEFAULT 'no',
    -- This code indicates the arrival point of the congress attendee.
  pt_arrival_point_code_c varchar(32),
    -- This code indicates the type of travel to the congress..
  pt_travel_type_to_cong_code_c varchar(32),
    -- Date of arrival at the conference.
  pm_arrival_d date,
    -- The hour of arrival.
  pm_arrival_hour_i integer DEFAULT 00,
    -- The minutes of arrival.
  pm_arrival_minute_i integer DEFAULT 00,
    -- Information concerning flight or bus numbers. 
  pm_to_cong_travel_info_c varchar(32),
    -- Indicates if transport from arrival point to congress needs to be arranged by Registrar.
  pm_arrival_transport_needed_l boolean DEFAULT 0,
    -- Gives further comments on arrival information.
  pm_arrival_comments_c varchar(160),
    -- Indicates the status of given departure details (undetermined, being planned,...).
  pm_departure_details_status_c varchar(24) DEFAULT 'no',
    -- This code indicates the departure point of the congress attendee.
  pt_departure_point_code_c varchar(32),
    -- This code indicates the type of travel from the congress.
  pt_travel_type_from_cong_code_c varchar(32),
    -- Date of departure from the conference.
  pm_departure_d date,
    -- The hour of departure.
  pm_departure_hour_i integer DEFAULT 00,
    -- The minutes of departure.
  pm_departure_minute_i integer DEFAULT 00,
    -- Information concerning flight or bus numbers. 
  pm_from_cong_travel_info_c varchar(32),
    -- Indicates if transport from congress to departure point needs to be arranged by Registrar.
  pm_departure_transport_needed_l boolean DEFAULT 0,
    -- Gives further comments on departure information.
  pm_departure_comments_c varchar(160),
    -- Applicant is interested if there would be a transport possibility from/to arrival/departure point.
  pm_transport_interest_l boolean DEFAULT 0,
    -- This code indicates what role they have during pre-congress.
  pm_st_pre_congress_code_c varchar(32),
    -- Indicates the role for the Congress.
  pm_st_congress_code_c varchar(32),
    -- Indicates any special information about the applicant.
  pm_st_special_applicant_c varchar(32),
    -- The role a participant has during a outreach
  pm_outreach_role_c varchar(32),
    -- Indicates if the person is a leader of a fellowship group.
  pm_st_fg_leader_l boolean DEFAULT 0,
    -- A free form field for group codes. 
  pm_st_fg_code_c varchar(32),
    -- Special Costs related to the outreach.
  pm_st_outreach_special_cost_i integer DEFAULT 0,
    -- Special Costs related to the congress.
  pm_st_cngrss_special_cost_i integer DEFAULT 0,
    -- This field idenifies the field to be charged for the costs.
  pm_st_field_charged_n bigint,
    -- This is the language spoken by the applicant at the Congress.
  pm_st_congress_language_c varchar(20),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_short_term_application_pk
    PRIMARY KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_short_term_application_nk
    UNIQUE (p_partner_key_n,pm_st_app_date_d,pm_st_application_type_c,pm_st_basic_outreach_id_c),
  CONSTRAINT pm_short_term_application_fk1
    FOREIGN KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n)
    REFERENCES pm_general_application(p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_short_term_application_fk2
    FOREIGN KEY (pm_st_special_applicant_c)
    REFERENCES pt_special_applicant(pt_code_c),
  CONSTRAINT pm_short_term_application_fk3
    FOREIGN KEY (pt_arrival_point_code_c)
    REFERENCES pt_arrival_point(pt_code_c),
  CONSTRAINT pm_short_term_application_fk4
    FOREIGN KEY (pt_departure_point_code_c)
    REFERENCES pt_arrival_point(pt_code_c),
  CONSTRAINT pm_short_term_application_fk5
    FOREIGN KEY (pt_travel_type_to_cong_code_c)
    REFERENCES pt_travel_type(pt_code_c),
  CONSTRAINT pm_short_term_application_fk6
    FOREIGN KEY (pt_travel_type_from_cong_code_c)
    REFERENCES pt_travel_type(pt_code_c),
  CONSTRAINT pm_short_term_application_fk7
    FOREIGN KEY (pm_st_pre_congress_code_c)
    REFERENCES pt_congress_code(pt_code_c),
  CONSTRAINT pm_short_term_application_fk8
    FOREIGN KEY (pm_st_congress_code_c)
    REFERENCES pt_congress_code(pt_code_c),
  CONSTRAINT pm_short_term_application_fk9
    FOREIGN KEY (pm_outreach_role_c)
    REFERENCES pt_congress_code(pt_code_c),
  CONSTRAINT pm_short_term_application_fk10
    FOREIGN KEY (pm_st_congress_language_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT pm_short_term_application_fk11
    FOREIGN KEY (pm_st_confirmed_option_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_short_term_application_fk12
    FOREIGN KEY (pm_st_current_field_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_short_term_application_fk13
    FOREIGN KEY (pm_st_field_charged_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_short_term_application_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_short_term_application_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_short_term_application_pk0 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
UNIQUE KEY inx_pm_short_term_application_nk1 
   (p_partner_key_n,pm_st_app_date_d,pm_st_application_type_c,pm_st_basic_outreach_id_c),
KEY inx_ort_term_application_fk2_key2 
   (pm_st_special_applicant_c),
KEY inx_ort_term_application_fk3_key3 
   (pt_arrival_point_code_c),
KEY inx_ort_term_application_fk4_key4 
   (pt_departure_point_code_c),
KEY inx_ort_term_application_fk5_key5 
   (pt_travel_type_to_cong_code_c),
KEY inx_ort_term_application_fk6_key6 
   (pt_travel_type_from_cong_code_c),
KEY inx_ort_term_application_fk7_key7 
   (pm_st_pre_congress_code_c),
KEY inx_ort_term_application_fk8_key8 
   (pm_st_congress_code_c),
KEY inx_ort_term_application_fk9_key9 
   (pm_outreach_role_c),
KEY inx_rt_term_application_fk10_key10 
   (pm_st_congress_language_c),
KEY inx_rt_term_application_fk11_key11 
   (pm_st_confirmed_option_n),
KEY inx_rt_term_application_fk12_key12 
   (pm_st_current_field_n),
KEY inx_rt_term_application_fk13_key13 
   (pm_st_field_charged_n),
KEY inx_rt_term_application_fkcr_key14 
   (s_created_by_c),
KEY inx_rt_term_application_fkmd_key15 
   (s_modified_by_c),
KEY inx_pm_short_term_application_nk0 
   (p_partner_key_n,pm_st_app_date_d,pm_st_application_type_c,pm_st_basic_outreach_id_c),
KEY pm_registration_office_k1 
   (pm_registration_office_n),
KEY partner_key_k2 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains information pertaining to the application process.
-- GROUP: personnel
CREATE TABLE pm_year_program_application (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer DEFAULT 0 NOT NULL,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint DEFAULT 0 NOT NULL,
    -- Date of application
  pm_yp_app_date_d date NOT NULL,
    -- Describes what the application is for, eg. conference, year program.
  pm_yp_basic_app_type_c varchar(32) NOT NULL,
    -- This field will not appear on the screen but will be updated when someone chooses to delete a record. Rather that an actual deletion, the record will be 'marked' for deletion after an agreed upon interval.
  pm_yp_basic_delete_flag_l boolean DEFAULT 0,
    -- Indicates booking at an orientation session.
  pm_ho_orient_conf_booking_key_c varchar(16),
    -- The agreed support figure.
  pm_yp_agreed_support_figure_n numeric(24, 10) DEFAULT 0,
    -- Indicates the agreed upon joining charge for the conference and / or the summer outreach.
  pm_yp_agreed_joining_charge_n numeric(24, 10) DEFAULT 0,
    -- Period of time the support covers..
  pm_yp_support_period_c varchar(24),
    -- Indicates which New Recruits Conference the applicant plans to attend.
  pm_yp_joining_conf_i integer DEFAULT 1,
    -- This is the expected date their commitment starts.
  pm_start_of_commitment_d date,
    -- This is the expected date their commitment ends.
  pm_end_of_commitment_d date,
    -- Indicates how long the applicant intends to stay with us
  pm_intended_com_length_months_i integer,
    -- Name of the position.
  pt_position_name_c varchar(60),
    -- Scope of this position.
  pt_position_scope_c varchar(24),
    -- Indicates if the assignment is in assistance to the given position.
  pt_assistant_to_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_year_program_application_pk
    PRIMARY KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_year_program_application_fk1
    FOREIGN KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n)
    REFERENCES pm_general_application(p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_year_program_application_fk2
    FOREIGN KEY (pt_position_name_c,pt_position_scope_c)
    REFERENCES pt_position(pt_position_name_c,pt_position_scope_c),
  CONSTRAINT pm_year_program_application_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_year_program_application_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__year_program_application_pk0 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
KEY inx__program_application_fk2_key1 
   (pt_position_name_c,pt_position_scope_c),
KEY inx_program_application_fkcr_key2 
   (s_created_by_c),
KEY inx_program_application_fkmd_key3 
   (s_modified_by_c),
KEY ypa_partner_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Document
-- GROUP: personnel
CREATE TABLE pm_document (

    -- This is the key that tell what site created this document
  p_site_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to identify the document
  pm_document_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This code indicates the type of document for a person.
  pm_doc_code_c varchar(40) NOT NULL,
    -- Document ID
  pm_document_id_c varchar(60),
    -- Place the document was issued.
  pm_place_of_issue_c varchar(80),
    -- The date the document was issued.
  pm_date_of_issue_d date,
    -- Date the document takes effect.
  pm_date_of_start_d date,
    -- Date the document expires
  pm_date_of_expiration_d date,
    -- Comments and details
  pm_doc_comment_c varchar(1000),
    -- ID of associated document
  pm_assoc_doc_id_c varchar(40),
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pm_contact_partner_key_n bigint DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_document_pk
    PRIMARY KEY (p_site_key_n,pm_document_key_n),
  CONSTRAINT pm_document_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_document_fk2
    FOREIGN KEY (pm_doc_code_c)
    REFERENCES pm_document_type(pm_doc_code_c),
  CONSTRAINT pm_document_fk3
    FOREIGN KEY (pm_contact_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT pm_document_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_document_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_document_pk0 
   (p_site_key_n,pm_document_key_n),
KEY inx_pm_document_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_document_fk2_key2 
   (pm_doc_code_c),
KEY inx_pm_document_fk3_key3 
   (pm_contact_partner_key_n),
KEY inx_pm_document_fkcr_key4 
   (s_created_by_c),
KEY inx_pm_document_fkmd_key5 
   (s_modified_by_c),
KEY pm_document_k0 
   (p_partner_key_n,pm_doc_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Passport Details
-- GROUP: personnel
CREATE TABLE pm_passport_details (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Passport Number
  pm_passport_number_c varchar(40) NOT NULL,
    -- Is this the main passport?
  pm_main_passport_l boolean DEFAULT 0,
    -- Indicates whether the passport is active or not.
  pm_active_flag_c varchar(6),
    -- Full passport name.
  pm_full_passport_name_c varchar(80),
    -- Date of Birth
  pm_passport_dob_d date,
    -- Location of birth, eg. Cleveland, Ohio, USA, or Oswestry,Shropshire, England.
  pm_place_of_birth_c varchar(60),
    -- Nationality of the passport holder.
  p_passport_nationality_code_c varchar(8),
    -- Date the passport expires
  pm_date_of_expiration_d date,
    -- Place the passport was issued.
  pm_place_of_issue_c varchar(40),
    -- Country the passport was issued.
  p_country_of_issue_c varchar(8),
    -- The date the passport was issued.
  pm_date_of_issue_d date,
    -- Details the type of passport, e.g. Residence, Diplomatic, etc. 
  pm_passport_details_type_c varchar(32),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_passport_details_pk
    PRIMARY KEY (p_partner_key_n,pm_passport_number_c),
  CONSTRAINT pm_passport_details_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_passport_details_fk2
    FOREIGN KEY (pm_passport_details_type_c)
    REFERENCES pt_passport_type(pt_code_c),
  CONSTRAINT pm_passport_details_fk3
    FOREIGN KEY (p_passport_nationality_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT pm_passport_details_fk4
    FOREIGN KEY (p_country_of_issue_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT pm_passport_details_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_passport_details_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_passport_details_pk0 
   (p_partner_key_n,pm_passport_number_c),
KEY inx_pm_passport_details_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_passport_details_fk2_key2 
   (pm_passport_details_type_c),
KEY inx_pm_passport_details_fk3_key3 
   (p_passport_nationality_code_c),
KEY inx_pm_passport_details_fk4_key4 
   (p_country_of_issue_c),
KEY inx_pm_passport_details_fkcr_key5 
   (s_created_by_c),
KEY inx_pm_passport_details_fkmd_key6 
   (s_modified_by_c),
UNIQUE KEY pm_passport_k0 
   (p_partner_key_n,pm_full_passport_name_c,pm_passport_number_c,p_passport_nationality_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains detail about the language the person speaks.
-- GROUP: personnel
CREATE TABLE pm_person_language (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the language(s) spoken.
  p_language_code_c varchar(20) NOT NULL,
    -- Years of experience this person has spoken this language.
  pm_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- The date the years of experience were up to date.
  pm_years_of_experience_as_of_d date,
    -- This field is a numeric representation of level of language.
  pt_language_level_i integer DEFAULT 99 NOT NULL,
  pm_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_language_pk
    PRIMARY KEY (p_partner_key_n,p_language_code_c),
  CONSTRAINT pm_person_language_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_language_fk2
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT pm_person_language_fk3
    FOREIGN KEY (pt_language_level_i)
    REFERENCES pt_language_level(pt_language_level_i),
  CONSTRAINT pm_person_language_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_language_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_language_pk0 
   (p_partner_key_n,p_language_code_c),
KEY inx_pm_person_language_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_person_language_fk2_key2 
   (p_language_code_c),
KEY inx_pm_person_language_fk3_key3 
   (pt_language_level_i),
KEY inx_pm_person_language_fkcr_key4 
   (s_created_by_c),
KEY inx_pm_person_language_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This details any previous ministry experience the individual has. 
-- GROUP: personnel
CREATE TABLE pm_past_experience (

    -- The connection to the site
  p_site_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to make editing these records easier
  pm_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Start date of previous experience.
  pm_start_date_d date,
    -- End date of previous experience.
  pm_end_date_d date,
    -- Location of previous work . 
  pm_prev_location_c varchar(60) NOT NULL,
    -- Describes role played in previous work.
  pm_prev_role_c varchar(60),
    -- Category/area of previous work
  pm_category_c varchar(60),
    -- Worked with which organisation before (if other than our organisation).
  pm_other_organisation_c varchar(50),
    -- Comments on previous experience.
  pm_past_exp_comments_c varchar(640),
    -- Indicates if past experience was with this organisation.
  pm_prev_work_here_l boolean DEFAULT 0,
    -- Indicates whether the individual has previous worked with similar organisations.
  pm_prev_work_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_past_experience_pk
    PRIMARY KEY (p_site_key_n,pm_key_n),
  CONSTRAINT pm_past_experience_uk
    UNIQUE (p_partner_key_n,pm_end_date_d,pm_start_date_d,pm_prev_location_c),
  CONSTRAINT pm_past_experience_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_past_experience_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_past_experience_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_past_experience_pk0 
   (p_site_key_n,pm_key_n),
KEY inx_pm_past_experience_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_past_experience_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_past_experience_fkmd_key3 
   (s_modified_by_c),
UNIQUE KEY pm_past_exp_k0 
   (p_partner_key_n,pm_end_date_d,pm_start_date_d,pm_prev_location_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table describes the ability or abilities one possesses.
-- GROUP: personnel
CREATE TABLE pm_person_ability (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the area of ability
  pt_ability_area_name_c varchar(32) NOT NULL,
    -- This field is a numeric representation of level of ability.
  pt_ability_level_i integer DEFAULT 0 NOT NULL,
    -- Years of experience this person has had this ability.
  pm_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- The date the years of experience were up to date.
  pm_years_of_experience_as_of_d date,
    -- Indicates whether the applicant is bringing his instrument.
  pm_bringing_instrument_l boolean DEFAULT 0,
  pm_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_ability_pk
    PRIMARY KEY (p_partner_key_n,pt_ability_area_name_c),
  CONSTRAINT pm_person_ability_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_ability_fk2
    FOREIGN KEY (pt_ability_area_name_c)
    REFERENCES pt_ability_area(pt_ability_area_name_c),
  CONSTRAINT pm_person_ability_fk3
    FOREIGN KEY (pt_ability_level_i)
    REFERENCES pt_ability_level(pt_ability_level_i),
  CONSTRAINT pm_person_ability_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_ability_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_ability_pk0 
   (p_partner_key_n,pt_ability_area_name_c),
KEY inx_pm_person_ability_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_person_ability_fk2_key2 
   (pt_ability_area_name_c),
KEY inx_pm_person_ability_fk3_key3 
   (pt_ability_level_i),
KEY inx_pm_person_ability_fkcr_key4 
   (s_created_by_c),
KEY inx_pm_person_ability_fkmd_key5 
   (s_modified_by_c),
UNIQUE KEY pm_person_ability_k0 
   (p_partner_key_n,pt_ability_level_i,pt_ability_area_name_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table provides detail on qualifications someone may possess.
-- GROUP: personnel
CREATE TABLE pm_person_qualification (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the area of qualification.
  pt_qualification_area_name_c varchar(32) NOT NULL,
    -- Years of experience this person has had this qualification.
  pm_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- The date the years of experience were up to date.
  pm_years_of_experience_as_of_d date,
    -- This field indicate whether the qualifications are the result of informal training.
  pm_informal_flag_l boolean DEFAULT 0 NOT NULL,
    -- This field is a numeric representation of level of qualification.
  pt_qualification_level_i integer DEFAULT 0 NOT NULL,
  pm_comment_c varchar(512),
    -- The date the person qualified.
  pm_qualification_date_d date,
    -- The date the qualification expires.
  pm_qualification_expiry_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_qualification_pk
    PRIMARY KEY (p_partner_key_n,pt_qualification_area_name_c),
  CONSTRAINT pm_person_qualification_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_qualification_fk2
    FOREIGN KEY (pt_qualification_area_name_c)
    REFERENCES pt_qualification_area(pt_qualification_area_name_c),
  CONSTRAINT pm_person_qualification_fk3
    FOREIGN KEY (pt_qualification_level_i)
    REFERENCES pt_qualification_level(pt_qualification_level_i),
  CONSTRAINT pm_person_qualification_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_qualification_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_qualification_pk0 
   (p_partner_key_n,pt_qualification_area_name_c),
KEY inx_person_qualification_fk1_key1 
   (p_partner_key_n),
KEY inx_person_qualification_fk2_key2 
   (pt_qualification_area_name_c),
KEY inx_person_qualification_fk3_key3 
   (pt_qualification_level_i),
KEY inx_erson_qualification_fkcr_key4 
   (s_created_by_c),
KEY inx_erson_qualification_fkmd_key5 
   (s_modified_by_c),
UNIQUE KEY pm_person_qualification_k0 
   (p_partner_key_n,pt_qualification_level_i,pt_qualification_area_name_c),
KEY pm_person_qualification_k21 
   (pm_qualification_expiry_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table describes the skills that a person has (including professional skills but also other ones).
-- GROUP: personnel
CREATE TABLE pm_person_skill (

  pm_person_skill_key_i integer NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Skill Category
  pm_skill_category_code_c varchar(60) NOT NULL,
    -- Description of skill in english language
  pm_description_english_c varchar(160) NOT NULL,
    -- Description of skill in local language
  pm_description_local_c varchar(160),
    -- Language that is used in field pm_description_local_c
  pm_description_language_c varchar(20),
    -- This field is a numeric representation of level of skill.
  pm_skill_level_i integer NOT NULL,
    -- Years of experience this person has had this skill.
  pm_years_of_experience_i integer DEFAULT 99,
    -- The date the years of experience were up to date.
  pm_years_of_experience_as_of_d date,
    -- Indicates if this is a professional skill
  pm_professional_skill_l boolean DEFAULT 0,
    -- Indicates if this is the person's current occupation
  pm_current_occupation_l boolean DEFAULT 0,
    -- Degree that is linked with the skill (if applicable)
  pm_degree_c varchar(160),
    -- Year the degree was obtained.
  pm_year_of_degree_i integer,
  pm_comment_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_skill_pk
    PRIMARY KEY (pm_person_skill_key_i),
  CONSTRAINT pm_person_skill_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_skill_fk2
    FOREIGN KEY (pm_skill_category_code_c)
    REFERENCES pt_skill_category(pt_code_c),
  CONSTRAINT pm_person_skill_fk3
    FOREIGN KEY (pm_description_language_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT pm_person_skill_fk4
    FOREIGN KEY (pm_skill_level_i)
    REFERENCES pt_skill_level(pt_level_i),
  CONSTRAINT pm_person_skill_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_skill_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_skill_pk0 
   (pm_person_skill_key_i),
KEY inx_pm_person_skill_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_person_skill_fk2_key2 
   (pm_skill_category_code_c),
KEY inx_pm_person_skill_fk3_key3 
   (pm_description_language_c),
KEY inx_pm_person_skill_fk4_key4 
   (pm_skill_level_i),
KEY inx_pm_person_skill_fkcr_key5 
   (s_created_by_c),
KEY inx_pm_person_skill_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table records the formal education that a person has
-- GROUP: personnel
CREATE TABLE pm_formal_education (

  pm_formal_education_key_i integer NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Education Category
  pm_education_category_c varchar(60),
    -- Description of degree (incl. title and subject)
  pm_degree_c varchar(160),
    -- Year the degree was obtained.
  pm_year_of_degree_i integer DEFAULT 99,
    -- Institution the degree was obtained from
  pm_institution_c varchar(160),
    -- Code of country in which the degree was obtained
  p_country_code_c varchar(8),
  pm_comment_c varchar(2000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_formal_education_pk
    PRIMARY KEY (pm_formal_education_key_i),
  CONSTRAINT pm_formal_education_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_formal_education_fk2
    FOREIGN KEY (p_country_code_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT pm_formal_education_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_formal_education_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_formal_education_pk0 
   (pm_formal_education_key_i),
KEY inx_pm_formal_education_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_formal_education_fk2_key2 
   (p_country_code_c),
KEY inx_pm_formal_education_fkcr_key3 
   (s_created_by_c),
KEY inx_pm_formal_education_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains personal data about our staff, eg. tax id.
-- GROUP: personnel
CREATE TABLE pm_personal_data (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- The person's height in cm
  pm_height_cm_i integer,
    -- The person's weight in kg
  pm_weight_kg_n numeric(5, 2),
    -- The person's eye colour
  pm_eye_colour_c varchar(40),
    -- The person's hair colour
  pm_hair_colour_c varchar(40),
    -- Information about the person's facial hair, e.g. beard, mustache
  pm_facial_hair_c varchar(60),
    -- Further physical information about the person like tatoos, piercings, scars or marks
  pm_physical_desc_c varchar(500),
    -- The person's blood type
  pm_blood_type_c varchar(20),
    -- Ethnic Origin
  pm_ethnic_origin_c varchar(60),
    -- Proof of life question 1
  pm_life_question_1_c varchar(200),
    -- Answer to proof of life question 1
  pm_life_answer_1_c varchar(200),
    -- Proof of life question 2
  pm_life_question_2_c varchar(200),
    -- Answer to proof of life question 2
  pm_life_answer_2_c varchar(200),
    -- Proof of life question 3
  pm_life_question_3_c varchar(200),
    -- Answer to proof of life question 3
  pm_life_answer_3_c varchar(200),
    -- Proof of life question 4
  pm_life_question_4_c varchar(200),
    -- Answer to proof of life question 4
  pm_life_answer_4_c varchar(200),
    -- User defined field-1 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld1_c varchar(50),
    -- User defined field-2 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld2_c varchar(50),
    -- User defined field-3 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld3_c varchar(50),
    -- User defined field-4 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld4_c varchar(50),
    -- User defined field-5 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld5_c varchar(50),
    -- User defined field-6 for personal information (not in use any longer, replaced by p_data_label_value_partner)
  pm_personal_fld6_c varchar(50),
    -- Name of the person's first language.
  p_language_code_c varchar(20),
    -- This is the year the person became a Believer.
  p_believer_since_year_i integer DEFAULT NULL,
    -- Comment about the year or how the person became a believer
  p_believer_since_comment_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_personal_data_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT pm_personal_data_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_personal_data_fk3
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT pm_personal_data_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_personal_data_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_personal_data_pk0 
   (p_partner_key_n),
KEY inx_pm_personal_data_fk3_key1 
   (p_language_code_c),
KEY inx_pm_personal_data_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_personal_data_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds the label values for partner related data.
-- GROUP: partner
CREATE TABLE p_data_label_value_partner (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- A sequence key for data labels.
  p_data_label_key_i integer NOT NULL,
    -- Label value for type Character.
  p_value_char_c varchar(8192),
    -- Label value for type Numeric.
  p_value_num_n numeric(24, 10) DEFAULT 0,
    -- Label value for type Currency.
  p_value_currency_n numeric(24, 10) DEFAULT 0,
    -- Label value for type Integer.
  p_value_int_i integer DEFAULT 0,
    -- Label value for type Boolean.
  p_value_bool_l boolean DEFAULT 0,
    -- Label value for type Date.
  p_value_date_d date,
    -- Label value for type Time.
  p_value_time_i integer DEFAULT 0,
    -- Label value for type Partner Key.
  p_value_partner_key_n bigint DEFAULT 0,
    -- Label value for type Lookup Value.
  p_value_lookup_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_value_partner_pk
    PRIMARY KEY (p_partner_key_n,p_data_label_key_i),
  CONSTRAINT p_data_label_value_partner_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_data_label_value_partner_fk2
    FOREIGN KEY (p_data_label_key_i)
    REFERENCES p_data_label(p_key_i),
  CONSTRAINT p_data_label_value_partner_fk3
    FOREIGN KEY (p_value_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_data_label_value_partner_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_value_partner_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__data_label_value_partner_pk0 
   (p_partner_key_n,p_data_label_key_i),
KEY inx__label_value_partner_fk1_key1 
   (p_partner_key_n),
KEY inx__label_value_partner_fk2_key2 
   (p_data_label_key_i),
KEY inx__label_value_partner_fk3_key3 
   (p_value_partner_key_n),
KEY inx_label_value_partner_fkcr_key4 
   (s_created_by_c),
KEY inx_label_value_partner_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds the label values for application related data.
-- GROUP: personnel
CREATE TABLE p_data_label_value_application (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer DEFAULT 0 NOT NULL,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint NOT NULL,
    -- A sequence key for data labels.
  p_data_label_key_i integer NOT NULL,
    -- Label value for type Character.
  p_value_char_c varchar(8192),
    -- Label value for type Numeric.
  p_value_num_n numeric(24, 10) DEFAULT 0,
    -- Label value for type Currency.
  p_value_currency_n numeric(24, 10) DEFAULT 0,
    -- Label value for type Integer.
  p_value_int_i integer DEFAULT 0,
    -- Label value for type Boolean.
  p_value_bool_l boolean DEFAULT 0,
    -- Label value for type Date.
  p_value_date_d date,
    -- Label value for type Time.
  p_value_time_i integer DEFAULT 0,
    -- Label value for type Partner Key.
  p_value_partner_key_n bigint DEFAULT 0,
    -- Label value for type Lookup Value.
  p_value_lookup_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_data_label_value_application_pk
    PRIMARY KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n,p_data_label_key_i),
  CONSTRAINT p_data_label_value_application_fk1
    FOREIGN KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n)
    REFERENCES pm_general_application(p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT p_data_label_value_application_fk2
    FOREIGN KEY (p_data_label_key_i)
    REFERENCES p_data_label(p_key_i),
  CONSTRAINT p_data_label_value_application_fk3
    FOREIGN KEY (p_value_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_data_label_value_application_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_data_label_value_application_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_a_label_value_application_pk0 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n,p_data_label_key_i),
KEY inx_el_value_application_fk1_key1 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
KEY inx_el_value_application_fk2_key2 
   (p_data_label_key_i),
KEY inx_el_value_application_fk3_key3 
   (p_value_partner_key_n),
KEY inx_l_value_application_fkcr_key4 
   (s_created_by_c),
KEY inx_l_value_application_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains information regarding personal progress reports.
-- GROUP: personnel
CREATE TABLE pm_person_evaluation (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This field describes the timing of the progress report, eg.  Semi-Annual, Annual, or Leaving.
  pm_evaluation_type_c varchar(24) NOT NULL,
    -- The date the evaluation was conducted.
  pm_evaluation_date_d date NOT NULL,
    -- Comments on the progress report.
  pm_evaluation_comments_c varchar(1000),
    -- Describe possible actions to take.
  pm_person_eval_action_c varchar(1000),
    -- Describes the person who conducted the progress report.
  pm_evaluator_c varchar(60) NOT NULL,
    -- Date of next evaluation.
  pm_next_evaluation_date_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_evaluation_pk
    PRIMARY KEY (p_partner_key_n,pm_evaluation_date_d,pm_evaluator_c),
  CONSTRAINT pm_person_evaluation_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_evaluation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_evaluation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_evaluation_pk0 
   (p_partner_key_n,pm_evaluation_date_d,pm_evaluator_c),
KEY inx_pm_person_evaluation_fk1_key1 
   (p_partner_key_n),
KEY inx_m_person_evaluation_fkcr_key2 
   (s_created_by_c),
KEY inx_m_person_evaluation_fkmd_key3 
   (s_modified_by_c),
UNIQUE KEY pm_pers_eval_k0 
   (p_partner_key_n,pm_evaluation_date_d,pm_evaluation_type_c,pm_evaluator_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table records the absence of a person (holiday, sickness, etc.)
-- GROUP: personnel
CREATE TABLE pm_person_absence (

  pm_person_absence_key_i integer NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- First day of abscence
  pm_absence_from_d date,
    -- Last day of abscence
  pm_absence_to_d date,
    -- Total number of working days absent
  pm_absence_days_i integer,
    -- Type of Absence
  pm_absence_type_c varchar(60),
  pm_comment_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_absence_pk
    PRIMARY KEY (pm_person_absence_key_i),
  CONSTRAINT pm_person_absence_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_absence_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_absence_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_absence_pk0 
   (pm_person_absence_key_i),
KEY inx_pm_person_absence_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_person_absence_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_person_absence_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table includes special medical or dietary needs someone may have.
-- GROUP: personnel
CREATE TABLE pm_special_need (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Contains special medical information if needed.
  pm_medical_comment_c varchar(5000),
    -- Contains special dietary information if needed.
  pm_dietary_comment_c varchar(5000),
    -- Contains any other special need that may be applicable.
  pm_other_special_need_c varchar(5000),
    -- Indicates if there are vegetarian needs.
  pm_vegetarian_flag_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_special_need_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT pm_special_need_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_special_need_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_special_need_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_special_need_pk0 
   (p_partner_key_n),
KEY inx_pm_special_need_fkcr_key1 
   (s_created_by_c),
KEY inx_pm_special_need_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains information regarding recruitment, home office, field office, etc. .
-- GROUP: personnel
CREATE TABLE pm_staff_data (

    -- The connection to the site
  p_site_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to make editing these records easier
  pm_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is a field indicating the status of the commitment
  pm_status_code_c varchar(40),
    -- This is the expected date their commitment starts.
  pm_start_of_commitment_d date NOT NULL,
    -- Is the commitment start date an approximate date?
  pm_start_date_approx_l boolean DEFAULT 0,
    -- This is the expected date their commitment ends.
  pm_end_of_commitment_d date,
    -- The office that recruited the partner. This is a unit of type field, not of type office.
  pm_office_recruited_by_n bigint NOT NULL,
    -- The home office of the person. This is a unit of type field, not of type office.
  pm_home_office_n bigint NOT NULL,
    -- The person's is serving for that field.
  pm_receiving_field_n bigint NOT NULL,
    -- The office they work at in the receiving field.
  pm_receiving_field_office_n bigint,
    -- Comments on commitment record.
  pm_staff_data_comments_c varchar(640),
    -- A free text field for a job title for a person. This is not the same as the person's role, however it may be auto generated from the roles.
  pm_job_title_c varchar(400),
    -- Phone extension of the person at this office
  pm_office_phone_ext_c varchar(50),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_staff_data_pk
    PRIMARY KEY (p_site_key_n,pm_key_n),
  CONSTRAINT pm_staff_data_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_staff_data_fk2
    FOREIGN KEY (pm_office_recruited_by_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_staff_data_fk3
    FOREIGN KEY (pm_home_office_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_staff_data_fk4
    FOREIGN KEY (pm_receiving_field_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_staff_data_fk5
    FOREIGN KEY (pm_receiving_field_office_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pm_staff_data_fk6
    FOREIGN KEY (pm_status_code_c)
    REFERENCES pm_commitment_status(pm_code_c),
  CONSTRAINT pm_staff_data_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_staff_data_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_staff_data_pk0 
   (p_site_key_n,pm_key_n),
KEY inx_pm_staff_data_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_staff_data_fk2_key2 
   (pm_office_recruited_by_n),
KEY inx_pm_staff_data_fk3_key3 
   (pm_home_office_n),
KEY inx_pm_staff_data_fk4_key4 
   (pm_receiving_field_n),
KEY inx_pm_staff_data_fk5_key5 
   (pm_receiving_field_office_n),
KEY inx_pm_staff_data_fk6_key6 
   (pm_status_code_c),
KEY inx_pm_staff_data_fkcr_key7 
   (s_created_by_c),
KEY inx_pm_staff_data_fkmd_key8 
   (s_modified_by_c),
UNIQUE KEY pm_staff_data_k0 
   (p_partner_key_n,pm_start_of_commitment_d,pm_end_of_commitment_d,pm_receiving_field_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table holds the commitment type history of a person
-- GROUP: personnel
CREATE TABLE pm_person_commitment_status (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Code for Status
  pm_status_code_c varchar(40) NOT NULL,
    -- This is the date since the status is valid
  pm_status_since_d date NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_commitment_status_pk
    PRIMARY KEY (p_partner_key_n,pm_status_code_c,pm_status_since_d),
  CONSTRAINT pm_person_commitment_status_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_commitment_status_fk2
    FOREIGN KEY (pm_status_code_c)
    REFERENCES pm_commitment_status(pm_code_c),
  CONSTRAINT pm_person_commitment_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_commitment_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__person_commitment_status_pk0 
   (p_partner_key_n,pm_status_code_c,pm_status_since_d),
KEY inx_on_commitment_status_fk1_key1 
   (p_partner_key_n),
KEY inx_on_commitment_status_fk2_key2 
   (pm_status_code_c),
KEY inx_n_commitment_status_fkcr_key3 
   (s_created_by_c),
KEY inx_n_commitment_status_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This table contains information concerning jobs within the unit.
-- GROUP: units
CREATE TABLE um_job (

    -- This is the partner key of the unit. It consists of the fund id followed by a computer generated six digit number.
  pm_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
    -- To make sure we can have two jobs in difference time-frames
  um_job_key_i integer NOT NULL,
    -- Indicates the normal length of commitment, eg. short-term.
  um_job_type_c varchar(40) DEFAULT 'Short Term' NOT NULL,
    -- Date from um_training_period.
  um_from_date_d date,
    -- Date the job posting is to.
  um_to_date_d date,
    -- Indicates the minimum number of staff required.
  um_minimum_i integer DEFAULT 0,
    -- Indicates the maximum number of staff required.
  um_maximum_i integer DEFAULT 0,
    -- Indicates the present number on staff.
  um_present_i integer DEFAULT 0,
    -- Number of part-timers acceptable.
  um_part_timers_i integer DEFAULT 0,
    -- Number of applications on file for this position. This field is driven from the pm_job_assignment.
  um_applications_i integer DEFAULT 0,
    -- Indicates if part-timers can be accepted for this position.
  um_part_time_flag_l boolean DEFAULT 0,
    -- Length of training required for this position.
  um_training_period_c varchar(30) DEFAULT 'One month' NOT NULL,
    -- Length of commitment required for this position.
  um_commitment_period_c varchar(30) DEFAULT 'Three months' NOT NULL,
    -- Is this position available to other systems.
  um_public_flag_l boolean DEFAULT 0,
    -- Describes where you want to advertise about a job opening, only within the Unit, to the whole organisation, or outside our organisation.
  um_job_publicity_i integer DEFAULT 0,
    -- Indicates whether previous experience with our organisation is required for this job. 
  um_previous_internal_exp_req_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_job_pk
    PRIMARY KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT um_job_fk1
    FOREIGN KEY (pm_unit_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_job_fk2
    FOREIGN KEY (pt_position_name_c,pt_position_scope_c)
    REFERENCES pt_position(pt_position_name_c,pt_position_scope_c),
  CONSTRAINT um_job_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_job_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_job_pk0 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_um_job_fk1_key1 
   (pm_unit_key_n),
KEY inx_um_job_fk2_key2 
   (pt_position_name_c,pt_position_scope_c),
KEY inx_um_job_fkcr_key3 
   (s_created_by_c),
KEY inx_um_job_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists abilities and experience required for various positions.
-- GROUP: units
CREATE TABLE um_job_requirement (

    -- This is the partner key of the unit to which this person is assigned. It consists of the fund id followed by a computer generated six digit number.
  pm_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
  um_job_key_i integer NOT NULL,
    -- Name of the area of ability
  pt_ability_area_name_c varchar(32) NOT NULL,
    -- Years of experience required for this position..
  um_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- This field is a numeric representation of level of ability.
  pt_ability_level_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_job_requirement_pk
    PRIMARY KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pt_ability_area_name_c),
  CONSTRAINT um_job_requirement_fk1
    FOREIGN KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i)
    REFERENCES um_job(pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT um_job_requirement_fk2
    FOREIGN KEY (pt_ability_area_name_c)
    REFERENCES pt_ability_area(pt_ability_area_name_c),
  CONSTRAINT um_job_requirement_fk3
    FOREIGN KEY (pt_ability_level_i)
    REFERENCES pt_ability_level(pt_ability_level_i),
  CONSTRAINT um_job_requirement_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_job_requirement_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_job_requirement_pk0 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pt_ability_area_name_c),
KEY inx_um_job_requirement_fk1_key1 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_um_job_requirement_fk2_key2 
   (pt_ability_area_name_c),
KEY inx_um_job_requirement_fk3_key3 
   (pt_ability_level_i),
KEY inx_um_job_requirement_fkcr_key4 
   (s_created_by_c),
KEY inx_um_job_requirement_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Language used on this job.
-- GROUP: units
CREATE TABLE um_job_language (

    -- This is the partner key of the unit to which this person is assigned. It consists of the fund id followed by a computer generated six digit number.
  pm_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
  um_job_key_i integer NOT NULL,
    -- Name of the language(s) spoken.
  p_language_code_c varchar(20) NOT NULL,
    -- Years of experience required using this language.
  um_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- This field is a numeric representation of level of language.
  pt_language_level_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_job_language_pk
    PRIMARY KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,p_language_code_c),
  CONSTRAINT um_job_language_fk1
    FOREIGN KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i)
    REFERENCES um_job(pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT um_job_language_fk2
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT um_job_language_fk3
    FOREIGN KEY (pt_language_level_i)
    REFERENCES pt_language_level(pt_language_level_i),
  CONSTRAINT um_job_language_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_job_language_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_job_language_pk0 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,p_language_code_c),
KEY inx_um_job_language_fk1_key1 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_um_job_language_fk2_key2 
   (p_language_code_c),
KEY inx_um_job_language_fk3_key3 
   (pt_language_level_i),
KEY inx_um_job_language_fkcr_key4 
   (s_created_by_c),
KEY inx_um_job_language_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of qualifications required for individual jobs.
-- GROUP: units
CREATE TABLE um_job_qualification (

    -- This is the partner key of the unit to which this person is assigned. It consists of the fund id followed by a computer generated six digit number.
  pm_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
  um_job_key_i integer NOT NULL,
    -- Name of the area of qualification.
  pt_qualification_area_name_c varchar(32) NOT NULL,
    -- Years of experience required using this qualification.
  um_years_of_experience_i integer DEFAULT 99,
    -- This field indicate whether the qualifications can be the result of informal training.
  pm_informal_flag_l boolean DEFAULT 0,
    -- This field is a numeric representation of level of qualification.
  pt_qualification_level_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_job_qualification_pk
    PRIMARY KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pt_qualification_area_name_c),
  CONSTRAINT um_job_qualification_fk1
    FOREIGN KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i)
    REFERENCES um_job(pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT um_job_qualification_fk2
    FOREIGN KEY (pt_qualification_area_name_c)
    REFERENCES pt_qualification_area(pt_qualification_area_name_c),
  CONSTRAINT um_job_qualification_fk3
    FOREIGN KEY (pt_qualification_level_i)
    REFERENCES pt_qualification_level(pt_qualification_level_i),
  CONSTRAINT um_job_qualification_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_job_qualification_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_job_qualification_pk0 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pt_qualification_area_name_c),
KEY inx_um_job_qualification_fk1_key1 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_um_job_qualification_fk2_key2 
   (pt_qualification_area_name_c),
KEY inx_um_job_qualification_fk3_key3 
   (pt_qualification_level_i),
KEY inx_m_job_qualification_fkcr_key4 
   (s_created_by_c),
KEY inx_m_job_qualification_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- This defines the position one fills.
-- GROUP: personnel
CREATE TABLE pm_job_assignment (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the partner key of the unit to which this person is assigned. It consists of the fund id followed by a computer generated six digit number.
  pm_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
  um_job_key_i integer NOT NULL,
    -- A number to make this unique as other wise we would need to add date from/to
  pm_job_assignment_key_i integer NOT NULL,
    -- Indicates if the assignment is in assistance to the given position.
  pt_assistant_to_l boolean DEFAULT 0,
    -- This defines the type of assignment.
  pt_assignment_type_code_c varchar(2),
    -- Can these costs be changed?
  um_costs_changed_flag_l boolean DEFAULT 0,
    -- This is the date from which this job is assigned.
  pm_from_date_d date NOT NULL,
    -- This is the date to which a job is expected to be filled.
  pm_to_date_d date,
    -- This field indicates if the hard copy details have been changed.
  pm_hrd_cpy_detail_change_flag_l boolean DEFAULT 0,
    -- This field indicates if the record is deletable.
  pm_deleteable_flag_l boolean DEFAULT 0,
    -- The office that entered this assignment
  pm_registration_office_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_job_assignment_pk
    PRIMARY KEY (p_partner_key_n,pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pm_job_assignment_key_i),
  CONSTRAINT pm_job_assignment_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT pm_job_assignment_fk2
    FOREIGN KEY (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i)
    REFERENCES um_job(pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT pm_job_assignment_fk3
    FOREIGN KEY (pt_assignment_type_code_c)
    REFERENCES pt_assignment_type(pt_assignment_type_code_c),
  CONSTRAINT pm_job_assignment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_job_assignment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_job_assignment_pk0 
   (p_partner_key_n,pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i,pm_job_assignment_key_i),
KEY inx_pm_job_assignment_fk1_key1 
   (p_partner_key_n),
KEY inx_pm_job_assignment_fk2_key2 
   (pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_pm_job_assignment_fk3_key3 
   (pt_assignment_type_code_c),
KEY inx_pm_job_assignment_fkcr_key4 
   (s_created_by_c),
KEY inx_pm_job_assignment_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of  the abilities within the unit.
-- GROUP: units
CREATE TABLE um_unit_ability (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the area of ability
  pt_ability_area_name_c varchar(32) NOT NULL,
    -- Years of experience this required for this ability.
  um_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- This field is a numeric representation of level of ability.
  pt_ability_level_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_unit_ability_pk
    PRIMARY KEY (p_partner_key_n,pt_ability_area_name_c),
  CONSTRAINT um_unit_ability_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_ability_fk2
    FOREIGN KEY (pt_ability_area_name_c)
    REFERENCES pt_ability_area(pt_ability_area_name_c),
  CONSTRAINT um_unit_ability_fk3
    FOREIGN KEY (pt_ability_level_i)
    REFERENCES pt_ability_level(pt_ability_level_i),
  CONSTRAINT um_unit_ability_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_unit_ability_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_unit_ability_pk0 
   (p_partner_key_n,pt_ability_area_name_c),
KEY inx_um_unit_ability_fk1_key1 
   (p_partner_key_n),
KEY inx_um_unit_ability_fk2_key2 
   (pt_ability_area_name_c),
KEY inx_um_unit_ability_fk3_key3 
   (pt_ability_level_i),
KEY inx_um_unit_ability_fkcr_key4 
   (s_created_by_c),
KEY inx_um_unit_ability_fkmd_key5 
   (s_modified_by_c),
UNIQUE KEY um_unit_ability_k0 
   (p_partner_key_n,pt_ability_area_name_c,pt_ability_level_i)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of the language used within this unit.
-- GROUP: units
CREATE TABLE um_unit_language (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the language(s) spoken.
  p_language_code_c varchar(20) NOT NULL,
    -- This field is a numeric representation of level of language.
  pt_language_level_i integer DEFAULT 0 NOT NULL,
    -- Years of experience required using this language.
  um_years_of_experience_i integer DEFAULT 99 NOT NULL,
    -- Contains comments pertaining to the language of the unit.
  um_unit_lang_comment_c varchar(80),
    -- Lists whether the languare is required or desired.
  um_unit_language_req_c varchar(16),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_unit_language_pk
    PRIMARY KEY (p_partner_key_n,p_language_code_c,pt_language_level_i),
  CONSTRAINT um_unit_language_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_language_fk2
    FOREIGN KEY (p_language_code_c)
    REFERENCES p_language(p_language_code_c),
  CONSTRAINT um_unit_language_fk3
    FOREIGN KEY (pt_language_level_i)
    REFERENCES pt_language_level(pt_language_level_i),
  CONSTRAINT um_unit_language_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_unit_language_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_unit_language_pk0 
   (p_partner_key_n,p_language_code_c,pt_language_level_i),
KEY inx_um_unit_language_fk1_key1 
   (p_partner_key_n),
KEY inx_um_unit_language_fk2_key2 
   (p_language_code_c),
KEY inx_um_unit_language_fk3_key3 
   (pt_language_level_i),
KEY inx_um_unit_language_fkcr_key4 
   (s_created_by_c),
KEY inx_um_unit_language_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details pertaining to the costs of being on in the unit.
-- GROUP: units
CREATE TABLE um_unit_cost (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Date from which these costs are applicable.
  um_valid_from_date_d date NOT NULL,
    -- Indicates amount it costs a single to be on the team.
  um_single_costs_period_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a couple to be on the team.
  um_couple_costs_period_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child1_costs_period_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child2_costs_period_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child3_costs_period_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for adults.
  um_adult_joining_charge_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for couples.
  um_couple_joining_charge_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for a child.
  um_child_joining_charge_intl_n numeric(24, 10) DEFAULT 0,
    -- Indicates the local currency.
  a_local_currency_code_c varchar(16),
    -- The charge period for the unit, eg. monthly, quarterly.
  um_charge_period_c varchar(24) NOT NULL,
    -- Indicates amount it costs a single to be on the team.
  um_single_costs_period_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a couple to be on the team.
  um_couple_costs_period_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child1_costs_period_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child2_costs_period_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates amount it costs a child to be on the team.
  um_child3_costs_period_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for adults.
  um_adult_joining_charge_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for couples.
  um_couple_joining_charge_base_n numeric(24, 10) DEFAULT 0,
    -- Indicates the joining charge for a child.
  um_child_joining_charge_base_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_unit_cost_pk
    PRIMARY KEY (p_partner_key_n,um_valid_from_date_d),
  CONSTRAINT um_unit_cost_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_cost_fk2
    FOREIGN KEY (a_local_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT um_unit_cost_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_unit_cost_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_unit_cost_pk0 
   (p_partner_key_n,um_valid_from_date_d),
KEY inx_um_unit_cost_fk1_key1 
   (p_partner_key_n),
KEY inx_um_unit_cost_fk2_key2 
   (a_local_currency_code_c),
KEY inx_um_unit_cost_fkcr_key3 
   (s_created_by_c),
KEY inx_um_unit_cost_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details pertaining to evaluation of the unit.
-- GROUP: units
CREATE TABLE um_unit_evaluation (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Indicates the date of the evaluation.
  um_date_of_evaluation_d date NOT NULL,
    -- The evaluation number is generated from a database sequence
  um_evaluation_number_n numeric(14, 0) DEFAULT 0 NOT NULL,
    -- Indicates whether the evaluator is married, single, etc.
  um_evaluator_family_status_c varchar(40) NOT NULL,
    -- The name of the evaluator's home country.
  p_evaluator_home_country_c varchar(40) NOT NULL,
    -- Age of the person conduction the unit evaluation.
  um_evaluator_age_n numeric(4, 0) DEFAULT 0 NOT NULL,
  um_evaluator_sex_c varchar(2) NOT NULL,
    -- Data regarding the unit evaluation.
  um_unit_evaluation_data_c varchar(160),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT um_unit_evaluation_pk
    PRIMARY KEY (p_partner_key_n,um_date_of_evaluation_d,um_evaluation_number_n),
  CONSTRAINT um_unit_evaluation_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT um_unit_evaluation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT um_unit_evaluation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_um_unit_evaluation_pk0 
   (p_partner_key_n,um_date_of_evaluation_d,um_evaluation_number_n),
KEY inx_um_unit_evaluation_fk1_key1 
   (p_partner_key_n),
KEY inx_um_unit_evaluation_fkcr_key2 
   (s_created_by_c),
KEY inx_um_unit_evaluation_fkmd_key3 
   (s_modified_by_c),
UNIQUE KEY um_unit_eval_k0 
   (p_partner_key_n,um_evaluation_number_n,um_date_of_evaluation_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Basic details about a conference
-- GROUP: conference
CREATE TABLE pc_conference (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint DEFAULT 0 NOT NULL,
  pc_outreach_prefix_c varchar(10),
  pc_start_d date,
  pc_end_d date,
    -- This defines which currency is being used
  a_currency_code_c varchar(16) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_conference_pk
    PRIMARY KEY (pc_conference_key_n),
  CONSTRAINT pc_conference_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pc_conference_fk2
    FOREIGN KEY (a_currency_code_c)
    REFERENCES a_currency(a_currency_code_c),
  CONSTRAINT pc_conference_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_conference_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_conference_pk0 
   (pc_conference_key_n),
KEY inx_pc_conference_fk2_key1 
   (a_currency_code_c),
KEY inx_pc_conference_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_conference_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists options that are set for a conference
-- GROUP: conference
CREATE TABLE pc_conference_option (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint NOT NULL,
    -- Unique name of the cost type
  pc_option_type_code_c varchar(32) NOT NULL,
    -- Description of the option type
  pc_option_set_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_conference_option_pk
    PRIMARY KEY (pc_conference_key_n,pc_option_type_code_c),
  CONSTRAINT pc_conference_option_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_conference_option_fk2
    FOREIGN KEY (pc_option_type_code_c)
    REFERENCES pc_conference_option_type(pc_option_type_code_c),
  CONSTRAINT pc_conference_option_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_conference_option_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_conference_option_pk0 
   (pc_conference_key_n,pc_option_type_code_c),
KEY inx_pc_conference_option_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_conference_option_fk2_key2 
   (pc_option_type_code_c),
KEY inx_c_conference_option_fkcr_key3 
   (s_created_by_c),
KEY inx_c_conference_option_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists optional discounts for a conference
-- GROUP: conference
CREATE TABLE pc_discount (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint NOT NULL,
    -- Unique name of the criteria that a person has to meet to get the discount
  pc_discount_criteria_code_c varchar(16) NOT NULL,
    -- Unique name of the cost type
  pc_cost_type_code_c varchar(32) NOT NULL,
    -- When is this discount valid (PRE, CONF, POST, ALWAYS)
  pc_validity_c varchar(6) NOT NULL,
    -- For discounts up to a certain age (mainly child discount). If age does not matter, set to -1.
  pc_up_to_age_i integer NOT NULL,
    -- Is the discount value given in percent (or total otherwise)
  pc_percentage_l boolean,
    -- Amount of discount (in percent or total)
  pc_discount_n numeric(24, 10),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_discount_pk
    PRIMARY KEY (pc_conference_key_n,pc_discount_criteria_code_c,pc_cost_type_code_c,pc_validity_c,pc_up_to_age_i),
  CONSTRAINT pc_discount_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_discount_fk2
    FOREIGN KEY (pc_discount_criteria_code_c)
    REFERENCES pc_discount_criteria(pc_discount_criteria_code_c),
  CONSTRAINT pc_discount_fk3
    FOREIGN KEY (pc_cost_type_code_c)
    REFERENCES pc_cost_type(pc_cost_type_code_c),
  CONSTRAINT pc_discount_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_discount_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_discount_pk0 
   (pc_conference_key_n,pc_discount_criteria_code_c,pc_cost_type_code_c,pc_validity_c,pc_up_to_age_i),
KEY inx_pc_discount_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_discount_fk2_key2 
   (pc_discount_criteria_code_c),
KEY inx_pc_discount_fk3_key3 
   (pc_cost_type_code_c),
KEY inx_pc_discount_fkcr_key4 
   (s_created_by_c),
KEY inx_pc_discount_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Lists the attendees at a conference
-- GROUP: conference
CREATE TABLE pc_attendee (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_home_office_key_n bigint DEFAULT 0 NOT NULL,
  pc_outreach_type_c varchar(12),
  pc_actual_arr_d date,
  pc_actual_dep_d date,
  pc_badge_print_d date,
  pc_details_print_d date,
  pc_comments_c varchar(1000),
  pc_discovery_group_c varchar(32),
  pc_work_group_c varchar(32),
  pc_registered_d date,
  pc_arrival_group_c varchar(40),
  pc_departure_group_c varchar(40),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_attendee_pk
    PRIMARY KEY (pc_conference_key_n,p_partner_key_n),
  CONSTRAINT pc_attendee_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_attendee_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pc_attendee_fk3
    FOREIGN KEY (pc_home_office_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pc_attendee_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_attendee_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_attendee_pk0 
   (pc_conference_key_n,p_partner_key_n),
KEY inx_pc_attendee_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_attendee_fk2_key2 
   (p_partner_key_n),
KEY inx_pc_attendee_fk3_key3 
   (pc_home_office_key_n),
KEY inx_pc_attendee_fkcr_key4 
   (s_created_by_c),
KEY inx_pc_attendee_fkmd_key5 
   (s_modified_by_c),
KEY pc_outreach_k0 
   (pc_conference_key_n,pc_outreach_type_c,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Charges for the various outreach options from a conference (currency held in conference master)
-- GROUP: conference
CREATE TABLE pc_conference_cost (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint DEFAULT 0 NOT NULL,
    -- 9999999999
  pc_option_days_i integer DEFAULT 0 NOT NULL,
  pc_charge_n numeric(24, 10) DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_conference_cost_pk
    PRIMARY KEY (pc_conference_key_n,pc_option_days_i),
  CONSTRAINT pc_conference_cost_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_conference_cost_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_conference_cost_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_conference_cost_pk0 
   (pc_conference_key_n,pc_option_days_i),
KEY inx_pc_conference_cost_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_conference_cost_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_conference_cost_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains extra conference costs for individual attendees
-- GROUP: conference
CREATE TABLE pc_extra_cost (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
    -- Key to identify the extra cost, along with conference and partner key
  pc_extra_cost_key_i integer DEFAULT 0 NOT NULL,
  pc_cost_type_code_c varchar(32),
  pc_cost_amount_n numeric(24, 10),
  pc_comment_c varchar(512),
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_authorising_field_n bigint,
    -- Indicate who authorised the extra cost.
  pc_authorising_person_c varchar(40),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_extra_cost_pk
    PRIMARY KEY (pc_conference_key_n,p_partner_key_n,pc_extra_cost_key_i),
  CONSTRAINT pc_extra_cost_fk1
    FOREIGN KEY (pc_conference_key_n,p_partner_key_n)
    REFERENCES pc_attendee(pc_conference_key_n,p_partner_key_n),
  CONSTRAINT pc_extra_cost_fk2
    FOREIGN KEY (pc_cost_type_code_c)
    REFERENCES pc_cost_type(pc_cost_type_code_c),
  CONSTRAINT pc_extra_cost_fk3
    FOREIGN KEY (pc_authorising_field_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT pc_extra_cost_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_extra_cost_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_extra_cost_pk0 
   (pc_conference_key_n,p_partner_key_n,pc_extra_cost_key_i),
KEY inx_pc_extra_cost_fk2_key1 
   (pc_cost_type_code_c),
KEY inx_pc_extra_cost_fk3_key2 
   (pc_authorising_field_n),
KEY inx_pc_extra_cost_fkcr_key3 
   (s_created_by_c),
KEY inx_pc_extra_cost_fkmd_key4 
   (s_modified_by_c),
KEY inx_pc_extra_cost_conf_partner0 
   (pc_conference_key_n,p_partner_key_n),
KEY inx_pc_extra_cost_conf_cost_type1 
   (pc_conference_key_n,pc_cost_type_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Discounts and Supplements for early or late registration
-- GROUP: conference
CREATE TABLE pc_early_late (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint DEFAULT 0 NOT NULL,
  pc_applicable_d date NOT NULL,
  pc_type_l boolean DEFAULT 1,
  pc_amount_percent_l boolean DEFAULT 1,
  pc_amount_n numeric(24, 10) DEFAULT 0,
  pc_percent_i integer DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_early_late_pk
    PRIMARY KEY (pc_conference_key_n,pc_applicable_d),
  CONSTRAINT pc_early_late_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_early_late_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_early_late_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_early_late_pk0 
   (pc_conference_key_n,pc_applicable_d),
KEY inx_pc_early_late_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_early_late_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_early_late_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains information about which groups individual attendees are assigned to
-- GROUP: conference
CREATE TABLE pc_group (

    -- This is the partner key of the conference. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint NOT NULL,
  pc_group_type_c varchar(40) NOT NULL,
  pc_group_name_c varchar(80) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_group_pk
    PRIMARY KEY (pc_conference_key_n,p_partner_key_n,pc_group_type_c,pc_group_name_c),
  CONSTRAINT pc_group_fk1
    FOREIGN KEY (pc_conference_key_n,p_partner_key_n)
    REFERENCES pc_attendee(pc_conference_key_n,p_partner_key_n),
  CONSTRAINT pc_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_group_pk0 
   (pc_conference_key_n,p_partner_key_n,pc_group_type_c,pc_group_name_c),
KEY inx_pc_group_fkcr_key1 
   (s_created_by_c),
KEY inx_pc_group_fkmd_key2 
   (s_modified_by_c),
KEY inx_pc_group_conf_partner0 
   (pc_conference_key_n,p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- outreach travel supplements (by outreach ID)
-- GROUP: conference
CREATE TABLE pc_supplement (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint DEFAULT 0 NOT NULL,
  pc_outreach_type_c varchar(12) NOT NULL,
  pc_supplement_n numeric(24, 10) DEFAULT 0,
    -- Apply conference fee discounts to this supplement
  pc_apply_discounts_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_supplement_pk
    PRIMARY KEY (pc_conference_key_n,pc_outreach_type_c),
  CONSTRAINT pc_supplement_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_supplement_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_supplement_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_supplement_pk0 
   (pc_conference_key_n,pc_outreach_type_c),
KEY inx_pc_supplement_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_supplement_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_supplement_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of building used for accomodation at a conference
-- GROUP: hospitality
CREATE TABLE pc_building (

  p_venue_key_n bigint NOT NULL,
  pc_building_code_c varchar(16) NOT NULL,
    -- This is a long description and is 80 characters long.
  pc_building_desc_c varchar(160),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_building_pk
    PRIMARY KEY (p_venue_key_n,pc_building_code_c),
  CONSTRAINT pc_building_fk1
    FOREIGN KEY (p_venue_key_n)
    REFERENCES p_venue(p_partner_key_n),
  CONSTRAINT pc_building_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_building_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_building_pk0 
   (p_venue_key_n,pc_building_code_c),
KEY inx_pc_building_fk1_key1 
   (p_venue_key_n),
KEY inx_pc_building_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_building_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Details of rooms used for accommodation at a conference
-- GROUP: hospitality
CREATE TABLE pc_room (

  p_venue_key_n bigint NOT NULL,
  pc_building_code_c varchar(16) NOT NULL,
  pc_room_number_c varchar(16) NOT NULL,
  pc_room_name_c varchar(100),
  pc_beds_i integer DEFAULT 0,
  pc_max_occupancy_i integer DEFAULT 0,
  pc_bed_charge_n numeric(24, 10) DEFAULT 0,
  pc_bed_cost_n numeric(24, 10) DEFAULT 0,
  pc_usage_c varchar(32),
    -- Gender that is preferred to use that room
  pc_gender_preference_c varchar(6),
    -- X Position for the room layout designer in pixels
  pc_layout_xpos_i integer,
    -- Y Position for the room layout designer in pixels
  pc_layout_ypos_i integer,
    -- Width for the room layout designer in pixels
  pc_layout_width_i integer,
    -- Height for the room layout designer in pixels
  pc_layout_height_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_room_pk
    PRIMARY KEY (p_venue_key_n,pc_building_code_c,pc_room_number_c),
  CONSTRAINT pc_room_fk1
    FOREIGN KEY (p_venue_key_n,pc_building_code_c)
    REFERENCES pc_building(p_venue_key_n,pc_building_code_c),
  CONSTRAINT pc_room_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_room_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_room_pk0 
   (p_venue_key_n,pc_building_code_c,pc_room_number_c),
KEY inx_pc_room_fk1_key1 
   (p_venue_key_n,pc_building_code_c),
KEY inx_pc_room_fkcr_key2 
   (s_created_by_c),
KEY inx_pc_room_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links rooms to attendees of a conference or a booking in the hospitality module
-- GROUP: hospitality
CREATE TABLE pc_room_alloc (

    -- Surrogate Primary Key; required because there can be several bookings per room, and not all guests might be linked to a partner
  pc_key_i integer NOT NULL,
  p_venue_key_n bigint NOT NULL,
  pc_building_code_c varchar(16) NOT NULL,
  pc_room_number_c varchar(16) NOT NULL,
    -- The room can be reserved for a conference
  pc_conference_key_n bigint,
    -- The partner key of the guest, can be null if group booking (see ph_booking for p_charged_key_n to find who is booking the room)
  p_partner_key_n bigint DEFAULT 0,
    -- This makes the room unavailable for other guests even if not all beds are used
  ph_book_whole_room_l boolean DEFAULT 1 NOT NULL,
    -- number of beds required by this allocation
  ph_number_of_beds_i integer DEFAULT 1,
    -- number of additional beds (e.g. mattrass, childrens cot, etc) required by this allocation
  ph_number_of_overflow_beds_i integer DEFAULT 0,
    -- possible values: couple, family, male, female
  ph_gender_c varchar(40),
  pc_in_d date NOT NULL,
  pc_out_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_room_alloc_pk
    PRIMARY KEY (pc_key_i),
  CONSTRAINT pc_room_alloc_fk1
    FOREIGN KEY (pc_conference_key_n,p_partner_key_n)
    REFERENCES pc_attendee(pc_conference_key_n,p_partner_key_n),
  CONSTRAINT pc_room_alloc_fk2
    FOREIGN KEY (p_venue_key_n,pc_building_code_c,pc_room_number_c)
    REFERENCES pc_room(p_venue_key_n,pc_building_code_c,pc_room_number_c),
  CONSTRAINT pc_room_alloc_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_room_alloc_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_room_alloc_pk0 
   (pc_key_i),
KEY inx_pc_room_alloc_fk1_key1 
   (pc_conference_key_n,p_partner_key_n),
KEY inx_pc_room_alloc_fk2_key2 
   (p_venue_key_n,pc_building_code_c,pc_room_number_c),
KEY inx_pc_room_alloc_fkcr_key3 
   (s_created_by_c),
KEY inx_pc_room_alloc_fkmd_key4 
   (s_modified_by_c),
KEY pc_room_alloc_k0 
   (pc_conference_key_n,p_partner_key_n,pc_in_d),
KEY pc_room_alloc_k21 
   (pc_conference_key_n,p_partner_key_n,p_venue_key_n,pc_in_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Attributes assigned to rooms used for accommodation at a conference
-- GROUP: hospitality
CREATE TABLE pc_room_attribute (

  p_venue_key_n bigint NOT NULL,
  pc_building_code_c varchar(16) NOT NULL,
  pc_room_number_c varchar(16) NOT NULL,
  pc_room_attr_type_code_c varchar(40) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_room_attribute_pk
    PRIMARY KEY (p_venue_key_n,pc_building_code_c,pc_room_number_c,pc_room_attr_type_code_c),
  CONSTRAINT pc_room_attribute_fk1
    FOREIGN KEY (p_venue_key_n,pc_building_code_c,pc_room_number_c)
    REFERENCES pc_room(p_venue_key_n,pc_building_code_c,pc_room_number_c),
  CONSTRAINT pc_room_attribute_fk2
    FOREIGN KEY (pc_room_attr_type_code_c)
    REFERENCES pc_room_attribute_type(pc_code_c),
  CONSTRAINT pc_room_attribute_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_room_attribute_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_room_attribute_pk0 
   (p_venue_key_n,pc_building_code_c,pc_room_number_c,pc_room_attr_type_code_c),
KEY inx_pc_room_attribute_fk1_key1 
   (p_venue_key_n,pc_building_code_c,pc_room_number_c),
KEY inx_pc_room_attribute_fk2_key2 
   (pc_room_attr_type_code_c),
KEY inx_pc_room_attribute_fkcr_key3 
   (s_created_by_c),
KEY inx_pc_room_attribute_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links venues to conferences
-- GROUP: conference
CREATE TABLE pc_conference_venue (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  pc_conference_key_n bigint NOT NULL,
    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_venue_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pc_conference_venue_pk
    PRIMARY KEY (pc_conference_key_n,p_venue_key_n),
  CONSTRAINT pc_conference_venue_fk1
    FOREIGN KEY (pc_conference_key_n)
    REFERENCES pc_conference(pc_conference_key_n),
  CONSTRAINT pc_conference_venue_fk2
    FOREIGN KEY (p_venue_key_n)
    REFERENCES p_venue(p_partner_key_n),
  CONSTRAINT pc_conference_venue_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pc_conference_venue_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pc_conference_venue_pk0 
   (pc_conference_key_n,p_venue_key_n),
KEY inx_pc_conference_venue_fk1_key1 
   (pc_conference_key_n),
KEY inx_pc_conference_venue_fk2_key2 
   (p_venue_key_n),
KEY inx_pc_conference_venue_fkcr_key3 
   (s_created_by_c),
KEY inx_pc_conference_venue_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- make sure charging works for a group or an individual; this summarises all the hospitality services that have to be paid for; also useful for planning meals in the kitchen and room preparation
-- GROUP: hospitality
CREATE TABLE ph_booking (

    -- Surrogate Primary Key; required because there can be several bookings per room and per group
  ph_key_i integer NOT NULL,
    -- the partner key of the visitor or the partner key of the organisation or group that is visiting; each room allocation can refer to the individual guest as well; this can be different from the partner that is charged in the invoice
  p_contact_key_n bigint,
    -- The ledger number that is needed for the invoice
  a_ledger_number_for_invoice_i integer DEFAULT 0,
    -- refers to an offer which will later be the invoice
  a_ar_invoice_key_i integer,
    -- This is a booking for n adults
  ph_number_of_adults_i integer DEFAULT 0,
    -- This is a booking for n children
  ph_number_of_children_i integer DEFAULT 0,
    -- The people that are part of this booking had n breakfasts; also useful for the kitchen
  ph_number_of_breakfast_i integer DEFAULT 0,
    -- The people that are part of this booking had n lunches
  ph_number_of_lunch_i integer DEFAULT 0,
    -- The people that are part of this booking had n suppers
  ph_number_of_supper_i integer DEFAULT 0,
    -- The number of linen that have been provided for this booking
  ph_number_of_linen_needed_i integer DEFAULT 0,
    -- this should be set to the date when the booking has been confirmed; required for early booking discounts
  ph_confirmed_d date,
  ph_in_d date NOT NULL,
  ph_out_d date,
  ph_time_arrival_i integer,
  ph_time_departure_i integer,
    -- Add notes about the stay or special requests by the guest
  ph_notes_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ph_booking_pk
    PRIMARY KEY (ph_key_i),
  CONSTRAINT ph_booking_fk1
    FOREIGN KEY (p_contact_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT ph_booking_fk2
    FOREIGN KEY (a_ledger_number_for_invoice_i,a_ar_invoice_key_i)
    REFERENCES a_ar_invoice(a_ledger_number_i,a_key_i),
  CONSTRAINT ph_booking_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ph_booking_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_ph_room_booking_fk1_ref0 
   (ph_key_i),
UNIQUE KEY inx_ph_booking_pk1 
   (ph_key_i),
KEY inx_ph_booking_fk1_key2 
   (p_contact_key_n),
KEY inx_ph_booking_fk2_key3 
   (a_ledger_number_for_invoice_i,a_ar_invoice_key_i),
KEY inx_ph_booking_fkcr_key4 
   (s_created_by_c),
KEY inx_ph_booking_fkmd_key5 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Links room allocations and a booking
-- GROUP: hospitality
CREATE TABLE ph_room_booking (

    -- details of the booking
  ph_booking_key_i integer NOT NULL,
    -- which room/beds are booked
  ph_room_alloc_key_i integer NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT ph_room_booking_pk
    PRIMARY KEY (ph_booking_key_i,ph_room_alloc_key_i),
  CONSTRAINT ph_room_booking_fk1
    FOREIGN KEY (ph_booking_key_i)
    REFERENCES ph_booking(ph_key_i),
  CONSTRAINT ph_room_booking_fk2
    FOREIGN KEY (ph_room_alloc_key_i)
    REFERENCES pc_room_alloc(pc_key_i),
  CONSTRAINT ph_room_booking_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT ph_room_booking_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_ph_room_booking_pk0 
   (ph_booking_key_i,ph_room_alloc_key_i),
KEY inx_ph_room_booking_fk1_key1 
   (ph_booking_key_i),
KEY inx_ph_room_booking_fk2_key2 
   (ph_room_alloc_key_i),
KEY inx_ph_room_booking_fkcr_key3 
   (s_created_by_c),
KEY inx_ph_room_booking_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Tax reference numbers
-- GROUP: partner
CREATE TABLE p_tax (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Tax or VAT
  p_tax_type_c varchar(16) NOT NULL,
    -- Tax Reference
  p_tax_ref_c varchar(100) NOT NULL,
  p_valid_from_d date,
  p_valid_until_d date,
  p_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_tax_pk
    PRIMARY KEY (p_partner_key_n,p_tax_type_c,p_tax_ref_c),
  CONSTRAINT p_tax_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_tax_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_tax_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_tax_pk0 
   (p_partner_key_n,p_tax_type_c,p_tax_ref_c),
KEY inx_p_tax_fk1_key1 
   (p_partner_key_n),
KEY inx_p_tax_fkcr_key2 
   (s_created_by_c),
KEY inx_p_tax_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Partner Area of Interest
-- GROUP: partner
CREATE TABLE p_partner_interest (

  p_partner_key_n bigint NOT NULL,
    -- Sequence number per Partner (required to make PK)
  p_interest_number_i integer NOT NULL,
    -- The Field the Partner is interested in
  p_field_key_n bigint,
    -- The Country the Partner is interested in
  p_country_c varchar(8),
    -- The Interest the Partner is interested in
  p_interest_c varchar(20),
    -- The level of interest
  p_level_i integer,
  p_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_interest_pk
    PRIMARY KEY (p_partner_key_n,p_interest_number_i),
  CONSTRAINT p_partner_interest_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_interest_fk2
    FOREIGN KEY (p_field_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_partner_interest_fk3
    FOREIGN KEY (p_country_c)
    REFERENCES p_country(p_country_code_c),
  CONSTRAINT p_partner_interest_fk4
    FOREIGN KEY (p_interest_c)
    REFERENCES p_interest(p_interest_c),
  CONSTRAINT p_partner_interest_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_interest_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_interest_pk0 
   (p_partner_key_n,p_interest_number_i),
KEY inx_p_partner_interest_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_interest_fk2_key2 
   (p_field_key_n),
KEY inx_p_partner_interest_fk3_key3 
   (p_country_c),
KEY inx_p_partner_interest_fk4_key4 
   (p_interest_c),
KEY inx_p_partner_interest_fkcr_key5 
   (s_created_by_c),
KEY inx_p_partner_interest_fkmd_key6 
   (s_modified_by_c),
KEY p_partner_interest_k10 
   (p_partner_key_n,p_field_key_n),
KEY p_partner_interest_k21 
   (p_partner_key_n,p_country_c),
KEY p_partner_interest_k32 
   (p_partner_key_n,p_interest_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Partner merge history
-- GROUP: partner
CREATE TABLE p_partner_merge (

    -- The partner that was merged.
  p_merge_from_n bigint NOT NULL,
    -- The partner they were merged into.
  p_merge_to_n bigint,
    -- The user who did the merge.
  s_merged_by_c varchar(20),
    -- Date of merge.
  s_merge_date_d date,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_merge_pk
    PRIMARY KEY (p_merge_from_n),
  CONSTRAINT p_partner_merge_fk1
    FOREIGN KEY (p_merge_from_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_merge_fk2
    FOREIGN KEY (p_merge_to_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_merge_fk3
    FOREIGN KEY (s_merged_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_merge_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_merge_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_merge_pk0 
   (p_merge_from_n),
KEY inx_p_partner_merge_fk2_key1 
   (p_merge_to_n),
KEY inx_p_partner_merge_fk3_key2 
   (s_merged_by_c),
KEY inx_p_partner_merge_fkcr_key3 
   (s_created_by_c),
KEY inx_p_partner_merge_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A reminder that can be linked to a Partner or Partner contact
-- GROUP: partner
CREATE TABLE p_partner_reminder (

    -- Primary Key of this reminder
  p_partner_reminder_id_i integer NOT NULL,
    -- Partner key of Partner to which the reminder is related
  p_partner_key_n bigint NOT NULL,
    -- ID of Contact to which reminder relates. Null means that reminder relates just to a Partner
  p_contact_id_i bigint,
    -- ID of this reminder (only unique per Partner/Contact)
  p_reminder_id_i integer DEFAULT 0 NOT NULL,
    -- The user that requires this reminder
  s_user_id_c varchar(20),
    -- This is a category, by which reminders can be grouped.
  p_category_code_c varchar(40),
    -- Type of action to take on getting the reminder (eg. Email etc)
  p_action_type_c varchar(16),
    -- Reason for the reminder (eg. birthday, etc.)
  p_reminder_reason_c varchar(600),
    -- Additional Comments
  p_comment_c varchar(5000),
    -- Date of event that reminder is about (if the reminder relates to a specific event like a birthday).
  p_event_date_d date NOT NULL,
    -- Date on which to send/display first reminder.
  p_first_reminder_date_d date,
    -- Frequency (in days) with which re-reminders should be sent.
  p_reminder_frequency_i integer DEFAULT 0,
    -- Date on which the last reminder was sent
  p_last_reminder_sent_d date,
    -- Date on which the next reminder will be sent
  p_next_reminder_date_d date,
    -- Is this reminder still active?
  p_reminder_active_l boolean DEFAULT 1 NOT NULL,
    -- Email address to which reminder should be sent
  p_email_address_c varchar(1000),
    -- Indicates whether or not the contact has restricted access. If it does then the access will be controlled by s_group_partner_reminder
  p_restricted_l boolean DEFAULT 0,
    -- Identifies a module. A module is any part of aprogram which is related to each menu entry or to the sub-system. Eg, partner administration, AP, AR etc.
  s_module_id_c varchar(20),
    -- If set, this contact is restricted to one user.
  s_user_restriction_c varchar(20),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_reminder_pk
    PRIMARY KEY (p_partner_reminder_id_i),
  CONSTRAINT p_partner_reminder_uk
    UNIQUE (p_partner_key_n,p_contact_id_i,p_reminder_id_i),
  CONSTRAINT p_partner_reminder_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_reminder_fk2
    FOREIGN KEY (p_contact_id_i)
    REFERENCES p_contact_log(p_contact_log_id_i),
  CONSTRAINT p_partner_reminder_fk3
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_reminder_fk4
    FOREIGN KEY (s_module_id_c)
    REFERENCES s_module(s_module_id_c),
  CONSTRAINT p_partner_reminder_fk5
    FOREIGN KEY (s_user_restriction_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_reminder_fk6
    FOREIGN KEY (p_category_code_c)
    REFERENCES p_reminder_category(p_code_c),
  CONSTRAINT p_partner_reminder_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_reminder_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_reminder_pk0 
   (p_partner_reminder_id_i),
UNIQUE KEY inx_p_partner_reminder_uk1 
   (p_partner_key_n,p_contact_id_i,p_reminder_id_i),
KEY inx_p_partner_reminder_fk1_key2 
   (p_partner_key_n),
KEY inx_p_partner_reminder_fk2_key3 
   (p_contact_id_i),
KEY inx_p_partner_reminder_fk3_key4 
   (s_user_id_c),
KEY inx_p_partner_reminder_fk4_key5 
   (s_module_id_c),
KEY inx_p_partner_reminder_fk5_key6 
   (s_user_restriction_c),
KEY inx_p_partner_reminder_fk6_key7 
   (p_category_code_c),
KEY inx_p_partner_reminder_fkcr_key8 
   (s_created_by_c),
KEY inx_p_partner_reminder_fkmd_key9 
   (s_modified_by_c),
KEY p_partner_reminder_k0 
   (p_partner_key_n,p_next_reminder_date_d)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Tracks the current gift destination for the Partner and maintains a history
-- GROUP: partner
CREATE TABLE p_partner_gift_destination (

    -- Surrogate Primary Key
  p_key_i integer NOT NULL,
    -- Partner key of Partner to which the field is assigned
  p_partner_key_n bigint NOT NULL,
    -- Field to which Partner is assigned
  p_field_key_n bigint NOT NULL,
    -- Date from which field assignment is valid
  p_date_effective_d date NOT NULL,
    -- Date on which field assignment expires
  p_date_expires_d date,
    -- Is the field assignment currently valid
  p_active_l boolean,
    -- Is this field the default gift destination?
  p_default_gift_destination_l boolean,
    -- Is this the field for a person or a family?
  p_partner_class_c varchar(24),
    -- Needed if this field assignment relates to a specific commitment
  p_commitment_site_key_n bigint,
    -- Needed if this field assignment relates to a specific commitment
  p_commitment_key_n bigint,
    -- Any comments relating to this field assignment
  p_comment_c varchar(400),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_gift_destination_pk
    PRIMARY KEY (p_key_i),
  CONSTRAINT p_partner_gift_destination_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_gift_destination_fk2
    FOREIGN KEY (p_field_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_partner_gift_destination_fk3
    FOREIGN KEY (p_partner_class_c)
    REFERENCES p_partner_classes(p_partner_class_c),
  CONSTRAINT p_partner_gift_destination_fk4
    FOREIGN KEY (p_commitment_site_key_n,p_commitment_key_n)
    REFERENCES pm_staff_data(p_site_key_n,pm_key_n),
  CONSTRAINT p_partner_gift_destination_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_gift_destination_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx__partner_gift_destination_pk0 
   (p_key_i),
KEY inx_ner_gift_destination_fk1_key1 
   (p_partner_key_n),
KEY inx_ner_gift_destination_fk2_key2 
   (p_field_key_n),
KEY inx_ner_gift_destination_fk3_key3 
   (p_partner_class_c),
KEY inx_ner_gift_destination_fk4_key4 
   (p_commitment_site_key_n,p_commitment_key_n),
KEY inx_er_gift_destination_fkcr_key5 
   (s_created_by_c),
KEY inx_er_gift_destination_fkmd_key6 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores one or more short codes for a Partner (eg. a donor or recipient code for online giving system)
-- GROUP: partner
CREATE TABLE p_partner_short_code (

    -- Partner key of Partner to which short code applies
  p_partner_key_n bigint NOT NULL,
    -- The short code which applies to the Partner
  p_partner_short_code_c varchar(30) NOT NULL,
    -- Field to which Partner is assigned
  p_field_key_n bigint NOT NULL,
    -- Is this a short code to identify a recipient?
  p_recipient_flag_l boolean,
    -- Is this a short code to identify a donor?
  p_donor_flag_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_short_code_pk
    PRIMARY KEY (p_partner_key_n,p_partner_short_code_c),
  CONSTRAINT p_partner_short_code_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_short_code_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_short_code_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_short_code_pk0 
   (p_partner_key_n,p_partner_short_code_c),
KEY inx_p_partner_short_code_fk1_key1 
   (p_partner_key_n),
KEY inx__partner_short_code_fkcr_key2 
   (s_created_by_c),
KEY inx__partner_short_code_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A particular state in which the Partner is or has been. The Partner may be in more than one state at one time if there are two different processes relating to them.
-- GROUP: partner
CREATE TABLE p_partner_state (

    -- Partner key of Partner to which state applies
  p_partner_key_n bigint NOT NULL,
    -- Unique identifier of this state for this partner
  p_state_index_i integer NOT NULL,
    -- Process that the state applies to
  p_process_code_c varchar(60),
    -- State of the Partner within the process
  p_state_code_c varchar(60),
    -- State of the Partner (freetext)
  p_state_freeform_c varchar(400),
    -- When did the Partner enter this state?
  p_state_start_date_d date,
    -- When will/did the Partner exit this state?
  p_state_end_date_d date,
    -- Has the Partner left this state?
  p_state_complete_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_state_pk
    PRIMARY KEY (p_partner_key_n,p_state_index_i),
  CONSTRAINT p_partner_state_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_state_fk2
    FOREIGN KEY (p_process_code_c,p_state_code_c)
    REFERENCES p_state(p_process_code_c,p_state_code_c),
  CONSTRAINT p_partner_state_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_state_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_state_pk0 
   (p_partner_key_n,p_state_index_i),
KEY inx_p_partner_state_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_state_fk2_key2 
   (p_process_code_c,p_state_code_c),
KEY inx_p_partner_state_fkcr_key3 
   (s_created_by_c),
KEY inx_p_partner_state_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A particular action which has been or needs to be applied to a Partner
-- GROUP: partner
CREATE TABLE p_partner_action (

    -- Partner key of Partner to which action applies
  p_partner_key_n bigint NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Sequential identifier for this action for this partner
  p_action_number_i integer NOT NULL,
    -- Process that the action is part of
  p_process_code_c varchar(60),
    -- Action to be applied to Partner
  p_action_code_c varchar(60),
    -- Action to be applied to Partner (freetext)
  p_action_freeform_c varchar(400),
    -- Date by which action should be performed. Reminders could be set up based on this
  p_perform_by_date_d date,
    -- Has the action already been completed?
  p_action_complete_l boolean,
    -- When was the action completed?
  p_action_complete_date_d date,
    -- Who should perform this action?
  p_user_to_perform_action_c varchar(20),
    -- Which group should perform this action?
  p_group_to_perform_action_c varchar(20),
    -- Who actually performed this action?
  p_user_that_performed_action_c varchar(20),
    -- Part of FK link to p_partner_reminder so can link this action to a reminder
  p_contact_id_i bigint,
    -- Part of FK link to p_partner_reminder so can link this action to a reminder
  p_reminder_id_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_action_pk
    PRIMARY KEY (p_partner_key_n,p_action_number_i),
  CONSTRAINT p_partner_action_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_action_fk2
    FOREIGN KEY (p_process_code_c,p_action_code_c)
    REFERENCES p_action(p_process_code_c,p_action_code_c),
  CONSTRAINT p_partner_action_fk3
    FOREIGN KEY (p_user_to_perform_action_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_action_fk4
    FOREIGN KEY (p_group_to_perform_action_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT p_partner_action_fk5
    FOREIGN KEY (p_user_that_performed_action_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_action_fk6
    FOREIGN KEY (p_partner_key_n,p_contact_id_i,p_reminder_id_i)
    REFERENCES p_partner_reminder(p_partner_key_n,p_contact_id_i,p_reminder_id_i),
  CONSTRAINT p_partner_action_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_action_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_action_pk0 
   (p_partner_key_n,p_action_number_i),
KEY inx_p_partner_action_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_action_fk2_key2 
   (p_process_code_c,p_action_code_c),
KEY inx_p_partner_action_fk3_key3 
   (p_user_to_perform_action_c),
KEY inx_p_partner_action_fk4_key4 
   (p_group_to_perform_action_c,s_group_unit_key_n),
KEY inx_p_partner_action_fk5_key5 
   (p_user_that_performed_action_c),
KEY inx_p_partner_action_fk6_key6 
   (p_partner_key_n,p_contact_id_i,p_reminder_id_i),
KEY inx_p_partner_action_fkcr_key7 
   (s_created_by_c),
KEY inx_p_partner_action_fkmd_key8 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Contains all Petra functions to which access can be granted or denied
-- GROUP: sysman
CREATE TABLE s_function (

    -- Identifier for the particular function
  s_function_id_c varchar(30) NOT NULL,
    -- Petra Module which contains the function
  s_module_name_c varchar(200) NOT NULL,
    -- Petra Sub-Module which contains the function
  s_sub_module_name_c varchar(200),
    -- Function name
  s_function_name_c varchar(200) NOT NULL,
    -- Filename associated with the function
  s_filename_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_function_pk
    PRIMARY KEY (s_function_id_c),
  CONSTRAINT s_function_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_function_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_s_group_function_fk2_ref0 
   (s_function_id_c),
UNIQUE KEY inx_s_function_pk1 
   (s_function_id_c),
KEY inx_s_function_fkcr_key2 
   (s_created_by_c),
KEY inx_s_function_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Gives the group access to a particular function
-- GROUP: sysman
CREATE TABLE s_group_function (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Identifier for the particular function
  s_function_id_c varchar(30) NOT NULL,
    -- Does the group have access to this function?
  s_can_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_function_pk
    PRIMARY KEY (s_group_id_c,s_unit_key_n,s_function_id_c),
  CONSTRAINT s_group_function_fk1
    FOREIGN KEY (s_group_id_c,s_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_function_fk2
    FOREIGN KEY (s_function_id_c)
    REFERENCES s_function(s_function_id_c),
  CONSTRAINT s_group_function_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_function_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_function_pk0 
   (s_group_id_c,s_unit_key_n,s_function_id_c),
KEY inx_s_group_function_fk1_key1 
   (s_group_id_c,s_unit_key_n),
KEY inx_s_group_function_fk2_key2 
   (s_function_id_c),
KEY inx_s_group_function_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_function_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Associates groups with roles
-- GROUP: sysman
CREATE TABLE s_job_group (

    -- Name of the position.
  pt_position_name_c varchar(60) NOT NULL,
    -- Scope of the position.
  pt_position_scope_c varchar(24) NOT NULL,
    -- Position record identifier
  um_job_key_i integer NOT NULL,
    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that this role/group assignment relates to
  s_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_job_group_pk
    PRIMARY KEY (pt_position_name_c,pt_position_scope_c,um_job_key_i,s_group_id_c,s_unit_key_n),
  CONSTRAINT s_job_group_fk1
    FOREIGN KEY (s_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i)
    REFERENCES um_job(pm_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
  CONSTRAINT s_job_group_fk2
    FOREIGN KEY (s_group_id_c,s_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_job_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_job_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_job_group_pk0 
   (pt_position_name_c,pt_position_scope_c,um_job_key_i,s_group_id_c,s_unit_key_n),
KEY inx_s_job_group_fk1_key1 
   (s_unit_key_n,pt_position_name_c,pt_position_scope_c,um_job_key_i),
KEY inx_s_job_group_fk2_key2 
   (s_group_id_c,s_unit_key_n),
KEY inx_s_job_group_fkcr_key3 
   (s_created_by_c),
KEY inx_s_job_group_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Defines a Partner set
-- GROUP: partner
CREATE TABLE p_partner_set (

    -- Identifier for the Partner Set
  p_partner_set_id_c varchar(40) NOT NULL,
    -- Field that the Partner Set relates to
  p_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Name of the Partner set
  p_partner_set_name_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_set_pk
    PRIMARY KEY (p_partner_set_id_c,p_unit_key_n),
  CONSTRAINT p_partner_set_fk1
    FOREIGN KEY (p_unit_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_partner_set_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_set_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_s_group_partner_set_fk2_ref0 
   (p_partner_set_id_c,p_unit_key_n),
UNIQUE KEY inx_p_partner_set_pk1 
   (p_partner_set_id_c,p_unit_key_n),
KEY inx_p_partner_set_fk1_key2 
   (p_unit_key_n),
KEY inx_p_partner_set_fkcr_key3 
   (s_created_by_c),
KEY inx_p_partner_set_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Associates a Group with a Partner Set. This may be an inclusive or exclusive association and may be read, write, delete
-- GROUP: sysman
CREATE TABLE s_group_partner_set (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Identifier for the Partner Set
  p_partner_set_id_c varchar(40) NOT NULL,
    -- Field that the Partner Set relates to
  p_partner_set_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Is the association between the Group and Partner Set inclusive or exclusive
  s_inclusive_or_exclusive_l boolean NOT NULL,
    -- If the association is inclusive this will grant read access to the Partners, otherwise it will deny read access to the Partners
  s_read_access_l boolean,
    -- If the association is inclusive this will grant write access to the Partners, otherwise it will deny write access to the Partners
  s_write_access_l boolean,
    -- If the association is inclusive this will grant delete access to the Partners, otherwise it will deny delete access to the Partners
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_partner_set_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_partner_set_id_c,p_partner_set_unit_key_n),
  CONSTRAINT s_group_partner_set_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_partner_set_fk2
    FOREIGN KEY (p_partner_set_id_c,p_partner_set_unit_key_n)
    REFERENCES p_partner_set(p_partner_set_id_c,p_unit_key_n),
  CONSTRAINT s_group_partner_set_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_partner_set_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_partner_set_pk0 
   (s_group_id_c,s_group_unit_key_n,p_partner_set_id_c,p_partner_set_unit_key_n),
KEY inx_s_group_partner_set_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_partner_set_fk2_key2 
   (p_partner_set_id_c,p_partner_set_unit_key_n),
KEY inx_s_group_partner_set_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_partner_set_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Places Partners in a Partner set
-- GROUP: partner
CREATE TABLE p_partner_set_partner (

    -- Identifier for the Partner Set
  p_partner_set_id_c varchar(40) NOT NULL,
    -- Field that the Partner Set relates to
  p_partner_set_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Partner Key
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_set_partner_pk
    PRIMARY KEY (p_partner_set_id_c,p_partner_set_unit_key_n,p_partner_key_n),
  CONSTRAINT p_partner_set_partner_fk1
    FOREIGN KEY (p_partner_set_id_c,p_partner_set_unit_key_n)
    REFERENCES p_partner_set(p_partner_set_id_c,p_unit_key_n),
  CONSTRAINT p_partner_set_partner_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_set_partner_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_set_partner_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_set_partner_pk0 
   (p_partner_set_id_c,p_partner_set_unit_key_n,p_partner_key_n),
KEY inx__partner_set_partner_fk1_key1 
   (p_partner_set_id_c,p_partner_set_unit_key_n),
KEY inx__partner_set_partner_fk2_key2 
   (p_partner_key_n),
KEY inx_partner_set_partner_fkcr_key3 
   (s_created_by_c),
KEY inx_partner_set_partner_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific gift, if the gift is restricted
-- GROUP: sysman
CREATE TABLE s_group_gift (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Ledger Number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- identifies which batch a transaction belongs to
  a_batch_number_i integer DEFAULT 0 NOT NULL,
    -- Identifies a transaction within a journal within a batch within a ledger
  a_gift_transaction_number_i integer DEFAULT 0 NOT NULL,
    -- Control read access to the gift
  s_read_access_l boolean,
    -- Control write access to the gift
  s_write_access_l boolean,
    -- Control delete access to the gift
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_gift_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT s_group_gift_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_gift_fk2
    FOREIGN KEY (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i)
    REFERENCES a_gift(a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
  CONSTRAINT s_group_gift_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_gift_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_gift_pk0 
   (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_s_group_gift_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_gift_fk2_key2 
   (a_ledger_number_i,a_batch_number_i,a_gift_transaction_number_i),
KEY inx_s_group_gift_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_gift_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to gifts with a specific motivation
-- GROUP: sysman
CREATE TABLE s_group_motivation (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Ledger Number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- This defines a motivation group.
  a_motivation_group_code_c varchar(16) NOT NULL,
    -- This defines the motivation detail within a motivation group.
  a_motivation_detail_code_c varchar(16) NOT NULL,
    -- Control read access to gifts with this motivation
  s_read_access_l boolean,
    -- Control write access to gifts with this motivation
  s_write_access_l boolean,
    -- Control delete access to gifts with this motivation
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_motivation_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT s_group_motivation_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_motivation_fk2
    FOREIGN KEY (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT s_group_motivation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_motivation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_motivation_pk0 
   (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_s_group_motivation_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_motivation_fk2_key2 
   (a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
KEY inx_s_group_motivation_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_motivation_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific Partner contact, if the contact is restricted
-- GROUP: sysman
CREATE TABLE s_group_partner_contact (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- identifying key for p_contact_log
  p_contact_id_i bigint DEFAULT 0 NOT NULL,
    -- Control read access to the contact
  s_read_access_l boolean,
    -- Control write access to the contact
  s_write_access_l boolean,
    -- Control delete access to the contact
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_partner_contact_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_contact_id_i),
  CONSTRAINT s_group_partner_contact_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_partner_contact_fk2
    FOREIGN KEY (p_contact_id_i)
    REFERENCES p_contact_log(p_contact_log_id_i),
  CONSTRAINT s_group_partner_contact_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_partner_contact_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_partner_contact_pk0 
   (s_group_id_c,s_group_unit_key_n,p_contact_id_i),
KEY inx_roup_partner_contact_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_roup_partner_contact_fk2_key2 
   (p_contact_id_i),
KEY inx_oup_partner_contact_fkcr_key3 
   (s_created_by_c),
KEY inx_oup_partner_contact_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific Partner reminder, if the reminder is restricted
-- GROUP: sysman
CREATE TABLE s_group_partner_reminder (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Primary Key of this reminder
  p_partner_reminder_id_i integer NOT NULL,
    -- Control read access to the reminder
  s_read_access_l boolean,
    -- Control write access to the reminder
  s_write_access_l boolean,
    -- Control delete access to the reminder
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_partner_reminder_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_partner_reminder_id_i),
  CONSTRAINT s_group_partner_reminder_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_partner_reminder_fk2
    FOREIGN KEY (p_partner_reminder_id_i)
    REFERENCES p_partner_reminder(p_partner_reminder_id_i),
  CONSTRAINT s_group_partner_reminder_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_partner_reminder_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_partner_reminder_pk0 
   (s_group_id_c,s_group_unit_key_n,p_partner_reminder_id_i),
KEY inx_oup_partner_reminder_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_oup_partner_reminder_fk2_key2 
   (p_partner_reminder_id_i),
KEY inx_up_partner_reminder_fkcr_key3 
   (s_created_by_c),
KEY inx_up_partner_reminder_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific location, if the Location is restricted
-- GROUP: sysman
CREATE TABLE s_group_location (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- This is the key that tell what site created this location, it will help to merge addresses when doing imports
  p_site_key_n bigint DEFAULT 0 NOT NULL,
  p_location_key_i integer DEFAULT 0 NOT NULL,
    -- Control read access to the location
  s_read_access_l boolean,
    -- Control write access to the location
  s_write_access_l boolean,
    -- Control delete access to the location
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_location_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_site_key_n,p_location_key_i),
  CONSTRAINT s_group_location_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_location_fk2
    FOREIGN KEY (p_site_key_n,p_location_key_i)
    REFERENCES p_location(p_site_key_n,p_location_key_i),
  CONSTRAINT s_group_location_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_location_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_location_pk0 
   (s_group_id_c,s_group_unit_key_n,p_site_key_n,p_location_key_i),
KEY inx_s_group_location_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_location_fk2_key2 
   (p_site_key_n,p_location_key_i),
KEY inx_s_group_location_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_location_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific partner location, if the Partner Location is restricted
-- GROUP: sysman
CREATE TABLE s_group_partner_location (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Partner key of Partner to which the location is related
  p_partner_key_n bigint NOT NULL,
    -- This is the key that tell what site created this location, it will help to merge addresses when doing imports
  p_site_key_n bigint DEFAULT 0 NOT NULL,
  p_location_key_i integer DEFAULT 0 NOT NULL,
    -- Control read access to the partner location
  s_read_access_l boolean,
    -- Control write access to the partner location
  s_write_access_l boolean,
    -- Control delete access to the partner location
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_partner_location_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_partner_key_n,p_site_key_n,p_location_key_i),
  CONSTRAINT s_group_partner_location_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_partner_location_fk2
    FOREIGN KEY (p_partner_key_n,p_site_key_n,p_location_key_i)
    REFERENCES p_partner_location(p_partner_key_n,p_site_key_n,p_location_key_i),
  CONSTRAINT s_group_partner_location_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_partner_location_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_partner_location_pk0 
   (s_group_id_c,s_group_unit_key_n,p_partner_key_n,p_site_key_n,p_location_key_i),
KEY inx_oup_partner_location_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_oup_partner_location_fk2_key2 
   (p_partner_key_n,p_site_key_n,p_location_key_i),
KEY inx_up_partner_location_fkcr_key3 
   (s_created_by_c),
KEY inx_up_partner_location_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Controls the access that a group has to a specific Data Label, if the Data Label is restricted
-- GROUP: sysman
CREATE TABLE s_group_data_label (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Identifying key for p_data_label
  p_data_label_key_i integer NOT NULL,
    -- Control read access to the data label
  s_read_access_l boolean,
    -- Control write access to the data label
  s_write_access_l boolean,
    -- Control delete access to the data label
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_data_label_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_data_label_key_i),
  CONSTRAINT s_group_data_label_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_data_label_fk2
    FOREIGN KEY (p_data_label_key_i)
    REFERENCES p_data_label(p_key_i),
  CONSTRAINT s_group_data_label_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_data_label_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_data_label_pk0 
   (s_group_id_c,s_group_unit_key_n,p_data_label_key_i),
KEY inx_s_group_data_label_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_data_label_fk2_key2 
   (p_data_label_key_i),
KEY inx_s_group_data_label_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_data_label_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Gives a group access to a specific ledger
-- GROUP: sysman
CREATE TABLE s_group_ledger (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Ledger Number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_ledger_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,a_ledger_number_i),
  CONSTRAINT s_group_ledger_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_ledger_fk2
    FOREIGN KEY (a_ledger_number_i)
    REFERENCES a_ledger(a_ledger_number_i),
  CONSTRAINT s_group_ledger_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_ledger_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_ledger_pk0 
   (s_group_id_c,s_group_unit_key_n,a_ledger_number_i),
KEY inx_s_group_ledger_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_ledger_fk2_key2 
   (a_ledger_number_i),
KEY inx_s_group_ledger_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_ledger_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Gives a group access to a specific cost centre (so that people without other finance access can access data on their own department)
-- GROUP: sysman
CREATE TABLE s_group_cost_centre (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Ledger Number
  a_ledger_number_i integer DEFAULT 0 NOT NULL,
    -- Cost Centre
  a_cost_centre_code_c varchar(24) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_cost_centre_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT s_group_cost_centre_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_cost_centre_fk2
    FOREIGN KEY (a_ledger_number_i,a_cost_centre_code_c)
    REFERENCES a_cost_centre(a_ledger_number_i,a_cost_centre_code_c),
  CONSTRAINT s_group_cost_centre_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_cost_centre_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_cost_centre_pk0 
   (s_group_id_c,s_group_unit_key_n,a_ledger_number_i,a_cost_centre_code_c),
KEY inx_s_group_cost_centre_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_cost_centre_fk2_key2 
   (a_ledger_number_i,a_cost_centre_code_c),
KEY inx_s_group_cost_centre_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_cost_centre_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Gives a group access to a specific extract
-- GROUP: sysman
CREATE TABLE s_group_extract (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- Identifier for the extract
  m_extract_id_i integer DEFAULT 0 NOT NULL,
    -- Control read access to the extract
  s_read_access_l boolean,
    -- Control write access to the extract
  s_write_access_l boolean,
    -- Control delete access to the extract
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_extract_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,m_extract_id_i),
  CONSTRAINT s_group_extract_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_extract_fk2
    FOREIGN KEY (m_extract_id_i)
    REFERENCES m_extract_master(m_extract_id_i),
  CONSTRAINT s_group_extract_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_extract_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_extract_pk0 
   (s_group_id_c,s_group_unit_key_n,m_extract_id_i),
KEY inx_s_group_extract_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_extract_fk2_key2 
   (m_extract_id_i),
KEY inx_s_group_extract_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_extract_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Records all database events (ie. insert, update, delete). Gets purged after each export
-- GROUP: sysman
CREATE TABLE s_change_event (

    -- Name of the database table where the event occurred
  s_table_name_c varchar(64) NOT NULL,
    -- Rowid of the record that the event applied to
  s_rowid_c varchar(40) NOT NULL,
    -- Type of event (I, U or D - Insert, Update, Delete)
  s_change_type_c varchar(2) NOT NULL,
    -- Concatenation of the natural key values for the affected record (the primary key is not enough where a surrogate key is used as this is not meaningful across sites)
  s_natural_key_c varchar(2000) NOT NULL,
    -- Date on which the event took place
  s_date_d date NOT NULL,
    -- Time at which event took place
  s_time_i integer NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_change_event_pk
    PRIMARY KEY (s_table_name_c,s_rowid_c),
  CONSTRAINT s_change_event_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_change_event_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_change_event_pk0 
   (s_table_name_c,s_rowid_c),
KEY inx_s_change_event_fkcr_key1 
   (s_created_by_c),
KEY inx_s_change_event_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Attributes for label paper
-- GROUP: sysman
CREATE TABLE s_label (

  p_label_name_c varchar(20) NOT NULL,
  s_label_description_c varchar(80),
    -- The form the label is designed for
  s_form_name_c varchar(20) NOT NULL,
    -- The disance from the top of the page to the top of the first label.
  s_top_margin_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The distance from the left edge of the page to the left edge of the first label.
  s_side_margin_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The distance from the top of the first label to the top of the next label.
  s_vertical_pitch_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The distance from the left edge of the first label to the left edge of the next label.
  s_horizontal_pitch_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The height of each label.
  s_label_height_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The width of each label.
  s_label_width_n numeric(6, 2) DEFAULT 0 NOT NULL,
    -- The number of labels across a page.
  s_labels_across_i integer DEFAULT 0 NOT NULL,
    -- The number of labels down a page.
  s_labels_down_i integer DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_label_pk
    PRIMARY KEY (p_label_name_c),
  CONSTRAINT s_label_fk1
    FOREIGN KEY (s_form_name_c)
    REFERENCES s_form(s_form_name_c),
  CONSTRAINT s_label_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_label_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_label_pk0 
   (p_label_name_c),
KEY inx_s_label_fk1_key1 
   (s_form_name_c),
KEY inx_s_label_fkcr_key2 
   (s_created_by_c),
KEY inx_s_label_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Allows multiple as well as arbitrary-length partner comments.
-- GROUP: partner
CREATE TABLE p_partner_comment (

  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Record Index that enables multiple comments per Partner
  p_index_i integer NOT NULL,
    -- Sequence number (is necessary to concatenate records to one comment)
  p_sequence_i integer NOT NULL,
    -- Comment
  p_comment_c longtext,
    -- Comment Type
  p_comment_type_c varchar(40),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_comment_pk
    PRIMARY KEY (p_partner_key_n,p_index_i,p_sequence_i),
  CONSTRAINT p_partner_comment_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_comment_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_comment_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_comment_pk0 
   (p_partner_key_n,p_index_i,p_sequence_i),
KEY inx_p_partner_comment_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_comment_fkcr_key2 
   (s_created_by_c),
KEY inx_p_partner_comment_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Submission type for foundation proposals e.g. EMAIL, LETTER.
-- GROUP: partner
CREATE TABLE p_proposal_submission_type (

  p_submission_type_code_c varchar(30) NOT NULL,
  p_submission_type_description_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_submission_type_pk
    PRIMARY KEY (p_submission_type_code_c),
  CONSTRAINT p_proposal_submission_type_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_proposal_submission_type_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_submission_type_pk0 
   (p_submission_type_code_c),
KEY inx_sal_submission_type_fkcr_key1 
   (s_created_by_c),
KEY inx_sal_submission_type_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Foundations - a type of ORGANISATION for fundraising proposals.
-- GROUP: partner
CREATE TABLE p_foundation (

    -- ORGANISATION key of the foundation.
  p_partner_key_n bigint NOT NULL,
    -- The Financial Development person who handles appeals to this foundation. Anyone else must get permission from the owner to contact the foundation.
  p_owner_1_key_n bigint,
    -- A second F.D. person who may do business with the foundation.
  p_owner_2_key_n bigint,
    -- Name of the contact for this Foundation
  p_key_contact_name_c varchar(70),
    -- The contact person's title.
  p_key_contact_title_c varchar(70),
    -- Contact email address
  p_key_contact_email_c varchar(120),
    -- Contact's phone number
  p_key_contact_phone_c varchar(40),
    -- Not part of original specification - provided in case contact does ever need to be a full Partner.
  p_contact_partner_n bigint,
  p_special_requirements_c varchar(700),
  p_proposal_formatting_c varchar(700),
    -- A lookup table would go with this. e.g. EMAIL, LETTER.
  p_proposal_submission_type_c varchar(30),
  p_special_instructions_c varchar(700),
    -- (Monthly | Quarterly | Annually)  These are the only ones we use now.
  p_review_frequency_c varchar(20),
    -- (Annually | Bi-Annually | No Restrictions)  This is a static list.
  p_submit_frequency_c varchar(30),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_foundation_pk
    PRIMARY KEY (p_partner_key_n),
  CONSTRAINT p_foundation_contact_fk1
    FOREIGN KEY (p_contact_partner_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_foundation_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_organisation(p_partner_key_n),
  CONSTRAINT p_foundation_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_owner_fk1
    FOREIGN KEY (p_owner_1_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_foundation_submission_fk1
    FOREIGN KEY (p_proposal_submission_type_c)
    REFERENCES p_proposal_submission_type(p_submission_type_code_c),
  CONSTRAINT p_foundation_owner_fk2
    FOREIGN KEY (p_owner_2_key_n)
    REFERENCES p_partner(p_partner_key_n),
UNIQUE KEY inx_p_foundation_pk0 
   (p_partner_key_n),
KEY inx_p_foundation_contact_fk1_key1 
   (p_contact_partner_n),
KEY inx_p_foundation_fkcr_key2 
   (s_created_by_c),
KEY inx_p_foundation_fkmd_key3 
   (s_modified_by_c),
KEY inx_p_foundation_owner_fk1_key4 
   (p_owner_1_key_n),
KEY inx_oundation_submission_fk1_key5 
   (p_proposal_submission_type_c),
KEY inx_p_foundation_owner_fk2_key6 
   (p_owner_2_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Foundation proposal status codes and descriptions
-- GROUP: partner
CREATE TABLE p_foundation_proposal_status (

  p_status_code_c varchar(30) NOT NULL,
  p_status_description_c varchar(80),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_proposal_status_pk
    PRIMARY KEY (p_status_code_c),
  CONSTRAINT p_foundation_proposal_status_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_proposal_status_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_proposal_status_pk0 
   (p_status_code_c),
KEY inx_ion_proposal_status_fkcr_key1 
   (s_created_by_c),
KEY inx_ion_proposal_status_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Proposals submitted to a Foundation
-- GROUP: partner
CREATE TABLE p_foundation_proposal (

  p_foundation_partner_key_n bigint NOT NULL,
  p_foundation_proposal_key_i integer NOT NULL,
  p_proposal_status_c varchar(30),
  p_proposal_notes_c varchar(700),
  p_submitted_date_d date,
  p_amount_requested_n numeric(19, 2),
  p_amount_approved_n numeric(19, 2),
  p_amount_granted_n numeric(19, 2),
  p_date_granted_d date,
    -- The F.D. person who sent the proposal
  p_partner_submitted_by_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_proposal_pk
    PRIMARY KEY (p_foundation_partner_key_n,p_foundation_proposal_key_i),
  CONSTRAINT p_foundation_proposal_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_proposal_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_proposal_status_fk1
    FOREIGN KEY (p_foundation_partner_key_n)
    REFERENCES p_foundation(p_partner_key_n),
  CONSTRAINT p_proposal_status_fk2
    FOREIGN KEY (p_proposal_status_c)
    REFERENCES p_foundation_proposal_status(p_status_code_c),
  CONSTRAINT p_proposal_submitted_fk3
    FOREIGN KEY (p_partner_submitted_by_n)
    REFERENCES p_partner(p_partner_key_n),
UNIQUE KEY inx_p_proposal_pk0 
   (p_foundation_partner_key_n,p_foundation_proposal_key_i),
KEY inx_foundation_proposal_fkcr_key1 
   (s_created_by_c),
KEY inx_foundation_proposal_fkmd_key2 
   (s_modified_by_c),
KEY inx_p_proposal_status_fk1_key3 
   (p_foundation_partner_key_n),
KEY inx_p_proposal_status_fk2_key4 
   (p_proposal_status_c),
KEY inx_p_proposal_submitted_fk3_key5 
   (p_partner_submitted_by_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A proposal may be for more than one project or key ministry.
-- GROUP: partner
CREATE TABLE p_foundation_proposal_detail (

  p_foundation_partner_key_n bigint NOT NULL,
  p_foundation_proposal_key_i integer NOT NULL,
  p_proposal_detail_id_i integer NOT NULL,
    -- Used when the proposal is for a key ministry
  p_key_ministry_key_n bigint,
    -- Ledger is part of the primary key for motivations.
  p_project_ledger_number_i integer,
    -- Used when the proposal is for a project
  p_project_motivation_group_c varchar(16),
    -- Used when the proposal is for a project
  p_project_motivation_detail_c varchar(16),
    -- The area that the project or key ministry is for
  p_area_partner_key_n bigint,
    -- The field that the project or key ministry is for if it applies
  p_field_partner_key_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_proposal_detail_pk
    PRIMARY KEY (p_foundation_partner_key_n,p_foundation_proposal_key_i,p_proposal_detail_id_i),
  CONSTRAINT p_area_fk1
    FOREIGN KEY (p_area_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_detail_proposal_fk1
    FOREIGN KEY (p_foundation_partner_key_n,p_foundation_proposal_key_i)
    REFERENCES p_foundation_proposal(p_foundation_partner_key_n,p_foundation_proposal_key_i),
  CONSTRAINT p_field_fk1
    FOREIGN KEY (p_field_partner_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_proposal_ministry_fk1
    FOREIGN KEY (p_key_ministry_key_n)
    REFERENCES p_unit(p_partner_key_n),
  CONSTRAINT p_proposal_motivation_fk1
    FOREIGN KEY (p_project_ledger_number_i,p_project_motivation_group_c,p_project_motivation_detail_c)
    REFERENCES a_motivation_detail(a_ledger_number_i,a_motivation_group_code_c,a_motivation_detail_code_c),
  CONSTRAINT p_detail_proposal_fk2
    FOREIGN KEY (p_foundation_partner_key_n)
    REFERENCES p_foundation(p_partner_key_n),
  CONSTRAINT p_foundation_proposal_detail_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_proposal_detail_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_proposal_detail_pk0 
   (p_foundation_partner_key_n,p_foundation_proposal_key_i,p_proposal_detail_id_i),
KEY inx_p_area_fk1_key1 
   (p_area_partner_key_n),
KEY inx_p_detail_proposal_fk1_key2 
   (p_foundation_partner_key_n,p_foundation_proposal_key_i),
KEY inx_p_field_fk1_key3 
   (p_field_partner_key_n),
KEY inx_p_proposal_ministry_fk1_key4 
   (p_key_ministry_key_n),
KEY inx__proposal_motivation_fk1_key5 
   (p_project_ledger_number_i,p_project_motivation_group_c,p_project_motivation_detail_c),
KEY inx_p_detail_proposal_fk2_key6 
   (p_foundation_partner_key_n),
KEY inx_ion_proposal_detail_fkcr_key7 
   (s_created_by_c),
KEY inx_ion_proposal_detail_fkmd_key8 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The month and day of reviews so that proposals can be sent in time
-- GROUP: partner
CREATE TABLE p_foundation_deadline (

  p_foundation_partner_key_n bigint NOT NULL,
  p_foundation_deadline_key_i integer NOT NULL,
    -- The month number
  p_deadline_month_i integer,
    -- The day number
  p_deadline_day_i integer,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_deadline_pk
    PRIMARY KEY (p_foundation_partner_key_n,p_foundation_deadline_key_i),
  CONSTRAINT p_foundation_deadline_fk1
    FOREIGN KEY (p_foundation_partner_key_n)
    REFERENCES p_foundation(p_partner_key_n),
  CONSTRAINT p_foundation_deadline_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_foundation_deadline_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_deadline_pk0 
   (p_foundation_partner_key_n,p_foundation_deadline_key_i),
KEY inx__foundation_deadline_fk1_key1 
   (p_foundation_partner_key_n),
KEY inx_foundation_deadline_fkcr_key2 
   (s_created_by_c),
KEY inx_foundation_deadline_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores the definition of each workflow. These may be pre-created, or created by the user.
-- GROUP: sysman
CREATE TABLE s_workflow_definition (

    -- Workflow ID
  s_workflow_id_i integer NOT NULL,
  s_name_c varchar(60) NOT NULL,
  s_description_c varchar(600),
    -- List of all modules within which workflow should be available
  s_module_list_c varchar(200),
    -- If workflow is dependent on a particular data item what sort of data item is it (eg. partner key, application, extract).
  s_type_of_shared_data_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_definition_pk
    PRIMARY KEY (s_workflow_id_i),
  CONSTRAINT s_workflow_definition_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_definition_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_definition_pk0 
   (s_workflow_id_i),
KEY inx_workflow_definition_fkcr_key1 
   (s_created_by_c),
KEY inx_workflow_definition_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Which users have access to this workflow
-- GROUP: sysman
CREATE TABLE s_workflow_user (

    -- Workflow ID
  s_workflow_id_i integer NOT NULL,
    -- The user that has access to the workflow
  s_user_id_c varchar(20) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_user_pk
    PRIMARY KEY (s_workflow_id_i,s_user_id_c),
  CONSTRAINT s_workflow_user_fk1
    FOREIGN KEY (s_workflow_id_i)
    REFERENCES s_workflow_definition(s_workflow_id_i),
  CONSTRAINT s_workflow_user_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_user_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_user_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_user_pk0 
   (s_workflow_id_i,s_user_id_c),
KEY inx_s_workflow_user_fk1_key1 
   (s_workflow_id_i),
KEY inx_s_workflow_user_fk2_key2 
   (s_user_id_c),
KEY inx_s_workflow_user_fkcr_key3 
   (s_created_by_c),
KEY inx_s_workflow_user_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Which groups have access to this workflow
-- GROUP: sysman
CREATE TABLE s_workflow_group (

    -- Workflow ID
  s_workflow_id_i integer NOT NULL,
    -- The group that has access to the workflow
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_group_pk
    PRIMARY KEY (s_workflow_id_i,s_group_id_c,s_group_unit_key_n),
  CONSTRAINT s_workflow_group_fk1
    FOREIGN KEY (s_workflow_id_i)
    REFERENCES s_workflow_definition(s_workflow_id_i),
  CONSTRAINT s_workflow_group_fk2
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_workflow_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_group_pk0 
   (s_workflow_id_i,s_group_id_c,s_group_unit_key_n),
KEY inx_s_workflow_group_fk1_key1 
   (s_workflow_id_i),
KEY inx_s_workflow_group_fk2_key2 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_workflow_group_fkcr_key3 
   (s_created_by_c),
KEY inx_s_workflow_group_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Stores each step of the workflow
-- GROUP: sysman
CREATE TABLE s_workflow_step (

    -- Workflow ID
  s_workflow_id_i integer NOT NULL,
    -- Indicates position within workflow (ie. 1 means first step, etc)
  s_step_number_i integer NOT NULL,
    -- Identifier for the particular function
  s_function_id_c varchar(30) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_step_pk
    PRIMARY KEY (s_workflow_id_i,s_step_number_i),
  CONSTRAINT s_workflow_step_fk1
    FOREIGN KEY (s_workflow_id_i)
    REFERENCES s_workflow_definition(s_workflow_id_i),
  CONSTRAINT s_workflow_step_fk2
    FOREIGN KEY (s_function_id_c)
    REFERENCES s_function(s_function_id_c),
  CONSTRAINT s_workflow_step_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_step_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_step_pk0 
   (s_workflow_id_i,s_step_number_i),
KEY inx_s_workflow_step_fk1_key1 
   (s_workflow_id_i),
KEY inx_s_workflow_step_fk2_key2 
   (s_function_id_c),
KEY inx_s_workflow_step_fkcr_key3 
   (s_created_by_c),
KEY inx_s_workflow_step_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Represents a relationship between two particular functions. Eg. Partner Find and Partner Edit
-- GROUP: sysman
CREATE TABLE s_function_relationship (

    -- Identifier for one particular function
  s_function_1_c varchar(30) NOT NULL,
    -- Identifier for related function
  s_function_2_c varchar(30) NOT NULL,
    -- Code to run if a workflow contains function_1 and function_2 as adjacent steps. This code will provide the glue to connect the two steps.
  s_code_to_run_c varchar(400),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_function_relationship_pk
    PRIMARY KEY (s_function_1_c,s_function_2_c),
  CONSTRAINT s_function_relationship_fk1
    FOREIGN KEY (s_function_1_c)
    REFERENCES s_function(s_function_id_c),
  CONSTRAINT s_function_relationship_fk2
    FOREIGN KEY (s_function_2_c)
    REFERENCES s_function(s_function_id_c),
  CONSTRAINT s_function_relationship_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_function_relationship_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_function_relationship_pk0 
   (s_function_1_c,s_function_2_c),
KEY inx_unction_relationship_fk1_key1 
   (s_function_1_c),
KEY inx_unction_relationship_fk2_key2 
   (s_function_2_c),
KEY inx_nction_relationship_fkcr_key3 
   (s_created_by_c),
KEY inx_nction_relationship_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- A running instance of a defined workflow
-- GROUP: sysman
CREATE TABLE s_workflow_instance (

    -- Workflow Instance ID
  s_workflow_instance_id_i integer NOT NULL,
    -- Workflow ID
  s_workflow_id_i integer NOT NULL,
    -- If there is a particular data item required through the workflow what is it. Eg. if a Partner Key is required what is that Partner Key
  s_key_data_item_c varchar(200),
    -- The type of the shared data item (eg. Partner Key)
  s_key_data_item_type_c varchar(200),
    -- Was this instance generated by the System rather than the user
  s_system_generated_l boolean,
    -- Has the workflow been completed
  s_complete_l boolean,
    -- User entered comment or note attached to the workflow
  s_note_c varchar(600),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_instance_pk
    PRIMARY KEY (s_workflow_instance_id_i),
  CONSTRAINT s_workflow_instance_fk1
    FOREIGN KEY (s_workflow_id_i)
    REFERENCES s_workflow_definition(s_workflow_id_i),
  CONSTRAINT s_workflow_instance_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_instance_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_instance_pk0 
   (s_workflow_instance_id_i),
KEY inx_s_workflow_instance_fk1_key1 
   (s_workflow_id_i),
KEY inx_s_workflow_instance_fkcr_key2 
   (s_created_by_c),
KEY inx_s_workflow_instance_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Each step of the running workflow, including the status of that step
-- GROUP: sysman
CREATE TABLE s_workflow_instance_step (

    -- Workflow Instance ID
  s_workflow_instance_id_i integer NOT NULL,
    -- Indicates current position within workflow instance
  s_step_number_i integer NOT NULL,
    -- Has this step been completed
  s_complete_l boolean,
    -- User who ran or is running this step
  s_user_id_c varchar(20) NOT NULL,
    -- Any output parameters from this step
  s_output_parameters_c varchar(200),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_workflow_instance_step_pk
    PRIMARY KEY (s_workflow_instance_id_i,s_step_number_i),
  CONSTRAINT s_workflow_instance_step_fk1
    FOREIGN KEY (s_workflow_instance_id_i)
    REFERENCES s_workflow_instance(s_workflow_instance_id_i),
  CONSTRAINT s_workflow_instance_step_fk2
    FOREIGN KEY (s_user_id_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_instance_step_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_workflow_instance_step_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_workflow_instance_step_pk0 
   (s_workflow_instance_id_i,s_step_number_i),
KEY inx_rkflow_instance_step_fk1_key1 
   (s_workflow_instance_id_i),
KEY inx_rkflow_instance_step_fk2_key2 
   (s_user_id_c),
KEY inx_kflow_instance_step_fkcr_key3 
   (s_created_by_c),
KEY inx_kflow_instance_step_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Volume to either represent a file directory or a volume relative to a parent volume
-- GROUP: sysman
CREATE TABLE s_volume (

    -- Name (Identifier) for the volume
  s_name_c varchar(160) NOT NULL,
    -- Name of the parent volume (use either Path or Parent Volume)
  s_parent_volume_name_c varchar(160),
    -- Path information (use either Path or Parent Volume)
  s_path_c varchar(4096),
    -- Comment for the user
  s_comment_c varchar(1000),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_volume_pk
    PRIMARY KEY (s_name_c),
  CONSTRAINT s_volume_fk1
    FOREIGN KEY (s_parent_volume_name_c)
    REFERENCES s_volume(s_name_c),
  CONSTRAINT s_volume_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_volume_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_file_info_fk2_ref0 
   (s_name_c),
UNIQUE KEY inx_s_volume_pk1 
   (s_name_c),
KEY inx_s_volume_fk1_key2 
   (s_parent_volume_name_c),
KEY inx_s_volume_fkcr_key3 
   (s_created_by_c),
KEY inx_s_volume_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Store information about the physical location of a file linked into Petra
-- GROUP: partner
CREATE TABLE p_file_info (

  p_key_n bigint NOT NULL,
    -- Partner key that the file is linked with (needed for performance reasons)
  p_partner_key_n bigint NOT NULL,
    -- Volume this file can be found in (use either volume or path)
  s_volume_name_c varchar(160),
    -- Path this file can be found in (use either volume or path)
  p_path_c varchar(4096),
    -- File name this file can be found under in given volume or path
  p_file_name_c varchar(1024),
    -- Internal name of the file, different from actual file name
  p_name_c varchar(160),
    -- Information about the file type
  p_file_type_c varchar(100),
    -- Comment for the user
  p_comment_c varchar(1000),
    -- Indicates whether or not the file info has restricted access. If it does then the access will be controlled by s_group_file_info.
  p_restricted_l boolean DEFAULT 0,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_file_info_pk
    PRIMARY KEY (p_key_n),
  CONSTRAINT p_file_info_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_file_info_fk2
    FOREIGN KEY (s_volume_name_c)
    REFERENCES s_volume(s_name_c),
  CONSTRAINT p_file_info_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_file_info_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_partner_graphic_fk2_ref0 
   (p_key_n),
UNIQUE KEY inx_p_file_info_pk1 
   (p_key_n),
KEY inx_p_file_info_fk2_key2 
   (s_volume_name_c),
KEY inx_p_file_info_fkcr_key3 
   (s_created_by_c),
KEY inx_p_file_info_fkmd_key4 
   (s_modified_by_c),
KEY p_file_info_k0 
   (p_partner_key_n),
KEY p_file_info_k21 
   (s_volume_name_c,p_file_name_c(50)),
KEY p_file_info_k32 
   (p_path_c(50),p_file_name_c(50))
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Any graphic associated to a partner can be stored here.
-- GROUP: partner
CREATE TABLE p_partner_graphic (

    -- This is the partner key assigned to each partner. It consists of the fund id followed by a computer generated six digit number.
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
  p_file_info_key_n bigint NOT NULL,
    -- The label for the graphic related to the partner.
  p_graphic_label_c varchar(64),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_graphic_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT p_partner_graphic_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_graphic_fk2
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT p_partner_graphic_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_graphic_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_graphic_pk0 
   (p_file_info_key_n),
KEY inx_p_partner_graphic_fk1_key1 
   (p_partner_key_n),
KEY inx_p_partner_graphic_fkcr_key2 
   (s_created_by_c),
KEY inx_p_partner_graphic_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link file with a partner
-- GROUP: partner
CREATE TABLE p_partner_file (

  p_file_info_key_n bigint NOT NULL,
    -- Partner key that the file is linked with (needed for performance reasons)
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_file_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT p_partner_file_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_file_fk2
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT p_partner_file_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_file_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_file_pk0 
   (p_file_info_key_n),
KEY inx_p_partner_file_fkcr_key1 
   (s_created_by_c),
KEY inx_p_partner_file_fkmd_key2 
   (s_modified_by_c),
KEY p_partner_file_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link file with a person
-- GROUP: partner
CREATE TABLE pm_person_file (

  p_file_info_key_n bigint NOT NULL,
    -- Partner key that the file is linked with (needed for performance reasons)
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_person_file_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT pm_person_file_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_person(p_partner_key_n),
  CONSTRAINT pm_person_file_fk2
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT pm_person_file_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_person_file_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_person_file_pk0 
   (p_file_info_key_n),
KEY inx_pm_person_file_fkcr_key1 
   (s_created_by_c),
KEY inx_pm_person_file_fkmd_key2 
   (s_modified_by_c),
KEY p_person_file_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link file with a partner contact
-- GROUP: partner
CREATE TABLE p_partner_contact_file (

  p_file_info_key_n bigint NOT NULL,
    -- Partner key that the file is linked with (needed for performance reasons)
  p_partner_key_n bigint NOT NULL,
    -- identifying key for Partner Contact
  p_contact_id_i bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_partner_contact_file_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT p_partner_contact_file_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_partner_contact_file_fk2
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT p_partner_contact_file_fk3
    FOREIGN KEY (p_contact_id_i)
    REFERENCES p_contact_log(p_contact_log_id_i),
  CONSTRAINT p_partner_contact_file_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_partner_contact_file_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_partner_contact_file_pk0 
   (p_file_info_key_n),
KEY inx_partner_contact_file_fk3_key1 
   (p_contact_id_i),
KEY inx_artner_contact_file_fkcr_key2 
   (s_created_by_c),
KEY inx_artner_contact_file_fkmd_key3 
   (s_modified_by_c),
KEY p_partner_contact_file_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link file with a personal document
-- GROUP: personnel
CREATE TABLE pm_document_file (

  p_file_info_key_n bigint NOT NULL,
    -- This is the key that tell what site created this document
  p_site_key_n bigint DEFAULT 0 NOT NULL,
    -- Key to identify the document
  pm_document_key_n bigint DEFAULT 0 NOT NULL,
    -- Partner key that the file is linked with (needed for performance reasons to find all records linked with a partner)
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_document_file_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT pm_document_file_fk1
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT pm_document_file_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT pm_document_file_fk3
    FOREIGN KEY (p_site_key_n,pm_document_key_n)
    REFERENCES pm_document(p_site_key_n,pm_document_key_n),
  CONSTRAINT pm_document_file_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_document_file_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_document_file_pk0 
   (p_file_info_key_n),
KEY inx_pm_document_file_fk3_key1 
   (p_site_key_n,pm_document_key_n),
KEY inx_pm_document_file_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_document_file_fkmd_key3 
   (s_modified_by_c),
KEY pm_document_file_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Link file with an application
-- GROUP: personnel
CREATE TABLE pm_application_file (

  p_file_info_key_n bigint NOT NULL,
    -- Partner key that the file is linked with
  p_partner_key_n bigint NOT NULL,
    -- Key to uniquely identify application
  pm_application_key_i integer,
    -- Partner key of office by which application was entered.
  pm_registration_office_n bigint,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT pm_application_file_pk
    PRIMARY KEY (p_file_info_key_n),
  CONSTRAINT pm_application_file_fk1
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT pm_application_file_fk2
    FOREIGN KEY (p_partner_key_n,pm_application_key_i,pm_registration_office_n)
    REFERENCES pm_general_application(p_partner_key_n,pm_application_key_i,pm_registration_office_n),
  CONSTRAINT pm_application_file_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT pm_application_file_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_pm_application_file_pk0 
   (p_file_info_key_n),
KEY inx_pm_application_file_fk2_key1 
   (p_partner_key_n,pm_application_key_i,pm_registration_office_n),
KEY inx_pm_application_file_fkcr_key2 
   (s_created_by_c),
KEY inx_pm_application_file_fkmd_key3 
   (s_modified_by_c),
KEY pm_application_file_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Set up a Group for use in Default Volumes
-- GROUP: sysman
CREATE TABLE s_volume_partner_group (

    -- Group Name this default volume applies to
  s_name_c varchar(60) NOT NULL,
    -- Description
  s_description_c varchar(160),
    -- Comment field
  s_comment_c varchar(600),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_volume_partner_group_pk
    PRIMARY KEY (s_name_c),
  CONSTRAINT s_volume_partner_group_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_volume_partner_group_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx__default_file_volume_fk1_ref0 
   (s_name_c),
UNIQUE KEY inx_s_volume_partner_group_pk1 
   (s_name_c),
KEY inx_olume_partner_group_fkcr_key2 
   (s_created_by_c),
KEY inx_olume_partner_group_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Set Default Volume for a user group in a specific area
-- GROUP: sysman
CREATE TABLE s_default_file_volume (

    -- Group Name this default volume applies to
  s_group_name_c varchar(60) NOT NULL,
    -- Area this default volume applies to (e.g. Partner, Application, Contacts, ...)
  s_area_c varchar(60) NOT NULL,
    -- Default Volume for combination of group and area
  s_volume_name_c varchar(160),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_default_file_volume_pk
    PRIMARY KEY (s_group_name_c,s_area_c),
  CONSTRAINT s_default_file_volume_fk1
    FOREIGN KEY (s_group_name_c)
    REFERENCES s_volume_partner_group(s_name_c),
  CONSTRAINT s_default_file_volume_fk2
    FOREIGN KEY (s_volume_name_c)
    REFERENCES s_volume(s_name_c),
  CONSTRAINT s_default_file_volume_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_default_file_volume_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_default_file_volume_pk0 
   (s_group_name_c,s_area_c),
KEY inx__default_file_volume_fk1_key1 
   (s_group_name_c),
KEY inx__default_file_volume_fk2_key2 
   (s_volume_name_c),
KEY inx_default_file_volume_fkcr_key3 
   (s_created_by_c),
KEY inx_default_file_volume_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Assign Partners to a Group for use in Default Volumes
-- GROUP: sysman
CREATE TABLE s_volume_partner_group_partner (

    -- Group Name this default volume applies to
  s_group_name_c varchar(60) NOT NULL,
    -- Partner key (one partner can be in several groups)
  p_partner_key_n bigint NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_volume_partner_group_ptn_pk
    PRIMARY KEY (s_group_name_c,p_partner_key_n),
  CONSTRAINT s_volume_partner_group_partner_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_volume_partner_group_partner_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_volume_partner_group_ptn_fk1
    FOREIGN KEY (s_group_name_c)
    REFERENCES s_volume_partner_group(s_name_c),
  CONSTRAINT s_volume_partner_group_ptn_fk2
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
UNIQUE KEY inx__volume_partner_group_ptn_pk0 
   (s_group_name_c,p_partner_key_n),
KEY inx_rtner_group_partner_fkcr_key1 
   (s_created_by_c),
KEY inx_rtner_group_partner_fkmd_key2 
   (s_modified_by_c),
KEY inx_me_partner_group_ptn_fk1_key3 
   (s_group_name_c),
KEY inx_me_partner_group_ptn_fk2_key4 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Associates a Group with an external document (file info). This may be read, write, delete
-- GROUP: sysman
CREATE TABLE s_group_file_info (

    -- identifies a system group
  s_group_id_c varchar(20) NOT NULL,
    -- Field that the group belongs to
  s_group_unit_key_n bigint DEFAULT 0 NOT NULL,
  p_file_info_key_n bigint NOT NULL,
    -- Control read access to the file information
  s_read_access_l boolean,
    -- Control write access to the file information
  s_write_access_l boolean,
    -- Control delete access to the file information
  s_delete_access_l boolean,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT s_group_file_info_pk
    PRIMARY KEY (s_group_id_c,s_group_unit_key_n,p_file_info_key_n),
  CONSTRAINT s_group_file_info_fk1
    FOREIGN KEY (s_group_id_c,s_group_unit_key_n)
    REFERENCES s_group(s_group_id_c,s_unit_key_n),
  CONSTRAINT s_group_file_info_fk2
    FOREIGN KEY (p_file_info_key_n)
    REFERENCES p_file_info(p_key_n),
  CONSTRAINT s_group_file_info_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT s_group_file_info_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_s_group_file_info_pk0 
   (s_group_id_c,s_group_unit_key_n,p_file_info_key_n),
KEY inx_s_group_file_info_fk1_key1 
   (s_group_id_c,s_group_unit_key_n),
KEY inx_s_group_file_info_fk2_key2 
   (p_file_info_key_n),
KEY inx_s_group_file_info_fkcr_key3 
   (s_created_by_c),
KEY inx_s_group_file_info_fkmd_key4 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- All possible channels to get data changes and consent
-- GROUP: partner
CREATE TABLE p_consent_channel (

    -- Unique key for channel
  p_channel_code_c varchar(40) NOT NULL,
    -- international name for i18n translation
  p_name_c varchar(128) DEFAULT '0' NOT NULL,
    -- comment for a channel
  p_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_consent_channel_pk
    PRIMARY KEY (p_channel_code_c),
  CONSTRAINT p_consent_channel_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_consent_channel_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_p_consent_history_fk2_ref0 
   (p_channel_code_c),
UNIQUE KEY inx_p_consent_channel_pk1 
   (p_channel_code_c),
KEY inx_p_consent_channel_fkcr_key2 
   (s_created_by_c),
KEY inx_p_consent_channel_fkmd_key3 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Keeps track of all data changes for GDPR
-- GROUP: partner
CREATE TABLE p_consent_history (

    -- Incremental key for entrys
  p_entry_id_i bigint DEFAULT 0 NOT NULL,
    -- Key for partner
  p_partner_key_n bigint DEFAULT 0 NOT NULL,
    -- Type for Data saved
  p_type_c varchar(128) NOT NULL,
    -- Value for type key
  p_value_c varchar(1024),
    -- Date the consent was given
  p_consent_date_d date NOT NULL,
    -- Code of channel
  p_channel_code_c varchar(40) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_consent_history_pk
    PRIMARY KEY (p_entry_id_i),
  CONSTRAINT p_consent_history_fk1
    FOREIGN KEY (p_partner_key_n)
    REFERENCES p_partner(p_partner_key_n),
  CONSTRAINT p_consent_history_fk2
    FOREIGN KEY (p_channel_code_c)
    REFERENCES p_consent_channel(p_channel_code_c),
  CONSTRAINT p_consent_history_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_consent_history_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_consent_history_pk0 
   (p_entry_id_i),
KEY inx_p_consent_history_fk2_key1 
   (p_channel_code_c),
KEY inx_p_consent_history_fkcr_key2 
   (s_created_by_c),
KEY inx_p_consent_history_fkmd_key3 
   (s_modified_by_c),
KEY p_consent_history_k0 
   (p_partner_key_n)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- Purposes for consent, for which the data may be used
-- GROUP: partner
CREATE TABLE p_consent_purpose (

    -- Unique key for purpose
  p_purpose_code_c varchar(40) NOT NULL,
    -- international name for i18n translation
  p_name_c varchar(128) DEFAULT '0' NOT NULL,
    -- comment for a purpose
  p_comment_c varchar(512),
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_consent_purpose_pk
    PRIMARY KEY (p_purpose_code_c),
  CONSTRAINT p_consent_purpose_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_consent_purpose_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
UNIQUE KEY inx_p_consent_purpose_pk0 
   (p_purpose_code_c),
KEY inx_p_consent_purpose_fkcr_key1 
   (s_created_by_c),
KEY inx_p_consent_purpose_fkmd_key2 
   (s_modified_by_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

-- The history of consent for personal data
-- GROUP: partner
CREATE TABLE p_consent_history_permission (

    -- ID for entrys
  p_consent_history_entry_i bigint DEFAULT 0 NOT NULL,
    -- Code for purpose
  p_purpose_code_c varchar(40) NOT NULL,
    -- The date the record was created.
  s_date_created_d date,
    -- User ID of who created this record.
  s_created_by_c varchar(20),
    -- The date the record was modified.
  s_date_modified_d date,
    -- User ID of who last modified this record.
  s_modified_by_c varchar(20),
    -- This identifies the current version of the record.
  s_modification_id_t timestamp DEFAULT CURRENT_TIMESTAMP,
  CONSTRAINT p_consent_history_permission_pk
    PRIMARY KEY (p_consent_history_entry_i,p_purpose_code_c),
  CONSTRAINT p_consent_history_permission_fk1
    FOREIGN KEY (p_consent_history_entry_i)
    REFERENCES p_consent_history(p_entry_id_i),
  CONSTRAINT p_consent_history_permission_fk2
    FOREIGN KEY (p_purpose_code_c)
    REFERENCES p_consent_purpose(p_purpose_code_c),
  CONSTRAINT p_consent_history_permission_fkcr
    FOREIGN KEY (s_created_by_c)
    REFERENCES s_user(s_user_id_c),
  CONSTRAINT p_consent_history_permission_fkmd
    FOREIGN KEY (s_modified_by_c)
    REFERENCES s_user(s_user_id_c),
KEY inx_t_history_permission_fk1_key0 
   (p_consent_history_entry_i),
KEY inx_t_history_permission_fk2_key1 
   (p_purpose_code_c),
KEY inx__history_permission_fkcr_key2 
   (s_created_by_c),
KEY inx__history_permission_fkmd_key3 
   (s_modified_by_c),
UNIQUE KEY p_consent_history_permission_k0 
   (p_consent_history_entry_i,p_purpose_code_c)
) ENGINE=InnoDB
 CHARACTER SET UTF8
;

CREATE TABLE seq_application (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_application VALUES(NULL, -1);
CREATE TABLE seq_ap_document (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_ap_document VALUES(NULL, -1);
CREATE TABLE seq_contact (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_contact VALUES(NULL, -1);
CREATE TABLE seq_extract_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_extract_number VALUES(NULL, -1);
CREATE TABLE seq_location_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_location_number VALUES(NULL, -1);
CREATE TABLE seq_pe_evaluation_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_pe_evaluation_number VALUES(NULL, -1);
CREATE TABLE seq_report_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_report_number VALUES(NULL, -1);
CREATE TABLE seq_general_ledger_master (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_general_ledger_master VALUES(NULL, -1);
CREATE TABLE seq_budget (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_budget VALUES(NULL, -1);
CREATE TABLE seq_bank_details (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_bank_details VALUES(NULL, -1);
CREATE TABLE seq_document (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_document VALUES(NULL, -1);
CREATE TABLE seq_past_experience (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_past_experience VALUES(NULL, -1);
CREATE TABLE seq_staff_data (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_staff_data VALUES(NULL, -1);
CREATE TABLE seq_job (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_job VALUES(NULL, -1);
CREATE TABLE seq_job_assignment (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_job_assignment VALUES(NULL, -1);
CREATE TABLE seq_data_label (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_data_label VALUES(NULL, -1);
CREATE TABLE seq_foundation_proposal (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_foundation_proposal VALUES(NULL, -1);
CREATE TABLE seq_proposal_detail (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_proposal_detail VALUES(NULL, -1);
CREATE TABLE seq_form_letter_insert (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_form_letter_insert VALUES(NULL, -1);
CREATE TABLE seq_workflow (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_workflow VALUES(NULL, -1);
CREATE TABLE seq_file_info (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_file_info VALUES(NULL, -1);
CREATE TABLE seq_person_skill (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_person_skill VALUES(NULL, -1);
CREATE TABLE seq_booking (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_booking VALUES(NULL, -1);
CREATE TABLE seq_room_alloc (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_room_alloc VALUES(NULL, -1);
CREATE TABLE seq_ar_invoice (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_ar_invoice VALUES(NULL, -1);
CREATE TABLE seq_match_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_match_number VALUES(NULL, -1);
CREATE TABLE seq_statement_number (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_statement_number VALUES(NULL, -1);
CREATE TABLE seq_login_process_id (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_login_process_id VALUES(NULL, -1);
CREATE TABLE seq_report_template (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_report_template VALUES(NULL, -1);
CREATE TABLE seq_partner_attribute_category_index (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_partner_attribute_category_index VALUES(NULL, -1);
CREATE TABLE seq_partner_attribute_type_index (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_partner_attribute_type_index VALUES(NULL, -1);
CREATE TABLE seq_partner_attribute_index (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_partner_attribute_index VALUES(NULL, -1);
CREATE TABLE seq_data_entry_id (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_data_entry_id VALUES(NULL, -1);
CREATE TABLE seq_partner_reminder (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_partner_reminder VALUES(NULL, -1);
CREATE TABLE seq_partner_membership (sequence INTEGER AUTO_INCREMENT, dummy INTEGER, PRIMARY KEY(sequence));
INSERT INTO seq_partner_membership VALUES(NULL, -1);
